/******************************************************************************************************

 *  Rohde&Schwarz Spectrum Analyzer instrument driver
 *  Callbacks
 *
 *  Original Release: September 2008
 *  By: Jiri Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *

*****************************************************************************************************/

#include "rsrfsiggen.h"
#include "rsrfsiggen_utility.h"
#include <stdlib.h>

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

#pragma warning (disable:4100) // unreferenced formal parameter

/******************************************************************************************************
 * Function: rsrfsiggen_hexPattern_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_hexPattern_ReadCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_BinOrHexPatternRead(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_hexPattern_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_hexPattern_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
    return RsCore_SpecialCallback_HexPatternWrite(instrSession, repCapName, attr, value);
}

/******************************************************************************************************
 * Function: rsrfsiggen_binPattern_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_binPattern_ReadCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr)
{
    return RsCore_SpecialCallback_BinOrHexPatternRead(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_binPattern_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_binPattern_WriteCallback(ViSession          instrSession,
                                             ViConstString      repCapName,
                                             RsCoreAttributePtr attr,
                                             void*              value)
{
    return RsCore_SpecialCallback_BinPatternWrite(instrSession, repCapName, attr, value);
}

/******************************************************************************************************
 * Function: rsrfsiggen_noQuotedString_ReadCallback
 * Purpose:  Obsolete, same as RsCore_ReadCallback
*****************************************************************************************************/
ViStatus rsrfsiggen_noQuotedString_ReadCallback(ViSession          instrSession,
                                                ViConstString      repCapName,
                                                RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_noQuotedString_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_noQuotedString_WriteCallback(ViSession          instrSession,
                                                 ViConstString      repCapName,
                                                 RsCoreAttributePtr attr,
                                                 void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " ", value, "");
}

/******************************************************************************************************
 * Function: rsrfsiggen_seconds_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_seconds_WriteCallback(ViSession          instrSession,
                                          ViConstString      repCapName,
                                          RsCoreAttributePtr attr,
                                          void*              value)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  cmd = NULL;

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    // Build the command string
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    switch (attr->dataType)
    {
    case RS_VAL_REAL64:
        RsCore_StrCatDynamicViReal64(instrSession, &cmd, " ", *(ViReal64 *)value * 1000, NULL);
        checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_binCommaSeparatedPattern_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_binCommaSeparatedPattern_ReadCallback(ViSession          instrSession,
                                                          ViConstString      repCapName,
                                                          RsCoreAttributePtr attr)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  i = 0;
    ViChar   *cmd = NULL,

           *p2buff = NULL,
           buffer[RS_MAX_MESSAGE_BUF_SIZE],
           result[RS_MAX_MESSAGE_BUF_SIZE] = "";

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    // Build the command string
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    // --- Attribute data type is used for VISA I/O ---
    switch (attr->dataType)
    {
    case RS_VAL_STRING:
        checkErr(RsCore_QueryAttributeCommand(instrSession, attr, cmd, buffer));
        RsCore_TrimString(buffer, RS_VAL_TRIM_WHITESPACES_AND_ALL_QUOTES);

        p2buff = strtok(buffer, ",");
        while (p2buff != NULL)
        {
            strcat(result, p2buff);
            p2buff = strtok(NULL, ",");
            i++;
        }
        RsCore_TrimString(buffer, RS_VAL_TRIM_WHITESPACES);
        checkErr(RsCore_SetAttributeP2Value(instrSession, attr, result));
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }

Error:
    if (cmd)
        free(cmd);

    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_binCommaSeparatedPattern_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_binCommaSeparatedPattern_WriteCallback(ViSession          instrSession,
                                                           ViConstString      repCapName,
                                                           RsCoreAttributePtr attr,
                                                           void*              value)
{
    ViStatus                        error = VI_SUCCESS;
    ViInt32                         i, j = 0;
    ViChar*                         cmd = NULL,
    buffer[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' },
    result[RS_MAX_MESSAGE_BUF_SIZE] = { '\0' };
    ViInt32                         strLength, patternLength = 0;

    // If no command available, exit the function
    if (attr->command == NULL)
        return VI_SUCCESS;

    // Build the command string
    checkErr(RsCore_BuildAttrCommandString(instrSession, attr, repCapName, &cmd));

    strLength = (ViInt32)strlen((ViString)value);

    if (attr->id == RSRFSIGGEN_ATTR_GSM_FIXED_PATTERN)
        patternLength = 142;
    else if (attr->id == RSRFSIGGEN_ATTR_GSM_SYNC_PATTERN)
        patternLength = 41;
    else
        patternLength = strLength;

    (void)sscanf((ViString)value, "%s", buffer);

    for (i = 0; i < patternLength; i++)
    {
        if (i < strLength)
            result[j++] = buffer[i];
        else
            result[j++] = '0';

        result[j++] = ',';
    }

    if (j > 0) j--;
    result[j] = '\0';

    switch (attr->dataType)
    {
    case RS_VAL_STRING:
        RsCore_StrCatDynamicString(instrSession, &cmd, " ", result, NULL);
        checkErr(RsCore_WriteAttributeCommand(instrSession, attr, cmd));
        break;
    default:
        checkErr(RsCore_GenerateInvalidDataTypeError(instrSession, attr->dataType, NULL));
    }

Error:
    if (cmd) free(cmd);
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_empty_ReadCallback
 * Purpose:  This function overrides standard callback function. Returns value
 *           from cache
*****************************************************************************************************/
ViStatus rsrfsiggen_empty_ReadCallback(ViSession          instrSession,
                                       ViConstString      repCapName,
                                       RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rsrfsiggen_empty_WriteCallback
 * Purpose:  This function overrides standard callback function. Does not write
 *           anything to instrument, just updates the cache
*****************************************************************************************************/
ViStatus rsrfsiggen_empty_WriteCallback(ViSession          instrSession,
                                        ViConstString      repCapName,
                                        RsCoreAttributePtr attr,
                                        void*              value)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rsrfsiggen_logging_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_logging_WriteCallback(ViSession          instrSession,
                                          ViConstString      repCapName,
                                          RsCoreAttributePtr attr,
                                          void*              value)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rsrfsiggen_logging_ReadCallback
 * Purpose:  This function overrides standard callback function. Returns value
 *           from cache
*****************************************************************************************************/
ViStatus rsrfsiggen_logging_ReadCallback(ViSession          instrSession,
                                         ViConstString      repCapName,
                                         RsCoreAttributePtr attr)
{
    return VI_SUCCESS;
}

/******************************************************************************************************
 * Function: rsrfsiggen_hex_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_hex_ReadCallback(ViSession          instrSession,
                                     ViConstString      repCapName,
                                     RsCoreAttributePtr attr)
{
   return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_hex_WriteCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_hex_WriteCallback(ViSession          instrSession,
                                      ViConstString      repCapName,
                                      RsCoreAttributePtr attr,
                                      void*              value)
{
   return RsCore_SpecialCallback_FormatNumberHexWrite(instrSession, repCapName, attr, value);
}

/******************************************************************************************************
 * Function: rsrfsiggen_hex_Int64_ReadCallback
 * Purpose:  This function overrides standard callback function.
*****************************************************************************************************/
ViStatus rsrfsiggen_hex_Int64_ReadCallback(ViSession          instrSession,
                                           ViConstString      repCapName,
                                           RsCoreAttributePtr attr)
{
   return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_hex_Int64_WriteCallback
 * Purpose:  This function overrides standard callback function.
 *****************************************************************************************************/
ViStatus rsrfsiggen_hex_Int64_WriteCallback(ViSession          instrSession,
                                            ViConstString      repCapName,
                                            RsCoreAttributePtr attr,
                                            void*              value)
{
   return RsCore_SpecialCallback_FormatNumberHexWrite(instrSession, repCapName, attr, value);
}

/******************************************************************************************************
 * Function: rsrfsiggen_doubleQuotedString_ReadCallback
 * Purpose:  This function overrides standard callback function. Reads string returned with apostrophes
 *****************************************************************************************************/
ViStatus rsrfsiggen_doubleQuotedString_ReadCallback(ViSession          instrSession,
                                                    ViConstString      repCapName,
                                                    RsCoreAttributePtr attr)
{
    return RsCore_ReadCallback(instrSession, repCapName, attr);
}

/******************************************************************************************************
 * Function: rsrfsiggen_doubleQuotedString_WriteCallback
 * Purpose:  This function overrides standard callback function. Writes parameter in double quotes
 *****************************************************************************************************/
ViStatus rsrfsiggen_doubleQuotedString_WriteCallback(ViSession          instrSession,
                                                     ViConstString      repCapName,
                                                     RsCoreAttributePtr attr,
                                                     void*              value)
{
    return RsCore_SpecialCallback_SurroundParamWrite(instrSession, repCapName, attr, " \"", value, "\"");
}
