/******************************************************************************************************

 *------------------------ Utility  -----------------------------------------*
 *****************************************************************************/
#include <float.h>
#include <math.h>
#include "rsrfsiggen.h"
#include "rsrfsiggen_internal.h"

#pragma warning( disable : 4152 )

/******************************************************************************************************

                                            Arrays

 *****************************************************************************************************/

static ViString pathArr[] = {"", "A", "B", "C", "D", NULL};
static ViString satNavStandardArr[] = {"GPS", "GAL", "GLON", "BEID", "QZSS", NULL};

/******************************************************************************************************
    rsrfsiggen_InitAttributes
    This function inits attributes to the desired values if needed.
 *****************************************************************************************************/
ViStatus rsrfsiggen_InitAttributes(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    // - Driver Identification -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 0, RSRFSIGGEN_SPECIFIC_DRIVER_DESCRIPTION));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_PREFIX, 0, RSRFSIGGEN_SPECIFIC_DRIVER_PREFIX));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_LOCATOR, 0, RSRFSIGGEN_SPECIFIC_DRIVER_LOCATOR));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_VENDOR, 0, RSRFSIGGEN_SPECIFIC_DRIVER_VENDOR));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MAJOR_VERSION, 0, RSRFSIGGEN_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_CLASS_SPEC_MINOR_VERSION, 0, RSRFSIGGEN_CLASS_SPEC_MINOR_VERSION));

    // - Driver Capabilities -
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_SUPPORTED_INSTRUMENT_MODELS, 0, RSRFSIGGEN_SUPPORTED_INSTRUMENT_MODELS));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_GROUP_CAPABILITIES, 0, RSRFSIGGEN_GROUP_CAPABILITIES));
    checkErr(RsCore_SetAttributeViString(instrSession, "", RS_ATTR_FUNCTION_CAPABILITIES, 0, RSRFSIGGEN_FUNCTION_CAPABILITIES));

    // - Version Info -
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MAJOR_VERSION, 0, RSRFSIGGEN_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_VERSION, 0, RSRFSIGGEN_MINOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_SPECIFIC_DRIVER_MINOR_MINOR_VERSION, 0, RSRFSIGGEN_MINOR_MINOR_VERSION));

    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MAJOR_VERSION, 0, RSRFSIGGEN_CLASS_SPEC_MAJOR_VERSION));
    checkErr(RsCore_SetAttributeViInt32(instrSession, "", RS_ATTR_CLASS_DRIVER_MINOR_VERSION, 0, RSRFSIGGEN_CLASS_SPEC_MINOR_VERSION));

    checkErr(RsCore_SetSpecificDriverRevision(instrSession, RS_ATTR_SPECIFIC_DRIVER_REVISION));

Error:
    return error;
}

/******************************************************************************************************
    rsrfsiggen_DefaultInstrSetup
    This function sends a default setup to the instrument. The
    rsrfsiggen_reset function calls this function. The
    rsrfsiggen_RsInit function calls this function when the
    user passes VI_FALSE for the reset parameter. This function is
    useful for configuring settings that other instrument driver
    functions require.

    Note:  Call this function only when the session is locked.
    *****************************************************************************************************/
ViStatus rsrfsiggen_DefaultInstrSetup(ViSession instrSession)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));

    // Set all the attributes to the default state. Do not update inherent attributes!
    checkErr(RsCore_ApplyAttributeDefaults(instrSession, VI_FALSE));

    // Init attributes
    checkErr(rsrfsiggen_InitAttributes(instrSession));

    rsSession->fastSweepInstrument = 0;
    checkErr(RsCore_ResetRegistersEseSre(instrSession));

Error:
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_CheckStatus
 * Purpose:  This function checks the status of the instrument to detect
 *           whether the instrument has encountered an error. This function
 *           is called at the end of most exported driver functions. If the
 *           instrument reports an error, this function returns
 *           RS_ERROR_INSTRUMENT_SPECIFIC. The user can set the
 *           rsrfsiggen_ATTR_QUERY_INSTRUMENT_STATUS attribute to VI_FALSE to disable this
 *           check and increase execution speed.
 *
 *           Note:  Call this function only when the session is locked.
 *****************************************************************************************************/
ViStatus rsrfsiggen_CheckStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    if (RsCore_QueryInstrStatus(instrSession) && !RsCore_Simulating(instrSession))
    {
        checkErr(RsCore_CheckStatus(instrSession, VI_SUCCESS));
    }

Error:
    return error;
}

/******************************************************************************************************
* Function: rsrfsiggen_SetOPCTimeout
* Purpose:  Sets OPC timeout
*****************************************************************************************************/
ViStatus rsrfsiggen_SetOPCTimeout(ViSession instrSession,
                                  ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/******************************************************************************************************
* Function: rsrfsiggen_GetOPCTimeout
* Purpose:  Return OPC timeout
*****************************************************************************************************/
ViStatus rsrfsiggen_GetOPCTimeout(ViSession instrSession,
                                  ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, timeout));

Error:
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_pathRepCap
 * Purpose:  This function checks instance repeated capability and
 *           creates proper string.
 *****************************************************************************************************/
ViStatus rsrfsiggen_pathRepCap(ViSession instrSession,
                               ViInt32   path,
                               ViChar    repCap[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (strlen(repCap) == 0)
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", pathArr[path]);
    else
        snprintf(repCap, RS_REPCAP_BUF_SIZE - strlen(repCap), "%s,%s", repCap, pathArr[path]);

Error:
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_satNavRepCap
 * Purpose:  This function checks SatNav standard repeated capability and
 *           creates proper string.
 *****************************************************************************************************/
ViStatus rsrfsiggen_satNavRepCap(ViSession instrSession,
                                 ViInt32   standard,
                                 ViChar    repCap[])
{  
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    if (strlen(repCap) == 0)
        snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s", satNavStandardArr[standard]);
    else
        snprintf(repCap, RS_REPCAP_BUF_SIZE - strlen(repCap), "%s,%s", repCap, satNavStandardArr[standard]);

Error:
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_satNavSVIDRepCap
 * Purpose:  This function checks SatNav SV ID repeated capability and
 *           creates proper string.
 *****************************************************************************************************/
ViStatus rsrfsiggen_satNavSVIDRepCap(ViSession instrSession,
                                     ViInt32   svid,
                                     ViChar    repCap[])
{  
    ViStatus error = VI_SUCCESS;
    ViChar   thisRepCap[RS_REPCAP_BUF_SIZE] = "";
    ViStatus repCapCount = 0;
    const    ViAttr attr = RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_REFERENCE_DATE;
    const    ViString repCapNameId = "SatNavSVID";

    if ((193 <= svid) && (svid <= 197))
    {
        svid -= 142; // 193 -> 51, etc
    }
    else
    {
        checkErr(RsCore_GetRepCapIdItemsCount(instrSession, repCapNameId, &repCapCount));
        checkErr(RsCore_InvalidViInt32Range(instrSession, svid, 1, repCapCount));
    }

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attr, repCapNameId, svid - 1, RS_REPCAP_BUF_SIZE, thisRepCap));

    if (strlen(repCap) == 0)
        strcpy(repCap, thisRepCap);
    else
        snprintf(repCap, RS_REPCAP_BUF_SIZE - strlen(repCap), "%s,%s", repCap, thisRepCap);

Error:
    return error;
}

/******************************************************************************************************
 * Function: rsrfsiggen_QueryMeasStatus
 * Purpose:  This function queries measurement status
 *****************************************************************************************************/
ViStatus rsrfsiggen_QueryMeasStatus(ViSession instrSession,
                                    ViString  cmd,
                                    ViInt32*  mainState,
                                    ViInt32*  syncState,
                                    ViInt32*  resourcesState)
{
    ViStatus      error = VI_SUCCESS;
    ViChar        response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*       p2buf;
    ViConstString mainStateArr[] = {"OFF", "RDY", "RUN",NULL};
    ViConstString syncStateArr[] = {"PEND", "ADJ", "INV",NULL};
    ViConstString resStateArr[] = {"QUE", "ACT", "INV",NULL};

    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;
    *mainState = RsCore_FindStringIndex(mainStateArr, p2buf);
    if (*mainState < 0)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;
    *syncState = RsCore_FindStringIndex(syncStateArr, p2buf);
    if (*syncState < 0)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;

    p2buf = strtok(NULL, ",\n");
    if (p2buf == NULL)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;
    *resourcesState = RsCore_FindStringIndex(resStateArr, p2buf);
    if (*resourcesState < 0)
        return RSRFSIGGEN_ERROR_INCORRECT_RESPONSE;

Error:
    return error;
}

/******************************************************************************************************
* Function: rsrfsiggen_IsSMW
* Purpose:  Returns VI_TRUE if the current istrument model is SMW
*****************************************************************************************************/
ViBoolean rsrfsiggen_IsSMW(ViSession instrSession)
{
    return RsCore_IsInstrumentModel(instrSession, "SMW");
}

/******************************************************************************************************
* Function: rsrfsiggen_IsSMBV
* Purpose:  Returns VI_TRUE if the current istrument model is SMBV
*****************************************************************************************************/
ViBoolean rsrfsiggen_IsSMBV(ViSession instrSession)
{
    return RsCore_IsInstrumentModel(instrSession, "SMBV");
}

ViStatus rsrfsiggen_utility_error_message(ViSession instrSession,
                                          ViStatus  errorCode,
                                          ViChar    errorMessage[])
{  
    ViStatus error = VI_SUCCESS;

    static RsCoreStringValueTable errorTable =
    {
        RSRFSIGGEN_ERROR_CODES_AND_MSGS,
        {0, NULL}
    };

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:
    return error;
}

