/*****************************************************************************
 *  Rohde&Schwarz RF Signal Generator
 *
 *  Original Release: November 2012
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rsrfsiggen.h"
#include "rsrfsiggen_internal.h"
#include "rsrfsiggen_utility.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
#include <sys/timeb.h>
#include <sys/types.h>
#include <unistd.h>
#define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif
#if defined (_CVI_)
#include <utility.h>
#endif
#define BUFFER_SIZE             2048L    /* File I/O buffer size */

 /*****************************************************************************
  *- Value Definition and Range Tables ---------------------------------------*
  *****************************************************************************/
static ViString pathArr[] = { "","A","B","C","D", NULL };
static ViString ggSystemArr[] = { "GPS","GAL","GLON","BEID", NULL };
static ViString enabledArr[] = { "OFF","ON", NULL };
static ViString clockSourceArr[] = { "INT", "EGC1", "EGC2", "ELC", NULL };
static ViString clockSourceLegacyArr[] = { "INT", "EXT", "AINT", NULL };
static ViString triggerSourceArr[] = { "INT", "INTB", "EGT1", "EGT2", "EGC1", "EGC2",
                                      "ELTR", "ELCL", NULL
};
static ViString triggerSourceLegacyArr[] = { "INT", "OBAS", "EXT", "BEXT", NULL };
static ViString frequencyArr[] = { "5MHZ", "10MHZ", "13MHZ", NULL };
static ViString rxTxArr[] = { "RX", "TX", NULL };
static ViString longitudeDirArr[] = { "EAST","WEST", NULL };
static ViString latitudeDirArr[] = { "NORTH","SOUTH", NULL };
static ViString timeBasisArr[] = { "UTC","GPS","GST","GLO", NULL };
static ViString pathMatrixArr[] = { "AA", "AB", "AC", "AD", "AE", "AF", "AG", "AH",
                                    "BA", "BB", "BC", "BD", "BE", "BF", "BG", "BH",
                                    "CA", "CB", "CC", "CD", "CE", "CF", "CG", "CH",
                                    "DA", "DB", "DC", "DD", "DE", "DF", "DG", "DH",
                                    "EA", "EB", "EC", "ED", "EE", "EF", "EG", "EH",
                                    "FA", "FB", "FC", "FD", "FE", "FF", "FG", "FH",
                                    "GA", "GB", "GC", "GD", "GE", "GF", "GG", "GH",
                                    "HA", "HB", "HC", "HD", "HE", "HF", "HG", "HH", NULL
};
static ViString satNavStandardArr[] = { "GPS","GAL","GLON","BEID","QZSS", NULL };
static ViString satNavRFBandArr[] = { "L1","L2", NULL };
static ViString satNavTestScenarioArr[] = { "GSMSIG","GSMPER1","GSMPER2","GSMPER3","W3GSIG","W3GPER1","W3GPER2","W3GPER3",
        "W3G2M","W3G2S","LTESIGST1","LTEPER1ST1","LTEPER2ST1","LTEPER5ST1","W3GSIGST4",
        "W3GPER1ST4","W3GPER2ST4","W3GPER5ST4","LTESIGST4","LTEPER1ST5","LTEPER2ST5",
        "LTEPER5ST5","USER", NULL
};
static ViString satNavSimulationModeArr[] = { "STAT","AUTO","USER", NULL };
static ViString triggerModeArr[] = { "AUTO","RETR","AAUT","ARET","SING", NULL };
static ViString satNavTriggerSourceArr[] = { "INT","OBAS","EXT","BEXT", NULL };
static ViString satNavMarkerModeArr[] = { "PULS","PATT","RAT","PPS","PP2S","TRIG","DIS","PPS10", NULL };
static ViString clockSyncMode[] = { "NONE","MAST","SLAV", NULL };
static ViString satNavClockMode[] = { "CHIP","MCH", NULL };
static ViString satNavVehicleTypeArr[] = { "HIL","SPAC","AIRC","SHIP","LVEH","PED", NULL };
static ViString satNavROModeArr[] = { "CYCL","OWAY","RTR", NULL };
static ViString satNavObscurationTypeArr[] = { "USER","CITY","SUB1","URB1","LTUN","MTUN","BR1","BR2","P1M","P10M","P1H","HIGH","CUTT", NULL };
static ViString satNavObscurationEnvArr[] = { "LOS","VOBS","RPL","FULL","GSR", NULL };
static ViString satNavObscurationPhysModelArr[] = { "OBSC","OMP", NULL };
static ViString satNavDataSourceArr[] = { "ZERO","ONE","PATT","PN9","PN11","PN15","PN16","PN20","PN21","PN23","DLIS","RND", NULL };
static ViString ADGTimeBasisArr[] = { "UTC","GPS","GST","GLO","BDT", NULL };
static ViString satNavPowerModeArr[] = { "USER","AUTO", NULL };
static ViString satNavReferenceStandardArr[] = { "GPS","GAL","GLON", NULL };
static ViString satNavTropoModelArr[] = { "NONE","STAN", NULL };
static ViString satNavIonModelArr[] = { "NONE","KLOB", NULL };
static ViString satNavSATSignalArr[] = { "CAC","PCOD","CAPQ","CAPI","E1D","RCA", NULL };
static ViString satNavCodeONL2Arr[] = { "RES","PCOD","CAC", NULL };
static ViString satNavRTDisplayTypeArr[] = { "SKY","MAP","POW","TRAJ","ATT", NULL };
static ViString satNavSDynConfigArr[] = { "USER","VEL1","VEL2", NULL };
static ViString satNavDynDopplerShiftUnitArr[] = { "HZ","MPS", NULL };
static ViString satNavDynProfileArr[] = { "CONS","HIGH", NULL };
static ViString satNavMConDataRateArr[] = { "D50HZ","D100HZ","D250HZ","D1000HZ", NULL };
static ViString satNavLogModeArr[] = { "RT","OFFL", NULL };
static ViString satNavLogUMotionStepArr[] = { "R1S","R2S","R5S","R10S","R02S","R04S","R08S", NULL };
static ViString SBASNavModeArr[] = { "MESS","RAW", NULL };
static ViString SBASRegionArr[] = { "WAAS","MSAS","EGNOS", NULL };
static ViString powerSensorMappingArr[] = { "SENS1","SENS2","SENS3","SENS4","UNM",NULL };

/*****************************************************************************
 * Function: rsrfsiggen_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rsrfsiggen_InitWithOptions function.
 *****************************************************************************/

ViStatus _VI_FUNC rsrfsiggen_init(ViRsrc resourceName, ViBoolean IDQuery,
    ViBoolean resetDevice, ViSession* newInstrSession)
{
    ViStatus    error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
            VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rsrfsiggen_InitWithOptions(resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rsrfsiggen_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rsrfsiggen_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = *newInstrSession;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
            "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr*)rsrfsiggen_RsCoreAttributeList;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rsrfsiggen", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        rsrfsiggen_RsCoreAttributeListLength, // Attributes count
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "SMW200A", VI_FALSE: "SMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        1000000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSRFSIGGEN_OPC_TIMEOUT, // OPC timeout
        10000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 4.7.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 4, 7, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rsrfsiggen_RsCoreRepCapTable, rsrfsiggen_RsCoreRepCapTableCount));

    // Query *IDN? string, parse it and set the following attributes:
    // - RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "SMW200A", false : "SMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    // The variable aliases is a comma-separated list of instrument names mapping.
    // If the instrument model full name is matching one of the names there, it is treated by the driver as the assigned instrument
    // Example: 'SGT100A -> SMW' - if the instrument full name is 'SGT100A', the driver will treat it as an 'SMW'
    checkErr(RsCore_QueryAndParseIDNstringWithAliases(instrSession, RSRFSIGGEN_SIMULATION_ID_QUERY, NULL, "SGS100A -> SMW, SGT100A -> SMW, SMCV100B -> SMW, SMM100A -> SMW", NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSRFSIGGEN_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rsrfsiggen_reset(instrSession));
    }
    else
    {
        checkErr(rsrfsiggen_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSRFSIGGEN_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_AfterErrorInInit(instrSession, sessionProperties, error, newInstrSession);

    return error;
}

/// HIFN  This function configures the frequency and the power level of the RF
/// HIFN  output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frequency/Sets the frequency of the RF output signal for CW mode
/// HIPAR frequency/(:SOURce:FREQuency:MODE CW). In Sweep mode (:SOURce:FREQuency:MODE
/// HIPAR frequency/SWEep), this value is linked to the current sweep frequency.
/// HIPAR powerLevel/Sets the RF output level in CW mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureRF(ViSession instrSession,
                                         ViInt32   path,
                                         ViReal64  frequency,
                                         ViReal64  powerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_FREQUENCY, frequency),
            3, "Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_AMPLITUDE, powerLevel),
            4, "Power Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
/// HIFN  This function configures the frequency of the RF
/// HIFN  output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frequency/Sets the frequency of the RF output signal for CW mode
/// HIPAR frequency/(:SOURce:FREQuency:MODE CW). In Sweep mode (:SOURce:FREQuency:MODE
/// HIPAR frequency/SWEep), this value is linked to the current sweep frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureRFFrequency(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_FREQUENCY, frequency),
            3, "Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the power level of the RF
/// HIFN  output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerLevel/Sets the RF output level in CW mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureRFAmplitude(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViReal64  powerLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_AMPLITUDE, powerLevel),
            3, "Power Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets RF level main  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR digitalAttenuation/Sets the Digital Attenuation.
/// HIPAR mode/Sets the RF level mode.
/// HIPAR settingCharacteristics/Selects the level behavior at the RF output over time.
/// HIPAR limit/Limits the maximum RF output level in CW and SWEEP mode.
/// HIPAR offset/Specifies the constant level offset of a downstream attenuator/amplifier.
ViStatus _VI_FUNC rsrfsiggen_ConfigureRFLevel(ViSession instrSession,
                                              ViInt32   path,
                                              ViReal64  digitalAttenuation,
                                              ViInt32   mode,
                                              ViInt32   settingCharacteristics,
                                              ViReal64  limit,
                                              ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_ATTENUATOR_DIGITAL_ATTENUATION, digitalAttenuation),
            3, "Digital Attenuation");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_LEVEL_MODE, mode),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_LEVEL_SETTING_CHARACTERISTICS, settingCharacteristics),
            5, "Setting Characteristics");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_LEVEL_LIMIT, limit),
            6, "Limit");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_RF_LEVEL_OFFSET, offset),
            7, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the RF level attenuator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Switches the mode of the attenuator at the RF output (Attenuator MODe).
/// HIPAR attenuation/Sets the attenuation value of the RF signal.
/// HIPAR RFOFFMode/Selects the attenuator's state after the RF signal is switched off.
ViStatus _VI_FUNC rsrfsiggen_ConfigureRFLevelAttenuator(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   mode,
                                                        ViReal64  attenuation,
                                                        ViInt32   RFOFFMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_ATTENUATOR_MODE, mode),
            3, "Mode");

    if (mode == RSRFSIGGEN_VAL_OUTPUT_AMODE_MANUAL)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_RF_ATTENUATOR_ATTENUATION, attenuation),
                4, "Attenuation");
    }
    else
    {
    }

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_ATTENUATOR_RF_OFF_MODE, RFOFFMode),
            5, "RF OFF Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the power-on of the RF Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerOnState/Selects the state which the RF output assumes when the instrument is switched on.
/// HIPAR displayLevelAsVoltageOfEMF/Displays the signal level as voltage of the EMF. The displayed
/// HIPAR displayLevelAsVoltageOfEMF/value represents the voltage over a 50 Ohm load.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerOnEMF(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   powerOnState,
                                                 ViBoolean displayLevelAsVoltageOfEMF)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_POWER_ON_ENABLED, powerOnState),
            3, "Power On State");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DISPLAY_LEVEL_AS_VOLTAGE_OF_EMF, displayLevelAsVoltageOfEMF),
            4, "Display Level As Voltage Of EMF");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the Automatic Level Control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR alcEnabled/Activates/deactivates automatic level control.
ViStatus _VI_FUNC rsrfsiggen_ConfigureALCEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   alcEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ALC_ENABLED, alcEnabled),
            3, "ALC Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the main parameters for ALC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR detectorSensitivity/Sets the sensitivity of the ALC detector.
/// HIPAR driverAmplifier/Selects the driver amplifier switching state.
/// HIPAR sampleLevel/Sets the sample level of automatic level control (ALC).
ViStatus _VI_FUNC rsrfsiggen_ConfigureALC(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   detectorSensitivity,
                                          ViInt32   driverAmplifier,
                                          ViInt32   sampleLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ALC_DETECTOR_SENSITIVITY, detectorSensitivity),
            3, "Detector Sensitivity");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ALC_DRIVER_AMPLIFIER, driverAmplifier),
            4, "Driver Amplifier");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ALC_SAMPLE_LEVEL, sampleLevel),
            5, "Sample Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates level correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR userCorrectionEnabled/Activates or deactivates level correction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureUserCorrectionEnabled(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViBoolean userCorrectionEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_USER_CORRECTION_DATA_ENABLED, userCorrectionEnabled),
            3, "User Correction Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Fills the selected user correction table with the level values
/// HIFN  measured by the power sensor for the given frequencies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sensor/Defines the used power sensor, i.e. the sensor whose values are used.
ViStatus _VI_FUNC rsrfsiggen_FillUserCorrectionDataWithSensor(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   sensor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sensor, 1, 4),
            3, "Sensor");

    snprintf(cmd, 512, ":SOUR%ld:CORR:CSET:DATA:SENS%ld:POW:SONC", path, sensor);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR outputEnabled/Activates and deactivates the RF output.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOutputEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean outputEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_OUTPUT_ENABLED, outputEnabled),
            3, "Output Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the frequency and power sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frequencyMode/Sets the frequency mode for generating the RF output signal in
/// HIPAR frequencyMode/the selected path. The selected mode determines the parameters
/// HIPAR frequencyMode/to be used for further frequency settings.
/// HIPAR powerMode/Selects the operating mode of the instrument to set the output level.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencyAndPowerMode(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   frequencyMode,
                                                            ViInt32   powerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_FREQUENCY_MODE, frequencyMode),
            3, "Frequency Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_RF_POWER_MODE, powerMode),
            4, "Power Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns an entry from the :SLISt[:LIST]? to one of the
/// HIFN  four sensor channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR sensor/Defines the used power sensor, i.e. the sensor whose values are used.
/// HIPAR mapping/Sets the Power Sensor Mapping for selected sensor.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerSensorMapping(ViSession instrSession,
                                                         ViInt32   sensor,
                                                         ViInt32   mapping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SLIS:ELEM%ld:MAPP %s", sensor, powerSensorMappingArr[mapping]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function assigns a sensor directly to one of the sensor channels,
/// HIFN  using the sensor name and
/// HIFN     serial number. To find out the the sensor name and ID, you can get it
/// HIFN  from the label of the R&S NRP,
/// HIFN     or using the command :SLISt:SCAN[:STATe]. This command detects all R&S
/// HIFN  NRP power sensors connected in the LAN or via USBTMC protocol
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR sensorID/Sets the sensor ID.
/// HIPAR mapping/Sets the Power Sensor Mapping for selected sensor.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerSensorMappingDirectly(
                            ViSession instrSession,
                            ViString  sensorID,
                            ViInt32   mapping)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SLIS:SENS:MAP '%s',%s", sensorID, powerSensorMappingArr[mapping]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets network parameters of the external frontend.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ipAddress/IP address of the external frontend.
/// HIPAR subnetMask/Bit group of the subnet in the host identifier.
/// HIPAR dhcp/DHCP state.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrontendNetwork(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViString  ipAddress,
                                                      ViString  subnetMask,
                                                      ViInt32   dhcp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s',%ld", ipAddress, subnetMask, dhcp);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_FRONTEND_NETWORK, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the level values to the selected table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR power/Enters the level values to the selected table.
ViStatus _VI_FUNC rsrfsiggen_UserCorrectionDataLevelValues(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   arraySize,
                                                           ViReal64  power[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:CORR:CSET:DATA:POW ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, power, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file from the default or the
/// HIFN  specified directory. Loaded are files with extension *.freqresp
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureCorrectionFrequencyLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_CORRECTION_FREQUENCY_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the current settings into the selected file. The
/// HIFN  file extension *.freqresp is assigned automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureCorrectionFrequencyStore(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_CORRECTION_FREQUENCY_STORE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the frequency response FR list files included in the current
/// HIFN  FR list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the frequency response FR list files included in the current
/// HIPAR catalog/FR list.
ViStatus _VI_FUNC rsrfsiggen_QueryCorrectionFrequencyFRListCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:CORR:FRES:RF:USER:FLIS:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects an existing frequency response list file (*.uco
/// HIFN  or *.fres) from the default directory or from the specific directory.
/// HIFN  Use 'none' to unload file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frList/Selects the FR List number.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureCorrectionFrequencyFRListSelect(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   frList,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frList, 1, 5),
            3, "FR List");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,FL%ld", pathArr[path], frList);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_CORRECTION_FREQUENCY_FR_LIST_SELECT, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the S-parameter files included in the current S-parameters
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the S-parameter files included in the current S-parameters
/// HIPAR catalog/list.
ViStatus _VI_FUNC rsrfsiggen_QueryCorrectionFrequencySListCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:CORR:FRES:RF:USER:SLIS:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Selects an existing S-parameter file (*.s<n>p) from the
/// HIFN  default directory or from the specific directory. Use 'none' to unload
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sList/Selects the S List number.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureCorrectionFrequencySListSelect(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   sList,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sList, 1, 10),
            3, "S List");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,SL%ld", pathArr[path], sList);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_CORRECTION_FREQUENCY_S_LIST_SELECT, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply amplitude
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR amEnabled/Activates/deactivates amplitude modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureAMEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean amEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_AM_ENABLED, amEnabled),
            3, "AM Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the coupling of an external source for
/// HIFN  amplitude modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR amExternalCoupling/Selects the coupling mode for the external modulation input (EXT MOD)
/// HIPAR amExternalCoupling/in the case of amplitude modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureAMExternalCoupling(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   amExternalCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_AM_EXTERNAL_COUPLING, amExternalCoupling),
            3, "AM External Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the modulation source, scaling, and depth for
/// HIFN  the signal generator's amplitude modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR depthType/Sets the used type of modulation depth.
/// HIPAR depth/Sets the modulation depth of the amplitude modulation in percent. The
/// HIPAR depth/modulation depth is limited by the maximum peak envelope power (PEP).
ViStatus _VI_FUNC rsrfsiggen_ConfigureAM(ViSession instrSession,
                                         ViInt32   path,
                                         ViInt32   depthType,
                                         ViReal64  depth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             depthType;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    /*viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_AM_DEPTH_TYPE,
                                                 depthType), 3, "Depth Type");
    */
    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_AM_DEPTH, depth),
            4, "Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the amplitude modulation.
/// HIFN
/// HIFN     Supported Instruments: SMW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sourceModPath1/Selects the modulation source for amplitude modulation.
/// HIPAR sourceModPath2/Selects the modulation source for amplitude modulation.
/// HIPAR modpath1State/Activates amplitude modulation.
/// HIPAR modpath2State/Activates amplitude modulation.
/// HIPAR depth/Sets the modulation depth of the amplitude modulation in percent. The
/// HIPAR depth/modulation depth is limited by the maximum peak envelope power (PEP).
/// HIPAR ratioModPath2/1/Sets the deviation ratio (path#2 to path#1) in percent.
ViStatus _VI_FUNC rsrfsiggen_ConfigureAMSMW(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   sourceModPath1,
                                            ViInt32   sourceModPath2,
                                            ViBoolean modpath1State,
                                            ViBoolean modpath2State,
                                            ViReal64  depth,
                                            ViReal64  ratioModPath21)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath1[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(repCap_modPath1, RS_REPCAP_BUF_SIZE, "%s,PMCH1", repCap);
    snprintf(repCap_modPath2, RS_REPCAP_BUF_SIZE, "%s,PMCH2", repCap);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_AM_DEPTH, depth),
            7, "Depth");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_AM_CHANNEL_ENABLED, modpath1State),
            5, "ModPath 1 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_AM_CHANNEL_SOURCE, sourceModPath1),
            3, "Source ModPath 1");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_AM_CHANNEL_ENABLED, modpath2State),
            6, "ModPath 2 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_AM_CHANNEL_SOURCE, sourceModPath2),
            4, "Source ModPath 2");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_AM_SOURCE_RATIO, ratioModPath21),
            8, "Ratio ModPath 2/1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply frequency
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fmEnabled/Activates/deactivates the frequency modulation.
/// HIPAR fmMode/Selects the mode for the frequency modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFMEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean fmEnabled,
                                                ViInt32   fmMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FM_ENABLED, fmEnabled),
            3, "FM Enabled");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FM_MODE, fmMode),
            4, "FM Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the coupling of an external source for
/// HIFN  frequency modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR fmExternalCoupling/Sets the coupling mode (AC or DC) for external feed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFMExternalCoupling(ViSession instrSession,
                                                         ViInt32   fmExternalCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "A", RSRFSIGGEN_ATTR_FM_EXTERNAL_COUPLING, fmExternalCoupling),
            3, "FM External Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the modulation deviation and modulation
/// HIFN  source for the signal generator's frequency modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR deviation/Sets the modulation depth in Hz.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFM(ViSession instrSession,
                                         ViReal64  deviation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "A", RSRFSIGGEN_ATTR_FM_DEVIATION, deviation),
            3, "Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the frequency modulation.
/// HIFN
/// HIFN     Supported Instruments: SMW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sourceModPath1/Selects the modulation source for frequency modulation.
/// HIPAR sourceModPath2/Selects the modulation source for frequency modulation.
/// HIPAR modpath1State/Activates frequency modulation.
/// HIPAR modpath2State/Activates frequency modulation.
/// HIPAR deviation/Sets the modulation depth in Hz.
/// HIPAR ratioModPath2/1/Sets the deviation ratio (path#2 to path#1) in percent.
/// HIPAR fmMode/Selects the mode for the frequency modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFMSMW(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   sourceModPath1,
                                            ViInt32   sourceModPath2,
                                            ViBoolean modpath1State,
                                            ViBoolean modpath2State,
                                            ViReal64  deviation,
                                            ViReal64  ratioModPath21,
                                            ViInt32   fmMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath1[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(repCap_modPath1, RS_REPCAP_BUF_SIZE, "%s,PMCH1", repCap);
    snprintf(repCap_modPath2, RS_REPCAP_BUF_SIZE, "%s,PMCH2", repCap);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FM_DEVIATION, deviation),
            7, "Deviation");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FM_MODE, fmMode),
            9, "FM Mode");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_FM_CHANNEL_ENABLED, modpath1State),
            5, "ModPath 1 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_FM_CHANNEL_SOURCE, sourceModPath1),
            3, "Source ModPath 1");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_FM_CHANNEL_ENABLED, modpath2State),
            6, "ModPath 2 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_FM_CHANNEL_SOURCE, sourceModPath2),
            4, "Source ModPath 2");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FM_SOURCE_RATIO, ratioModPath21),
            8, "Ratio ModPath 2/1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply phase
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pmEnabled/Activates/deactivates phase modulation.
/// HIPAR pmMode/Selects the mode for the phase modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePMEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean pmEnabled,
                                                ViInt32   pmMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_PM_PATH_ENABLED, pmEnabled),
            3, "PM Enabled");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_PM_PATH_MODE, pmMode),
            4, "PM Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the coupling of an external source for phase
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR pmExternalCoupling/Sets the coupling mode (AC or DC) for external feed.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePMExternalCoupling(ViSession instrSession,
                                                         ViInt32   pmExternalCoupling)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_PM_EXTERNAL_COUPLING, pmExternalCoupling),
            3, "PM External Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the attribute that control the signal generator's phase
/// HIFN  modulation. The attributes are the modulation deviation and the
/// HIFN  modulating source(s).
/// HIFN
/// HIFN     This function configures the modulation deviation and modulation
/// HIFN  source for the signal generator's phase modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR deviation/Sets the the modulation depth in RAD.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePM(ViSession instrSession,
                                         ViReal64  deviation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_PM_DEVIATION, deviation),
            3, "Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the phase modulation.
/// HIFN
/// HIFN     Supported Instruments: SMW
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sourceModPath1/Selects the modulation source for phase modulation signal.
/// HIPAR sourceModPath2/Selects the modulation source for phase modulation signal.
/// HIPAR modpath1State/Activates phase modulation.
/// HIPAR modpath2State/Activates phase modulation.
/// HIPAR deviation/Sets the the modulation depth in RAD.
/// HIPAR ratioModPath2/1/Sets PM Ratio.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePMSMW(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   sourceModPath1,
                                            ViInt32   sourceModPath2,
                                            ViBoolean modpath1State,
                                            ViBoolean modpath2State,
                                            ViReal64  deviation,
                                            ViReal64  ratioModPath21)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath1[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_modPath2[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(repCap_modPath1, RS_REPCAP_BUF_SIZE, "%s,PMCH1", repCap);
    snprintf(repCap_modPath2, RS_REPCAP_BUF_SIZE, "%s,PMCH2", repCap);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_PM_PATH_DEVIATION, deviation),
            7, "Deviation");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_PM_CHANNEL_ENABLED, modpath1State),
            5, "ModPath 1 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath1, RSRFSIGGEN_ATTR_PM_CHANNEL_SOURCE, sourceModPath1),
            3, "Source ModPath 1");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_PM_CHANNEL_ENABLED, modpath2State),
            6, "ModPath 2 State");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap_modPath2, RSRFSIGGEN_ATTR_PM_CHANNEL_SOURCE, sourceModPath2),
            4, "Source ModPath 2");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_PM_RATIO, ratioModPath21),
            8, "Ratio ModPath 2/1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply pulse
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pulseModulationEnabled/Activates/deactivates the pulse modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseModulationEnabled(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViBoolean pulseModulationEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_PULM_ENABLED, pulseModulationEnabled),
            3, "Pulse Modulation Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the source for pulse modulation of the RF output
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the source for pulse modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseModulationSource(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_PULM_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the polarity of the external source signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR externalPolarity/Selects the active level of the gate signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseModulationExternalPolarity(
                            ViSession instrSession,
                            ViInt32   externalPolarity)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_PULM_GATE_INPUT_POLARITY, externalPolarity),
            2, "externalPolarity");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator pulse modulation to the
/// HIFN  RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR period/Sets the period of the generated pulse. The period determines the
/// HIPAR period/repetition frequency of the internal signal.
/// HIPAR width/Sets the width of the generated pulse. The width determines the pulse
/// HIPAR width/length. The pulse width must be at least 20 ns less than the set pulse
/// HIPAR width/period.
/// HIPAR delay/Sets the pulse delay. The pulse delay determines the time that elapses
/// HIPAR delay/after a trigger event before pulse modulation starts. The pulse delay
/// HIPAR delay/is not effective for double pulse generation.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseModulation(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViReal64  period,
                                                      ViReal64  width,
                                                      ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_PULM_PERIOD, period),
            3, "Period");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_PULM_WIDTH, width),
            4, "Width");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_PULM_DELAY, delay),
            5, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the Input and Trigger settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR externalImpedance/Selects the active level of the gate signal.
/// HIPAR triggerMode/Selects the trigger mode for pulse modulation.
/// HIPAR triggerInputSlope/Sets the active slope of an externally applied trigger signal at the
/// HIPAR triggerInputSlope/TRIGGER input (BNC connector at the rear of the instrument).
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseModulationInputandTriggerSettings(
                            ViSession instrSession,
                            ViInt32   externalImpedance,
                            ViInt32   triggerMode,
                            ViInt32   triggerInputSlope)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_PULM_EXTERNAL_INPUT_IMPEDANCE, externalImpedance),
            2, "External Impedance");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_PULM_TRIGGER_MODE, triggerMode),
            3, "Trigger Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_PULM_EXT_TRIGGER_INPUT_SLOPE, triggerInputSlope),
            4, "Trigger Input Slope");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates/deactivates the pulse generator. The pulse
/// HIFN  generator and the pulse modulation are activated separately.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR pulseGeneratorEnabled/Activates/deactivates the pulse generator. The pulse generator and the
/// HIPAR pulseGeneratorEnabled/pulse modulation are activated separately.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePulseGeneratorEnabled(ViSession instrSession,
                                                            ViBoolean pulseGeneratorEnabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, "", RSRFSIGGEN_ATTR_PGEN_ENABLED, pulseGeneratorEnabled),
            2, "Pulse Generator Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the LF generators output frequency and
/// HIFN  waveform.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR frequency/Sets the frequency of the LF signal for CW mode (:SOUR::MODE CW mode).
/// HIPAR frequency/In Sweep mode (:SOUR:LFO:FREQ:MODE SWE), the frequency is linked to the
/// HIPAR frequency/sweep frequency.
/// HIPAR shape/Selects the shape of the LF generator.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFOutput(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   output,
                                               ViReal64  frequency,
                                               ViInt32   shape)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_output[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, output, 1, 2),
            3, "Output");
    snprintf(repCap_output, RS_REPCAP_BUF_SIZE, ",LF%ld", output);
    strcat(repCap, repCap_output);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LFGEN_FREQUENCY, frequency),
            4, "Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LF_OUTPUT_SHAPE, shape),
            5, "Shape");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the output voltage and enables the LF generator
/// HIFN  within the RF signal generator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR amplitude/Sets the output voltage of the LF output.
/// HIPAR enabled/Activates/deactivates the LF output.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFGeneratorOutput(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   output,
                                                        ViReal64  amplitude,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_output[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, output, 1, 2),
            3, "Output");
    snprintf(repCap_output, RS_REPCAP_BUF_SIZE, ",LF%ld", output);
    strcat(repCap, repCap_output);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LFGEN_VOLTAGE, amplitude),
            2, "Amplitude");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LF_OUTPUT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the mode for the output of the LF generator
/// HIFN  frequency, and determines the commands to be used for frequency
/// HIFN  settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR sweepState/Enables or disables the LF sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFSweepState(ViSession instrSession,
                                                   ViBoolean sweepState)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "",
        RSRFSIGGEN_ATTR_LF_SWEEP_STATE,
        (sweepState == VI_TRUE) ?
        RSRFSIGGEN_VAL_LF_SWEEP_STATE_SWEEP : RSRFSIGGEN_VAL_LF_SWEEP_STATE_CW),
        2, "Sweep State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator LF sweep mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR mode/This control sets the mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFSweep(ViSession instrSession,
                                              ViInt32   mode)
{
    ViStatus                         error = VI_SUCCESS;
    ViInt32                          attr_param[5][2] = { {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_AUTO},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_SINGLE},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_SINGLE},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL}
                                     };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_MODE_EXT_STEP),
            2, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_MODE, attr_param[mode][0]),
            2, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_TRIGGER_SOURCE, attr_param[mode][1]),
            2, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and stop frequencies for a LF
/// HIFN  frequency sweep. If the stop frequency is less than the start
/// HIFN  frequency, the frequency decreases during the sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR start/Sets the start frequency for the LF Sweep mode.
/// HIPAR stop/Sets the stop frequency for the LF sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFFrequencySweepStartStop(
                            ViSession instrSession,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_START_FREQUENCY, start),
            2, "Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_STOP_FREQUENCY, stop),
            3, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that control the step
/// HIFN  frequencies of the generator's RF output signal. These settings are
/// HIFN  start and stop frequency, step size, and lin/log scaling. If the stop
/// HIFN  frequency is less than the start frequency, the frequency decreases
/// HIFN  during the sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR start/Sets the start frequency for the LF Sweep mode.
/// HIPAR stop/Sets the stop frequency for the LF sweep.
/// HIPAR scaling/Specifies the spacing of the steps.
/// HIPAR stepSize/Sets the selected step size.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFFrequencyStepStartStop(
                            ViSession instrSession,
                            ViReal64  start,
                            ViReal64  stop,
                            ViInt32   scaling,
                            ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, scaling, RSRFSIGGEN_VAL_LF_FREQUENCY_STEP_SCALING_LINEAR, RSRFSIGGEN_VAL_LF_FREQUENCY_STEP_SCALING_LOGARITHMIC),
            4, "Scaling");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_START_FREQUENCY, start),
            2, "Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_STOP_FREQUENCY, stop),
            3, "Stop");
    if (scaling == RSRFSIGGEN_VAL_LF_FREQUENCY_STEP_SCALING_LOGARITHMIC)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_STEP_LOG, stepSize),
                5, "Step Size");
    }
    else
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_STEP_LIN, stepSize),
                5, "Step Size");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures how the frequency sweep advances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR dwell/Sets the dwell time for each frequency step of the sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLFFrequencyStepDwell(ViSession instrSession,
                                                           ViReal64  dwell)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_LF_SWEEP_DWELL_TIME, dwell),
            2, "Dwell");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator sweep mode and trigger
/// HIFN  source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR type/Specifies the sweep type applied to the output signal.
/// HIPAR mode/This control sets the mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSweep(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   type,
                                            ViInt32   mode)
{
    ViStatus                           error = VI_SUCCESS;
    ViChar                             repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr                             attr[2][2] = { {RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_MODE, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_TRIGGER_SOURCE},
    {RSRFSIGGEN_ATTR_LEVEL_SWEEP_MODE, RSRFSIGGEN_ATTR_LEVEL_SWEEP_TRIGGER_SOURCE}
                                       };
    ViInt32                            attr_param[5][2] = { {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_AUTO},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO,   RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_SINGLE},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP,   RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_SINGLE},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO,   RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP,   RSRFSIGGEN_VAL_SWEEP_TRIGGER_SOURCE_EXTERNAL}
                                       };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSRFSIGGEN_VAL_SWEEP_MODE_FREQUENCY_SWEEP, RSRFSIGGEN_VAL_SWEEP_MODE_POWER_SWEEP),
            3, "Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_MODE_EXT_STEP),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, attr[type][0], attr_param[mode][0]),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, attr[type][1], attr_param[mode][1]),
            4, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the frequency sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sweepState/Enables or disables the frequency sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencySweepState(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViBoolean sweepState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap,
        RSRFSIGGEN_ATTR_RF_FREQUENCY_MODE,
        (sweepState == VI_TRUE) ?
        RSRFSIGGEN_VAL_RF_FREQUENCY_SWEEP : RSRFSIGGEN_VAL_RF_FREQUENCY_CW),
        3, "Sweep State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and stop frequencies for a
/// HIFN  frequency sweep. If the stop frequency is less than the s?tart
/// HIFN  frequency, the frequency decreases during the sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR start/Sets the start frequency for the sweep mode. STARt can be greater than
/// HIPAR start/STOP.
/// HIPAR stop/Sets the stop frequency for the sweep mode. STOP can be less than
/// HIPAR stop/STARt.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencySweepStartStop(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  start,
                            ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_START_FREQUENCY, start),
            3, "Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_STOP_FREQUENCY, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the center frequency and span for a frequency
/// HIFN  sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR center/Sets the center frequency of the sweep. This setting in combination
/// HIPAR center/with the span setting ([SOURce:]FREQuency:SPAN) defines the sweep
/// HIPAR center/range.
/// HIPAR span/Specifies the span for the sweep. This setting in combination with the
/// HIPAR span/center frequency setting ([SOUR]:FREQ:CENT) defines the sweep range.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencySweepCenterSpan(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  center,
                            ViReal64  span)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_CENTER_FREQUENCY, center),
            3, "Center");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_SPAN, span),
            4, "Span");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that control the step
/// HIFN  frequencies of the generator's RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR scaling/Selects linear or logarithmic sweep spacing.
/// HIPAR stepSize/Sets the step size.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencyStep(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   scaling,
                                                    ViReal64  stepSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attribute = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_SPACING, scaling),
            3, "Scaling");

    switch (scaling)
    {
    case RSRFSIGGEN_VAL_FREQ_SWEEP_SPACING_LINEAR:
        attribute = RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_STEP_LINEAR;
        break;

    case RSRFSIGGEN_VAL_FREQ_SWEEP_SPACING_LOGARITHMIC:
        attribute = RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_STEP_LOG;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, scaling), 3, "Scaling");
        break;
    }

    if (attribute != 0)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, attribute, stepSize),
                4, "Step Size");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures how the frequency sweep advances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dwell/Sets the time taken for each frequency step of the sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFrequencyStepDwell(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViReal64  dwell)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FREQUENCY_SWEEP_DWELL_TIME, dwell),
            3, "Dwell");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the power sweep..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sweepState/Enables or disables the power sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerSweepState(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViBoolean sweepState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap,
        RSRFSIGGEN_ATTR_RF_POWER_MODE,
        (sweepState == VI_TRUE) ?
        RSRFSIGGEN_VAL_RF_POWER_SWEEP : RSRFSIGGEN_VAL_RF_POWER_CW),
        3, "Sweep State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the start and stop power for a power sweep.
/// HIFN  If the stop power is less than the start power, the power decreases in
/// HIFN  value during the sweep.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR start/Sets the RF start level in Sweep mode.
/// HIPAR stop/Sets the stop level in Sweep mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerSweepStartStop(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViReal64  start,
                                                          ViReal64  stop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LEVEL_SWEEP_START_LEVEL, start),
            3, "Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LEVEL_SWEEP_STOP_LEVEL, stop),
            4, "Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures how the power sweep advances.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dwell/Sets the time taken for each level step of the sweep.
ViStatus _VI_FUNC rsrfsiggen_ConfigurePowerStepDwell(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  dwell)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LEVEL_SWEEP_DWELL_TIME, dwell),
            3, "Dwell");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the list state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR listState/Enables or disables the list state.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListState(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean listState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap,
        RSRFSIGGEN_ATTR_RF_FREQUENCY_MODE,
        (listState == VI_TRUE) ?
        RSRFSIGGEN_VAL_RF_FREQUENCY_LIST : RSRFSIGGEN_VAL_RF_FREQUENCY_CW),
        3, "List State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a named list of frequency values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR name/Selects the specified list. If a new list is to be created, the name
/// HIPAR name/can be entered here. The list is created if it does not yet exist. The
/// HIPAR name/list selected here is available for the further processing steps
/// HIPAR name/(editing) and is used in the instrument when the List mode is
/// HIPAR name/activated.
/// HIPAR frequencies/Fills the FREQuency part of the selected list with frequency values.
/// HIPAR arraySize/Sets the size of the Frequencies array.
ViStatus _VI_FUNC rsrfsiggen_CreateFrequencyList(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViString  name,
                                                 ViReal64  frequencies[],
                                                 ViInt32   arraySize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViString freqList = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_SELECT_LIST, name),
            3, "Name");

    checkErr(RsCore_Convert_ViReal64array_To_String(instrSession, frequencies, arraySize, NULL, &freqList));
    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_FREQUENCY_VALUES, freqList),
            4, "Frequencies");

Error:
    if (freqList)
        free(freqList);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a named list of power values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR name/Selects the specified list. If a new list is to be created, the name
/// HIPAR name/can be entered here. The list is created if it does not yet exist. The
/// HIPAR name/list selected here is available for the further processing steps
/// HIPAR name/(editing) and is used in the instrument when the List mode is
/// HIPAR name/activated.
/// HIPAR powers/Fills the power part of the selected list with power values.
/// HIPAR arraySize/Sets the size of the Powers array.
ViStatus _VI_FUNC rsrfsiggen_CreatePowerList(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  name,
                                             ViReal64  powers[],
                                             ViInt32   arraySize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViString powerList = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_SELECT_LIST, name),
            3, "Name");

    checkErr(RsCore_Convert_ViReal64array_To_String(instrSession, powers, arraySize, NULL, &powerList));
    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_POWER_VALUES, powerList),
            4, "Powers");

Error:
    if (powerList)
        free(powerList);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a named list of dwell time values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR name/Selects the specified list. If a new list is to be created, the name
/// HIPAR name/can be entered here. The list is created if it does not yet exist. The
/// HIPAR name/list selected here is available for the further processing steps
/// HIPAR name/(editing) and is used in the instrument when the List mode is
/// HIPAR name/activated.
/// HIPAR dwellTimes/The dwell time values in the selected list in s.
/// HIPAR arraySize/Sets the size of the Dwell Times array.
ViStatus _VI_FUNC rsrfsiggen_CreateDwellList(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  name,
                                             ViReal64  dwellTimes[],
                                             ViInt32   arraySize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViString dwellList = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_SELECT_LIST, name),
            3, "Name");

    checkErr(RsCore_Convert_ViReal64array_To_String(instrSession, dwellTimes, arraySize, NULL, &dwellList));
    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_DWELL_TIME_VALUES, dwellList),
            4, "Dwell Times");

Error:
    if (dwellList)
        free(dwellList);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select a named list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR name/Selects the specified list. If a new list is to be created, the name
/// HIPAR name/can be entered here. The list is created if it does not yet exist. The
/// HIPAR name/list selected here is available for the further processing steps
/// HIPAR name/(editing) and is used in the instrument when the List mode is
/// HIPAR name/activated.
ViStatus _VI_FUNC rsrfsiggen_SelectList(ViSession instrSession,
                                        ViInt32   path,
                                        ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_SELECT_LIST, name),
            3, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the time for which the instrument retains a
/// HIFN  setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dwell/Sets the time for which the instrument retains a setting.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListDwell(ViSession instrSession,
                                                ViInt32   path,
                                                ViReal64  dwell)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_DWELL_TIME, dwell),
            3, "Dwell");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the list mode and Dwell time mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the list mode.
/// HIPAR dwellTimeMode/Selects the dwell time mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListDwellMode(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   mode,
                                                    ViInt32   dwellTimeMode)
{
    ViStatus                         error = VI_SUCCESS;
    ViChar                           repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32                          attr_param[5][2] = { {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_LIST_TRIGGER_SOURCE_AUTO},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_LIST_TRIGGER_SOURCE_SINGLE},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP, RSRFSIGGEN_VAL_LIST_TRIGGER_SOURCE_BUS},
    {RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_LIST_TRIGGER_SOURCE_EXTERNAL},
    {RSRFSIGGEN_VAL_SWEEP_MODE_STEP, RSRFSIGGEN_VAL_LIST_TRIGGER_SOURCE_EXTERNAL}
                                     };

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSRFSIGGEN_VAL_SWEEP_MODE_AUTO, RSRFSIGGEN_VAL_SWEEP_MODE_EXT_STEP),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_MODE, attr_param[mode][0]),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_TRIGGER_SOURCE, attr_param[mode][1]),
            3, "Mode");

    if (!rsrfsiggen_IsSMBV(instrSession))
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_DWELL_TIME_MODE, dwellTimeMode),
                4, "Dwell Time Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the current state of the list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR listState/Returns the current state of the list mode.
ViStatus _VI_FUNC rsrfsiggen_ListCurrentState(ViSession  instrSession,
                                              ViInt32    path,
                                              ViBoolean* listState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_GetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_CURRENT_STATE, listState),
            3, "List State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of the S-parameter data sets that have been loaded to
/// HIFN  the power sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR sensor/Defines the used power sensor, i.e. the sensor whose values are used.
/// HIPAR arraySize/Array size parameter.
/// HIPAR sParameterList/Queries the S-parameter files included in the current S-parameters
/// HIPAR sParameterList/list.
ViStatus _VI_FUNC rsrfsiggen_QueryPowerSParameterList(ViSession instrSession,
                                                      ViInt32   sensor,
                                                      ViInt32   arraySize,
                                                      ViChar    sParameterList[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sensor, 1, 4),
            2, "Sensor");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Sense%ld", sensor);

    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_POWER_SPARAMETER_LIST,
            arraySize, sParameterList),
            4, "S Parameter List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Several S-parameter tables can be stored in a sensor. This function
/// HIFN  selects a loaded data set for S-parameter correction for the
/// HIFN  corresponding sensor.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR sensor/Defines the used power sensor, i.e. the sensor whose values are used.
/// HIPAR sList/Selects the S List number.
ViStatus _VI_FUNC rsrfsiggen_PowerSParameterSelect(ViSession instrSession,
                                                   ViInt32   sensor,
                                                   ViInt32   sList)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sensor, 1, 4),
            2, "Sensor");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "Sense%ld", sensor);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_POWER_SPARAMETER_SELECT, sList),
            3, "S List");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the list to the starting point.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
ViStatus _VI_FUNC rsrfsiggen_ResetList(ViSession instrSession,
                                       ViInt32   path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_RESET_LIST, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures List Import Export.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the dexchange list mode.
/// HIPAR name/Calls the File Manager for selecting the list mode list to be exported
/// HIPAR name/(source) into an ASCII file or the destination for the ASCII file to be
/// HIPAR name/imported (destination) in.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListImportExport(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   mode,
                                                       ViString  name)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_LIST_MODE, mode),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_LIST_FILE, name),
            4, "Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures List Import Export ASCII.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR name/Selects the specified list. If a new list is to be created, the name
/// HIPAR name/can be entered here. The list is created if it does not yet exist. The
/// HIPAR name/list selected here is available for the further processing steps
/// HIPAR name/(editing) and is used in the instrument when the List mode is
/// HIPAR name/activated.
/// HIPAR extension/Selects the file extension of the ASCII file to be imported or
/// HIPAR extension/exported. Selection TXT (text file) or CSV (Excel file) is available.
/// HIPAR columnSeparator/Selects the separator between the frequency and level column of the
/// HIPAR columnSeparator/ASCII table.
/// HIPAR decimalSeparator/Selects the decimal separator used in the ASCII data between '.'
/// HIPAR decimalSeparator/(decimal point) and ',' (comma) with floating-point numerals.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListImportExportASCII(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViString  name,
                                                            ViInt32   extension,
                                                            ViInt32   columnSeparator,
                                                            ViInt32   decimalSeparator)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_ASCII_FILE, name),
            3, "Name");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_ASCII_EXTENSION, extension),
            4, "Extension");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_ASCII_COLUMN_SEPARATOR, columnSeparator),
            5, "Column Separator");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_IMPORT_EXPORT_ASCII_COLUMN_DECIMAL_SEPARATOR, decimalSeparator),
            6, "Decimal Separator");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function starts the export or import of the selected file.
/// HIFN     When import is selected, the ASCII file is imported as list mode list.
/// HIFN
/// HIFN     When export is selected, the list mode list is exported into the
/// HIFN  selected ASCII file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
ViStatus _VI_FUNC rsrfsiggen_ExecuteListImportExport(ViSession instrSession,
                                                     ViInt32   path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_IMPORT_EXPORT_EXECUTE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the list mode data parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR startListRange/Sets the start index of the index range which defines a subgroup of
/// HIPAR startListRange/frequency/level value pairs in the current list.
/// HIPAR stopListRange/Sets the stop index of the index range which defines a subgroup
/// HIPAR stopListRange/of frequency/level value pairs in the current list.
/// HIPAR runMode/Selects the run mode for processing the list.
ViStatus _VI_FUNC rsrfsiggen_ConfigureListModeData(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   startListRange,
                                                   ViInt32   stopListRange,
                                                   ViInt32   runMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_START_INDEX, startListRange),
            3, "Start List Range");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_STOP_INDEX, stopListRange),
            4, "Stop List Range");

    if (!rsrfsiggen_IsSMBV(instrSession))
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LIST_RUN_MODE, runMode),
                5, "Run Mode");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator's reference oscillator.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR source/This function selects between internal and external reference
/// HIPAR source/frequency.
/// HIPAR frequency/Valid Values:
ViStatus _VI_FUNC rsrfsiggen_ConfigureReferenceOscillator(ViSession instrSession,
                                                          ViInt32   source,
                                                          ViInt32   frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_REFERENCE_OSCILLATOR_SOURCE, source),
            2, "Source");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frequency, RSRFSIGGEN_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_5MHZ, RSRFSIGGEN_VAL_SMU_ROSC_EXTERNAL_FREQUENCY_13MHZ),
            3, "Frequency");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:ROSC:EXT:FREQ %s", frequencyArr[frequency]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply IQ (vector)
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqEnabled/Activates and deactivates IQ modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean iqEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_ENABLED, iqEnabled),
            3, "IQ Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the signal generator to apply IQ (vector)
/// HIFN  modulation to the RF output signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the input signal for the IQ modulator. This selection is only
/// HIPAR source/available for Path A (SOURce[1]). The input signal for the IQ modulator
/// HIPAR source/of Path B is always the internal baseband signal (BASeband).
/// HIPAR swapEnabled/When set to ON, this command swaps the I and Q channel.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQ(ViSession instrSession,
                                         ViInt32   path,
                                         ViInt32   source,
                                         ViBoolean swapEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_MODULATION_SOURCE, source),
            3, "Source");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_SWAP_ENABLED, swapEnabled),
            4, "Swap Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the IQ (vector) modulation to allow controlled
/// HIFN  impairment for test or external corrections.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqImpairmentEnabled/Activates (ON) and deactivates (OFF) the three impairment or
/// HIPAR iqImpairmentEnabled/correction values LEAKage, QUADrature and IQRatio for the analog signal
/// HIPAR iqImpairmentEnabled/in the IQ modulator.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQImpairmentEnabled(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViBoolean iqImpairmentEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_IMPAIRMENTS_ENABLED, iqImpairmentEnabled),
            3, "IQ Impairment Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the settings that simulate or correct
/// HIFN  impairment for the signal generator's IQ modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iOffset/Sets the carrier leakage amplitude for the I-channel of the IQ
/// HIPAR iOffset/modulation.
/// HIPAR qOffset/Sets the carrier leakage amplitude for the Q-channel of the IQ
/// HIPAR qOffset/modulation.
/// HIPAR gainImbalance/Sets the ratio of I modulation to Q modulation (amplification
/// HIPAR gainImbalance/"imbalance").
/// HIPAR quadratureOffset/Sets the quadrature offset for the IQ modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQImpairment(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViReal64  iOffset,
                                                   ViReal64  qOffset,
                                                   ViReal64  gainImbalance,
                                                   ViReal64  quadratureOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_I_OFFSET, iOffset),
            3, "I Offset");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_Q_OFFSET, qOffset),
            4, "Q Offset");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_GAIN_IMBALANCE, gainImbalance),
            5, "Gain Imbalance");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_QUADRATURE_OFFSET, quadratureOffset),
            6, "Quadrature Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the IQ Analog Wideband Input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR gain/Specifies the gain for the analog wideband I/Q input.
/// HIPAR crestFactor/Specifies the crest factor of the external analog signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQAnalogWidebandInput(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   gain,
                                                            ViReal64  crestFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_ANALOG_WIDEBAND_GAIN, gain),
            3, "Gain");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_ANALOG_WIDEBAND_CREST_FACTOR, crestFactor),
            4, "Crest Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates optimization of level accuracy and imbalance for
/// HIFN  RF Output.
/// HIFN
/// HIFN     Level accuracy and imbalance can be either optimized for RF output
/// HIFN  (active) or for differential outputs (not active).
/// HIFN
/// HIFN     With a second option Baseband Main Module (B13) it is possible to
/// HIFN  optimize a signal for  for RF output in one path and for the
/// HIFN  differential output in the other path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqSignalsforRFOutput/Activates/deactivates optimization of level accuracy and imbalance for
/// HIPAR iqSignalsforRFOutput/RF Output.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputOptimizeIQSignalsforRFOutputEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean iqSignalsforRFOutput)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_OPTIMIZE_IQ_SIGNALS_FOR_RF_OUTPUT_ENABLED, iqSignalsforRFOutput),
            3, "IQ Signals for RF Output");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the I/Q output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqOutputType/Sets the I/Q output type.
/// HIPAR mode/Selects the mode for setting the outputs - independent or together.
/// HIPAR mode/The menu changes depending on the selection here.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutput(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   iqOutputType,
                                               ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_TYPE, iqOutputType),
            3, "IQ Output Type");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_MODE, mode),
            4, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the IQ Analog Output state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqAnalogEnabled/Switches on and off the analog I/Q output signals.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputAnalogEnabled(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViBoolean iqAnalogEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_ANALOG_ENABLED, iqAnalogEnabled),
            3, "IQ Analog Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the I/Q output analog.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR biasI/Sets the bias. A DC voltage is superimposed upon the symmetric I or Q
/// HIPAR biasI/signal. The maximum overall voltage (Level EMF + Bias + Offset) is 4
/// HIPAR biasI/volts.
/// HIPAR offsetI/Sets an offset between the inverting and the non-inverting output. The
/// HIPAR offsetI/maximum overall voltage (Level EMF + Bias + Offset) is 4 Volt.  The set
/// HIPAR offsetI/value is set half in the positive and half in the negative direction:
/// HIPAR biasQ/Sets the bias. A DC voltage is superimposed upon the symmetric I or Q
/// HIPAR biasQ/signal. The maximum overall voltage (Level EMF + Bias + Offset) is 4
/// HIPAR biasQ/volts.
/// HIPAR offsetQ/Sets an offset between the inverting and the non-inverting output. The
/// HIPAR offsetQ/maximum overall voltage (Level EMF + Bias + Offset) is 4 Volt.  The set
/// HIPAR offsetQ/value is set half in the positive and half in the negative direction:
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputAnalog(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  biasI,
                                                     ViReal64  offsetI,
                                                     ViReal64  biasQ,
                                                     ViReal64  offsetQ)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_BIAS_I, biasI),
            3, "Bias I");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_OFFSET_I, offsetI),
            4, "Offset I");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_BIAS_Q, biasQ),
            5, "Bias Q");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_OFFSET_Q, offsetQ),
            6, "Offset Q");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the VCC value for the selected <xValue>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR xValue/Sets the value on the x-axis. Value range depends on the selected
/// HIPAR xValue/PEPinMin and PEPinMax values.
/// HIPAR xUnit/Selects the x unit.
/// HIPAR xScale/Selects the x scale.
/// HIPAR vcc/Returns the delta phase value.
ViStatus _VI_FUNC rsrfsiggen_QueryIQEnvelopeVccForX(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViReal64  xValue,
                                                    ViInt32   xUnit,
                                                    ViInt32   xScale,
                                                    ViReal64* vcc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xValue, -100.0, 100.0), 3, "X Value");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xUnit, RSRFSIGGEN_VAL_ENVELOPE_XUNIT_NORMALIZED, RSRFSIGGEN_VAL_ENVELOPE_XUNIT_V),
            4, "X Unit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xScale, RSRFSIGGEN_VAL_ENVELOPE_XSCALE_VOLTAGE, RSRFSIGGEN_VAL_ENVELOPE_XSCALE_POWER),
            5, "X Scale");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:IQ:OUTP:ANAL:ENV:VCC:VAL? %.12lG,%s,%s", path, xValue, RsRFSigGen_rngIQEnvelopeVccForXUnit.entries[xUnit].cmdString, RsRFSigGen_rngIQEnvelopeVccForXScale.entries[xScale].cmdString);
    checkErr(rsrfsiggen_QueryViReal64(instrSession, cmd, vcc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the shaping values into a file with the selected file name and
/// HIFN  loads it. The file is stored in the default directory or in the
/// HIFN  directory specified with the absolute file path. If the file does not
/// HIFN  yet exist, a new file is created. The file extension is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputEnvelopeTablesNew(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:OUTP:ANAL:ENV:SHAP:FILE:NEW '%s',", path, fileName);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the shaping function in a raw data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputEnvelopeTablesData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:OUTP:ANAL:ENV:SHAP:FILE:DATA ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the shaping values into a file with the selected file name and
/// HIFN  loads it. The file is stored in the default directory or in the
/// HIFN  directory specified with the absolute file path. If the file does not
/// HIFN  yet exist, a new file is created. The file extension is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputEnvelopeTablesAutoNew(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:OUTP:ANAL:ENV:SHAP:PV:FILE:NEW '%s',", path, fileName);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the shaping function in a raw data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputEnvelopeTablesAutoData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:OUTP:ANAL:ENV:SHAP:PV:FILE:DATA ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the polynomial coefficients.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR coefficients/Sets the polynomial constant a0 and the polynomial coefficients a1 to
/// HIPAR coefficients/an. The polynomial constant and coefficients influence the envelope
/// HIPAR coefficients/shape.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputEnvelopePolynomialCoefficients(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  coefficients[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:OUTP:ANAL:ENV:SHAP:COEF ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, coefficients, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the IQ Digital Output state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqDigitalEnabled/Switches on and off the digital I/Q output signals.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQOutputDigitalEnabled(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViBoolean iqDigitalEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_OUTPUT_DIGITAL_ENABLED, iqDigitalEnabled),
            3, "IQ Digital Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the IQ Digital Output state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR digitalPredistortionEnabled/Enabels or disables the generation of digital predistored signals.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean digitalPredistortionEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_ENABLED, digitalPredistortionEnabled),
            3, "Digital Predistortion Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the main parameters for the IQ digital predistortion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR referenceLevel/Sets whether the selected Level value corresponds to the signal level
/// HIPAR referenceLevel/before or after the predistortion.
/// HIPAR AMAMFirst/Sets that the AM/AM predistortion is applied prior to the AM/PM.
/// HIPAR maximumOutputLevelError/ Sets the allowed maximum error.
/// HIPAR maximumNumberOfIterations/Sets the manximum number of performed iteration to achieving the
/// HIPAR maximumNumberOfIterations/requiered errror set
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortion(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   referenceLevel,
                                                             ViBoolean AMAMFirst,
                                                             ViReal64  maximumOutputLevelError,
                                                             ViInt32   maximumNumberOfIterations)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_REFERENCE_LEVEL, referenceLevel),
            3, "Reference Level");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_AMAM_FIRST, AMAMFirst),
            4, "AM/AM First");

    if (referenceLevel == RSRFSIGGEN_VAL_IQ_LEVEL_AFTER_DIGITAL_PREDISTORTION)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_MAXIMUM_OUTPUT_LEVEL_ERROR, maximumOutputLevelError),
                5, "Maximum Output Level Error");
    }

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_MAXIMUM_NUMBER_OF_ITERATIONS, maximumNumberOfIterations),
            6, "Maximum Number Of Iterations");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the main parameters for predistortion correction coefficients.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR shaping/Selects the method to define the correction coefficients.
/// HIPAR interpolation/Enables a linear (voltage or power) interpolation between the defined correction values.
/// HIPAR invertCorrectionValues/Inverts the defined correction values.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionCorrection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   shaping,
                            ViInt32   interpolation,
                            ViBoolean invertCorrectionValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_CORRECTION_COEFFICIENTS_SHAPING, shaping),
            3, "Shaping");

    if (shaping == RSRFSIGGEN_VAL_IQ_COEFFICIENTS_CORRECTION_FROM_TABLE || shaping == RSRFSIGGEN_VAL_IQ_COEFFICIENTS_CORRECTION_NORMALIZED)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_INTERPOLATION, interpolation),
                4, "Interpolation");
    }

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_INVERT_CORRECTION_VALUES, invertCorrectionValues),
            5, "Invert Correction Values");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the minimum and the maximum values of the input power (PEPin).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR minimumRange_PEPin/Sets the minimum value of the input power.
/// HIPAR maximumRange_PEPin/Sets the maximum value of the input power.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionInputPower(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  minimumRange_PEPin,
                            ViReal64  maximumRange_PEPin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_INPUT_POWER_MINIMUM, minimumRange_PEPin),
            3, "minimumRange_PEPin");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_IQ_DIGITAL_PREDISTORTION_INPUT_POWER_MAXIMUM, maximumRange_PEPin),
            4, "Maximum Range PEPin");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the predistortion function in a raw data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionDPDShapingTablesData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DPD:SHAP:TABL:%s:FILE:DATA ", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM");
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the correction values into a file with the selected file name
/// HIFN  and loads it. The file is stored in the default directory or in the
/// HIFN  directory specified with the absolute file path. If the file does not
/// HIFN  yet exist, a new file is created. The file extension is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR fileName/File Name parameter.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionDPDShapingTablesNewFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViString  fileName,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DPD:SHAP:TABL:%s:FILE:NEW '%s',", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM", fileName);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the delta phase and delta power values for the
/// HIFN  selected <xValue>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR xValue/Sets the value on the x-axis. Value range depends on the selected
/// HIPAR xValue/PEPinMin and PEPinMax values.
/// HIPAR xUnit/Selects the unit.
/// HIPAR deltaPhase/Returns the delta phase value.
/// HIPAR deltaPower/Returns the delta power value.
ViStatus _VI_FUNC rsrfsiggen_QueryIQDigitalPredistortionDPDDeltaPhaseDeltaPowerForX(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViReal64  xValue,
                            ViInt32   xUnit,
                            ViReal64* deltaPhase,
                            ViReal64* deltaPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 data[2] = { 0.0, 0.0 };
    ViInt32  returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xValue, -100.0, 100.0), 3, "X Value");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xUnit, RSRFSIGGEN_VAL_PREDIST_UNIT_DBM, RSRFSIGGEN_VAL_PREDIST_UNIT_V),
            4, "X Unit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DPD:%s:VAL? %.12lG,%s", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM", xValue, RsRFSigGen_rngPredistortionXUnit.entries[xUnit].cmdString);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 2, data, &returnedValues));

    *deltaPhase = data[0];
    if (returnedValues > 1)
        *deltaPower = data[1];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the polynomial coefficients.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR coefficients/Sets the list of up to 10 comma separated value pairs.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalDPDShapingPolynomialCoefficients(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  coefficients[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DPD:SHAP:POLY:COEF ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, coefficients, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the normalized predistortion function in a raw data format
/// HIFN  (binary data).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pinmax/Maximum absolute input power Pinmax.
/// HIPAR vinvmax/Normalized values Vin/Vmax.
/// HIPAR deltaV/Normalized values deltaV/V.
/// HIPAR deltaPhase/Normalized values deltaPhase[deg].
/// HIPAR arraySize/Array size parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalPredistortionNormalizedData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  pinmax,
                            ViReal64  vinvmax[],
                            ViReal64  deltaV[],
                            ViReal64  deltaPhase[],
                            ViInt32   arraySize)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64* data = NULL;
    ViInt32   i = 0;
    ViByte*   outBinData = NULL;
    ViInt32   outDataSize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DPD:SHAP:NORM:DATA ", path);

    viCheckAlloc(data = (ViReal64*)malloc(sizeof(ViReal64) * (3 * arraySize + 2)));

    data[0] = pinmax;
    data[1] = (ViReal64)arraySize;
    for (i = 0; i < arraySize; i++)
    {
        data[2 + (3 * i)] = vinvmax[i];
        data[2 + (3 * i + 1)] = deltaV[i];
        data[2 + (3 * i + 2)] = deltaPhase[i];
    }

    checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, data, arraySize + 2, &outBinData, &outDataSize, VI_FALSE));
    checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, outBinData, (ViInt64)outDataSize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (data) free(data);

    if (outBinData) free(outBinData);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the predistortion function in a raw data format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalDohertyShapingTablesData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DOH:SHAP:TABL:%s:FILE:DATA ", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM");
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the correction values into a file with the selected file name
/// HIFN  and loads it. The file is stored in the default directory or in the
/// HIFN  directory specified with the absolute file path. If the file does not
/// HIFN  yet exist, a new file is created. The file extension is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR fileName/File Name parameter.
/// HIPAR arraySize/Array size parameter.
/// HIPAR data/Table data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalDohertyShapingTablesNewFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViString  fileName,
                            ViInt32   arraySize,
                            ViReal64  data[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DOH:SHAP:TABL:%s:FILE:NEW '%s',", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM", fileName);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, data, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the delta phase and delta power values for the
/// HIFN  selected <xValue>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR diagram/Selects the diagram.
/// HIPAR xValue/Sets the value on the x-axis. Value range depends on the selected
/// HIPAR xValue/PEPinMin and PEPinMax values.
/// HIPAR xUnit/Selects the unit.
/// HIPAR deltaPhase/Returns the delta phase value.
/// HIPAR deltaPower/Returns the delta power value.
ViStatus _VI_FUNC rsrfsiggen_QueryIQDigitalDohertyDeltaPhaseDeltaPowerForX(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   diagram,
                            ViReal64  xValue,
                            ViInt32   xUnit,
                            ViReal64* deltaPhase,
                            ViReal64* deltaPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 data[2] = { 0.0, 0.0 };
    ViInt32  returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, diagram, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM, RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_PM),
            3, "Diagram");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, xValue, -100.0, 100.0), 3, "X Value");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, xUnit, RSRFSIGGEN_VAL_PREDIST_UNIT_DBM, RSRFSIGGEN_VAL_PREDIST_UNIT_V),
            4, "X Unit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DOH:%s:VAL? %.12lG,%s", path, (diagram == RSRFSIGGEN_VAL_PREDIST_DIAGRAM_AM_AM) ? "AMAM" : "AMPM", xValue, RsRFSigGen_rngPredistortionXUnit.entries[xUnit].cmdString);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 2, data, &returnedValues));

    *deltaPhase = data[0];
    if (returnedValues > 1)
        *deltaPower = data[1];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the polynomial coefficients.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR coefficients/Sets the list of up to 10 comma separated value pairs.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitalDohertyShapingPolynomialCoefficients(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  coefficients[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DOH:SHAP:POLY:COEF ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, coefficients, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the normalized predistortion function in a raw data format
/// HIFN  (binary data).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pinmax/Maximum absolute input power Pinmax.
/// HIPAR vinvmax/Normalized values Vin/Vmax.
/// HIPAR deltaV/Normalized values deltaV/V.
/// HIPAR deltaPhase/Normalized values deltaPhase[deg].
/// HIPAR arraySize/Array size parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureIQDigitaDohertylPredistortionNormalizedData(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  pinmax,
                            ViReal64  vinvmax[],
                            ViReal64  deltaV[],
                            ViReal64  deltaPhase[],
                            ViInt32   arraySize)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64* data = NULL;
    ViInt32   i = 0;
    ViByte*   outBinData = NULL;
    ViInt32   outDataSize = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:IQ:DOH:SHAP:NORM:DATA ", path);

    viCheckAlloc(data = (ViReal64*)malloc(sizeof(ViReal64) * (3 * arraySize + 2)));

    data[0] = pinmax;
    data[1] = (ViReal64)arraySize;
    for (i = 0; i < arraySize; i++)
    {
        data[2 + (3 * i)] = vinvmax[i];
        data[2 + (3 * i + 1)] = deltaV[i];
        data[2 + (3 * i + 2)] = deltaPhase[i];
    }

    checkErr(RsCore_Convert_Float32Array_To_BinaryDataBlock(instrSession, data, arraySize + 2, &outBinData, &outDataSize, VI_FALSE));
    checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, outBinData, (ViInt64)outDataSize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (data) free(data);

    if (outBinData) free(outBinData);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function do selected file operation for selected subsystem.
/// HIFN
/// HIFN     The default folder is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified folder are
/// HIFN  read. Only files with the file extension *.'subsystem' will be listed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR subsystem/Sets the subsystem for selected operation.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Reads out the files with selected subsystem.
ViStatus _VI_FUNC rsrfsiggen_FileCatalog(ViSession instrSession,
                                         ViInt32   path,
                                         ViInt32   subsystem,
                                         ViInt32   arraySize,
                                         ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attribute = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    if (arraySize < 1)
    {
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, arraySize), 1, "Array Size");
    }

    switch (subsystem)
    {
    case RSRFSIGGEN_VAL_SUBSYSTEM_GSM:
        attribute = RSRFSIGGEN_ATTR_GSM_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_BTO:
        attribute = RSRFSIGGEN_ATTR_BT_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_W3GPP:
        attribute = RSRFSIGGEN_ATTR_W3GPP_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_C2K:
        attribute = RSRFSIGGEN_ATTR_C2K_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_TDSCDMA:
        attribute = RSRFSIGGEN_ATTR_TDSCDMA_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLAN:
        attribute = RSRFSIGGEN_ATTR_WLAN_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLANN:
        attribute = RSRFSIGGEN_ATTR_WLANN_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WIMAX:
        attribute = RSRFSIGGEN_ATTR_WIMAX_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_DVB:
        attribute = RSRFSIGGEN_ATTR_DVB_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_DAB:
        attribute = RSRFSIGGEN_ATTR_DAB_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_DM:
        attribute = RSRFSIGGEN_ATTR_DM_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_GPS:
        attribute = RSRFSIGGEN_ATTR_GPS_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_STEREO:
        attribute = RSRFSIGGEN_ATTR_STEREO_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_XMR:
        attribute = RSRFSIGGEN_ATTR_XMR_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_SIRIUS:
        attribute = RSRFSIGGEN_ATTR_SIRIUS_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_EUTRA:
        attribute = RSRFSIGGEN_ATTR_EUTRA_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_TETRA:
        attribute = RSRFSIGGEN_ATTR_TETRA_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_FADING:
        attribute = RSRFSIGGEN_ATTR_FSIM_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_EVDO:
        attribute = RSRFSIGGEN_ATTR_EVDO_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_OFDM:
        attribute = RSRFSIGGEN_ATTR_OFDM_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLANAD:
        attribute = RSRFSIGGEN_ATTR_WLANAD_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_ONEWEB:
        attribute = RSRFSIGGEN_ATTR_ONEWEB_CATALOG_SETTINGS;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_EXTSEQ:
        attribute = RSRFSIGGEN_ATTR_EXTSEQ_CATALOG_SETTINGS;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_V5G:
        attribute = RSRFSIGGEN_ATTR_V5G_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_5GNR:
        attribute = RSRFSIGGEN_ATTR_5G_NR_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_LORA:
        attribute = RSRFSIGGEN_ATTR_LORA_SETTINGS_CATALOG;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_HRP_UWB:
        attribute = RSRFSIGGEN_ATTR_HRP_UWB_SETTINGS_CATALOG;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, subsystem), 3, "Subsystem");
        break;
    }

    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, repCap, attribute,
            arraySize, catalog),
            5, "Catalog");

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function do selected file operation for selected subsystem.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.'subsystem' will be selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR action/Sets the action with selected file.
/// HIPAR subsystem/Sets the subsystem for selected operation.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_FileSettings(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   action,
                                          ViInt32   subsystem,
                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attribute = RSRFSIGGEN_ATTR_GSM_DELETE_SETTINGS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, action, 0, 2),
            3, "Action");

    switch (action)
    {
    case RSRFSIGGEN_VAL_FILE_ACTION_DELETE:
    {
        switch (subsystem)
        {
        case RSRFSIGGEN_VAL_SUBSYSTEM_GSM:
            attribute = RSRFSIGGEN_ATTR_GSM_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_BTO:
            attribute = RSRFSIGGEN_ATTR_BT_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_W3GPP:
            attribute = RSRFSIGGEN_ATTR_W3GPP_DELETE_3GPP_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_C2K:
            attribute = RSRFSIGGEN_ATTR_C2K_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TDSCDMA:
            attribute = RSRFSIGGEN_ATTR_TDSCDMA_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLAN:
            attribute = RSRFSIGGEN_ATTR_WLAN_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANN:
            attribute = RSRFSIGGEN_ATTR_WLANN_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WIMAX:
            attribute = RSRFSIGGEN_ATTR_WIMAX_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DVB:
            attribute = RSRFSIGGEN_ATTR_DVB_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DAB:
            attribute = RSRFSIGGEN_ATTR_DAB_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DM:
            attribute = RSRFSIGGEN_ATTR_DM_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_GPS:
            attribute = RSRFSIGGEN_ATTR_GPS_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_STEREO:
            attribute = RSRFSIGGEN_ATTR_STEREO_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_XMR:
            attribute = RSRFSIGGEN_ATTR_XMR_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_SIRIUS:
            attribute = RSRFSIGGEN_ATTR_SIRIUS_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EUTRA:
            attribute = RSRFSIGGEN_ATTR_EUTRA_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TETRA:
            attribute = RSRFSIGGEN_ATTR_TETRA_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_FADING:
            attribute = RSRFSIGGEN_ATTR_FSIM_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EVDO:
            attribute = RSRFSIGGEN_ATTR_EVDO_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_OFDM:
            attribute = RSRFSIGGEN_ATTR_OFDM_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANAD:
            attribute = RSRFSIGGEN_ATTR_WLANAD_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EXTSEQ:
            attribute = RSRFSIGGEN_ATTR_EXTSEQ_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_V5G:
            attribute = RSRFSIGGEN_ATTR_V5G_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_5GNR:
            attribute = RSRFSIGGEN_ATTR_5G_NR_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_LORA:
            attribute = RSRFSIGGEN_ATTR_LORA_DELETE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_HRP_UWB:
            attribute = RSRFSIGGEN_ATTR_HRP_UWB_DELETE_SETTINGS;
            break;

        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 4, "Subsystem");
            break;
        }
        break;
    }

    case RSRFSIGGEN_VAL_FILE_ACTION_RECALL:
    {
        switch (subsystem)
        {
        case RSRFSIGGEN_VAL_SUBSYSTEM_GSM:
            attribute = RSRFSIGGEN_ATTR_GSM_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_BTO:
            attribute = RSRFSIGGEN_ATTR_BT_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_W3GPP:
            attribute = RSRFSIGGEN_ATTR_W3GPP_RECALL_3GPP_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_C2K:
            attribute = RSRFSIGGEN_ATTR_C2K_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TDSCDMA:
            attribute = RSRFSIGGEN_ATTR_TDSCDMA_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLAN:
            attribute = RSRFSIGGEN_ATTR_WLAN_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANN:
            attribute = RSRFSIGGEN_ATTR_WLANN_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WIMAX:
            attribute = RSRFSIGGEN_ATTR_WIMAX_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DVB:
            attribute = RSRFSIGGEN_ATTR_DVB_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DAB:
            attribute = RSRFSIGGEN_ATTR_DAB_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DM:
            attribute = RSRFSIGGEN_ATTR_DM_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_GPS:
            attribute = RSRFSIGGEN_ATTR_GPS_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_STEREO:
            attribute = RSRFSIGGEN_ATTR_STEREO_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_XMR:
            attribute = RSRFSIGGEN_ATTR_XMR_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_SIRIUS:
            attribute = RSRFSIGGEN_ATTR_SIRIUS_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EUTRA:
            attribute = RSRFSIGGEN_ATTR_EUTRA_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TETRA:
            attribute = RSRFSIGGEN_ATTR_TETRA_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_FADING:
            attribute = RSRFSIGGEN_ATTR_FSIM_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EVDO:
            attribute = RSRFSIGGEN_ATTR_EVDO_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_OFDM:
            attribute = RSRFSIGGEN_ATTR_OFDM_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANAD:
            attribute = RSRFSIGGEN_ATTR_WLANAD_RECALL_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EXTSEQ:
            attribute = RSRFSIGGEN_ATTR_EXTSEQ_LOAD_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_ONEWEB:
            attribute = RSRFSIGGEN_ATTR_ONEWEB_LOAD_SETTINGS;
            break;
        case RSRFSIGGEN_VAL_SUBSYSTEM_V5G:
            attribute = RSRFSIGGEN_ATTR_V5G_RECALL_SETTINGS;
            break;
        case RSRFSIGGEN_VAL_SUBSYSTEM_5GNR:
            attribute = RSRFSIGGEN_ATTR_5G_NR_RECALL_SETTINGS;
            break;
        case RSRFSIGGEN_VAL_SUBSYSTEM_LORA:
            attribute = RSRFSIGGEN_ATTR_LORA_RECALL_SETTINGS;
            break;
        case RSRFSIGGEN_VAL_SUBSYSTEM_HRP_UWB:
            attribute = RSRFSIGGEN_ATTR_HRP_UWB_LOAD_SETTINGS;
            break;

        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 4, "Subsystem");
            break;
        }
        break;
    }

    case RSRFSIGGEN_VAL_FILE_ACTION_SAVE:
    {
        switch (subsystem)
        {
        case RSRFSIGGEN_VAL_SUBSYSTEM_GSM:
            attribute = RSRFSIGGEN_ATTR_GSM_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_BTO:
            attribute = RSRFSIGGEN_ATTR_BT_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_W3GPP:
            attribute = RSRFSIGGEN_ATTR_W3GPP_SAVE_3GPP_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_C2K:
            attribute = RSRFSIGGEN_ATTR_C2K_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TDSCDMA:
            attribute = RSRFSIGGEN_ATTR_TDSCDMA_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLAN:
            attribute = RSRFSIGGEN_ATTR_WLAN_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANN:
            attribute = RSRFSIGGEN_ATTR_WLANN_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WIMAX:
            attribute = RSRFSIGGEN_ATTR_WIMAX_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DVB:
            attribute = RSRFSIGGEN_ATTR_DVB_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DAB:
            attribute = RSRFSIGGEN_ATTR_DAB_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_DM:
            attribute = RSRFSIGGEN_ATTR_DM_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_GPS:
            attribute = RSRFSIGGEN_ATTR_GPS_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_STEREO:
            attribute = RSRFSIGGEN_ATTR_STEREO_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_XMR:
            attribute = RSRFSIGGEN_ATTR_XMR_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_SIRIUS:
            attribute = RSRFSIGGEN_ATTR_SIRIUS_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EUTRA:
            attribute = RSRFSIGGEN_ATTR_EUTRA_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_TETRA:
            attribute = RSRFSIGGEN_ATTR_TETRA_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EVDO:
            attribute = RSRFSIGGEN_ATTR_EVDO_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_OFDM:
            attribute = RSRFSIGGEN_ATTR_OFDM_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_WLANAD:
            attribute = RSRFSIGGEN_ATTR_WLANAD_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_EXTSEQ:
            attribute = RSRFSIGGEN_ATTR_EXTSEQ_STORE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_ONEWEB:
            attribute = RSRFSIGGEN_ATTR_ONEWEB_STORE_SETTINGS;
            break;
        case RSRFSIGGEN_VAL_SUBSYSTEM_V5G:
            attribute = RSRFSIGGEN_ATTR_V5G_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_5GNR:
            attribute = RSRFSIGGEN_ATTR_5G_NR_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_LORA:
            attribute = RSRFSIGGEN_ATTR_LORA_SAVE_SETTINGS;
            break;

        case RSRFSIGGEN_VAL_SUBSYSTEM_HRP_UWB:
            attribute = RSRFSIGGEN_ATTR_HRP_UWB_SAVE_SETTINGS;
            break;

        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 4, "Subsystem");
            break;
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, action), 3, "Action");
        break;
    }
    }

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, attribute, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads the indicated directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will
/// HIPAR directory/be performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns the files in the indicated directory as
/// HIPAR output/comma separated strings.
ViStatus _VI_FUNC rsrfsiggen_FileDirectory(ViSession instrSession,
                                           ViString  directory,
                                           ViInt32   bufferSize,
                                           ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViUInt32 count = 0;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, directory),
        2, "Directory");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:CAT? '%s'", directory);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    checkErr(rsrfsiggen_CheckStatus(instrSession));
    //strncpy(output, pbuffer, (size_t) bufferSize);
    if (count >= (ViUInt32)bufferSize)
    {
        strncpy(output, pbuffer, (size_t)(bufferSize - 1));
        output[bufferSize - 1] = '\0';
        error = (ViStatus)(count + 1);
    }
    else
    {
        strncpy(output, pbuffer, (size_t)count);
        output[count] = '\0';
    }

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the number of files and subdirectories of the current or of a
/// HIFN  specified directory. The number includes the parent directory strings
/// HIFN  "." and ".." so that it corresponds to the number of strings returned
/// HIFN  by the rsscope_FileDirectoryContent function after the initial numeric
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberOfFilesAndFolders/Returns the number of files and subdirectories of the current or of a
/// HIPAR numberOfFilesAndFolders/specified directory.
ViStatus _VI_FUNC rsrfsiggen_FileCount(ViSession instrSession,
                                       ViString  directory,
                                       ViInt32*  numberOfFilesAndFolders)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG?");

    if (!isNullOrEmpty(directory))
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG? '%s'", directory);
    else
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:CAT:LENG?");

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfFilesAndFolders));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads out the subdirectories of the specified directory.
/// HIFN  If no directory is specified, the default directory selected with the
/// HIFN  MMEM:CDIR command is read out. The directories are output in a list
/// HIFN  (the list entries are separated by commas).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Output parameter.
/// HIPAR output/This indicator returns names of the subdirectories separated by
/// HIPAR output/colons. The first two strings are related to the parent directory.
ViStatus _VI_FUNC rsrfsiggen_FileSubDirectory(ViSession instrSession,
                                              ViString  directory,
                                              ViInt32   bufferSize,
                                              ViChar    output[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, directory),
        2, "Directory");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DCAT? '%s'", directory);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, bufferSize, output));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns out the number of subdirectories in the specified directory.
/// HIFN  If no directory is specified, the directory selected with the MMEM:CDIR
/// HIFN  command is read out.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/This control specifies the directory in which the command will be
/// HIPAR directory/performed.
/// HIPAR numberofFilesAndFolders/Returns the number number of parent and subdirectories.
ViStatus _VI_FUNC rsrfsiggen_FileSubDirectoryCount(ViSession instrSession,
                                                   ViString  directory,
                                                   ViInt32*  numberOfFilesAndFolders)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = cmd;

    checkErr(RsCore_LockSession(instrSession));

    viCheckAlloc(numberOfFilesAndFolders);

    p2buf += sprintf(p2buf, "MMEM:DCAT:LENG?");

    if ((directory != NULL) && (strlen(directory) > 0))
        sprintf(p2buf, " '%s'", directory);

    checkErr(RsCore_QueryViInt32(instrSession, cmd, numberOfFilesAndFolders));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Renames an existing file ('Source') if no path is specified for
/// HIFN  'Destination'. Otherwise the file is moved to the specified path and
/// HIFN  stored under the original file name or, if specified, a new file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Define file source.
/// HIPAR destination/Define file destination.
ViStatus _VI_FUNC rsrfsiggen_MoveFile(ViSession instrSession,
                                      ViString  source,
                                      ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:MOVE '%s','%s'", source, destination);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies the first specified file to the second specified file. Instead
/// HIFN  of just a file, this function can also be used to copy a complete
/// HIFN  directory together with all its files.
/// HIFN
/// HIFN     If <destination> is not specified, <source> is copied to the MMEM:MSIS
/// HIFN  drive and the MMEM:CDIR directory. Files which already exist with the
/// HIFN  same name in the destination directory are overwritten without an error
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/Define file source.
/// HIPAR destination/Define file destination.
ViStatus _VI_FUNC rsrfsiggen_CopyFile(ViSession instrSession,
                                      ViString  source,
                                      ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:COPY '%s','%s'", source, destination);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the specified file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR filename/Deletes the specified file.
ViStatus _VI_FUNC rsrfsiggen_FileDelete(ViSession instrSession,
                                        ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_DELETE, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Changes the default directory. This directory is used for all
/// HIFN  subsequent MMEM commands if no path is specified with them. It is also
/// HIFN  possible to change to a higher directory using two dots '..'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/Changes the default directory. This directory is used for all
/// HIPAR directory/subsequent MMEM commands if no path is specified with them. It is also
/// HIPAR directory/possible to change to a higher directory using two dots '..'.
ViStatus _VI_FUNC rsrfsiggen_ChangeDirectory(ViSession instrSession,
                                             ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_CHANGE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Creates a new subdirectory in the specified directory. If no directory
/// HIFN  is specified, a subdirectory is created in the default directory. This
/// HIFN  function can also be used to create a directory tree.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/Creates a new subdirectory in the specified directory. If no directory
/// HIPAR directory/is specified, a subdirectory is created in the default directory. This
/// HIPAR directory/function can also be used to create a directory tree.
ViStatus _VI_FUNC rsrfsiggen_MakeDirectory(ViSession instrSession,
                                           ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_MAKE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the specified subdirectory in the specified directory. If no
/// HIFN  directory is specified, the subdirectory with the specified name is
/// HIFN  deleted in the default directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR directory/Deletes the specified subdirectory in the specified directory. If no
/// HIPAR directory/directory is specified, the subdirectory with the specified name is
/// HIPAR directory/deleted in the default directory.
ViStatus _VI_FUNC rsrfsiggen_DeleteDirectory(ViSession instrSession,
                                             ViString  directory)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_REMOVE_DIRECTORY, directory),
            2, "Directory");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the user directory, that means the directory the R&S SMW
/// HIFN  stores user files on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR arraySize/Array size parameter.
/// HIPAR userPath/Queries the user directory, that means the directory the R&S SMW
/// HIPAR userPath/stores user files on.
ViStatus _VI_FUNC rsrfsiggen_GetUserPath(ViSession instrSession,
                                         ViInt32   arraySize,
                                         ViChar    userPath[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_USER_PATH,
            arraySize, userPath),
            3, "User Path");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the drive (or network resource in the case of networks) using
/// HIFN  <msus> (MSIS = Mass Storage Identification String). This setting is
/// HIFN  effective for all MMEMory functions where the drive is not explicitly
/// HIFN  specified in the parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR drive/Sets the drive (or network resource in the case of networks) using
/// HIPAR drive/<msus> (MSIS = Mass Storage Identification String). This setting is
/// HIPAR drive/effective for all MMEMory functions where the drive is not explicitly
/// HIPAR drive/specified in the parameter.
ViStatus _VI_FUNC rsrfsiggen_ChangeDrive(ViSession instrSession,
                                         ViString  drive)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_MMEMORY_DRIVE, drive),
            2, "Drive");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the specified file stored under the specified name in an
/// HIFN  internal memory. After the file has been loaded, the instrument setting
/// HIFN  must be activated using an *RCL command.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR number/Determines to the specific <number> to be used with the *RCL command,
/// HIPAR number/e.g. *RCL 4.
/// HIPAR filename/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_LoadState(ViSession instrSession,
                                       ViInt32   number,
                                       ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:LOAD:STAT %ld,'%s'\n", number, filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current instrument setting in the specified file. The
/// HIFN  instrument setting must first be stored in an internal memory with the
/// HIFN  same number using the common command *SAV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR number/Corresponds to the specific <number> defined with the *SAV command,
/// HIPAR number/e.g. *SAV 4.
/// HIPAR filename/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_StoreState(ViSession instrSession,
                                        ViInt32   number,
                                        ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:STOR:STAT %ld,'%s'\n", number, filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Generates waveform file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR subsystem/Sets the subsystem for selected operation.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_GenerateWaveformFile(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   subsystem,
                                                  ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attribute = RSRFSIGGEN_ATTR_GSM_GENERATE_WAVEFORM_FILE;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subsystem, 0, 16),
            3, "Subsystem");

    switch (subsystem)
    {
    case RSRFSIGGEN_VAL_SUBSYSTEM_GSM:
        attribute = RSRFSIGGEN_ATTR_GSM_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_BTO:
        attribute = RSRFSIGGEN_ATTR_BT_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_W3GPP:
        attribute = RSRFSIGGEN_ATTR_W3GPP_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_C2K:
        attribute = RSRFSIGGEN_ATTR_C2K_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_TDSCDMA:
        attribute = RSRFSIGGEN_ATTR_TDSCDMA_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLAN:
        attribute = RSRFSIGGEN_ATTR_WLAN_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLANN:
        attribute = RSRFSIGGEN_ATTR_WLANN_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WIMAX:
        attribute = RSRFSIGGEN_ATTR_WIMAX_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_DVB:
        attribute = RSRFSIGGEN_ATTR_DVB_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_DM:
        attribute = RSRFSIGGEN_ATTR_DM_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_EUTRA:
        attribute = RSRFSIGGEN_ATTR_EUTRA_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_TETRA:
        attribute = RSRFSIGGEN_ATTR_TETRA_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_EVDO:
        attribute = RSRFSIGGEN_ATTR_EVDO_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_OFDM:
        RSRFSIGGEN_ATTR_OFDM_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_WLANAD:
        attribute = RSRFSIGGEN_ATTR_WLANAD_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_ONEWEB:
        attribute = RSRFSIGGEN_ATTR_ONEWEB_STORE_WAVEFORM;
        break;
    case RSRFSIGGEN_VAL_SUBSYSTEM_V5G:
        attribute = RSRFSIGGEN_ATTR_V5G_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_5GNR:
        attribute = RSRFSIGGEN_ATTR_5G_NR_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_LORA:
        attribute = RSRFSIGGEN_ATTR_LORA_GENERATE_WAVEFORM_FILE;
        break;

    case RSRFSIGGEN_VAL_SUBSYSTEM_HRP_UWB:
        attribute = RSRFSIGGEN_ATTR_HRP_UWB_GENERATE_WAVEFORM_FILE;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, subsystem), 3, "Subsystem");
        break;
    }

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, attribute, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Read To File From Instrument
 * Purpose:  This function is used to read data from the instrument and
 *           write it to a user specified file.
*****************************************************************************/
ViStatus _VI_FUNC rsrfsiggen_ReadToFileFromInstrument(ViSession instrSession,
                                                      ViString  source,
                                                      ViString  destination)
{
    return rsrfsiggen_readToFile(instrSession, source, destination);
}

/*****************************************************************************
 * Function: Write From File To Instrument
 * Purpose:  This function is used to read data from a user specified file
 *           and write it to the instrument.
*****************************************************************************/
ViStatus _VI_FUNC rsrfsiggen_WriteFromFileToInstrument(ViSession instrSession,
                                                       ViString  source,
                                                       ViString  destination)
{
    return rsrfsiggen_writeFromFile(instrSession, source, destination);
}

/// HIFN  Loads a file with previously saved instrument settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR filename/Loads a file with previously saved instrument settings.
ViStatus _VI_FUNC rsrfsiggen_RecallInstrumentSettings(ViSession instrSession,
                                                      ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_SYSTEM_LOAD_INSTRUMENT_SETTINGS, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current instrument settings to a file with defined filename.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR filename/Saves the current instrument settings to a file with defined filename.
ViStatus _VI_FUNC rsrfsiggen_SaveInstrumentSettings(ViSession instrSession,
                                                    ViString  filename)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_SYSTEM_SAVE_INSTRUMENT_SETTINGS, filename),
            2, "Filename");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the ARB generator by specifying the sample
/// HIFN  frequency, selected waveform and state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR waveform/Selects the waveform file. If the file is not on the default path, the
/// HIPAR waveform/path must be specified at the same time. If no file of the specified
/// HIPAR waveform/name exists, it is created. The file extension may be omitted. Only
/// HIPAR waveform/files with the file extension *.wv will be created or loaded.
/// HIPAR clockFrequency/Sets the clock rate in samples. Loading a waveform sets the clock rate
/// HIPAR clockFrequency/that is defined in the waveform tag 'clock'. This function subsequently
/// HIPAR clockFrequency/changes the clock rate; see data sheet for value range.
/// HIPAR arbEnabled/Switches the ARB generator on. Any other standards or digital
/// HIPAR arbEnabled/modulation that may be in the ON state are automatically turned OFF.
/// HIPAR arbEnabled/The selected waveform is generated straight away (ARB:SEQ
/// HIPAR arbEnabled/AUTO|RETRigger) or after the first trigger event (ARB:SEQ  AAUT|ARET),
/// HIPAR arbEnabled/depending on the trigger setting.
ViStatus _VI_FUNC rsrfsiggen_ConfigureArb(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  waveform,
                                          ViReal64  clockFrequency,
                                          ViUInt16  arbEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ARB_WAVEFORM_SELECT, waveform),
            3, "Waveform");
    if (clockFrequency != 0.0)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ARB_CLOCK_FREQUENCY, clockFrequency),
                4, "Clock Frequency");
    }

    if (arbEnabled == RSRFSIGGEN_VAL_ON_WITHOUT_WAIT)
    {
        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ARB_ENABLED_NO_WAIT, VI_TRUE),
                5, "ARB Enabled");
    }
    else
    {
        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ARB_ENABLED, arbEnabled),
                5, "ARB Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function appends information on the ARB clock rate to specified
/// HIFN  waveform file. The file must contain I/Q and/or marker data and have
/// HIFN  been created with the command: MMEMory:DATA:UNPRotected
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter is a string. It completes the file path and file
/// HIPAR fileName/name with extention (*.wv).
/// HIPAR clock/Sets the Clock parameter.
ViStatus _VI_FUNC rsrfsiggen_WriteARBClockWaveformFile(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViString  fileName,
                                                       ViReal64  clock)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, clock, 400.0, 100.0e+06), 4, "Clock");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:WAV:CLOC \"%s\",%lf", path, fileName, clock);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function adds a blank segment to the multi segment file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sampleCount/Specifies the number of samples.
/// HIPAR frequency/Determines the clock rate.
ViStatus _VI_FUNC rsrfsiggen_ARBMultisegmentAddBlankSegment(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViReal64  sampleCount,
                                                            ViReal64  frequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sampleCount, 512.0, 1.0e+7), 3, "Sample Count");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:WSEG:CONF:BLAN:APP %lf,%lf", path, sampleCount, frequency);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function appends a new segment to the selected sequencing play
/// HIFN  list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Activates/deactivates the appended segment.
/// HIPAR segment/Indicates the number of the segment as in the multi segment waveform
/// HIPAR segment/file.
/// HIPAR repetition/Defines how many times this segment is repeated.
/// HIPAR next/Determines the action after completing the current segment, like for
/// HIPAR next/instance which segment is processed after the processing of the current
/// HIPAR next/one is finished.
ViStatus _VI_FUNC rsrfsiggen_ARBMultisegmentAppendSegmentToPlayList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean state,
                            ViInt32   segment,
                            ViInt32   repetition,
                            ViInt32   next)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetition, 1, 1048575),
            5, "Sample Count");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:WSEG:SEQ:APP %s,%ld,%ld,%ld\n", path, state ? "ON" : "OFF", segment, repetition, next);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets all ARB generator parameters to their default values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
ViStatus _VI_FUNC rsrfsiggen_ClearAllArbWaveforms(ViSession instrSession,
                                                  ViInt32   path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ARB_SET_TO_DEFAULT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all statistic results in a comma-separated list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR statisticsAll/Queries all statistic results in a comma-separated list.
ViStatus _VI_FUNC rsrfsiggen_QueryARBEthernetUploadStatistics(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   statisticsAll[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   outValue[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32* arraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_GetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_ARB_ETHERNET_UPLOAD_STATISTIC_ALL,
                                                 RS_VAL_DIRECT_USER_CALL,
                                                 RS_MAX_MESSAGE_BUF_SIZE,
                                                 outValue), 3, "Wrong parameters");

    RsCore_Convert_String_To_ViInt32Array(instrSession, outValue, &statisticsAll, arraySize);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries waveform tags defined in the waveform file with extension
/// HIFN  *.wv.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tagName/Sets the tag name.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Output parameter.
/// HIPAR tag/Queries all statistic results in a comma-separated list.
/// HIPAR returnedValues/Number of values returned from the instrument.
ViStatus _VI_FUNC rsrfsiggen_QueryARBEthernetUploadWaveformTag(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  tagName,
                            ViInt32   bufferSize,
                            ViChar    tag[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf (param, RS_PARAM_BUF_SIZE, "'%s'", tagName);

    viCheckParm(RsCore_GetAttributeViRawStringWithQparam(instrSession, repCap,
                                                         RSRFSIGGEN_ATTR_HIDDEN_ARB_ETHERNET_UPLOAD_WAVEFORM_TAG,
                                                         param, bufferSize, tag, returnedValues), 5, "tag");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureARBClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureARBClockSourceLegacy(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function writes the block data <binary block data> to the file
/// HIFN  identified by <file_name>.
/// HIFN
/// HIFN     Using this command, waveforms can be sent directly to the instrument.
/// HIFN
/// HIFN     Only the file name has to be entered, the file extension may be
/// HIFN  omitted. Waveform data is stored only in files with the specific file
/// HIFN  extensions *.wv.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
/// HIPAR waveformData/Sets waveform data.
ViStatus _VI_FUNC rsrfsiggen_ARBWaveformDataWrite(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViString  fileName,
                                                  ViInt32   arraySize,
                                                  ViUInt8   waveformData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FORM PACK;:SOUR%ld:BB:ARB:WAV:DATA '%s',", path, fileName);
    checkErr(RsCore_WriteBinaryDataBlock(instrSession, cmd, waveformData, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads the block data (binary block data) from the file
/// HIFN  identified by <file name> and <waveform tag> parameter.
/// HIFN
/// HIFN     Using this function, waveforms can be sent directly from the
/// HIFN  instrument to the host computer.
/// HIFN
/// HIFN     Only the file name has to be entered, the file extension may be
/// HIFN  omitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
/// HIPAR waveformTag/Sets waveform tag.
/// HIPAR waveformData/Queries waveform data.
ViStatus _VI_FUNC rsrfsiggen_ARBWaveformDataRead(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViString  fileName,
                                                 ViString  waveformTag,
                                                 ViInt32   bufferSize,
                                                 ViChar    waveformData[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViByte*  response = NULL;
    ViInt64  responseLen = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:ARB:WAV:DATA? '%s','%s'", path, fileName, waveformTag);
    checkErr(RsCore_QueryBinaryDataBlock(instrSession, cmd, VI_TRUE, NULL, &response, &responseLen));

    checkErr(rsrfsiggen_CheckStatus(instrSession));
    checkErr(RsCore_CopyToUserBufferBinData((ViBuf)waveformData, bufferSize, response, (ViInt32)responseLen));

Error:
    if (response)
        free(response);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_ARB_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureARBTriggerDateTime(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   year,
                                                         ViInt32   month,
                                                         ViInt32   day,
                                                         ViInt32   hour,
                                                         ViInt32   minute,
                                                         ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_ARB_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_ARB_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches on digital modulation. Switching on
/// HIFN  digital modulation turns off all the other digital standards on
/// HIFN  the same path (SOURce1 or SOURce2).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR DMEnabled/Switches on digital modulation. Switching on digital modulation
/// HIPAR DMEnabled/turns off all the other digital standards on the same path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean DMEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DM_ENABLED, DMEnabled),
            3, "DM Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function specifies the actual standard used by the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Selects the standard. After selection, modulation parameters
/// HIPAR standard/Modulation Type, Symbol Rate, Filter and Coding are automatically set
/// HIPAR standard/in accordance with the standard. The standard values are listed in
/// HIPAR standard/section  Set acc. to Standard - Digital Modulation.
ViStatus _VI_FUNC rsrfsiggen_SelectDMStandard(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_ACC_TO_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the DM clock settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMClockSource(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DM:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the source of the data for the digital
/// HIFN  modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the data source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMDataSource(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_DATA_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the type of the PRBS used as data for the
/// HIFN  digital modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR type/Defines the length of the pseudo-random sequence in accordance with
/// HIPAR type/the following equation:
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMPRBSType(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_PRBS_TYPE, type),
            3, "Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function creates a waveform using the current settings of the DM
/// HIFN  menu. The file is stored with the predefined file extension *.wv. The
/// HIFN  file name and the directory it is stored in are user-definable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_DMGenerateWaveformFile(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_DM_GENERATE_WAVEFORM_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the modulation type. Modulation types are described in
/// HIFN  section Modulation Types - Custom Digital Mod.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR modulation/Selects the modulation type. Modulation types are described in
/// HIPAR modulation/section Modulation Types - Custom Digital Mod.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMModulationType(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   modulation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_MODULATION_TYPE, modulation),
            3, "Modulation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables power ramping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Enables or disables power ramping.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMPowerRampEnabled(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMPING_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects power ramping in the baseband only or mixed
/// HIFN  power ramping in the baseband and the RF section. The ON setting is
/// HIFN  mandatory if, with power ramping active, only the baseband signal is
/// HIFN  output (I/Q outputs), or if a baseband signal is applied to two RF
/// HIFN  paths (RF A and RF B). Only then can a signal with a defined,
/// HIFN  predictable level be output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR attenuation/Selects power ramping in the baseband only or mixed power ramping in
/// HIPAR attenuation/the baseband and the RF section. The ON setting is mandatory if, with
/// HIPAR attenuation/power ramping active, only the baseband signal is output (I/Q outputs),
/// HIPAR attenuation/or if a baseband signal is applied to two RF paths (RF A and RF B).
/// HIPAR attenuation/Only then can a signal with a defined, predictable level be output.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMPowerRampAttenuation(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViBoolean attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_IN_BASEBAND_ONLY, attenuation),
            3, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures digital modulation power ramping.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the source for the power ramp control signals.
/// HIPAR function/Sets the edge shape of the ramp envelope.
/// HIPAR time/Sets the power ramping rise time and fall time for a burst.
/// HIPAR riseDelay/Sets the delay in the rising edge. A positive value gives rise to a
/// HIPAR riseDelay/delay and a negative value causes an advance. The setting is expressed
/// HIPAR riseDelay/in symbols.
/// HIPAR fallDelay/Sets the delay in the falling edge. A positive value gives rise to a
/// HIPAR fallDelay/delay and a negative value causes an advance. The setting is expressed
/// HIPAR fallDelay/in symbols.
/// HIPAR attenuation/Sets the level attenuation for signal ranges that are flagged with
/// HIPAR attenuation/level attribute attenuated by the LEV_ATT control signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMPowerRamp(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   source,
                                                  ViInt32   function,
                                                  ViReal64  time,
                                                  ViReal64  riseDelay,
                                                  ViReal64  fallDelay,
                                                  ViReal64  attenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_SOURCE, source),
            3, "Source");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_FUNCTION, function),
            4, "Function");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_TIME, time),
            5, "Time");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_RISE_DELAY, riseDelay),
            6, "Rise Delay");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_FALL_DELAY, fallDelay),
            7, "Fall Delay");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DM_POWER_RAMP_ATTENUATION, attenuation),
            8, "Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMClockSourceLegacy(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DM:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_DM_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMTriggerDateTime(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minute,
                                                        ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_DM_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_DM_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the input connector at that the signal is fed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR digitalInterface/Selects the input connector at that the signal is fed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBBINDigitalInterface(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   digitalInterface)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BBIN_DIGITAL_INTERFACE, digitalInterface),
            3, "Digital Interface");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the baseband to that the channel is mapped.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Sets the channel number. Up to 8 channels per HS DIG I/Q interface and
/// HIPAR channel/max. 8 channels for both interfaces.
/// HIPAR activateChannel/Activates the channel.
/// HIPAR bbChannel/This control is reserved.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBBINChannelMapping(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   channel,
                                                         ViBoolean activateChannel,
                                                         ViInt32   BBChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
             BBChannel;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 0, 7),
            3, "Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,BCh%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_BBIN_ACTIVATE_CHANNEL, activateChannel),
            4, "Activate Channel");

    /*    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BBIN_MAPPED_BB_CHANNEL, BBChannel),
                5, "BB Channel");
    */

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the peak level per channel relative to full scale
/// HIFN  and crest factor of the individual channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Sets the channel number. Up to 8 channels per HS DIG I/Q interface and
/// HIPAR channel/max. 8
/// HIPAR crestFactor/Sets the crest factor of the individual channels.
/// HIPAR peakLevel/Sets the peak level per channel relative to full scale.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBBINChannelPower(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   channel,
                                                       ViReal64  crestFactor,
                                                       ViReal64  peakLevel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 0, 7),
            3, "Channel");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,BCh%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_BBIN_CHANNEL_CREST_FACTOR, crestFactor),
            4, "Crest Factor");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_BBIN_CHANNEL_PEAK_LEVEL, peakLevel),
            5, "Peak Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches on the modulation in accordance with the GSM
/// HIFN  standard. Any other standards or digital modulation that may be in the
/// HIFN  ON state will be automatically turned OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR gsmEnabled/Switches on the modulation in accordance with the GSM standard. Any
/// HIPAR gsmEnabled/other standards or digital modulation that may be in the ON state will
/// HIPAR gsmEnabled/be automatically turned OFF.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean gsmEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_ENABLED, gsmEnabled),
            3, "GSM Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures GSM/EDGE subsystem.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sequenceMode/Selects GSM mode.
/// HIPAR symbolRateMode/Sets the symbol rate mode, i.e. determines whether a normal bursts
/// HIPAR symbolRateMode/(NB) or higher symbol rate bursts (HB) will be generated.
/// HIPAR simulationMode/Selects the modulation signal for the mode Unframed (see
/// HIPAR simulationMode/RSRFSIGGEN_ATTR_GSM_SIMULATION_MODE). The modulation type and filter
/// HIPAR simulationMode/type are set in accordance with the selection.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSM(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   sequenceMode,
                                          ViInt32   symbolRateMode,
                                          ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_MODE, sequenceMode),
            3, "Sequence Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_SYMBOL_RATE_MODE, symbolRateMode),
            4, "Symbol Rate Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file with predefined frame settings.
/// HIFN
/// HIFN     Loads the selected file with predefined frame settings. The folder is
/// HIFN  pre-set, therefore a path cannot be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Loads the selected file with predefined frame settings.
ViStatus _VI_FUNC rsrfsiggen_RecallGSMPredefinedFrame(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_RECALL_PREDEFINED_FRAME, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file with userdefined frame settings
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_fu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Loads the selected file with userdefined frame settings.
ViStatus _VI_FUNC rsrfsiggen_RecallGSMUserFrame(ViSession instrSession,
                                                ViInt32   path,
                                                ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_RECALL_USER_FRAME, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the current frame settings into the selected
/// HIFN  file.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_fu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Stores the current frame settings into the selected file.
ViStatus _VI_FUNC rsrfsiggen_SaveGSMUserFrame(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_SAVE_USER_FRAME, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the selected file with userdefined frame
/// HIFN  settings.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_fu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Deletes the selected file with userdefined frame settings.
ViStatus _VI_FUNC rsrfsiggen_DeleteGSMUserFrame(ViSession instrSession,
                                                ViInt32   path,
                                                ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_DELETE_USER_FRAME, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file with predefined slot settings.
/// HIFN
/// HIFN     Loads the selected file with predefined frame settings. The folder is
/// HIFN  pre-set, therefore a path cannot be specified.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Sets the frame number.
/// HIPAR slot/Sets the slot number.
/// HIPAR fileName/Loads the selected file with predefined slot settings.
ViStatus _VI_FUNC rsrfsiggen_RecallGSMPredefinedSlot(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   frame,
                                                     ViInt32   slot,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 2),
            3, "Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slot, 0, 7),
            3, "Slot");
    snprintf(repCap_frame, RS_REPCAP_BUF_SIZE, ",F%ld,S%ld", frame, slot);
    strcat(repCap, repCap_frame);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_RECALL_PREDEFINED_SLOT, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file with userdefined slot settings.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_slu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Sets the frame number.
/// HIPAR slot/Sets the slot number.
/// HIPAR fileName/Loads the selected file with userdefined slot settings.
ViStatus _VI_FUNC rsrfsiggen_RecallGSMUserSlot(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   frame,
                                               ViInt32   slot,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 2),
            3, "Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slot, 0, 7),
            3, "Slot");
    snprintf(repCap_frame, RS_REPCAP_BUF_SIZE, ",F%ld,S%ld", frame, slot);
    strcat(repCap, repCap_frame);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_RECALL_USER_SLOT, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the current slot settings into the selected file.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_slu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Sets the frame number.
/// HIPAR slot/Sets the slot number.
/// HIPAR fileName/Stores the current slot settings into the selected file.
ViStatus _VI_FUNC rsrfsiggen_SaveGSMUserSlot(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   frame,
                                             ViInt32   slot,
                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 2),
            3, "Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slot, 0, 7),
            3, "Slot");
    snprintf(repCap_frame, RS_REPCAP_BUF_SIZE, ",F%ld,S%ld", frame, slot);
    strcat(repCap, repCap_frame);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_SAVE_USER_SLOT, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the selected file with user defined slot
/// HIFN  settings.
/// HIFN
/// HIFN     The folder is set using command MMEM:CDIRectory. A path can also be
/// HIFN  specified, in which case the files in the specified folder are read.
/// HIFN  The file extension may be omitted. Only files with the file extension
/// HIFN  *.gsm_slu will be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Sets the frame number.
/// HIPAR slot/Sets the slot number.
/// HIPAR fileName/Deletes the selected file with user defined slot settings.
ViStatus _VI_FUNC rsrfsiggen_DeleteGSMUserSlot(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   frame,
                                               ViInt32   slot,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_frame[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 2),
            3, "Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slot, 0, 7),
            3, "Slot");
    snprintf(repCap_frame, RS_REPCAP_BUF_SIZE, ",F%ld,S%ld", frame, slot);
    strcat(repCap, repCap_frame);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_DELETE_USER_SLOT, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the content of the specified marker in the
/// HIFN  selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Sets the frame number.
/// HIPAR slot/Sets the slot number.
/// HIPAR user/Sets the slot number.
/// HIPAR tag/Returns the content of the specified marker in the selected file.
/// HIPAR arraySize/Array size parameter.
/// HIPAR response/Returns the content of the specified marker in the selected file.
/// HIPAR returnedValues/Returns the number of values returned in the Response control.
ViStatus _VI_FUNC rsrfsiggen_QueryGSMUserSlotMarkerDefinition(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   frame,
                            ViInt32   slot,
                            ViInt32   user,
                            ViString  tag,
                            ViInt32   arraySize,
                            ViChar    response[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 2),
            3, "Frame");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, slot, 0, 7),
            4, "Slot");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, user, 1, 2),
            5, "User");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GSM:FRAM%ld:SLOT%ld:USER%ld:TRIG:OUTP:TAG? '%s'", path, frame, slot, user, tag);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, response, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the modulation in accordance with the Bluetooth
/// HIFN  standard. Activating this standard deactivates all the other digital
/// HIFN  standards and digital modulation modes (in case of two-path
/// HIFN  instruments, this affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR btoEnabled/Sets the modulation in accordance with the Bluetooth standard.
/// HIPAR btoEnabled/Activating this standard deactivates all the other digital standards
/// HIPAR btoEnabled/and digital modulation modes (in case of two-path instruments, this
/// HIPAR btoEnabled/affects the same path).
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTEnabled(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean btoEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_BT_ENABLED, btoEnabled),
            3, "BTO Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Determines the digital Bluetooth standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR bluetoothMode/Determines the digital Bluetooth standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTMode(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   bluetoothMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BT_MODE, bluetoothMode),
            3, "Bluetooth Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the transport mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR transportMode/Sets the transport mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTBasic(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   transportMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BT_TRANSPORT_MODE, transportMode),
            3, "Transport Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the channel type. Advertising and data are
/// HIFN  available.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channelType/Sets the channel type. Advertising and data are available.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTLowEnergy(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   channelType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BT_LE_CHANNEL_TYPE, channelType),
            3, "Channel Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables / disables the features 0 to 16 for the used feature set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR featureSet/Select the feature.
/// HIPAR state/Enables / disables the features 0 to 16 for the used feature set.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTLowEnergyFeatureSet(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   featureSet,
                                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, featureSet, 0, 16),
            3, "Feature Set");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BTO:ECON:PCON:FSB%ld:STAT %s", path, featureSet, enabledArr[state ? 1 : 0]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects GSM trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GSM trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GSM:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GSM trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMTriggerSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GSM:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMTrigger(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean sync,
                                                 ViReal64  delay,
                                                 ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_GSM_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMTriggerDateTime(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   year,
                                                         ViInt32   month,
                                                         ViInt32   day,
                                                         ViInt32   hour,
                                                         ViInt32   minute,
                                                         ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_GSM_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_GSM_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR marker/Sets the marker number.
/// HIPAR mode/Defines the signal for the selected marker output.
/// HIPAR period/Sets the repetition rate for the frame clock at the marker outputs.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMMarkerMode(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   marker,
                                                    ViInt32   mode,
                                                    ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");
    snprintf(repCap_marker, RS_REPCAP_BUF_SIZE, ",M%ld", marker);
    strcat(repCap, repCap_marker);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_MARKER_MODE, mode),
            4, "mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_MARKER_FRAME_PERIOD, period),
            5, "period");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR marker/Sets the marker number.
/// HIPAR delay/Defines the delay between the signal on the marker outputs and the start of the
/// HIPAR delay/frame/slot, expressed in terms of symbols.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMMarkerDelay(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   marker,
                                                     ViInt32   delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_marker[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");
    snprintf(repCap_marker, RS_REPCAP_BUF_SIZE, ",M%ld", marker);
    strcat(repCap, repCap_marker);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GSM_MARKER_DELAY, delay),
            4, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GSM:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGSMClockSourceLegacy(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K40"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GSM:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects Bluetooth trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTTriggerMode(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BT_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects Bluetooth trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTTriggerSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BTO:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects Bluetooth trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTTriggerSourceLegacy(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BTO:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTTrigger(ViSession instrSession,
                                                ViInt32   path,
                                                ViBoolean sync,
                                                ViReal64  delay,
                                                ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_BT_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_BT_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_BT_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_BT_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTTriggerDateTime(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minute,
                                                        ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_BT_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_BT_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTClockSource(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BTO:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBTClockSourceLegacy(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K60"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BTO:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates modulation in accordance with the 3GPP FDD
/// HIFN  standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR w3gppEnabled/Activates modulation in accordance with the 3GPP FDD standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPEnabled(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean w3gppEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_ENABLED, w3gppEnabled),
            3, "W3GPP Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the transmission direction.
/// HIFN
/// HIFN     The signal either corresponds to that of a base station (FORWard |
/// HIFN  DOWN) or that of a user equipment (REVerse | UP).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPLinkDirection(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures Basestation OCNS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ocns/Activates/deactivates OCNS channels, as defined in the standard.
/// HIPAR ocnsMode/Selects the scenario for setting the OCNS channels.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPBasestationOCNS(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViBoolean ocns,
                                                           ViInt32   ocnsMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_OCNS_ADD_BS, ocns),
            3, "OCNS");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_OCNS_MODE_BS, ocnsMode),
            4, "OCNS Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the specified files with stored user channel
/// HIFN  codings.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_dl in a
/// HIFN  directory of the user's choice. The directory applicable to the
/// HIFN  function is defined with the command MMEMory:CDIR. To access the files
/// HIFN  in this directory, you only have to give the file name, without the
/// HIFN  path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Deletes the specified files with stored user channel codings.
ViStatus _VI_FUNC rsrfsiggen_DeleteW3GPPChannelCodingUserCoding(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_BS1_CHANNEL_CODING_USER_CODING_DELETE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the specified files with stored user channel
/// HIFN  codings.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_dl in a
/// HIFN  directory of the user's choice. The directory applicable to the
/// HIFN  function is defined with the command MMEMory:CDIR. To access the files
/// HIFN  in this directory, you only have to give the file name, without the
/// HIFN  path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Sets the channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_RecallW3GPPChannelCodingUserCoding(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 11, 13),
            3, "Channel");
    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, ",DPCH%ld", channel);
    strcat(repCap, repCap_channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_BS1_CHANNEL_CODING_USER_CODING_RECALL, fileName),
            4, "fileName");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the current settings for channel coding as user
/// HIFN  channel coding in the specified file.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_dl in a
/// HIFN  directory of the user's choice. The directory in which the file is
/// HIFN  stored is defined with the command MMEMory:CDIR. To store the files in
/// HIFN  this directory, you only have to give the file name, without the path
/// HIFN  and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Sets the channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_SaveW3GPPChannelCodingUserCoding(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_channel[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 11, 13),
            3, "Channel");
    snprintf(repCap_channel, RS_REPCAP_BUF_SIZE, ",DPCH%ld", channel);
    strcat(repCap, repCap_channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_BS1_CHANNEL_CODING_USER_CODING_SAVE, fileName),
            4, "fileName");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the specified files with stored user channel
/// HIFN  codings.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_ul in a
/// HIFN  directory of the user's choice. The directory applicable to This
/// HIFN  functions is defined with the command MMEMory:CDIR. To access the files
/// HIFN  in this directory, you only have to give the file name, without the
/// HIFN  path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_DeleteW3GPPGlobalEnhancedChannels(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_UE1_USER_CODING_DELETE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the specified files with stored user channel
/// HIFN  codings.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_ul in a
/// HIFN  directory of the user's choice. The directory applicable to This
/// HIFN  functions is defined with the command MMEMory:CDIR. To access the files
/// HIFN  in this directory, you only have to give the file name, without the
/// HIFN  path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_RecallW3GPPGlobalEnhancedChannels(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_UE1_USER_CODING_RECALL, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the specified files with stored user channel
/// HIFN  codings.
/// HIFN
/// HIFN     The files are stored with the fixed file extensions *.3g_ccod_ul in a
/// HIFN  directory of the user's choice. The directory applicable to This
/// HIFN  functions is defined with the command MMEMory:CDIR. To access the files
/// HIFN  in this directory, you only have to give the file name, without the
/// HIFN  path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_SaveW3GPPGlobalEnhancedChannels(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_UE1_USER_CODING_SAVE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects 3GPP FDD trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPTriggerMode(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects 3GPP FDD trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPTriggerSource(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:W3GP:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects 3GPP FDD trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPTriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:W3GP:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPTrigger(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean sync,
                                                   ViReal64  delay,
                                                   ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_W3GPP_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPClockSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:W3GP:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureW3GPPClockSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K42"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:W3GP:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates modulation in accordance with the  CDMA2000
/// HIFN  standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR c2kEnabled/Activates modulation in accordance with the  CDMA2000 standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean c2kEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_ENABLED, c2kEnabled),
            3, "C2K Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the transmission direction.
/// HIFN
/// HIFN     The signal either corresponds to that of a base station (FORWard |
/// HIFN  DOWN) or that of a user equipment (REVerse | UP).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kLinkDirection(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the station that has data to be copied.
/// HIFN
/// HIFN     Whether the station copied is a base or user equipment depends on
/// HIFN  which transmission direction is selected (command C2K:LINK UP | DOWN).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR station/Selects the station that has data to be copied.
ViStatus _VI_FUNC rsrfsiggen_C2kCopyFromSource(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   station)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_COPY_FROM_SOURCE, station),
            3, "Station");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the station to which data is to be copied.
/// HIFN
/// HIFN     Whether the data is copied to a base station or a user equipment
/// HIFN  depends on which transmission direction is selected (command C2K:LINK
/// HIFN  UP | DOWN).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR station/Selects the station that has data to be copied.
ViStatus _VI_FUNC rsrfsiggen_C2kToDestination(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   station)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_TO_DESTINATION, station),
            3, "Station");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects CDMA2000 trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects CDMA2000 trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K46"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:C2K:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects CDMA2000 trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kTriggerSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K46"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:C2K:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kTrigger(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean sync,
                                                 ViReal64  delay,
                                                 ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_C2K_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K46"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:C2K:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureC2kClockSourceLegacy(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K46"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:C2K:TRIG:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates modulation in accordance with the  TD-SCDMA
/// HIFN  standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tdscdmaEnabled/Activates modulation in accordance with the  TD-SCDMA standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMAEnabled(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViBoolean tdscdmaEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_ENABLED, tdscdmaEnabled),
            3, "TD-SCDMA Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the transmission direction.
/// HIFN
/// HIFN     The signal either corresponds to that of a base station (FORWard |
/// HIFN  DOWN) or that of a mobile station (REVerse | UP).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMALinkDirection(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the file with the test models defined in the
/// HIFN  TD-SCDMA standard or a self-defined test setup.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Selects the file with the test models defined in the TD-SCDMA standard
/// HIPAR fileName/or a self-defined test setup.
ViStatus _VI_FUNC rsrfsiggen_TestModelTDSCDMASettings(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_SETTINGS_TEST_MODEL, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects TD-SCDMA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMATriggerMode(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects TD-SCDMA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMATriggerSource(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TDSC:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects TD-SCDMA trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMATriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TDSC:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMATrigger(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViBoolean sync,
                                                     ViReal64  delay,
                                                     ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TDSCDMA_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMAClockSource(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TDSC:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTDSCDMAClockSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K50"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TDSC:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates modulation in accordance with
/// HIFN  the1xEV-DO standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR evdoEnabled/Activates or deactivates modulation in accordance with the 1xEV-DO
/// HIPAR evdoEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean evdoEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_ENABLED, evdoEnabled),
            3, "EVDO Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configres 1xEV-DO standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
/// HIPAR pnOffset/Sets the PN Offset of the 1xEV-DO signal.
/// HIPAR systemTime/Sets the System Time value of the 1xEV-DO signal and the base station.
/// HIPAR systemTime/The System Time value is expressed in units of 1.67 ms intervals (80
/// HIPAR systemTime/ms/ 48).
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDO(ViSession instrSession,
                                           ViInt32   path,
                                           ViInt32   linkDirection,
                                           ViInt32   pnOffset,
                                           ViReal64  systemTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_PN_OFFSET, pnOffset),
            4, "PN Offset");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_SYSTEM_TIME, systemTime),
            5, "System Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the UL setting of Terminal 1 to one of the
/// HIFN  predefined configurations.
/// HIFN
/// HIFN     Note(s):
/// HIFN     This function is valid only for Uplink Link Direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR predefinedSettings/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOTerminalPredefinedSettings(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   predefinedSettings)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_TERMINAL_PREDEFINED_SETTINGS, predefinedSettings),
            3, "Predefined Settings");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function generates waveform file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Generates waveform file.
ViStatus _VI_FUNC rsrfsiggen_GenerateEVDOWaveformFile(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_GENERATE_WAVEFORM_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects EV-DO trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects EV-DO trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K47"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EVDO:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects EV-DO trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOTriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K47"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EVDO:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViReal64  delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EVDO_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K47"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EVDO:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEVDOClockSourceLegacy(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K47"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EVDO:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches on the modulation in accordance with the
/// HIFN  IEEE 802.11a-g WLAN standard. Activating this standard
/// HIFN  deactivates all the other digital standards and digital
/// HIFN  modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR WLANEnabled/Switches on the modulation in accordance with the GSM standard. Any
/// HIPAR WLANEnabled/other standards or digital modulation that may be in the ON state will
/// HIPAR WLANEnabled/be automatically turned OFF.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean WLANEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_ENABLED, WLANEnabled),
            3, "WLAN Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base WLAN settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Selects the standard according to which the signal is simulated.
/// HIPAR physicalLayer/Selects the Physical Layer Mode.
/// HIPAR simulationMode/Selects the Simulation Mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLAN(ViSession instrSession,
                                           ViInt32   path,
                                           ViInt32   standard,
                                           ViInt32   physicalLayer,
                                           ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_STANDARD, standard),
            3, "Standard");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_PHYSICAL_LAYER_MODE, physicalLayer),
            4, "Physical Layer");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures frame settings, when simulation mode is set
/// HIFN  to "Framed".
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR predefinedFrame/Selects the frame type. The selection defines parameters of the MAC
/// HIPAR predefinedFrame/layer, e.g. the values of the MAC Header bit fields.
/// HIPAR sequenceLength/Selects the number of frames.
/// HIPAR idleTime/Sets the idle time, i.e. the time between two PPDU bursts.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANFrame(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   predefinedFrame,
                                                ViInt32   sequenceLength,
                                                ViReal64  idleTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_PREDEFINED_FRAMES, predefinedFrame),
            3, "Predefined Frame");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_SEQUENCE_LENGTH, sequenceLength),
            4, "Sequence Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_IDLE_TIME, idleTime),
            5, "Idle Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects WLAN trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects WLAN trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K48"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:WLAN:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects WLAN trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANTriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K48"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:WLAN:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViInt32   delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLAN_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K48"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:WLAN:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANClockSourceLegacy(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K48"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:WLAN:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches on the modulation in accordance with the
/// HIFN  IEEE 802.11a-n WLAN-N standard. Activating this standard
/// HIFN  deactivates all the other digital standards and digital
/// HIFN  modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR WLANNEnabled/Switches on the modulation in accordance with the GSM standard. Any
/// HIPAR WLANNEnabled/other standards or digital modulation that may be in the ON state will
/// HIPAR WLANNEnabled/be automatically turned OFF.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANNEnabled(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean WLANNEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WLANN_ENABLED, WLANNEnabled),
            3, "WLAN-N Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base WLAN N settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Selects the transmission bandwidth.
/// HIPAR configureBasebandBfromA/Activates or deactivates coupling of both baseband paths.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANN(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   standard,
                                            ViBoolean configureBasebandBfromA)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANN_TRANSMISSION_BANDWIDTH, standard),
            3, "Standard");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WLANN_COUPLING_ENABLED, configureBasebandBfromA),
            4, "Configure Baseband B from A");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the IEEE 802.11 WLAN standard.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Frame block.
/// HIPAR standard/Sets the IEEE 802.11 WLAN standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANNStandard(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   frame,
                                                    ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   capability[15] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 100),
            3, "Frame");

    sprintf(capability, "%s,FB%ld", repCap, frame);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, capability, RSRFSIGGEN_ATTR_WLANN_FRAME_BLOCK_STANDARD, standard),
            4, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function saves the IQ chain in a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR txChain/Tx chain.
/// HIPAR fileName/The IQ chain file.
ViStatus _VI_FUNC rsrfsiggen_SaveWLANAntennasChainMappingFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   txChain,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   capability[15] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, txChain, 1, 4),
            3, "Tx Chain");

    sprintf(capability, "%s,O%ld", repCap, txChain);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, capability, RSRFSIGGEN_ATTR_WLANN_ANTENNAS_CHAIN_MAPPING_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected file for beamforming.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frame/Frame block.
/// HIPAR fileName/The file for beamforming.
ViStatus _VI_FUNC rsrfsiggen_WLANNPPDUSpatialMappingBeamformingFileSelection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   frame,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   capability[15] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, frame, 1, 100),
            3, "Frame");

    sprintf(capability, "%s,FB%ld", repCap, frame);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, capability, RSRFSIGGEN_ATTR_WLANN_SPATIAL_MAPPING_BEAMFORMING_FILE_SELECTION, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_WLANN_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANNTriggerDateTime(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   year,
                                                           ViInt32   month,
                                                           ViInt32   day,
                                                           ViInt32   hour,
                                                           ViInt32   minute,
                                                           ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_WLANN_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_WLANN_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the standard and deactivates all the other digital standards
/// HIFN  and digital modulation modes in the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the standard and deactivates all the other digital standards
/// HIPAR enabled/and digital modulation modes in the same path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANADEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IEEE 802.11ad.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR frameType/Sets the IEEE 802.11ad frame type.
/// HIPAR dmgPHYMode/Sets the DMG PHY mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANAD(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   frameType,
                                             ViInt32   dmgPHYMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_FRAME_TYPE, frameType),
            3, "Frame Type");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_DMG_PHY_MODE, dmgPHYMode),
            4, "DMG PHY Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IEEE 802.11ad Trigger In.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed. Provided are internal triggering by means of a
/// HIPAR source/command, external trigger signal via one of the provided local or
/// HIPAR source/global connectors and triggering by a signal from the other
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANADTriggerIn(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_TRIGGER_MODE, mode),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_TRIGGER_SOURCE, source),
            4, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_WLANAD_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANADTriggerDateTime(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   year,
                                                            ViInt32   month,
                                                            ViInt32   day,
                                                            ViInt32   hour,
                                                            ViInt32   minute,
                                                            ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_WLANAD_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_WLANAD_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IEEE 802.11ad Marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR marker/Sets the marker.
/// HIPAR mode/Sets the marker mode.
/// HIPAR delay/Defines the delay between the signal on the marker outputs and the
/// HIPAR delay/start of the signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANADMarker(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   marker,
                                                   ViInt32   mode,
                                                   ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   capabilities[20] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    sprintf(capabilities, "%s,M%ld", repCap, marker);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, capabilities, RSRFSIGGEN_ATTR_WLANAD_MARKER_MODE, mode),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, capabilities, RSRFSIGGEN_ATTR_WLANAD_MARKER_DELAY, delay),
            5, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the IEEE 802.11ad Frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sequenceLength/Sets the sequence length.
/// HIPAR idleTime/Sets the idle time, the time delay between the frames.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWLANADFrame(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   sequenceLength,
                                                  ViReal64  idleTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_FRAME_SEQUENCE_LENGTH, sequenceLength),
            3, "Sequence Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_WLANAD_FRAME_IDLE_TIME, idleTime),
            4, "Idle Time");

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates modulation in accordance with the IEEE 802.16
/// HIFN  WIMAX standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR wimaxEnabled/Activates modulation in accordance with the IEEE 802.16 WIMAX
/// HIPAR wimaxEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXEnabled(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean wimaxEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_ENABLED, wimaxEnabled),
            3, "WiMAX Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the Physical Layer Mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR physicalLayerMode/Selects the Physical Layer Mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXPhysicalLayerMode(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   physicalLayerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_PHYSICAL_LAYER_MODE, physicalLayerMode),
            3, "Physical Layer Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the version of the IEEE 802.16 WiMAX standard
/// HIFN  underlying the definitions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR version/Sets the version of the IEEE 802.16 WiMAX standard underlying the
/// HIPAR version/definitions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXVersion(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   version)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_VERSION, version),
            3, "Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures OFDM settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR duplexing/Selects the duplexing. The duplexing mode determines how the uplink
/// HIPAR duplexing/and downlink signal are separated.
/// HIPAR linkDirection/Defines the transmission direction.
/// HIPAR frameDuration/Selects the frame duration. For test reasons, continuous generation or
/// HIPAR frameDuration/generation for a freely selectable duration (USER) are available. In
/// HIPAR frameDuration/continuous mode, the frame duration equals the sum of the burst
/// HIPAR frameDuration/durations.
/// HIPAR userFrameDuration/Sets the frame duration to a freely selectable value.
/// HIPAR sequenceLength/Selects the number of frames.
/// HIPAR predefinedFrames/Selects the frame type.
/// HIPAR levelReference/Selects the level reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXOFDM(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   duplexing,
                                                ViInt32   linkDirection,
                                                ViInt32   frameDuration,
                                                ViReal64  userFrameDuration,
                                                ViInt32   sequenceLength,
                                                ViInt32   predefinedFrames,
                                                ViInt32   levelReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_DUPLEXING, duplexing),
            3, "Duplexing");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_LINK_DIRECTION, linkDirection),
            4, "Link Direction");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_FRAME_DURATION, frameDuration),
            5, "Frame Duration");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_USER_FRAME_DURATION, userFrameDuration),
            6, "User Frame Duration");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_SEQUENCE_LENGTH, sequenceLength),
            7, "Sequence Length");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_PREDEFINED_FRAMES, predefinedFrames),
            8, "Predefined Frames");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_OFDM_LEVEL_REFERENCE, levelReference),
            9, "Level Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures OFDM settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
/// HIPAR basebandAB/Activates control of both paths via the WiMAX menu.
/// HIPAR frameDuration/Selects the frame duration. For test reasons, continuous generation or
/// HIPAR frameDuration/generation for a freely selectable duration (USER) are available. In
/// HIPAR frameDuration/continuous mode, the frame duration equals the sum of the burst
/// HIPAR frameDuration/durations.
/// HIPAR userFrameDuration/Sets the frame duration to a freely selectable value.
/// HIPAR sequenceLength/Selects the number of frames.
/// HIPAR predefinedFrames/Selects the frame type.
/// HIPAR levelReference/Selects the level reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXOFDMA(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   linkDirection,
                                                 ViBoolean basebandAB,
                                                 ViInt32   frameDuration,
                                                 ViReal64  userFrameDuration,
                                                 ViInt32   sequenceLength,
                                                 ViInt32   predefinedFrames,
                                                 ViInt32   levelReference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_USE_BASEBAND_AB, basebandAB),
            3, "Baseband AB");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_FRAME_DURATION, frameDuration),
            4, "Frame Duration");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_USER_FRAME_DURATION, userFrameDuration),
            5, "User Frame Duration");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_SEQUENCE_LENGTH, sequenceLength),
            6, "Predefined Frames");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_PREDEFINED_FRAMES, predefinedFrames),
            7, "Predefined Frames");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_OFDMA_LEVEL_REFERENCE, levelReference),
            8, "Level Reference");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the UL-Map file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Sets the UL-Map file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXOFDMUlmapFile(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_OFDM_ULMAP_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the UL-Map file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Sets the UL-Map file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureWiMAXOFDMAUlmapFile(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   zone,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_zone[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, zone, 0, 7),
            3, "Zone");
    snprintf(repCap_zone, RS_REPCAP_BUF_SIZE, ",Zone%ld", zone);
    strcat(repCap, repCap_zone);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_WIMAX_OFDMA_ULMAP_FILE, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the DVB standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes on the same path.
/// HIFN     The DVB signal is generated according to the performed settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dvbEnabled/Activates or deactivates the DVB standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean dvbEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_ENABLED, dvbEnabled),
            3, "DVB Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures DVB settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dvbStandard/Selects the DVB standard to be used to generate the modulation signal.
/// HIPAR numberOfSuperFrames/Sets the number of the transmitted super-frames. Each super-frame
/// HIPAR numberOfSuperFrames/consists of four OFDM frames.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVB(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   dvbStandard,
                                          ViInt32   numberOfSuperFrames)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    if (dvbStandard == RSRFSIGGEN_VAL_DVB_STANDARD_DVBT)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_T_NUMBER_OF_SUPER_FRAMES, numberOfSuperFrames),
                4, "Number Of Super Frames");
    }
    else
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_H_NUMBER_OF_SUPER_FRAMES, numberOfSuperFrames),
                4, "Number Of Super Frames");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects DVB trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects DVB trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DVB:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects DVB trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBTriggerSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DVB:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBTrigger(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean sync,
                                                 ViReal64  delay,
                                                 ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DVB_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_DVB_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBTriggerDateTime(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   year,
                                                         ViInt32   month,
                                                         ViInt32   day,
                                                         ViInt32   hour,
                                                         ViInt32   minute,
                                                         ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_DVB_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_DVB_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DVB:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDVBClockSourceLegacy(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K52"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DVB:CLOCk:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates modulation in accordance with
/// HIFN  the DAB standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dabEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR dabEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean dabEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_ENABLED, dabEnabled),
            3, "DAB Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures DAB settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dataSource/Sets the DAB Data Source.
/// HIPAR transportMode/Selects the transport mode. Depending on the transport mode selected,
/// HIPAR transportMode/signal with different sequence length will be generated.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDAB(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   dataSource,
                                          ViInt32   transportMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_DATA_SOURCE, dataSource),
            3, "Data Source");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_TRANSPORT_MODE, transportMode),
            4, "Transport Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures DAB ETI Files settings.
/// HIFN
/// HIFN     Note(s):
/// HIFN     This function is valid only for Data Source set to ETI File.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR etiFile/Selects the ETI file for the data source selection.
/// HIPAR numberOfETIFrames/Sets the desired sequence lenght in form of ETI frames.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABETI(ViSession instrSession,
                                             ViInt32   path,
                                             ViString  etiFile,
                                             ViInt32   numberOfETIFrames)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_ETI_FILE, etiFile),
            3, "ETI File");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_NUMBER_OF_ETI_FRAMES, numberOfETIFrames),
            4, "Number Of ETI Frames");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects DAB trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects DAB trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K53"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DAB:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects DAB trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABTriggerSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K53"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DAB:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABTrigger(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean sync,
                                                 ViInt32   delay,
                                                 ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DAB_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K53"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DAB:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDABClockSourceLegacy(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K53"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:DAB:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures carrier settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierSpacing/Sets the carrier spacing. The carriers are generated symmetrically
/// HIPAR carrierSpacing/around the RF carrier. The total bandwidth (Number of carriers - 1) *
/// HIPAR carrierSpacing/Carrier spacing is 80 MHz. The maximum carrier spacing that can be set
/// HIPAR carrierSpacing/is dependent on the chosen number of carriers.
/// HIPAR optimizeCrestFactorMode/Sets the mode by which automatic settings will minimize the crest
/// HIPAR optimizeCrestFactorMode/factor or hold it at a chosen value.
/// HIPAR noOfCarriers/Sets the number of carriers in the Multicarrier CW signal. The total
/// HIPAR noOfCarriers/bandwidth (Number of carriers - 1) * Carrier spacing is 80 MHz. The
/// HIPAR noOfCarriers/number of carriers entered therefore defines the maximum carrier
/// HIPAR noOfCarriers/spacing (:BB:MCCW:CARRier:SPACing).
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWCarrierSetup(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   noOfCarriers,
                                                       ViReal64  carrierSpacing,
                                                       ViInt32   optimizeCrestFactorMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_NO_OF_CARRIERS, noOfCarriers),
            3, "No Of Carriers");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_CARRIER_SPACING, carrierSpacing),
            4, "Carrier Spacing");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_OPTIMIZE_CREST_FACTOR_MODE, optimizeCrestFactorMode),
            5, "Optimize Crest Factor Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures start stop carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierStop/Selects the last carrier in the carrier range to which the settings
/// HIPAR carrierStop/with the :BB:MCCW:EDIT:CARR:.. commands shall apply.
/// HIPAR carrierStart/Selects the first carrier in the carrier range to which the settings
/// HIPAR carrierStart/with the :BB:MCCW:EDIT:CARR:.. commands shall apply.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTableCarrier(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   carrierStart,
                                                       ViInt32   carrierStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_START, carrierStart),
            3, "Carrier Start");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_STOP, carrierStop),
            4, "Carrier Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures carriers phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR phaseStep/Sets the step width by which the start phases of the carriers in the
/// HIPAR phaseStep/defined carrier range will be incremented.
/// HIPAR phaseStart/Sets the start phase for the individual carriers in the defined
/// HIPAR phaseStart/carrier range. If the command :BB:MCCW:EDIT:CARR:PHAS:STEP is used to
/// HIPAR phaseStart/define a step width, the phase entered here applies only to the
/// HIPAR phaseStart/starting carrier. The phases of the remaining carriers are stepped up
/// HIPAR phaseStart/or down by the phase value specified in the
/// HIPAR phaseStart/:BB:MCCW:EDIT:CARR:PHAS:STEP command.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTablePhase(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  phaseStart,
                                                     ViReal64  phaseStep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_PHASE_START, phaseStart),
            3, "Phase Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_PHASE_STEP, phaseStep),
            4, "Phase Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures carriers power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerStep/Sets the step width by which the starting power of the carriers in the
/// HIPAR powerStep/defined carrier range will be incremented.
/// HIPAR powerStart/Sets the power for the individual carriers in the defined carrier
/// HIPAR powerStart/range. If the command :BB:MCCW:EDIT:CARR:POW:STEP is used to define a
/// HIPAR powerStart/step width, the power entered here applies only to the starting
/// HIPAR powerStart/carrier. The power of the remaining carriers is stepped up or down by
/// HIPAR powerStart/the power specified in the :BB:MCCW:EDIT:CARR:POW:STEP command.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTablePower(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  powerStart,
                                                     ViReal64  powerStep)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_POWER_START, powerStart),
            3, "Power Start");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_EDIT_CARRIER_POWER_STEP, powerStep),
            4, "Power Step");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function switches the selected carrier on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierIndex/Selects the carrier index.
/// HIPAR enabled/Switches the selected carrier on or off.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWCarrierEnabled(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   carrierIndex,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrierIndex, 1, 8192),
            3, "Carrier Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:STAT %ld,%s", path, carrierIndex, enabledArr[enabled ? 1 : 0]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures start phase of the selected carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierIndex/Selects the carrier index.
/// HIPAR phase/Sets the start phase of the selected carrier.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWCarrierPhase(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   carrierIndex,
                                                       ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrierIndex, 1, 8192),
            3, "Carrier Index");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.0, 359.9), 4, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:PHAS %ld,%le", path, carrierIndex, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures power of the selected carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierIndex/Selects the carrier index.
/// HIPAR power/Sets the power of the selected carrier.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWCarrierPower(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   carrierIndex,
                                                       ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrierIndex, 1, 8192),
            3, "Carrier Index");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -80.0, 0.0), 4, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:POW %ld,%le", path, carrierIndex, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  his function switches the carrier on or off with the aid of a value
/// HIFN  list. The first value in the list is assigned to the carrier with index
/// HIFN  0, the second value to the carrier with index 1, etc. The maximum
/// HIFN  length corresponds to the maximum number of multicarriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the size of the Enable array.
/// HIPAR enabled/Switches the carrier on or off with the aid of a value list
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWMulticarrierCarrierEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViBoolean enabled[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:LIST:STAT ", path);
    checkErr(RsCore_WriteAsciiViBooleanArray(instrSession, cmd, enabled, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the start phase of the carrier with the aid of a
/// HIFN  value list. The first value in the list is assigned to the carrier with
/// HIFN  index 0, the second value to the carrier with index 1, etc. The maximum
/// HIFN  length corresponds to the maximum number of multicarriers. There is no
/// HIFN  need to enter all the values every time. Values not set by the value
/// HIFN  list are set with the default values provided they have already been
/// HIFN  explicitly set by a previous command. If this is the case, the values
/// HIFN  continue to apply until overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the size of the Phase array.
/// HIPAR phase/Sets the start phase of the carrier with the aid of a value list.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWMulticarrierCarrierPhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  phase[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:LIST:PHAS ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, phase, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the power of the carrier with the aid of a value
/// HIFN  list. The first value in the list is assigned to the carrier with index
/// HIFN  0, the second value to the carrier with index 1, etc. The maximum
/// HIFN  length corresponds to the maximum number of multicarriers. There is no
/// HIFN  need to enter all the values every time. Values not set by the value
/// HIFN  list are set with the default values provided they have already been
/// HIFN  explicitly set by a previous command. If this is the case, the values
/// HIFN  continue to apply until overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the size of the Power array.
/// HIPAR power/Sets the power of the carrier with the aid of a value list.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWMulticarrierCarrierPower(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  power[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CARR:LIST:POW ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, power, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects Multi Carrier trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects Multi Carrier trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects Multi Carrier trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViReal64  delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_MCCW_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_MCCW_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWTriggerDateTime(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   year,
                                                          ViInt32   month,
                                                          ViInt32   day,
                                                          ViInt32   hour,
                                                          ViInt32   minute,
                                                          ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_MCCW_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_MCCW_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureMCCWClockSourceLegacy(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K61"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:MCCW:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the FM-STEREO standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR stereoEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR stereoEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureStereoEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean stereoEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_ENABLED, stereoEnabled),
            3, "Stereo Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures audio settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR audioMode/Selects the operating mode.
/// HIPAR audioLevel/Sets the level of the audio signal.
/// HIPAR preemphasis/Enables or disables and sets the preemphasis parameter value to 50 s
/// HIPAR preemphasis/or 75s.
/// HIPAR deviation/Sets the level of the audio signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureStereoAudioSettings(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViReal64  deviation,
                                                          ViInt32   audioMode,
                                                          ViReal64  audioLevel,
                                                          ViInt32   preemphasis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_DEVIATION, deviation),
            3, "Deviation");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_AUDIO_MODE, audioMode),
            4, "Audio Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_AUDIO_LEVEL, audioLevel),
            5, "Audio Level");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_PREEMPHASIS, preemphasis),
            6, "Preemphasis");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures audio source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the audio source for the FM-Stereo signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureStereoAudioSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_AUDIO_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates or deactivates the pilot tone.
/// HIFN
/// HIFN     Note:
/// HIFN     Stereo operating mode is possible with enabled pilot tone only.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Enabled or disables the pilot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureStereoPilotEnable(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_PILOT_ENABLE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures stereo pilot tone settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR phase/Sets the phase of the pilot tone (with respect to the 38 kHz
/// HIPAR phase/subcarrier).
/// HIPAR deviation/Sets the frequency deviation of the pilot tone.
ViStatus _VI_FUNC rsrfsiggen_ConfigureStereoPilotTone(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViReal64  deviation,
                                                      ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_PILOT_DEVIATION, deviation),
            3, "Deviation");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_STEREO_PILOT_PHASE, phase),
            4, "Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the XM-RADIO standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR xmrEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR xmrEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureXMREnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean xmrEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_ENABLED, xmrEnabled),
            3, "XMR Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base XM Radio settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR physicalLayerMode/Selects the XM-Radio physical layer mode.
/// HIPAR dataSource/Determines the data source for the data field.
ViStatus _VI_FUNC rsrfsiggen_ConfigureXMR(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   physicalLayerMode,
                                          ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_PHYSICAL_LAYER_MODE, physicalLayerMode),
            3, "Physical Layer Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_DATA_SOURCE, dataSource),
            4, "Data Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellites parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR delay/Sets the signal delay in physical layer transmission satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureXMRSatellite(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_SATELLITE_DELAY, delay),
            3, "Delay");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures terrestrial parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR delay/Sets the signal delay in physical layer mode terrestrial.
/// HIPAR levelAMSS/Sets the signal delay in physical layer mode terrestrial.
/// HIPAR levelMCM/Sets the level for the multicarrier modulation (MCM).
ViStatus _VI_FUNC rsrfsiggen_ConfigureXMRTerrestrial(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViReal64  delay,
                                                     ViReal64  levelAMSS,
                                                     ViReal64  levelMCM)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_TERRESTRIAL_DELAY, delay),
            3, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_TERRESTRIAL_LEVEL_AMSS, levelAMSS),
            4, "Level AMSS");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_XMR_TERRESTRIAL_LEVEL_MCM, levelMCM),
            5, "Level MCM");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the Sirius standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR siriusEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR siriusEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSiriusEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean siriusEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_ENABLED, siriusEnabled),
            3, "Sirius Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base Sirius settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR physicalLayer/Selects the physical layer mode for the transmission of the Sirius
/// HIPAR physicalLayer/signal.
/// HIPAR physicalLayerTransmission/Selects the physical layer transmission for the transmission of the
/// HIPAR physicalLayerTransmission/Sirius signal.
/// HIPAR dataSource/Determines the data source for the data field.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSirius(ViSession instrSession,
                                             ViInt32   path,
                                             ViInt32   physicalLayer,
                                             ViInt32   physicalLayerTransmission,
                                             ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_PHYSICAL_LAYER, physicalLayer),
            3, "Physical Layer");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_PHYSICAL_LAYER_TRANSMISSION, physicalLayerTransmission),
            4, "Physical Layer Transmission");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_DATA_SOURCE, dataSource),
            5, "Data Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellites parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR delay/Sets the signal delay in physical layer transmission satellite.
/// HIPAR modulation/Sets the modulation used in the selected satellite band.
/// HIPAR overlayAngularOffset/Sets the overlay angular offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSiriusSatellite(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViReal64  delay,
                                                      ViInt32   modulation,
                                                      ViInt32   overlayAngularOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_SATELLITE_DELAY, delay),
            3, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_SATELLITE_MODULATION, modulation),
            4, "Modulation");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_SATELLITE_OVERLAY_ANGULAR_OFFSET, overlayAngularOffset),
            5, "Overlay Angular Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures terrestrial parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR delay/Sets the signal delay in physical layer transmission satellite.
/// HIPAR overlayAngularOffset/Sets the overlay angular offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSiriusTerrestrial(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViReal64  delay,
                                                        ViInt32   overlayAngularOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_TERRESTRIAL_DELAY, delay),
            3, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SIRIUS_TERRESTRIAL_OVERLAY_ANGULAR_OFFSET, overlayAngularOffset),
            4, "Overlay Angular Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates modulation in accordance with the EUTRA/LTE standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR eutraEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR eutraEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRAEnabled(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean eutraEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_ENABLED, eutraEnabled),
            3, "EUTRA Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base EUTRA settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR duplexing/Selects the duplexing mode. The duplexing mode determines how the
/// HIPAR duplexing/uplink and downlink signal are separated.
/// HIPAR linkDirection/Defines the transmission direction. The signal either corresponds to
/// HIPAR linkDirection/that of a base station (DOWN) or that of a subscriber station (UP).
/// HIPAR sequenceLength/Sets the sequence length of the signal in number of frames. The signal
/// HIPAR sequenceLength/is calculated in advance and output in the arbitrary waveform
/// HIPAR sequenceLength/generator. The maximum number of frames is calculated as follows:
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRA(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   duplexing,
                                            ViInt32   linkDirection,
                                            ViInt32   sequenceLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_DUPLEXING, duplexing),
            3, "Duplexing");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_LINK_DIRECTION, linkDirection),
            4, "Link Direction");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_SEQUENCE_LENGTH, sequenceLength),
            5, "Sequence Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the codebook index for mapping mode Codebook.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR user/Selects the user.
/// HIPAR index/The number of available codebook indices depends on the number of
/// HIPAR index/antenna ports with mapped user-specific reference symbols, the number
/// HIPAR index/of layers and number of Tx antennas per component carrier.
/// HIPAR codebookIndex/Sets the codebook index for mapping mode Codebook.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRADLUserAPMCodebookIndex(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   user,
                            ViInt32   index,
                            ViInt32   codebookIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, user, 1, 4),
            3, "User");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EUTR:DL:USER%ld:APM:CBIN%ld %ld", path, user, index, codebookIndex);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects EUTRA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRATriggerMode(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects EUTRA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRATriggerSource(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K55"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EUTR:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects EUTRA trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRATriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K55"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:EUTR:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRATrigger(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean sync,
                                                   ViReal64  delay,
                                                   ViInt64   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt64(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_INTERNAL_TRIGGER_INHIBIT, inhibit),
        5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_EUTRA_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRATriggerDateTime(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   year,
                                                           ViInt32   month,
                                                           ViInt32   day,
                                                           ViInt32   hour,
                                                           ViInt32   minute,
                                                           ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_EUTRA_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_EUTRA_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRAClockSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, RSRFSIGGEN_REPCAP_PATH_A, pathCount),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EUTRA_CLOCK_SOURCE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureEUTRAClockSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathCount));
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, RSRFSIGGEN_REPCAP_PATH_A, pathCount),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EUTRA_CLOCK_SOURCE_LEGACY,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EUTRA_CLOCK_SOURCE_LEGACY, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the TETRA standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tetraEnabled/Activates or deactivates modulation in accordance with the DAB
/// HIPAR tetraEnabled/standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRAEnabled(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean tetraEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_ENABLED, tetraEnabled),
            3, "TETRA Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures base TETRA settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR testMode/Selects the test mode.
/// HIPAR linkDirection/Defines the transmission direction. The signal either corresponds to
/// HIPAR linkDirection/that of a base station (DOWN) or that of a subscriber station (UP).
/// HIPAR sequenceLength/Sets the sequence length of the arbitrary waveform file in the number
/// HIPAR sequenceLength/of multiframes. One multiframe is the minimum sequence length for a T1
/// HIPAR sequenceLength/signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRA(ViSession instrSession,
                                            ViInt32   path,
                                            ViInt32   testMode,
                                            ViInt32   linkDirection,
                                            ViInt32   sequenceLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_TEST_MODE, testMode),
            3, "Test Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_LINK_DIRECTION, linkDirection),
            4, "Link Direction");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_SEQUENCE_LENGTH_SYMBOLS, sequenceLength),
            5, "Sequence Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures channel type
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) Channel type is available only for test modes T1 and T4.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR testMode/Selects the channel type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRAChannelType(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   testMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_CHANNEL_TYPE, testMode),
            3, "Test Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures modulation type
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for test modes T2 and User.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR modulationType/Selects the modulation type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRAModulationType(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   modulationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_MODULATION_TYPE, modulationType),
            3, "Modulation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures downlink burst type.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) This function is available only for downlink direction of test
/// HIFN  modes T2 and User.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR burstType/Selects the burst type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRADownlinkBurstType(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   burstType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_DL_BURST_TYPE, burstType),
            3, "Burst Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects TETRA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRATriggerMode(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects TETRA trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRATriggerSource(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K68"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_TRIGGER_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TETR:TRIG:SOUR %s", path, triggerSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects TETRA trigger mode for legacy instruments.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRATriggerSourceLegacy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K68"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_TRIGGER_SOURCE_BEXTERNAL),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TETR:TRIG:SOUR %s", path, triggerSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Sets the state of the signal synchronous to the external trigger
/// HIPAR sync/event.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of samples) for
/// HIPAR delay/triggering by the trigger signal from the second path.
/// HIPAR inhibit/Specifies the number of samples by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event. This function applies only for triggering by
/// HIPAR inhibit/the second path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRATrigger(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViBoolean sync,
                                                   ViReal64  delay,
                                                   ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_TETRA_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRAClockSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K68"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_CLOCK_SOURCE_ELC),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TETR:CLOC:SOUR %s", path, clockSourceArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureTETRAClockSourceLegacy(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentModel(instrSession, "!SMW"));
    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K68"));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_INTERNAL, RSRFSIGGEN_VAL_SMU_CLOCK_SOURCE_AINT),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:TETR:CLOC:SOUR %s", path, clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the TETRA standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR gpsEnabled/Activates or deactivates the GPS standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSEnabled(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViBoolean gpsEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ENABLED, gpsEnabled),
            3, "gpsEnabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rfBand/Selects the center carrier frequency of the instrument.
/// HIPAR testScenario/Selects the file with the predefined A-GPS/A-GNSS test scenario.
/// HIPAR simulationMode/Sets the simulation mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPS(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   rfBand,
                                                ViInt32   testScenario,
                                                ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_RF_BAND, rfBand),
            3, "RF Band");

    if (RsCore_HasInstrumentOptions(instrSession, "K65") == VI_TRUE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_TEST_SCENARIO, testScenario),
                4, "Test Scenario");
    }

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS GNSS.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR galileo/Defines the navigation standards that will be part of the hybrid
/// HIPAR galileo/configuration.
/// HIPAR glonass/Defines the navigation standards that will be part of the hybrid
/// HIPAR glonass/configuration.
/// HIPAR rfFrequency/Use this command if several R&S SMBV instruments are connected to
/// HIPAR rfFrequency/generate GNSS signal in the same GNSS band.
/// HIPAR pCode/Enables the configuration of GPS signals modulates with P code.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSGNSS(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean galileo,
                                                    ViBoolean glonass,
                                                    ViBoolean rfFrequency,
                                                    ViBoolean pCode,
                                                    ViBoolean beidou)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_GNSS_ENABLED_GALILEO, galileo),
            3, "Galileo");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_GNSS_ENABLED_GLONASS, glonass),
            4, "Glonass");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_GNSS_ENABLED_BEIDOU, beidou),
            7, "BeiDou");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_GNSS_USE_COMMON_RF_FREQUENCY, rfFrequency),
            5, "RF Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_GNSS_P_CODE, pCode),
            6, "P-Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSAlmanacFileGPS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ALMANAC_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSAlmanacFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ALMANAC_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSAlmanacFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ALMANAC_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSRINEXFileGPS(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_RINEX_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSRINEXFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_RINEX_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSRINEXFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_RINEX_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSTriggerMode(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIFN
/// HIFN     Note(s): This function is valid only for SMBV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GPS trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSTriggerSource(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSClockSource(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected vehicle description file. Use the command
/// HIFN  MMEM:CDIRectory to define the directory. Only files with the file
/// HIFN  extension *.xvd are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSUserEnvironmentVehicleLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_USER_ENVIRONMENT_VEHICLE_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefind waypoint file. A waypoint file must have the
/// HIFN  extension .txt, .nmea or .xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSLocWaypointFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_LOC_WAYPOINT_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSLocCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 3, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 4, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:LOC:COOR:DEC %lf,%lf,%lf", path, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSLocCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            3, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            4, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 5, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            6, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            7, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            8, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 9, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            10, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 11, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected antenna pattern file. Only files with the file
/// HIFN  extension *.ant_pat are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSAntennaLoad(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ANTENNA_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the software to schedule a real time switch of the used
/// HIFN  antenna at a specific future time, as specified in the antenna pattern
/// HIFN  (*.ant_pat) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR elapsedTime/Elapsed time from the simulation start time at which a switch to the
/// HIPAR elapsedTime/defined <AntennaId> is performed.
/// HIPAR antennaID/Use the command <subsystem>:APATtern:ANTenna:LIST? to query the
/// HIPAR antennaID/antenna IDs defined in the currently selected antenna pattern file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSAntennaUpdate(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  elapsedTime,
                                                             ViInt32   antennaID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elapsedTime, 0.0, 99999999.0), 3, "Elapsed Time");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaID, 1, 100),
            4, "Antenna ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:UPD:ANT %lf,%ld", path, elapsedTime, antennaID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date for the simulation in DD.MM.YYYY format of the
/// HIFN  Gregorian calendar.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSNavigationDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    /*viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
        2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");
    */

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:NAV:SIM:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the exact simulation start time in UTC time format.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and
/// HIFN     <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSNavigationTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Seconds");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:NAV:SIM:TIME %ld,%ld,%lf", path, hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGRefCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:ADG:%s:LOC:COOR:DEC %lf,%lf,%lf", path, ggSystemArr[system], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGRefCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:ADG:%s:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, ggSystemArr[system], longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as almanac file with the specific file extensions
/// HIFN  *.rs_al or *.rs_yuma.
/// HIFN     The default extension is *.rs_al and can be omitted in the file name.
/// HIFN     To save an almanac file as file with extension *.rs_yuma, however,
/// HIFN  this extension has to be specified in the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGGenerateAlmanacFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ADG_GENERATE_ALMANAC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGGenerateNavigationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ADG_GENERATE_NAVIGATION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as acquisition file with the specific file
/// HIFN  extensions *.rs_acq.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGGenerateAcquisitionFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ADG_GENERATE_ACQUISITION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  ionospheric file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as ionospheric file with the specific file
/// HIFN  extensions *.rs_ion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGGenerateIonosphericFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ADG_GENERATE_IONOSPHERIC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected UTC
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as UTC file with the specific file extensions
/// HIFN  *.rs_utc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSADGGenerateUTCFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_ADG_GENERATE_UTC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerMode/Determines whether the power is calculated automatically or is based
/// HIPAR powerMode/on user defined settings.
/// HIPAR referencePower/Sets the power level that is used as a reference for the calculation
/// HIPAR referencePower/of the power level of the satellites.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGPSSat(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   powerMode,
                                                   ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_SAT_POWER_MODE, powerMode),
            3, "Power Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GPS_SAT_REFERENCE_POWER, referencePower),
            4, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the receiver attitude parameters yaw, pitch, roll in degrees
/// HIFN  for the selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR yaw/Returns the Yaw parameter.
/// HIPAR pitch/Returns the Pitch parameter.
/// HIPAR roll/Returns the Roll parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTAttitude(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   timeBasis,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minutes,
                                                      ViReal64  seconds,
                                                      ViInt32   weekNumber,
                                                      ViReal64  timeOfWeek,
                                                      ViReal64* yaw,
                                                      ViReal64* pitch,
                                                      ViReal64* roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:RATT? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf, %lf, %lf", yaw, pitch, roll);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitude/Queries the satellite longitude in the selected moment of time.
/// HIPAR latitude/Queries the satellite latitude in the selected moment of time.
/// HIPAR altitude/Queries the satellite altitude in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTDecimal(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   timeBasis,
                                                     ViInt32   year,
                                                     ViInt32   month,
                                                     ViInt32   day,
                                                     ViInt32   hour,
                                                     ViInt32   minutes,
                                                     ViReal64  seconds,
                                                     ViInt32   weekNumber,
                                                     ViReal64  timeOfWeek,
                                                     ViReal64* longitude,
                                                     ViReal64* latitude,
                                                     ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf, %lf, %lf", longitude, latitude, altitude);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitudeDeg/Queries the satellite longitude degrees.
/// HIPAR longitudeMin/Queries the satellite longitude minutes in the selected moment of
/// HIPAR longitudeMin/time.
/// HIPAR longitudeSec/Queries the satellite longitude seconds in the selected moment of
/// HIPAR longitudeSec/time.
/// HIPAR longitudeDir/Queries the satellite longitude direction in the selected moment of
/// HIPAR longitudeDir/time.
/// HIPAR latitudeDeg/Queries the satellite latitude degrees in the selected moment of time.
/// HIPAR latitudeMin/Queries the satellite latitude minutes in the selected moment of time.
/// HIPAR latitudeSec/Queries the satellite latitude seconds in the selected moment of time.
/// HIPAR latitudeDir/Queries the satellite latitude direction in the selected moment of
/// HIPAR latitudeDir/time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTDMS(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   timeBasis,
                                                 ViInt32   year,
                                                 ViInt32   month,
                                                 ViInt32   day,
                                                 ViInt32   hour,
                                                 ViInt32   minutes,
                                                 ViReal64  seconds,
                                                 ViInt32   weekNumber,
                                                 ViReal64  timeOfWeek,
                                                 ViInt32*  longitudeDeg,
                                                 ViInt32*  longitudeMin,
                                                 ViReal64* longitudeSec,
                                                 ViChar    longitudeDir[],
                                                 ViInt32*  latitudeDeg,
                                                 ViInt32*  latitudeMin,
                                                 ViReal64* latitudeSec,
                                                 ViChar    latitudeDir[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%ld,%ld,%lf,%s,%ld,%ld,%lf,%s", longitudeDeg, longitudeMin, longitudeSec, longitudeDir, latitudeDeg, latitudeMin, latitudeSec, latitudeDir);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTAzimuth(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   svid,
                                                     ViInt32   system,
                                                     ViInt32   timeBasis,
                                                     ViInt32   year,
                                                     ViInt32   month,
                                                     ViInt32   day,
                                                     ViInt32   hour,
                                                     ViInt32   minutes,
                                                     ViReal64  seconds,
                                                     ViInt32   weekNumber,
                                                     ViReal64  timeOfWeek,
                                                     ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite's elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR elevation/Queries the satellite's elevation in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTElevation(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   svid,
                                                       ViInt32   system,
                                                       ViInt32   timeBasis,
                                                       ViInt32   year,
                                                       ViInt32   month,
                                                       ViInt32   day,
                                                       ViInt32   hour,
                                                       ViInt32   minutes,
                                                       ViReal64  seconds,
                                                       ViInt32   weekNumber,
                                                       ViReal64  timeOfWeek,
                                                       ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute power of a specified Tap of a specific satellite
/// HIFN  at a selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR tap/Sets the Tap value.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR absolutePower/Queries the absolute power of a specified Tap of a specific satellite
/// HIPAR absolutePower/at a selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGPSRTPower(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   svid,
                                                   ViInt32   system,
                                                   ViInt32   tap,
                                                   ViInt32   timeBasis,
                                                   ViInt32   year,
                                                   ViInt32   month,
                                                   ViInt32   day,
                                                   ViInt32   hour,
                                                   ViInt32   minutes,
                                                   ViReal64  seconds,
                                                   ViInt32   weekNumber,
                                                   ViReal64  timeOfWeek,
                                                   ViReal64* absolutePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            5, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            6, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            7, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            8, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            9, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            10, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            11, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 12, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            13, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 14, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:%s:SVID%ld:TAP%ld:POW:ABS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, tap, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, absolutePower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the GALILEO standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR galEnabled/Activates or deactivates the GALILEO standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALEnabled(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViBoolean galEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ENABLED, galEnabled),
            3, "GAL Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rfBand/Selects the center carrier frequency of the instrument.
/// HIPAR testScenario/Selects the file with the predefined A-GPS/A-GNSS test scenario.
/// HIPAR simulationMode/Sets the simulation mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGAL(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   rfBand,
                                                ViInt32   testScenario,
                                                ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_RF_BAND, rfBand),
            3, "RF band");

    if (RsCore_HasInstrumentOptions(instrSession, "K67") == VI_TRUE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_TEST_SCENARIO, testScenario),
                4, "Test Scenario");
    }

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS GNSS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR gps/Defines the navigation standards that will be part of the hybrid
/// HIPAR gps/configuration.
/// HIPAR glonass/Defines the navigation standards that will be part of the hybrid
/// HIPAR glonass/configuration.
/// HIPAR rfFrequency/Use this command if several R&S SMBV instruments are connected to
/// HIPAR rfFrequency/generate GNSS signal in the same GNSS band.
/// HIPAR pCode/Enables the configuration of GPS signals modulates with P code.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALGNSS(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean gps,
                                                    ViBoolean glonass,
                                                    ViBoolean beiDou,
                                                    ViBoolean rfFrequency,
                                                    ViBoolean pCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_GNSS_ENABLED_GPS, gps),
            3, "GPS");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_GNSS_ENABLED_GLONASS, glonass),
            4, "GLONASS");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_GNSS_ENABLED_BEIDOU, beiDou),
            5, "BeiDou");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_GNSS_USE_COMMON_RF_FREQUENCY, rfFrequency),
            6, "RF Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_GNSS_P_CODE, pCode),
            7, "P Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALAlmanacFileGPS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ALMANAC_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALAlmanacFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ALMANAC_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALAlmanacFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ALMANAC_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALRINEXFileGPS(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ALMANAC_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALRINEXFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_RINEX_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALRINEXFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_RINEX_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALTriggerMode(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GPS trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALTriggerSource(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALClockSource(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected vehicle description file. Use the command
/// HIFN  MMEM:CDIRectory to define the directory. Only files with the file
/// HIFN  extension *.xvd are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALUserEnvironmentVehicleLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_USER_ENVIRONMENT_VEHICLE_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefind waypoint file. A waypoint file must have the
/// HIFN  extension .txt, .nmea or .xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALLocWaypointFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_LOC_WAYPOINT_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALLocCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 3, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 4, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:LOC:COOR:DEC %lf,%lf,%lf", path, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALLocCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            3, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            4, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 5, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            6, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            7, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            8, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 9, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            10, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 11, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected antenna pattern file. Only files with the file
/// HIFN  extension *.ant_pat are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALAntennaLoad(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ANTENNA_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the software to schedule a real time switch of the used
/// HIFN  antenna at a specific future time, as specified in the antenna pattern
/// HIFN  (*.ant_pat) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR elapsedTime/Elapsed time from the simulation start time at which a switch to the
/// HIPAR elapsedTime/defined <AntennaId> is performed.
/// HIPAR antennaID/Use the command <subsystem>:APATtern:ANTenna:LIST? to query the
/// HIPAR antennaID/antenna IDs defined in the currently selected antenna pattern file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALAntennaUpdate(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  elapsedTime,
                                                             ViInt32   antennaID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elapsedTime, 0.0, 99999999.0), 3, "Elapsed Time");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaID, 1, 100),
            4, "Antenna ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:UPD:ANT %lf,%ld", path, elapsedTime, antennaID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date for the simulation in DD.MM.YYYY format of the
/// HIFN  Gregorian calendar.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALNavigationDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    /*viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
        2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");
    */

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:NAV:SIM:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the exact simulation start time in UTC time format.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and
/// HIFN     <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALNavigationTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:NAV:SIM:TIME %ld,%ld,%lf", path, hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGRefCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:ADG:%s:LOC:COOR:DEC %lf,%lf,%lf", path, ggSystemArr[system], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGRefCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:ADG:%s:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, ggSystemArr[system], longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as almanac file with the specific file extensions
/// HIFN  *.rs_al or *.rs_yuma.
/// HIFN     The default extension is *.rs_al and can be omitted in the file name.
/// HIFN     To save an almanac file as file with extension *.rs_yuma, however,
/// HIFN  this extension has to be specified in the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGGenerateAlmanacFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ADG_GENERATE_ALMANAC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGGenerateNavigationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ADG_GENERATE_NAVIGATION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as acquisition file with the specific file
/// HIFN  extensions *.rs_acq.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGGenerateAcquisitionFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ADG_GENERATE_ACQUISITION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  ionospheric file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as ionospheric file with the specific file
/// HIFN  extensions *.rs_ion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGGenerateIonosphericFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ADG_GENERATE_IONOSPHERIC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected UTC
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as UTC file with the specific file extensions
/// HIFN  *.rs_utc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALADGGenerateUTCFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_ADG_GENERATE_UTC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerMode/Determines whether the power is calculated automatically or is based
/// HIPAR powerMode/on user defined settings.
/// HIPAR referencePower/Sets the power level that is used as a reference for the calculation
/// HIPAR referencePower/of the power level of the satellites.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGALSat(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   powerMode,
                                                   ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_SAT_POWER_MODE, powerMode),
            3, "Power Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GAL_SAT_REFERENCE_POWER, referencePower),
            4, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the receiver attitude parameters yaw, pitch, roll in degrees
/// HIFN  for the selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR yaw/Returns the Yaw parameter.
/// HIPAR pitch/Returns the Pitch parameter.
/// HIPAR roll/Returns the Roll parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTAttitude(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   timeBasis,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minutes,
                                                      ViReal64  seconds,
                                                      ViInt32   weekNumber,
                                                      ViReal64  timeOfWeek,
                                                      ViReal64* yaw,
                                                      ViReal64* pitch,
                                                      ViReal64* roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:RATT? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    (void)sscanf(response, "%lf,%lf,%lf", yaw, pitch, roll);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitude/Queries the satellite longitude in the selected moment of time.
/// HIPAR latitude/Queries the satellite latitude in the selected moment of time.
/// HIPAR altitude/Queries the satellite altitude in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTDecimal(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   timeBasis,
                                                     ViInt32   year,
                                                     ViInt32   month,
                                                     ViInt32   day,
                                                     ViInt32   hour,
                                                     ViInt32   minutes,
                                                     ViReal64  seconds,
                                                     ViInt32   weekNumber,
                                                     ViReal64  timeOfWeek,
                                                     ViReal64* longitude,
                                                     ViReal64* latitude,
                                                     ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", longitude, latitude, altitude);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitudeDeg/Queries the satellite longitude degrees.
/// HIPAR longitudeMin/Queries the satellite longitude minutes in the selected moment of
/// HIPAR longitudeMin/time.
/// HIPAR longitudeSec/Queries the satellite longitude seconds in the selected moment of
/// HIPAR longitudeSec/time.
/// HIPAR longitudeDir/Queries the satellite longitude direction in the selected moment of
/// HIPAR longitudeDir/time.
/// HIPAR latitudeDeg/Queries the satellite latitude degrees in the selected moment of time.
/// HIPAR latitudeMin/Queries the satellite latitude minutes in the selected moment of time.
/// HIPAR latitudeSec/Queries the satellite latitude seconds in the selected moment of time.
/// HIPAR latitudeDir/Queries the satellite latitude direction in the selected moment of
/// HIPAR latitudeDir/time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTDMS(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   timeBasis,
                                                 ViInt32   year,
                                                 ViInt32   month,
                                                 ViInt32   day,
                                                 ViInt32   hour,
                                                 ViInt32   minutes,
                                                 ViReal64  seconds,
                                                 ViInt32   weekNumber,
                                                 ViReal64  timeOfWeek,
                                                 ViInt32*  longitudeDeg,
                                                 ViInt32*  longitudeMin,
                                                 ViReal64* longitudeSec,
                                                 ViChar    longitudeDir[],
                                                 ViInt32*  latitudeDeg,
                                                 ViInt32*  latitudeMin,
                                                 ViReal64* latitudeSec,
                                                 ViChar    latitudeDir[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%ld,%ld,%lf,%s,%ld,%ld,%lf,%s", longitudeDeg, longitudeMin, longitudeSec, longitudeDir, latitudeDeg, latitudeMin, latitudeSec, latitudeDir);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTAzimuth(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   svid,
                                                     ViInt32   system,
                                                     ViInt32   timeBasis,
                                                     ViInt32   year,
                                                     ViInt32   month,
                                                     ViInt32   day,
                                                     ViInt32   hour,
                                                     ViInt32   minutes,
                                                     ViReal64  seconds,
                                                     ViInt32   weekNumber,
                                                     ViReal64  timeOfWeek,
                                                     ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite's elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR elevation/Queries the satellite's elevation in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTElevation(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   svid,
                                                       ViInt32   system,
                                                       ViInt32   timeBasis,
                                                       ViInt32   year,
                                                       ViInt32   month,
                                                       ViInt32   day,
                                                       ViInt32   hour,
                                                       ViInt32   minutes,
                                                       ViReal64  seconds,
                                                       ViInt32   weekNumber,
                                                       ViReal64  timeOfWeek,
                                                       ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute power of a specified Tap of a specific satellite
/// HIFN  at a selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR tap/Sets the Tap value.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR absolutePower/Queries the absolute power of a specified Tap of a specific satellite
/// HIPAR absolutePower/at a selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGALRTPower(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   svid,
                                                   ViInt32   system,
                                                   ViInt32   tap,
                                                   ViInt32   timeBasis,
                                                   ViInt32   year,
                                                   ViInt32   month,
                                                   ViInt32   day,
                                                   ViInt32   hour,
                                                   ViInt32   minutes,
                                                   ViReal64  seconds,
                                                   ViInt32   weekNumber,
                                                   ViReal64  timeOfWeek,
                                                   ViReal64* absolutePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            5, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            6, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            7, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            8, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            9, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            10, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            11, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 12, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            13, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 14, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GAL:RT:%s:SVID%ld:TAP%ld:POW:ABS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, tap, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, absolutePower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the GLONASS standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR glonEnabled/Activates or deactivates the GLONASS standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONEnabled(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViBoolean glonEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ENABLED, glonEnabled),
            3, "GLON Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rfBand/Selects the center carrier frequency of the instrument.
/// HIPAR testScenario/Selects the file with the predefined A-GPS/A-GNSS test scenario.
/// HIPAR simulationMode/Sets the simulation mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLON(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   rfBand,
                                                 ViInt32   testScenario,
                                                 ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_RF_BAND, rfBand),
            3, "RF Band");

    if (RsCore_HasInstrumentOptions(instrSession, "K95") == VI_TRUE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_TEST_SCENARIO, testScenario),
                4, "Test Scenario");
    }

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation GPS GNSS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR gps/Defines the navigation standards that will be part of the hybrid
/// HIPAR gps/configuration.
/// HIPAR galileo/Defines the navigation standards that will be part of the hybrid
/// HIPAR galileo/configuration.
/// HIPAR rfFrequency/Use this command if several R&S SMBV instruments are connected to
/// HIPAR rfFrequency/generate GNSS signal in the same GNSS band.
/// HIPAR pCode/Enables the configuration of GPS signals modulates with P code.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONGNSS(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViBoolean gps,
                                                     ViBoolean galileo,
                                                     ViBoolean beiDou,
                                                     ViBoolean rfFrequency,
                                                     ViBoolean pCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_GNSS_ENABLED_GPS, gps),
            3, "GPS");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_GNSS_ENABLED_GALILEO, galileo),
            4, "GALILEO");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_GNSS_ENABLED_BEIDOU, beiDou),
            5, "BeiDou");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_GNSS_USE_COMMON_RF_FREQUENCY, rfFrequency),
            6, "RF Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_GNSS_P_CODE, pCode),
            7, "P Code");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONAlmanacFileGPS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ALMANAC_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONAlmanacFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ALMANAC_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLOAlmanacFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ALMANAC_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONRINEXFileGPS(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_RINEX_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONRINEXFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_RINEX_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLORINEXFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_RINEX_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONTriggerMode(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GPS trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONTriggerSource(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONClockSource(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected vehicle description file. Use the command
/// HIFN  MMEM:CDIRectory to define the directory. Only files with the file
/// HIFN  extension *.xvd are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONUserEnvironmentVehicleLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_USER_ENVIRONMENT_VEHICLE_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefind waypoint file. A waypoint file must have the
/// HIFN  extension .txt, .nmea or .xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONLocWaypointFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_LOC_WAYPOINT_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONLocCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 3, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 4, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:LOC:COOR:DEC %lf,%lf,%lf", path, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONLocCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            3, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            4, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 5, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            6, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            7, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            8, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 9, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            10, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 11, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected antenna pattern file. Only files with the file
/// HIFN  extension *.ant_pat are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONAntennaLoad(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ANTENNA_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the software to schedule a real time switch of the used
/// HIFN  antenna at a specific future time, as specified in the antenna pattern
/// HIFN  (*.ant_pat) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR elapsedTime/Elapsed time from the simulation start time at which a switch to the
/// HIPAR elapsedTime/defined <AntennaId> is performed.
/// HIPAR antennaID/Use the command <subsystem>:APATtern:ANTenna:LIST? to query the
/// HIPAR antennaID/antenna IDs defined in the currently selected antenna pattern file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONAntennaUpdate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  elapsedTime,
                            ViInt32   antennaID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elapsedTime, 0.0, 99999999.0), 3, "Elapsed Time");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaID, 1, 100),
            4, "Antenna ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:UPD:ANT %lf,%ld", path, elapsedTime, antennaID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date for the simulation in DD.MM.YYYY format of the
/// HIFN  Gregorian calendar.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONNavigationDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    /*viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
        2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");
    */

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:NAV:SIM:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the exact simulation start time in UTC time format.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and
/// HIFN     <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONNavigationTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:NAV:SIM:TIME %ld,%ld,%lf", path, hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGRefCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:ADG:%s:LOC:COOR:DEC %lf,%lf,%lf", path, ggSystemArr[system], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGRefCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:ADG:%s:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, ggSystemArr[system], longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as almanac file with the specific file extensions
/// HIFN  *.rs_al or *.rs_yuma.
/// HIFN     The default extension is *.rs_al and can be omitted in the file name.
/// HIFN     To save an almanac file as file with extension *.rs_yuma, however,
/// HIFN  this extension has to be specified in the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGGenerateAlmanacFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ADG_GENERATE_ALMANAC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGGenerateNavigationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ADG_GENERATE_NAVIGATION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as acquisition file with the specific file
/// HIFN  extensions *.rs_acq.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGGenerateAcquisitionFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ADG_GENERATE_ACQUISITION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  ionospheric file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as ionospheric file with the specific file
/// HIFN  extensions *.rs_ion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGGenerateIonosphericFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ADG_GENERATE_IONOSPHERIC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected UTC
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as UTC file with the specific file extensions
/// HIFN  *.rs_utc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONADGGenerateUTCFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_ADG_GENERATE_UTC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerMode/Determines whether the power is calculated automatically or is based
/// HIPAR powerMode/on user defined settings.
/// HIPAR referencePower/Sets the power level that is used as a reference for the calculation
/// HIPAR referencePower/of the power level of the satellites.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGLONSat(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   powerMode,
                                                    ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_SAT_POWER_MODE, powerMode),
            3, "Power Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GLON_SAT_REFERENCE_POWER, referencePower),
            4, "Reference Power");

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLONRTAttitude(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   timeBasis,
                                                       ViInt32   year,
                                                       ViInt32   month,
                                                       ViInt32   day,
                                                       ViInt32   hour,
                                                       ViInt32   minutes,
                                                       ViReal64  seconds,
                                                       ViInt32   weekNumber,
                                                       ViReal64  timeOfWeek,
                                                       ViReal64* yaw,
                                                       ViReal64* pitch,
                                                       ViReal64* roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:RATT? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", yaw, pitch, roll);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitude/Queries the satellite longitude in the selected moment of time.
/// HIPAR latitude/Queries the satellite latitude in the selected moment of time.
/// HIPAR altitude/Queries the satellite altitude in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLORTDecimal(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   timeBasis,
                                                     ViInt32   year,
                                                     ViInt32   month,
                                                     ViInt32   day,
                                                     ViInt32   hour,
                                                     ViInt32   minutes,
                                                     ViReal64  seconds,
                                                     ViInt32   weekNumber,
                                                     ViReal64  timeOfWeek,
                                                     ViReal64* longitude,
                                                     ViReal64* latitude,
                                                     ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", longitude, latitude, altitude);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitudeDeg/Queries the satellite longitude degrees.
/// HIPAR longitudeMin/Queries the satellite longitude minutes in the selected moment of
/// HIPAR longitudeMin/time.
/// HIPAR longitudeSec/Queries the satellite longitude seconds in the selected moment of
/// HIPAR longitudeSec/time.
/// HIPAR longitudeDir/Queries the satellite longitude direction in the selected moment of
/// HIPAR longitudeDir/time.
/// HIPAR latitudeDeg/Queries the satellite latitude degrees in the selected moment of time.
/// HIPAR latitudeMin/Queries the satellite latitude minutes in the selected moment of time.
/// HIPAR latitudeSec/Queries the satellite latitude seconds in the selected moment of time.
/// HIPAR latitudeDir/Queries the satellite latitude direction in the selected moment of
/// HIPAR latitudeDir/time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLORTDMS(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   timeBasis,
                                                 ViInt32   year,
                                                 ViInt32   month,
                                                 ViInt32   day,
                                                 ViInt32   hour,
                                                 ViInt32   minutes,
                                                 ViReal64  seconds,
                                                 ViInt32   weekNumber,
                                                 ViReal64  timeOfWeek,
                                                 ViInt32*  longitudeDeg,
                                                 ViInt32*  longitudeMin,
                                                 ViReal64* longitudeSec,
                                                 ViChar    longitudeDir[],
                                                 ViInt32*  latitudeDeg,
                                                 ViInt32*  latitudeMin,
                                                 ViReal64* latitudeSec,
                                                 ViChar    latitudeDir[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%ld, %ld, %lf,%s,%ld,%ld,%lf,%s", longitudeDeg, longitudeMin, longitudeSec, longitudeDir, latitudeDeg, latitudeMin, latitudeSec, latitudeDir);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLONRTAzimuth(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   svid,
                                                      ViInt32   system,
                                                      ViInt32   timeBasis,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minutes,
                                                      ViReal64  seconds,
                                                      ViInt32   weekNumber,
                                                      ViReal64  timeOfWeek,
                                                      ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite's elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR elevation/Queries the satellite's elevation in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLONRTElevation(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   svid,
                                                        ViInt32   system,
                                                        ViInt32   timeBasis,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minutes,
                                                        ViReal64  seconds,
                                                        ViInt32   weekNumber,
                                                        ViReal64  timeOfWeek,
                                                        ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf",
        path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute power of a specified Tap of a specific satellite
/// HIFN  at a selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR tap/Sets the Tap value.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR absolutePower/Queries the absolute power of a specified Tap of a specific satellite
/// HIPAR absolutePower/at a selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGLONRTPower(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   svid,
                                                    ViInt32   system,
                                                    ViInt32   tap,
                                                    ViInt32   timeBasis,
                                                    ViInt32   year,
                                                    ViInt32   month,
                                                    ViInt32   day,
                                                    ViInt32   hour,
                                                    ViInt32   minutes,
                                                    ViReal64  seconds,
                                                    ViInt32   weekNumber,
                                                    ViReal64  timeOfWeek,
                                                    ViReal64* absolutePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            5, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            6, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            7, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            8, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            9, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            10, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            11, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 12, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            13, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 14, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GLON:RT:%s:SVID%ld:TAP%ld:POW:ABS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf",
        path, ggSystemArr[system], svid, tap, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, absolutePower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates or deactivates the BeiDou standard.
/// HIFN
/// HIFN     Activating this standard deactivates all the other digital standards
/// HIFN  and digital modulation modes (in case of two-path instruments, this
/// HIFN  affects the same path).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR beidouEnabled/Activates or deactivates the BeiDou standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouEnabled(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViBoolean beidouEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ENABLED, beidouEnabled),
            3, "BeiDou Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation BeiDou.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rfBand/Selects the center carrier frequency of the instrument.
/// HIPAR simulationMode/Sets the simulation mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDou(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   rfBand,
                                                   ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_RF_BAND, rfBand),
            3, "RF Band");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_SIMULATION_MODE, simulationMode),
            5, "Simulation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures satellite navigation BeiDou GNSS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR galileo/Defines the navigation standards that will be part of the hybrid
/// HIPAR galileo/configuration.
/// HIPAR glonass/Defines the navigation standards that will be part of the hybrid
/// HIPAR glonass/configuration.
/// HIPAR rfFrequency/Use this command if several R&S SMBV instruments are connected to
/// HIPAR rfFrequency/generate GNSS signal in the same GNSS band.
/// HIPAR pCode/Enables the configuration of GPS signals modulates with P code.
/// HIPAR gps/Defines the navigation standards that will be part of the hybrid
/// HIPAR gps/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouGNSS(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViBoolean galileo,
                                                       ViBoolean glonass,
                                                       ViBoolean rfFrequency,
                                                       ViBoolean pCode,
                                                       ViBoolean gps)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_GNSS_ENABLED_GALILEO, galileo),
            3, "Galileo");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_GNSS_ENABLED_GLONASS, glonass),
            4, "Glonass");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_GNSS_USE_COMMON_RF_FREQUENCY, rfFrequency),
            5, "RF Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_GNSS_P_CODE, pCode),
            6, "P Code");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_GNSS_ENABLED_GPS, gps),
            7, "GPS");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAlmanacFileBeiDou(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ALMANAC_FILE_BEIDOU, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAlmanacFileGPS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ALMANAC_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAlmanacFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ALMANAC_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAlmanacFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ALMANAC_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouRINEXFileBeiDou(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_RINEX_FILE_BEIDOU, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouRINEXFileGPS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_RINEX_FILE_GPS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouRINEXFileGALILEO(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_RINEX_FILE_GALILEO, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the RINEX file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouRINEXFileGLONASS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_RINEX_FILE_GLONASS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouTriggerMode(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source for legacy instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects GPS trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouTriggerSource(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouClockSource(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected vehicle description file. Use the command
/// HIFN  MMEM:CDIRectory to define the directory. Only files with the file
/// HIFN  extension *.xvd are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouUserEnvironmentVehicleLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_USER_ENVIRONMENT_VEHICLE_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefind waypoint file. A waypoint file must have the
/// HIFN  extension .txt, .nmea or .xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouLocWaypointFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_LOC_WAYPOINT_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouLocDecimal(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  longitude,
                                                             ViReal64  latitude,
                                                             ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 3, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 4, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:LOC:COOR:DEC %lf,%lf,%lf", path, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouLocDMS(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   longitudeDeg,
                                                         ViInt32   longitudeMin,
                                                         ViReal64  longitudeSec,
                                                         ViInt32   longitudeDir,
                                                         ViInt32   latitudeDeg,
                                                         ViInt32   latitudeMin,
                                                         ViReal64  latitudeSec,
                                                         ViInt32   latitudeDir,
                                                         ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            3, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            4, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 5, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            6, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            7, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            8, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 9, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            10, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 11, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected antenna pattern file. Only files with the file
/// HIFN  extension *.ant_pat are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAntennaLoad(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ANTENNA_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the software to schedule a real time switch of the used
/// HIFN  antenna at a specific future time, as specified in the antenna pattern
/// HIFN  (*.ant_pat) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR elapsedTime/Elapsed time from the simulation start time at which a switch to the
/// HIPAR elapsedTime/defined <AntennaId> is performed.
/// HIPAR antennaID/Use the command <subsystem>:APATtern:ANTenna:LIST? to query the
/// HIPAR antennaID/antenna IDs defined in the currently selected antenna pattern file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouAntennaUpdate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  elapsedTime,
                            ViInt32   antennaID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elapsedTime, 0.0, 99999999.0), 3, "Elapsed Time");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaID, 1, 100),
            4, "Antenna ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GPS:RT:UPD:ANT %lf,%ld", path, elapsedTime, antennaID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date for the simulation in DD.MM.YYYY format of the
/// HIFN  Gregorian calendar.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouNavigationDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    /*viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
        2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");
    */

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:NAV:SIM:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the exact simulation start time in UTC time format.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and
/// HIFN     <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouNavigationTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:NAV:SIM:TIME %ld,%ld,%lf", path, hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGRefDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   system,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:ADG:%s:LOC:COOR:DEC %lf,%lf,%lf", path, ggSystemArr[system], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR system/Selects the SatNav System.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGRefDMS(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   system,
                                                            ViInt32   longitudeDeg,
                                                            ViInt32   longitudeMin,
                                                            ViReal64  longitudeSec,
                                                            ViInt32   longitudeDir,
                                                            ViInt32   latitudeDeg,
                                                            ViInt32   latitudeMin,
                                                            ViReal64  latitudeSec,
                                                            ViInt32   latitudeDir,
                                                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:ADG:%s:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, ggSystemArr[system], longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as almanac file with the specific file extensions
/// HIFN  *.rs_al or *.rs_yuma.
/// HIFN     The default extension is *.rs_al and can be omitted in the file name.
/// HIFN     To save an almanac file as file with extension *.rs_yuma, however,
/// HIFN  this extension has to be specified in the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGAlmFile(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ADG_GENERATE_ALMANAC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGNavFile(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ADG_GENERATE_NAVIGATION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as acquisition file with the specific file
/// HIFN  extensions *.rs_acq.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGAcqFile(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ADG_GENERATE_ACQUISITION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  ionospheric file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as ionospheric file with the specific file
/// HIFN  extensions *.rs_ion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGIonFile(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ADG_GENERATE_IONOSPHERIC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected UTC
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as UTC file with the specific file extensions
/// HIFN  *.rs_utc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouADGUTCFile(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_ADG_GENERATE_UTC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR powerMode/Determines whether the power is calculated automatically or is based
/// HIPAR powerMode/on user defined settings.
/// HIPAR referencePower/Sets the power level that is used as a reference for the calculation
/// HIPAR referencePower/of the power level of the satellites.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavBeiDouSat(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   powerMode,
                                                      ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_SAT_POWER_MODE, powerMode),
            3, "Power Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_BEIDOU_SAT_REFERENCE_POWER, referencePower),
            4, "Reference Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the receiver attitude parameters yaw, pitch, roll in degrees
/// HIFN  for the selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR yaw/Returns the Yaw parameter.
/// HIPAR pitch/Returns the Pitch parameter.
/// HIPAR roll/Returns the Roll parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTAttitude(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   timeBasis,
                                                         ViInt32   year,
                                                         ViInt32   month,
                                                         ViInt32   day,
                                                         ViInt32   hour,
                                                         ViInt32   minutes,
                                                         ViReal64  seconds,
                                                         ViInt32   weekNumber,
                                                         ViReal64  timeOfWeek,
                                                         ViReal64* yaw,
                                                         ViReal64* pitch,
                                                         ViReal64* roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:RATT? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", yaw, pitch, roll);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitude/Queries the satellite longitude in the selected moment of time.
/// HIPAR latitude/Queries the satellite latitude in the selected moment of time.
/// HIPAR altitude/Queries the satellite altitude in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTDecimal(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   timeBasis,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minutes,
                                                        ViReal64  seconds,
                                                        ViInt32   weekNumber,
                                                        ViReal64  timeOfWeek,
                                                        ViReal64* longitude,
                                                        ViReal64* latitude,
                                                        ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", longitude, latitude, altitude);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitudeDeg/Queries the satellite longitude degrees.
/// HIPAR longitudeMin/Queries the satellite longitude minutes in the selected moment of
/// HIPAR longitudeMin/time.
/// HIPAR longitudeSec/Queries the satellite longitude seconds in the selected moment of
/// HIPAR longitudeSec/time.
/// HIPAR longitudeDir/Queries the satellite longitude direction in the selected moment of
/// HIPAR longitudeDir/time.
/// HIPAR latitudeDeg/Queries the satellite latitude degrees in the selected moment of time.
/// HIPAR latitudeMin/Queries the satellite latitude minutes in the selected moment of time.
/// HIPAR latitudeSec/Queries the satellite latitude seconds in the selected moment of time.
/// HIPAR latitudeDir/Queries the satellite latitude direction in the selected moment of
/// HIPAR latitudeDir/time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTDMS(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   timeBasis,
                                                    ViInt32   year,
                                                    ViInt32   month,
                                                    ViInt32   day,
                                                    ViInt32   hour,
                                                    ViInt32   minutes,
                                                    ViReal64  seconds,
                                                    ViInt32   weekNumber,
                                                    ViReal64  timeOfWeek,
                                                    ViInt32*  longitudeDeg,
                                                    ViInt32*  longitudeMin,
                                                    ViReal64* longitudeSec,
                                                    ViChar    longitudeDir[],
                                                    ViInt32*  latitudeDeg,
                                                    ViInt32*  latitudeMin,
                                                    ViReal64* latitudeSec,
                                                    ViChar    latitudeDir[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%ld, %ld, %lf,%s,%ld,%ld,%lf,%s", longitudeDeg, longitudeMin, longitudeSec, longitudeDir, latitudeDeg, latitudeMin, latitudeSec, latitudeDir);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTAzimuth(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   svid,
                                                        ViInt32   system,
                                                        ViInt32   timeBasis,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minutes,
                                                        ViReal64  seconds,
                                                        ViInt32   weekNumber,
                                                        ViReal64  timeOfWeek,
                                                        ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite's elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR elevation/Queries the satellite's elevation in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTElevation(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   svid,
                                                          ViInt32   system,
                                                          ViInt32   timeBasis,
                                                          ViInt32   year,
                                                          ViInt32   month,
                                                          ViInt32   day,
                                                          ViInt32   hour,
                                                          ViInt32   minutes,
                                                          ViReal64  seconds,
                                                          ViInt32   weekNumber,
                                                          ViReal64  timeOfWeek,
                                                          ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute power of a specified Tap of a specific satellite
/// HIFN  at a selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR tap/Sets the Tap value.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR absolutePower/Queries the absolute power of a specified Tap of a specific satellite
/// HIPAR absolutePower/at a selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavBeiDouRTPower(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   svid,
                                                      ViInt32   system,
                                                      ViInt32   tap,
                                                      ViInt32   timeBasis,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minutes,
                                                      ViReal64  seconds,
                                                      ViInt32   weekNumber,
                                                      ViReal64  timeOfWeek,
                                                      ViReal64* absolutePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            5, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            6, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            7, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            8, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            9, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            10, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            11, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 12, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            13, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 14, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:BEID:RT:%s:SVID%ld:TAP%ld:POW:ABS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", path, ggSystemArr[system], svid, tap, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, absolutePower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the selected signal simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables or disables the selected signal simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavEnabled(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the RF frequency in the Frequency display in the header of the
/// HIFN  instrument to the carrier frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavUpdateRFFrequency(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:PRFF\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the center carrier frequency of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR rfBand/Selects the center carrier frequency of the instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRFBand(ViSession instrSession,
                                                   ViInt32   standard,
                                                   ViInt32   rfBand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rfBand, 0, 1),
            3, "RF Band");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RFB %s\n", satNavStandardArr[standard], satNavRFBandArr[rfBand]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the file with the predefined A-GPS/A-GNSS test scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR testScenario/Selects the file with the predefined A-GPS/A-GNSS test scenario.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTestScenario(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   testScenario)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, testScenario, 0, 22),
            3, "Test Scenario");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATSC %s\n", satNavStandardArr[standard], satNavTestScenarioArr[testScenario]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the simulation mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR simulationMode/Sets the simulation mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSimulationMode(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   simulationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, simulationMode, 0, 2),
            3, "Simulation Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SMOD %s\n", satNavStandardArr[standard], satNavSimulationModeArr[simulationMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Restores the default settings. The default values correspond to the
/// HIFN  *RST values in the descriptions of the SCPI commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSetToDefault(ViSession instrSession,
                                                         ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:PRES\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the navigation standards that will be part of the hybrid
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Defines the navigation standards that will be part of the hybrid
/// HIPAR enabled/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSEnabledGPS(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HYBR:GPS %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the navigation standards that will be part of the hybrid
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Defines the navigation standards that will be part of the hybrid
/// HIPAR enabled/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSEnabledGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HYBR:GAL %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the navigation standards that will be part of the hybrid
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Defines the navigation standards that will be part of the hybrid
/// HIPAR enabled/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSEnabledGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HYBR:GLON %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the navigation standards that will be part of the hybrid
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Defines the navigation standards that will be part of the hybrid
/// HIPAR enabled/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSEnabledBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HYBR:BEID %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the navigation standards that will be part of the hybrid
/// HIFN  configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Defines the navigation standards that will be part of the hybrid
/// HIPAR enabled/configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSEnabledQZSS(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HYBR:QZSS %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Use this command if several R&S SMBV instruments are connected to
/// HIFN  generate GNSS signal in the same GNSS band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR useCommonRFFrequency/Use this command if several R&S SMBV instruments are connected to
/// HIPAR useCommonRFFrequency/generate GNSS signal in the same GNSS band.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSUseCommonRFFrequency(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean useCommonRFFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:UCRF %s\n", satNavStandardArr[standard], (useCommonRFFrequency == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the configuration of GPS signals modulates with P code.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR pCode/Enables the configuration of GPS signals modulates with P code.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPCode(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViBoolean pCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:UMGP %s\n", satNavStandardArr[standard], (pCode == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates anti spoofing flag in the GPS navigation message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR antiSpoofing/Activates anti spoofing flag in the GPS navigation message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAntiSpoofing(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean antiSpoofing)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT:ASP %s\n", satNavStandardArr[standard], (antiSpoofing == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the SV IDs of all valid satellites for the GNSS system.
/// HIFN     The query lists SV IDs of the satellites included in the satellite
/// HIFN  constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/The path for which is this function is performed.
/// HIPAR standard/Sets the satellite navigation standard.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR list/Queries the list of all available SVIDs.
/// HIPAR returnedValues/Displays the number of returned values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSSVIDListValid(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   standard,
                                                          ViInt32   arraySize,
                                                          ViInt32   list[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attrArr[] =
             {
             RSRFSIGGEN_ATTR_SATNAV_GNSS_GPS_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_GALILEO_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_GLONASS_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_BEIDOU_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_QZSS_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SVID_LIST_VALID,
             RSRFSIGGEN_ATTR_SATNAV_GNSS_NAVIC_SVID_LIST_VALID
             };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                4, "Array Size");

    checkErr(RsCore_GetAttributeViStringAsViInt32ArrayToUserBuffer(instrSession, repCap, attrArr[standard], NULL,
                                                                    arraySize, list, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAlmanacFileGPS(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAlmanacFileGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAlmanacFileGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GLON:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAlmanacFileBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:BEID:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAlmanacFileQZSS(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:QZSS:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This setting is only available for GPS and Galileo satellites.
/// HIFN     Queries the week number of the selected almanac file for the
/// HIFN  navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/This setting is ony available for GPS and Galileo satellites.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekNumberGPS(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViReal64* weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:WNUM?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This setting is only available for GPS and Galileo satellites.
/// HIFN     Queries the week number of the selected almanac file for the
/// HIFN  navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/This setting is ony available for GPS and Galileo satellites.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekNumberGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64* weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:WNUM?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start date of the week span of the selected almanac file
/// HIFN  for the navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR begin/Queries the start date of the week span of the selected almanac file
/// HIPAR begin/for the navigation standard. The file is selected using
/// HIPAR begin/<subsystem>:NAVigation:ALManac:<GNSS>:FILE.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekSpanBeginGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    begin[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:DATE:BEG?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(begin, 128, response));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start date of the week span of the selected almanac file
/// HIFN  for the navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR begin/Queries the start date of the week span of the selected almanac file
/// HIPAR begin/for the navigation standard. The file is selected using
/// HIPAR begin/<subsystem>:NAVigation:ALManac:<GNSS>:FILE.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekSpanBeginGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    begin[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:DATE:BEG?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, begin, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start date of the week span of the selected almanac file
/// HIFN  for the navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR end/Queries the start date of the week span of the selected almanac file
/// HIPAR end/for the navigation standard. The file is selected using
/// HIPAR end/<subsystem>:NAVigation:ALManac:<GNSS>:FILE.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekSpanEndGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    end[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:DATE:END?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, end, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the start date of the week span of the selected almanac file
/// HIFN  for the navigation standard. The file is selected using
/// HIFN  <subsystem>:NAVigation:ALManac:<GNSS>:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR end/Queries the start date of the week span of the selected almanac file
/// HIPAR end/for the navigation standard. The file is selected using
/// HIPAR end/<subsystem>:NAVigation:ALManac:<GNSS>:FILE.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacWeekSpanEndGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    end[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:DATE:END?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, end, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Week Number for which the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Determines the Week Number for which the selected almanac is used for
/// HIPAR weekNumber/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPWeekNumberGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32*  weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:TOAP:WNUM?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Week Number for which the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Determines the Week Number for which the selected almanac is used for
/// HIPAR weekNumber/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPWeekNumberGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32*  weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:TOAP:WNUM?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeOfWeek/Determines the Time of Week (TOW) the selected almanac is used for
/// HIPAR timeOfWeek/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPTimeOfWeekGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    timeOfWeek[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GPS:TOAP:TOW?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, timeOfWeek, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeOfWeek/Determines the Time of Week (TOW) the selected almanac is used for
/// HIPAR timeOfWeek/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPTimeOfWeekGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    timeOfWeek[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GAL:TOAP:TOW?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, timeOfWeek, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR date/Determines the Time of Week (TOW) the selected almanac is used for
/// HIPAR date/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPDateGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    date[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GLON:TOAP:DATE?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, date, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the selected almanac is used for
/// HIFN  (time of applicability).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR time/Determines the Time of Week (TOW) the selected almanac is used for
/// HIPAR time/(time of applicability).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavAlmanacTOAPTimeGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViChar    time[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:ALM:GLON:TOAP:TIME?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, time, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and activates one RINEX file per navigation standard. Perform
/// HIFN  <subsystem>:NAVigation:RINex:IMPort to upload the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXFileGPS(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GPS:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a RINEX configuration file previously selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Activates a RINEX configuration file previously selected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXEnabledGPS(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GPS:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and activates one RINEX file per navigation standard. Perform
/// HIFN  <subsystem>:NAVigation:RINex:IMPort to upload the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXFileGALILEO(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GAL:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a RINEX configuration file previously selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Activates a RINEX configuration file previously selected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXEnabledGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GAL:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and activates one RINEX file per navigation standard. Perform
/// HIFN  <subsystem>:NAVigation:RINex:IMPort to upload the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXFileGLONASS(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GLON:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a RINEX configuration file previously selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Activates a RINEX configuration file previously selected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXEnabledGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:GLON:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and activates one RINEX file per navigation standard. Perform
/// HIFN  <subsystem>:NAVigation:RINex:IMPort to upload the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXFileBeiDou(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:BEID:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a RINEX configuration file previously selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Activates a RINEX configuration file previously selected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXEnabledBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:BEID:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and activates one RINEX file per navigation standard. Perform
/// HIFN  <subsystem>:NAVigation:RINex:IMPort to upload the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXFileQZSS(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:QZSS:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates a RINEX configuration file previously selected.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Activates a RINEX configuration file previously selected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXEnabledQZSS(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:QZSS:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the time conversion and the atmospheric parameters to the
/// HIFN  corresponding values retrieved from the RINEX file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR utc/Synchronizes the time conversion and the atmospheric parameters to the
/// HIPAR utc/corresponding values retrieved from the RINEX file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXUTC(ViSession instrSession,
                                                     ViInt32   standard,
                                                     ViBoolean utc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:UUAS %s\n", satNavStandardArr[standard], (utc == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Updates the frequency number for Glonass satellites automatically at
/// HIFN  import of a RINEX File.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR updateFrequency/Updates the frequency number for Glonass satellites automatically at
/// HIPAR updateFrequency/import of a RINEX File.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXUpdateFrequencyGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean updateFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:UFNS %s\n", satNavStandardArr[standard], (updateFrequency == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Uploads the RINEX file selected by
/// HIFN  <subsystem>:NAVigation:RINex:GPS:FILE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRINEXImport(ViSession instrSession,
                                                        ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:RIN:IMP\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the almanac file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSMOPSFile(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_IONOSPHERIC_MOPS_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_IONOSPHERIC_MOPS_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Saves the current ionospheric grid configuration in a file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSMOPSExport(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_IONOSPHERIC_MOPS_EXPORT,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_IONOSPHERIC_MOPS_EXPORT, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Add *.ems, *.nstb or *.iono_grid files to an import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSMOPSImportAddEMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_EMS,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_EMS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Add *.ems, *.nstb or *.iono_grid files to an import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSMOPSImportAddNSTB(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_NSTB,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_NSTB, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Add *.ems, *.nstb or *.iono_grid files to an import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSMOPSImportAddGRID(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_GRID,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_MOPS_IMPORT_ADD_GRID, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the affected parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGSynchronize(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:SYNC", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference frame.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceFrame/Sets the reference frame.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGReferenceFrame(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   referenceFrame)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:COOR:RFR %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSADGReferenceFrame.entries[referenceFrame].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the format in which the coordinates of the reference location are
/// HIFN  set.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR positionFormat/Sets the format in which the coordinates of the reference location are
/// HIPAR positionFormat/set.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGPositionFormat(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   positionFormat)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:COOR:FORM %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngGPSPositionFormat.entries[positionFormat].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCoordinatesDecimalPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180, 180), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90, 90), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 6, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:COOR:DEC:PZ %.12lG,%.12lG,%.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180, 180), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90, 90), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 6, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:COOR:DEC %.12lG,%.12lG,%.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in degrees, minutes and
/// HIFN  seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR longitudeDeg/Sets the longitude deg.
/// HIPAR longitudeMin/Sets the longitude min.
/// HIPAR longitudeSec/Sets the longitude Seconds.
/// HIPAR longitudeDirection/Sets the longitude direction.
/// HIPAR latitudeDeg/Sets the latitude deg.
/// HIPAR latitudeMin/Sets the latitude min.
/// HIPAR latitudeSec/Sets the latitude Seconds.
/// HIPAR latitudeDirection/Sets the latitude direction.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCoordinatesDMSPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDirection,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDirection,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDirection, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDirection, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:COOR:DMS:PZ %ld,%ld,%.12lG,%s,%ld,%ld,%.12lG,%s,%.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, longitudeDeg, longitudeMin, longitudeSec, RsRFSigGen_rngGPSLongitudeDirection.entries[longitudeDirection].cmdString,
        latitudeDeg, latitudeMin, latitudeSec, RsRFSigGen_rngGPSLatitudeDirection.entries[latitudeDirection].cmdString, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in degrees, minutes and
/// HIFN  seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR longitudeDeg/Sets the longitude deg.
/// HIPAR longitudeMin/Sets the longitude min.
/// HIPAR longitudeSec/Sets the longitude Seconds.
/// HIPAR longitudeDirection/Sets the longitude direction.
/// HIPAR latitudeDeg/Sets the latitude deg.
/// HIPAR latitudeMin/Sets the latitude min.
/// HIPAR latitudeSec/Sets the latitude Seconds.
/// HIPAR latitudeDirection/Sets the latitude direction.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDirection,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDirection,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDirection, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDirection, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:COOR:DMS %ld,%ld,%.12lG,%s,%ld,%ld,%.12lG,%s,%.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, longitudeDeg, longitudeMin, longitudeSec, RsRFSigGen_rngGPSLongitudeDirection.entries[longitudeDirection].cmdString,
        latitudeDeg, latitudeMin, latitudeSec, RsRFSigGen_rngGPSLatitudeDirection.entries[latitudeDirection].cmdString, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Uncertainty Radius, i.e. sets the maximum radius of the area
/// HIFN  within which the two-dimensional location of the UE is bounded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR uncertainlyRadius/Sets the Uncertainty Radius.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGUncertainlyRadius(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   uncertainlyRadius)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, uncertainlyRadius, 0, 1000000),
            4, "Uncertainly Radius");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:LOC:URAD %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, uncertainlyRadius);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the timebase used to enter the time of assistance data
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeBasis/Sets the time basis.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGTimeBasis(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   standard,
                                                             ViInt32   timeBasis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, RSRFSIGGEN_VAL_SATNAV_ADG_TBAS_UTC, RSRFSIGGEN_VAL_SATNAV_ADG_TBAS_BDT),
            4, "Time Basis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:TBAS %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngADGTimeBasis.entries[timeBasis].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enabled for UTC or GLONASS timebase. Enters the date for the
/// HIFN  assistance data in DMS format of the Gregorian calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGDate(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   standard,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:DATE %ld,%ld,%ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enabled for UTC or GLONASS timebase. Enters the date for the
/// HIFN  assistance data in DMS format of the Gregorian calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGTime(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   standard,
                                                        ViInt32   hour,
                                                        ViInt32   minute,
                                                        ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            4, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            5, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999), 6, "Seconds");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:TIME %ld,%ld,%.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, hour, minute, second);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the duration of the assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR duration/Sets the duration of the assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGDuration(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   standard,
                                                            ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, duration, 1.0e-3, 5.0e3), 4, "Duration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:DUR %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, duration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the resolution of the assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR resolution/Sets the resolution of the assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGResolution(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, resolution, 1.0e-3, 5.0), 4, "Resolution");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:RES %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, resolution);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the assistance data is generated
/// HIFN  for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeofWeek/Determines the Time of Week (TOW) the assistance data is generated
/// HIPAR timeofWeek/for.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGTimeofWeek(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViReal64  timeofWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeofWeek, -604800, 604800), 4, "Time of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:TOW %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, timeofWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the Time of Week (TOW) the assistance data is generated
/// HIFN  for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Sets the week number (WN) the assistance data is generated for.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGWeekNumber(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViReal64  weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, weekNumber, 0.0, 9999.0e53), 4, "Week Number");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:TOAD:WNUM %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, weekNumber);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The file extension (*.rs_acq) is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCreateAcquisitionFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_ACQUISITION_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_ACQUISITION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCreateAlmanacFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_ALMANAC_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_ALMANAC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCreateUTCFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_UTC_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_UTC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. Assistance data settings are stored as navigation file
/// HIFN  with the specific file extensions *.rs_nav or into RINEX files with
/// HIFN  extension .10n.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCreateNavigationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_NAVIGATION_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_NAVIGATION_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  acquisition file. The file extension (*.rs_ion) is assigned
/// HIFN  automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGCreateIonosphericFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_IONOSPHERIC_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ADG_CREATE_IONOSPHERIC_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates satellites so that they are included in the generated
/// HIFN  assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR enabled/Activates satellites so that they are included in the generated
/// HIPAR enabled/assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSADGSVIDEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:ADG:%s:SVID%ld:STAT %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svid, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the reference epoch time tb.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR hour/Queries the hour parameter.
/// HIPAR minute/Queries the minute parameter.
/// HIPAR seconds/Queries the seconds parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSOrbitGLONASST(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   svid,
                                                          ViInt32*  hour,
                                                          ViInt32*  minute,
                                                          ViReal64* seconds)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 time = NULL;
    ViInt32   retCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            4, "SVID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:GLON:NMES:NAV:EPH:TOE?", path, svid);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &time, &retCnt));

    *hour = (ViInt32)time[0];
    *minute = (ViInt32)time[1];
    *seconds = time[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (time) free(time);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the sine difference of inclination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR iodc/Sets the issue of data, clock (IODC).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSOrbitIODC(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   svid,
                                                          ViInt32   standard,
                                                          ViInt32   iodc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:IODC %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, iodc);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_BEIDOU:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:DNAV:EPH:IODC %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, iodc);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:IODC %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, iodc);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SV health.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR svHealth/Sets the SV health.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataSVHealth(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   svHealth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:HEAL %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svHealth);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_BEIDOU:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:DNAV:EPH:HEAL %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svHealth);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:HEAL %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svHealth);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GLONASS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:HEAL %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svHealth);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the user range accuracy index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR ura/Sets the user range accuracy index.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataURA(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   ura)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_BEIDOU:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:URA %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, ura);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:URA %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, ura);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:URA %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, ura);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:URA %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, ura);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GLONASS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:URA %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, ura);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SV config.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR asFlag/Sets the SV config.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataAsFlag(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   asFlag)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:SVC %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, asFlag);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:SVC %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, asFlag);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the code on L2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR codeOnL2Mode/Sets the code on L2.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataCodeOnL2Mode(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   codeOnL2Mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:CLTM %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSCodeOnL2Mode.entries[codeOnL2Mode].cmdString);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:CLTM %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSCodeOnL2Mode.entries[codeOnL2Mode].cmdString);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the L2 P data flag.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR l2P/Sets the L2 P data flag.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataL2P(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean l2P)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:LTPD %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, l2P ? "ON" : "OFF");
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:LTPD %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, l2P ? "ON" : "OFF");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the fit interval flag.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fitInterval/Sets the fit interval flag.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataFitInterval(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean fitInterval)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:FIFL %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, fitInterval ? "ON" : "OFF");
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:FIFL %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, fitInterval ? "ON" : "OFF");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the subframe 1 (reserved 1 to 4).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR reservedValue/Selects the Reserved value.
/// HIPAR subframeValue/Sets the subframe 1 (reserved 1 to 4).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAdditionalDataSubframe1(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   reservedValue,
                            ViInt32   subframeValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    switch (standard)
    {
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:LNAV:EPH:SF1R%ld %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, reservedValue, subframeValue);
        break;
    case RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:NMES:NAV:EPH:SF1R%ld %ld", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, reservedValue, subframeValue);
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, standard), 4, "Channel");
        break;
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the time base is UTC, defines the date for the simulation in
/// HIFN  DD.MM.YYYY format of the Gregorian calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionStartDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If the time base is UTC, sets the simulation start time in UTC time
/// HIFN  format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionStartTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0, 59.999), 5, "Second");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:TIME %ld,%ld,%.12lG", path, hour, minute, second);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date at the simulation start time of the selected
/// HIFN  navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Displays the year.
/// HIPAR month/Displays the month.
/// HIPAR day/Displays the day.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionStartUTCDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32*  year,
                            ViInt32*  month,
                            ViInt32*  day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  retCnt = 0;
    ViAInt32 date = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:UTC:DATE?", path);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &date, &retCnt));

    *year = date[0];
    *month = date[1];
    *day = date[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (date) free(date);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the simulation start time of the selected navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Displays the hour.
/// HIPAR minute/Displays the minute.
/// HIPAR seconds/Displays the seconds.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionStartUTCTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32*  hour,
                            ViInt32*  minute,
                            ViReal64* seconds)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 time = NULL;
    ViInt32   retCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:UTC:TIME?", path);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &time, &retCnt));

    *hour = (ViInt32)time[0];
    *minute = (ViInt32)time[1];
    *seconds = time[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (time) free(time);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date at the simulation start time of the selected
/// HIFN  navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Displays the year.
/// HIPAR month/Displays the month.
/// HIPAR day/Displays the day.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionStartGLONASSDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32*  year,
                            ViInt32*  month,
                            ViInt32*  day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  retCnt = 0;
    ViAInt32 date = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:GLON:DATE?", path);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &date, &retCnt));

    *year = date[0];
    *month = date[1];
    *day = date[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (date) free(date);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the simulation start time of the selected navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR hour/Displays the hour.
/// HIPAR minute/Displays the minute.
/// HIPAR seconds/Displays the seconds.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionStartGLONASSTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32*  hour,
                            ViInt32*  minute,
                            ViReal64* seconds)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32   retCnt = 0;
    ViAReal64 time = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:GLON:TIME?", path);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &time, &retCnt));

    *hour = (ViInt32)time[0];
    *minute = (ViInt32)time[1];
    *seconds = time[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (time) free(time);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date for the UTC-UTC(SU) data in DMS format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Displays the year.
/// HIPAR month/Displays the month.
/// HIPAR day/Displays the day.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionStartUTCSDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32*  year,
                            ViInt32*  month,
                            ViInt32*  day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAInt32 date = NULL;
    ViInt32  retCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:UTCS:UTC:DATE?", path);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &date, &retCnt));

    *year = date[0];
    *month = date[1];
    *day = date[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (date) free(date);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date of the next UTC time correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionLeapDate(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:LEAP:DATE %ld,%ld,%ld", path, year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the week number at the simulation start of the selected
/// HIFN  navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR weekNumber/Queries the week number at the simulation start of the selected
/// HIPAR weekNumber/navigation standard.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionWeekNumber(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32*  weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:%s:WNUM?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:WNUM?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the time of week at the simulation start of the selected
/// HIFN  navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR timeOfWeek/Queries the time of week at the simulation start of the selected
/// HIPAR timeOfWeek/navigation standard.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionTimeOfWeek(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32*  timeOfWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:%s:TOW?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:TOW?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, timeOfWeek));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the time offset between the time in the navigation standard
/// HIFN  and UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR offset/Queries the time offset between the time in the navigation standard
/// HIPAR offset/and UTC.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionOffset(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViReal64* offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:%s:OFFS?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:STAR:%s:OFFS?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    }

    checkErr(RsCore_QueryViReal64(instrSession, cmd, offset));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the UTC data reference week number, WNt.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR wnot/Sets the UTC data reference week number, WNt.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionWNot(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   wnot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wnot, 0, 255),
            5, "WNot");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:WNOT %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, wnot);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:WNOT %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, wnot);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the UTC data reference time of week, tot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR tot/Sets the UTC data reference time of week, tot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionTot(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 255),
            5, "Tot");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:TOT %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, tot);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:TOT %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, tot);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the UTC data reference time of week, tot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR tot/Sets the UTC data reference time of week, tot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionTotUnscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 255),
            5, "Tot");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:TOT:UNSC %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, tot);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:TOT:UNSC %ld", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, tot);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the integer offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR integerOffset/Queries the integer offset.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSTimeConversionIntegerOffset(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32*  integerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:IOFF?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:IOFF?", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    }

    checkErr(RsCore_QueryViInt32(instrSession, cmd, integerOffset));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant term of polynomial, A0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a0/Sets the constant term of polynomial, A0.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionA0(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViReal64  a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a0, -2147483648.0, 2147483647.0), 5, "a0");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:AZER %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a0);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:AZER %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, a0);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant term of polynomial, A0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a0/Sets the constant term of polynomial, A0.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionA0Unscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViReal64  a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a0, -2147483648.0, 2147483647.0), 5, "a0");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:AZER:UNSC %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a0);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:AZER:UNSC %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, a0);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant term of polynomial, A1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a1/Sets the constant term of polynomial, A1.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionA1(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViReal64  a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a1, -8388608, 8388607), 5, "a1");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:AONE %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a1);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:AONE %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, a1);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant term of polynomial, A1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a1/Sets the constant term of polynomial, A1.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionA1Unscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViReal64  a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 5),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a1, -8388608, 8388607), 5, "a1");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:%s:UTC:AONE:UNSC %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a1);
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:TIME:CONV:%s:UTC:AONE:UNSC %.12lG", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, a1);
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter WNot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR wnot/Sets the parameter WNot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASWNot(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   wnot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wnot, 0, 529947),
            4, "wnot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:WNOT %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, wnot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter Tot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR tot/Sets the parameter Tot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASTot(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 65535),
            4, "Tot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:TOT %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, tot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter Tot.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR tot/Sets the parameter Tot.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASTotUnscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 65535),
            4, "Tot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:TOT:UNSC %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, tot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter A1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a1/Sets the parameter A1.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASA1(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, a1, -4096, 4096), 4, "A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:AONE %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter A1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a1/Sets the parameter A1.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASA1Unscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, a1, -4096, 4096), 4, "A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:AONE:UNSC %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter A0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a0/Sets the parameter A0.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASA0(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, a0, -32768, 32767), 4, "A0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:AZER %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter A0.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR a0/Sets the parameter A0.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSTimeConversionSBASA0Unscaled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, a0, -32768, 32767), 4, "A0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:SBAS:%s:NMES:NAV:TIME:CONV:UTC:AZER:UNSC %ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, a0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines if satellites from the selected GNSS system are included in
/// HIFN  the simulated satellites constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SatNav SBAS Region.
/// HIPAR enabled/Defines if satellites from the selected GNSS system are included in
/// HIPAR enabled/the simulated satellites constellation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalStandardEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 3),
            4, "Region");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_SBAS)
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SYST:%s:%s %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, enabled ? "ON" : "OFF");
    }
    else
    {
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SYST:%s %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, enabled ? "ON" : "OFF");
    }

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the SV ID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR enabled/Defines if satellites from the selected GNSS system are included in
/// HIPAR enabled/the simulated satellites constellation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalStandardSVEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:STAT %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Includes the SV ID in the currents constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR enabled/Includes the SV ID in the currents constellation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalStandardPresentInConstellation(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRES %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  If enabled, the power of the SV ID signals is reduced to account for
/// HIFN  the free space attenuation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR enabled/If enabled, the power of the SV ID signals is reduced to account for
/// HIPAR enabled/the free space attenuation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalStandardPathLossEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:POW:PLOS %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reduces the signal of the selected SV ID by the defined value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR offset/Reduces the signal of the selected SV ID by the defined value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPowerStandardOffset(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offset, -21, 0), 5, "Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:POW:OFFS %.12lG", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, offset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SV ID to that the configuration form the current satellite
/// HIFN  (SVID<ch>) is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR copySVID/Sets the SV ID to that the configuration form the current satellite
/// HIPAR copySVID/(SVID<ch>) is applied.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPowerCopySVID(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   copySVID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, copySVID, 0, 200),
            5, "Copy SVID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:POW:COPY:SVID %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSMonitorShowSVID.entries[copySVID].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the configuration of the current satellite (SVID<ch>:<GNSS
/// HIFN  system>) to the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPowerCopySVIDExecute(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:POW:COPY:EXEC", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SV ID to that the configuration form the current satellite
/// HIFN  (SVID<ch>) is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR copySVID/Sets the SV ID to that the configuration form the current satellite
/// HIPAR copySVID/(SVID<ch>) is applied.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSModulationControlCopySVID(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid,
                            ViInt32   copySVID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, copySVID, 0, 200),
            5, "Copy SVID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MCON:COPY:SVID %s", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSMonitorShowSVID.entries[copySVID].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the configuration of the current satellite (SVID<ch>:<GNSS
/// HIFN  system>) to the satellite defined with the command
/// HIFN  [:SOURce<hw>]:BB:GNSS:SVID<ch>:GPS:MCONtrol:COPY:SVID.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSModulationControlCopyExecute(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svid)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 6),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MCON:COPY:EXEC", path, svid, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalSBASL1BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    snprintf(repCap_svid, RS_REPCAP_BUF_SIZE, ",SVID%ld", svid);

    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_SBAS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalQZSSL1BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    snprintf(repCap_svid, RS_REPCAP_BUF_SIZE, ",SVID%ld", svid);

    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_QZSS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalQZSSL2BandL2CNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    snprintf(repCap_svid, RS_REPCAP_BUF_SIZE, ",SVID%ld", svid);

    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_QZSS_L2_BAND_L2C_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalQZSSL5BandL5SNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    snprintf(repCap_svid, RS_REPCAP_BUF_SIZE, ",SVID%ld", svid);

    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_QZSS_L5_BAND_L5S_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalNavICL5BandSPSNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_NAVIC_L5_BAND_SPS_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_NAVIC_L5_BAND_SPS_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL1BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL1BandPNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L1_BAND_P_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L1_BAND_P_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL2BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_CA_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL2BandL2CNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_L2C_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_L2C_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL2BandPNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_P_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L2_BAND_P_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGPSL5BandL5SNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L5_BAND_L5S_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GPS_L5_BAND_L5S_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGLONASSL1BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GLONASS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GLONASS_L1_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGLONASSL2BandCANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GLONASS_L2_BAND_CA_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GLONASS_L2_BAND_CA_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGALILEOL1BandE1OSNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L1_BAND_E1OS_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L1_BAND_E1OS_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGALILEOL2BandE6SNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L2_BAND_E6S_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L2_BAND_E6S_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGALILEOL5BandE5ANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L5_BAND_E5A_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L5_BAND_E5A_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalGALILEOL5BandE5BNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L5_BAND_E5B_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_GALILEO_L5_BAND_E5B_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalBeiDouL1BandB1CNavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L1_BAND_B1C_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L1_BAND_B1C_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalBeiDouL1BandB1INavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L1_BAND_B1I_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L1_BAND_B1I_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalBeiDouL2BandB3INavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L2_BAND_B3I_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L2_BAND_B3I_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalBeiDouL5BandB2ANavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L5_BAND_B2A_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L5_BAND_B2A_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR fileName/Sets a file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalBeiDouL5BandB2INavMessageDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svid,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_svid[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L5_BAND_B2I_NAV_MESSAGE_DATA_LIST,
            "SatNavSVID", svid- 1, RS_REPCAP_BUF_SIZE, repCap_svid),
            3, "SVID");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    strcat(repCap, ",");
    strcat(repCap, repCap_svid);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIGNAL_BEIDOU_L5_BAND_B2I_NAV_MESSAGE_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets hardware in loop (HIL) position, motion (velocity, acceleration,
/// HIFN  jerk) and attitude in Earth Fixed Earth Centered ECEF coordinates.
/// HIFN     The yaw/heading, pitch/elevation, roll/bank rotation angles and their
/// HIFN  derivatives are obtained by rotating the body (XYZ) frame. The frame is
/// HIFN  rotated starting from an aligned state with the local NED frame. Three
/// HIFN  consecutive Euler rotations are: around the z axis, the y axis and the
/// HIFN  x axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the array size.
/// HIPAR modeA/Sets hardware in loop (HIL) position, motion (velocity, acceleration,
/// HIPAR modeA/jerk) and attitude in Earth Fixed Earth Centered ECEF coordinates.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSHILModeA(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   arraySize,
                                                         ViReal64  modeA[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:RT:REC:HILP:MODE:A ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, modeA, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets hardware in loop (HIL) position, motion (velocity, acceleration,
/// HIFN  jerk) and attitude in North East Down (NED) coordinates.
/// HIFN     The yaw/heading, pitch/elevation, roll/bank rotation angles and their
/// HIFN  derivatives are obtained by rotating the body (XYZ) frame. The frame is
/// HIFN  rotated starting from an aligned state with the local NED frame. Three
/// HIFN  consecutive Euler rotations are: around the z axis, the y axis and the
/// HIFN  x axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the array size.
/// HIPAR modeB/Sets hardware in loop (HIL) position, motion (velocity, acceleration,
/// HIPAR modeB/jerk) and attitude in North East Down (NED) coordinates.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSHILModeB(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   arraySize,
                                                         ViReal64  modeB[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:RT:REC:HILP:MODE:B ", path);
    checkErr(RsCore_WriteAsciiViReal64Array(instrSession, cmd, modeB, arraySize));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current latency tcal.latency,i and statistics on the
/// HIFN  latency values.
/// HIFN     This command returns also information on the minimum and maximal
/// HIFN  deviation from zero latency and the number of non-zero latency values
/// HIFN  measured since the last time this query was sent.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR lastLatency/Displays the last latency.
/// HIPAR maxLatency/Displays the maximum latency.
/// HIPAR minLatency/Displays the minimum latency.
/// HIPAR noZeroValues/Displays the no zero values.
/// HIPAR totalValues/Displays the total values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSHILLatencyStatistics(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64* lastLatency,
                            ViReal64* maxLatency,
                            ViReal64* minLatency,
                            ViInt32*  noZeroValues,
                            ViInt32*  totalValues)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32   retCnt = 0;
    ViAReal64 values = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:RT:REC:HILP:LAT:STAT?", path);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &values, &retCnt));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

    *lastLatency = values[0];
    *maxLatency = values[1];
    *minLatency = values[2];
    *noZeroValues = (ViInt32)values[3];
    *totalValues = (ViInt32)values[4];

Error:
    if (values) free(values);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSReceiverLocationCoordinatesDecimalPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64* longitude,
                            ViReal64* latitude,
                            ViReal64* altitude)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 results = NULL;
    ViInt32   returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:REC:LOC:COOR:DEC:PZ?", path);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &results, &returnedValues));

    *longitude = results[0];
    *latitude = results[1];
    *altitude = results[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (results)
        free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSReceiverLocationCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64* longitude,
                            ViReal64* latitude,
                            ViReal64* altitude)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 results = NULL;
    ViInt32   returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:REC:LOC:COOR:DEC?", path);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &results, &returnedValues));

    *longitude = results[0];
    *latitude = results[1];
    *altitude = results[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (results)
        free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees,  minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR DMSResults/Displays values of the DMS.
/// HIPAR returnedValues/Displays the number of returned values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSReceiverLocationCoordinatesDMSPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  DMSResults[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:REC:LOC:COOR:DMS:PZ?", path);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, DMSResults, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees,  minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR DMSResults/Displays values of the DMS.
/// HIPAR returnedValues/Displays the number of returned values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSReceiverLocationCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViReal64  DMSResults[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:REC:LOC:COOR:DMS?", path);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, DMSResults, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefined or user-defined waypoint files to simulate a
/// HIFN  moving scenario.
/// HIFN     Waypoint files have the extension *.txt, *.nmea, *.kml or *.xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverLocationWaypointsFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_LOCATION_WAYPOINTS_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_LOCATION_WAYPOINTS_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefined or user-defined vehicle description (*.xvd) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverLocationVehicleFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_LOCATION_VEHICLE_FILE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_LOCATION_VEHICLE_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverEnvironmentVerticalLoadFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_VERTICAL_LOAD_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverEnvironmentRoadsideLoadFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_ROADSIDE_LOAD_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverEnvironmentLandMobileLoadFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_LAND_MOBILE_LOAD_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Copies the multipath configuration of the source GNSS System and SV ID
/// HIFN  to the target SV ID and GNSS system or to all SV IDs from a system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects the path.
/// HIPAR sourceStandard/Sets the source GNSS system.
/// HIPAR sourceSVID/Sets the source SV ID number.
/// HIPAR targetStandard/Sets the target GNSS system.
/// HIPAR targetSVID/Sets the target SV ID number.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSReceiverEnvironmentStaticMultipathCopy(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   sourceStandard,
                            ViInt32   sourceSVID,
                            ViInt32   targetStandard,
                            ViInt32   targetSVID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_STATIC_MULTIPATH_STANDARD, sourceStandard),
            3, "Source Standard");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_STATIC_MULTIPATH_SVID, sourceSVID),
            4, "Source SVID");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_STATIC_MULTIPATH_COPY_TARGET_STANDARD, targetStandard),
            5, "Target Standard");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_STATIC_MULTIPATH_COPY_TARGET_SVID, targetSVID),
            6, "Target SVID");
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_RECEIVER_ENVIRONMENT_STATIC_MULTIPATH_COPY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ant_pat/*.body_mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSAntennaFile(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   antenna,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,SA%ld", pathArr[path], antenna);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_ANTENNA_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ant_pat/*.body_mask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSBodyFile(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   antenna,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,SA%ld", pathArr[path], antenna);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_BODY_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
/// HIPAR weekNumber/Sets the week number.
/// HIPAR timeofWeek/Sets the time of week.
/// HIPAR longitude/Displays the longitude.
/// HIPAR latitude/Displays the latitude.
/// HIPAR altitude/Displays the altitude.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   timeBasis,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second,
                            ViInt32   weekNumber,
                            ViReal64  timeOfWeek,
                            ViReal64* longitude,
                            ViReal64* latitude,
                            ViReal64* altitude)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 values = NULL;
    ViInt32   outArraySize = 0;
    ViPChar   p2timeBasis;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        3, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            10, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        11, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:GNSS:RT:REC:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG", path, p2timeBasis, year, month, day, hour, minute, second,
        weekNumber, timeOfWeek);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &values, &outArraySize));

    *longitude = values[0];
    *latitude = values[1];
    *altitude = values[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (values) free(values);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
/// HIPAR weekNumber/Sets the week number.
/// HIPAR timeofWeek/Sets the time of week.
/// HIPAR longitude/Displays the longitude values.
/// HIPAR longitudeDirection/Displays the longitude direction.
/// HIPAR latitude/Displays the latitude values.
/// HIPAR latitudeDirection/Displays the latitude direction.
/// HIPAR altitude/Displays the altitude.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTCoordinatesDMS(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   timeBasis,
                                                             ViInt32   year,
                                                             ViInt32   month,
                                                             ViInt32   day,
                                                             ViInt32   hour,
                                                             ViInt32   minute,
                                                             ViReal64  second,
                                                             ViInt32   weekNumber,
                                                             ViReal64  timeOfWeek,
                                                             ViReal64  longitude[],
                                                             ViInt32*  longitudeDirection,
                                                             ViReal64  latitude[],
                                                             ViInt32*  latitudeDirection,
                                                             ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2timeBasis;
    ViPChar  response = NULL;
    ViPChar  p2tok;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        3, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            10, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        11, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:GNSS:RT:REC:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG", path, p2timeBasis, year, month, day, hour, minute, second,
        weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

    p2tok = strtok(response, ",");
    longitude[0] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    longitude[1] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    longitude[2] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsRFSigGen_rngGPSLongitudeDirection, p2tok, longitudeDirection));

    p2tok = strtok(NULL, ",");
    latitude[0] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    latitude[1] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    latitude[2] = RsCore_Convert_String_To_ViReal64(p2tok);

    p2tok = strtok(NULL, ",");
    checkErr(RsCore_FindRangeTableDiscreteNumericValue(instrSession, &RsRFSigGen_rngGPSLatitudeDirection, p2tok, latitudeDirection));

    p2tok = strtok(NULL, ",");
    *altitude = RsCore_Convert_String_To_ViReal64(p2tok);

Error:
    if (response)
        free(response);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the velocity of the receiver location for the selected moment
/// HIFN  of time.
/// HIFN     The required query parameters depend on the selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeBasis/Sets the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
/// HIPAR weekNumber/Sets the week number.
/// HIPAR timeofWeek/Sets the time of week.
/// HIPAR velocity/Displays the velocity.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTReceiverVelocity(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   timeBasis,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second,
                            ViInt32   weekNumber,
                            ViReal64  timeofWeek,
                            ViReal64* velocity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2timeBasis;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        3, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            10, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeofWeek, 0, 604799.999),
        11, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:GNSS:RT:REC:RVEL? %s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG", path, p2timeBasis, year, month, day, hour, minute, second,
        weekNumber, timeofWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, velocity));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the receiver attitude parameters yaw, pitch, roll in degrees
/// HIFN  for the selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR timeBasis/Selects the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the seconds.
/// HIPAR weekNumber/Sets the week number. Required for GPS, GST or BDT time basis.
/// HIPAR timeOfWeek/Sets the time of week. Required for GPS, GST or BDT time basis.
/// HIPAR pitch/Displays the pitch.
/// HIPAR roll/Displays the roll.
/// HIPAR yaw/Displays the yaw.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTReceiverAttitude(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   timeBasis,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minutes,
                            ViReal64  seconds,
                            ViInt32   weekNumber,
                            ViReal64  timeOfWeek,
                            ViReal64* yaw,
                            ViReal64* pitch,
                            ViReal64* roll)
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   p2timeBasis;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar    qparam[RS_MAX_MESSAGE_LEN] = "";
    ViPReal64 results = NULL;
    ViInt32   resultCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        3, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999),
        9, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            10, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        11, "Time Of Week");

    snprintf(qparam, RS_MAX_MESSAGE_LEN, "%s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG",
            p2timeBasis, year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);

    checkErr(RsCore_GetAttributeViStringAsViReal64Array(instrSession, repCap,
            RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RT_RECEIVER_ATTITUDE,
            qparam, &results, &resultCount));

    checkErr((resultCount == 3) ? error : RS_ERROR_UNEXPECTED_RESPONSE);
    *yaw = results[0];
    *pitch = results[1];
    *roll = results[2];

Error:
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the velocity and the coordinates of the receiver location in
/// HIFN  decimal format for the selected moment of time.
/// HIFN     The required parameters depend on the selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR timeBasis/Selects the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the seconds.
/// HIPAR weekNumber/Sets the week number. Required for GPS, GST or BDT time basis.
/// HIPAR timeOfWeek/Sets the time of week. Required for GPS, GST or BDT time basis.
/// HIPAR longitude/Displays the longitude.
/// HIPAR latitude/Displays the latitude.
/// HIPAR altitude/Displays the altitude.
/// HIPAR velocity/Displays the altitude.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTReceiverPositionAndVelocity(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   timeBasis,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minutes,
                            ViReal64  seconds,
                            ViInt32   weekNumber,
                            ViReal64  timeOfWeek,
                            ViReal64* longitude,
                            ViReal64* latitude,
                            ViReal64* altitude,
                            ViReal64* velocity)
{
    ViStatus  error = VI_SUCCESS;
    ViPChar   p2timeBasis;
    ViChar    repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar    qparam[RS_MAX_MESSAGE_LEN] = "";
    ViPReal64 results = NULL;
    ViInt32   resultCount = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        3, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999),
        9, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            10, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        11, "Time Of Week");

    snprintf(qparam, RS_MAX_MESSAGE_LEN, "%s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG",
            p2timeBasis, year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);

    checkErr(RsCore_GetAttributeViStringAsViReal64Array(instrSession, repCap,
            RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RECEIVER_POSITION_AND_VELOCITY,
            qparam, &results, &resultCount));

    checkErr((resultCount == 4) ? error : RS_ERROR_UNEXPECTED_RESPONSE);
    *longitude = results[0];
    *latitude = results[1];
    *altitude = results[2];
    *velocity = results[3];

Error:
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR timeBasis/Sets the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
/// HIPAR weekNumber/Sets the week number.
/// HIPAR timeofWeek/Sets the time of week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time. The
/// HIPAR azimuth/required query parameters, depend on the selected timebase.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTAzimuth(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   standard,
                                                      ViInt32   svid,
                                                      ViInt32   timeBasis,
                                                      ViInt32   year,
                                                      ViInt32   month,
                                                      ViInt32   day,
                                                      ViInt32   hour,
                                                      ViInt32   minute,
                                                      ViReal64  second,
                                                      ViInt32   weekNumber,
                                                      ViReal64  timeOfWeek,
                                                      ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;
    ViPChar  p2timeBasis;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        4, "Time Basis");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                5, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                5, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            10, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:GNSS:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG", path, p2standard, svid, p2timeBasis,
        year, month, day, hour, minute, second, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected timebase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the SVID parameter.
/// HIPAR timeBasis/Sets the time basis.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minutes.
/// HIPAR second/Sets the seconds parameter.
/// HIPAR weekNumber/Sets the week number.
/// HIPAR timeofWeek/Sets the time of week.
/// HIPAR elevation/Queries the satellite elevation in the selected moment of time. The
/// HIPAR elevation/required query parameters, depend on the selected timebase.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSRTElevation(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   standard,
                                                        ViInt32   svid,
                                                        ViInt32   timeBasis,
                                                        ViInt32   year,
                                                        ViInt32   month,
                                                        ViInt32   day,
                                                        ViInt32   hour,
                                                        ViInt32   minute,
                                                        ViReal64  second,
                                                        ViInt32   weekNumber,
                                                        ViReal64  timeOfWeek,
                                                        ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;
    ViPChar  p2timeBasis;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngADGTimeBasis, timeBasis, &p2timeBasis),
        4, "Time Basis");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 193, 197),
                5, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
                5, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            10, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0, 604799.999),
        13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:GNSS:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%.12lG,%ld,%.12lG", path, p2standard, svid, p2timeBasis,
        year, month, day, hour, minute, second, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the RTK base
/// HIFN  station in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSRTKBaseLocationDecimalPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   base,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBase[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 6, "Altitude");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_PZ,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");
     viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_PZ,
            "Base", base - 1, RS_REPCAP_BUF_SIZE, repCapBase),
            3, "Base");
    strcat (repCap, ",");
    strcat (repCap, repCapBase);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG", longitude, latitude, altitude);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_PZ, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the RTK base
/// HIFN  station in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSRTKBaseLocationDecimalWGS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   base,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBase[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10.0e3, 50.0e6), 6, "Altitude");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_WGS,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");
     viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_WGS,
            "Base", base - 1, RS_REPCAP_BUF_SIZE, repCapBase),
            3, "Base");
    strcat (repCap, ",");
    strcat (repCap, repCapBase);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%.12lG,%.12lG,%.12lG", longitude, latitude, altitude);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DECIMAL_WGS, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the coordinates of the geographic location of the RTK base
/// HIFN  station in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSRTKBaseLocationDMSPZ(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   base,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBase[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_PZ,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");
     viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_PZ,
            "Base", base - 1, RS_REPCAP_BUF_SIZE, repCapBase),
            3, "Base");
    strcat (repCap, ",");
    strcat (repCap, repCapBase);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld,%.12lG,%s,%ld,%ld,%.12lG,%s,%.12lG", longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_PZ, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the RTK base
/// HIFN  station in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSRTKBaseLocationDMSWGS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   base,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapBase[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_PZ,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");
     viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_PZ,
            "Base", base - 1, RS_REPCAP_BUF_SIZE, repCapBase),
            3, "Base");
    strcat (repCap, ",");
    strcat (repCap, repCapBase);

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "%ld,%ld,%.12lG,%s,%ld,%ld,%.12lG,%s,%.12lG", longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_RTK_BASE_LOCATION_DMS_WGS, cmd));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Applies the configuration of the current satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
ViStatus _VI_FUNC rsrfsiggen_ApplySatNavGNSSPseudorange(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   standard,
                                                        ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:COPY:EXEC", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SV ID to that the configuration form the current satellite
/// HIFN  (SVID<ch>) is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR svid/Sets the SV ID.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeSVID(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   svids)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:COPY:SVID %ld", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, svids);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets how the pseudorange errors are defined.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR mode/Sets how the pseudorange errors are defined.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeMode(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, RSRFSIGGEN_VAL_PSEUDORANGE_MODE_FSBAS, RSRFSIGGEN_VAL_PSEUDORANGE_MODE_PROFILE),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:MODE %s", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSPseudorangeMode.entries[mode].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant pseudorange error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorangeValue/Sets the constant pseudorange error.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeValue(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  pseudorangeValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:VAL %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorangeValue);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads a pseudorange error file with extension *.rs_perr from the
/// HIFN  default directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR filename/Sets the file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeErrorsFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViString  filename)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:FILE '%s'", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, filename);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Appends a row in the profile table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
ViStatus _VI_FUNC rsrfsiggen_AppendSatNavGNSSPseudorangeProfileTableRow(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF:APP", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the number of entry in the profile table.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR entries/Displays the number of entry in the profile table.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSPseudorangeProfileTableEntries(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32*  entries)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF:COUN?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, entries));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a start time offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR offset/Adds a start time offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeProfileStartTimeOffset(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF:OFFS %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, offset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Deletes the selected pseudorange error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorange/Sets the pseudorange.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeProfileDelete(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   pseudorange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pseudorange, 1, 10),
            5, "Pseudorange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF%ld:DEL", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorange);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Inserts a row befor the selected pseudorange error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorange/Sets the pseudorange.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeProfileInsert(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   pseudorange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pseudorange, 1, 10),
            5, "Pseudorange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF%ld:INS", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorange);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference time for the pseudorange error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorange/Sets the pseudorange.
/// HIPAR time/Sets the reference time for the pseudorange error.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeProfileReferenceTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   pseudorange,
                            ViReal64  time)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pseudorange, 1, 10),
            5, "Pseudorange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF%ld:REF %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorange, time);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the pseudorange error value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorange/Sets the pseudorange.
/// HIPAR profileValue/Sets the pseudorange error value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSPseudorangeProfileValue(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   pseudorange,
                            ViReal64  profileValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, pseudorange, 1, 10),
            5, "Pseudorange");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, profileValue, -1000, 1000),
        6, "Profile Value");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:PRER:PROF%ld:VAL %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorange, profileValue);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects between the predefined velocity profiles or a user-defined
/// HIFN  one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR configuration/Selects between the predefined velocity profiles or a user-defined
/// HIPAR configuration/one.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsPredefinedConfiguration(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   configuration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, configuration, RSRFSIGGEN_VAL_GNSS_SIGNAL_DYNAMICS_CONFIG_USER, RSRFSIGGEN_VAL_GNSS_SIGNAL_DYNAMICS_CONFIG_VEL2),
            5, "Configuration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:CONF %s", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSSignalDynamicsPredefinedConfig.entries[configuration].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the dynamics profile type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR profile/Selects the dynamics profile type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsVelocityProfile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   profile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, RSRFSIGGEN_VAL_GNSS_VELOCITY_PROFILE_CONSTANT, RSRFSIGGEN_VAL_GNSS_VELOCITY_PROFILE_HIGH),
            5, "Profile");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:PROF %s", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, RsRFSigGen_rngSatNavGNSSSignalDynamicsVelocityProfile.entries[profile].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates the initial velocity, used at the beginning of the profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR velocity/Indicates the initial velocity, used at the beginning of the profile.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsInitialVelocity(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  velocity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, velocity, -19042, 19042),
        5, "Velocity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:IVEL %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, velocity);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant velocity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR velocity/Sets the constant velocity.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsVelocity(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  velocity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, velocity, -100e-3, 100e3),
        5, "Velocity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:VEL %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, velocity);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time duration during that acceleration is applied and thus
/// HIFN  the velocity varies.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR acceleration/Sets the time duration during that acceleration is applied and thus
/// HIPAR acceleration/the velocity varies.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsConstantAccelerationPeriod(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  acceleration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, acceleration, 0.1, 10800.0),
        5, "Acceleration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:CAP %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, acceleration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time period during that the velocity is kept constant.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR velocity/Sets the time period during that the velocity is kept constant.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsConstantVelocityPeriod(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  velocity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, velocity, 0.1, 10800.0),
        5, "Velocity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:CVP %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, velocity);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum acceleration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR maxAcceleration/Sets the maximum acceleration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsMaxAcceleration(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  maxAcceleration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, maxAcceleration, 0.01, 1000.0),
        5, "Max Acceleration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:ACCEL:MAX %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, maxAcceleration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum jerk.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR maxJerk/Sets the maximum jerk.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsMaxJerk(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  maxJerk)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, maxJerk, 0.1, 7e4),
        5, "Max Jerk");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:JERK:MAX %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, maxJerk);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the duration of the profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR repetitionPeriod/Queries the duration of the profile.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSSignalDynamicsRepetitionPeriod(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64* repetitionPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:RPER?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, repetitionPeriod));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the initial carrier phase.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR carrierPhase/Sets the initial carrier phase.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsInitialCarrierPhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  carrierPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, carrierPhase, 0.0, 6.28),
        5, "Carrier Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:CPH %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, carrierPhase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the initial pseudorange.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR pseudorange/Sets the initial pseudorange.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsInitialPseudorange(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  pseudorange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, pseudorange, 0, 119900000),
            5, "Pseudorange");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:PRAN %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, pseudorange);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets a time delay before the profile is applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR timeOffset/Sets a time delay before the profile is applied.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSignalDynamicsTimeOffset(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOffset, 0, 9e4),
            5, "Time Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SDYN:TOFF %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, timeOffset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference week.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR referenceWeek/Sets the reference week.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitReferenceWeek(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   referenceWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceWeek, 0, 10000),
            5, "Reference Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:WNOE %ld", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, referenceWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference time of week.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR referenceTimeofWeek/Sets the reference time of week.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitReferenceTimeofWeek(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  referenceTimeofWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, referenceTimeofWeek, 0, 982980),
            5, "Reference Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:TOE %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, referenceTimeofWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the square root of semi-major axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR sqrtA/Sets the square root of semi-major axis.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitSqrtA(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  sqrtA)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sqrtA, 0.190735, 8192),
        5, "Sqrt A");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:SQRA %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, sqrtA);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the eccentricity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR eccentricity/Sets the eccentricity.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitEccentricity(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  eccentricity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, eccentricity, 0, 4294967295),
            5, "Eccentricity");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:ECC %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, eccentricity);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets inclination angle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR i0/Sets inclination angle.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitInclinationAngle(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  i0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, i0, -1.000, 1.000),
        5, "i0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:IZER %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, i0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the longitude of ascending node.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR omega0/Sets the longitude of ascending node.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitLongitudeOfAscendingNode(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  omega0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, omega0, -1.000, 1.000),
        5, "Omega0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:OZER %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, omega0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the argument of perigee.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR omega/Sets the argument of perigee.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitArgumentOfPerigee(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  omega)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, omega, -1.000, 1.000),
        5, "Omega");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:OMEG %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, omega);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the mean anomaly.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR m0/Sets the mean anomaly.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitMeanAnomaly(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  m0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, m0, -1.000, 1.000),
        5, "M0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:MZER %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, m0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the rate of inclination angle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR idot/Sets the rate of inclination angle.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitRateOfInclinationAngle(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  idot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, idot, -9313e-15, 9312e-15),
        5, "iDot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:IDOT %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, idot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the rate of right ascension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR omegadot/Sets the rate of right ascension.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitRateOfRightAscension(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  omegadot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, omegadot, -0.0000009536743, 0.0000009536742),
        5, "omegaDot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:ODOT %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, omegadot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the mean motion difference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR deltan/Sets the mean motion difference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitMeanMotionDifference(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  deltan)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, deltan, -0.0000000037253, 0.0000000037252),
        5, "delta n");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:NDEL %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, deltan);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference date and time for PZ-90 coordinate reference.
/// HIFN     The orbital parameters can be set with
/// HIFN  rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitGLONASSPZ90Parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/The path for which is this function is performed.
/// HIPAR svID/Sets the SV ID.
/// HIPAR year/Year. Range 1996 to 9999
/// HIPAR month/Month. Range 1 to 12
/// HIPAR day/Day. Range 1 to 31
/// HIPAR hour/Hour. Range 0 to 23
/// HIPAR minute/Minute. Range 0 to 59
/// HIPAR second/Second. Range 0 to 59.999, Increment 0.001
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitGLONASSPZ90ReferenceDateTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1996, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "second");

    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", year, month, day);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_REFERENCE_DATE, param));
    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%.12lG", hour, minute, second);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_REFERENCE_TIME, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the simulated orbit parameters for PZ-90 coordinate reference.
/// HIFN     The reference time can be set with
/// HIFN  rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitGLONASSPZ90ReferenceDateTime.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR svID/Sets the SV ID.
/// HIPAR xn/Sets the XG coordinate.
/// HIPAR yn/Sets the YG coordinate.
/// HIPAR zn/Sets the ZG coordinate.
/// HIPAR xndot/Sets the velocity component X'G.
/// HIPAR yndot/Sets the velocity component Z'G.
/// HIPAR zndot/Sets the velocity component Z'G.
/// HIPAR xndotdot/Sets the acceleration component X''G.
/// HIPAR yndotdot/Sets the acceleration component Y''G.
/// HIPAR zndotdot/Sets the acceleration component Z''G.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitGLONASSPZ90Parameters(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViReal64  xn,
                            ViReal64  yn,
                            ViReal64  zn,
                            ViReal64  xndot,
                            ViReal64  yndot,
                            ViReal64  zndot,
                            ViReal64  xndotdot,
                            ViReal64  yndotdot,
                            ViReal64  zndotdot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_XN, xn),
            4, "Xn");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_YN, yn),
            5, "Yn");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_ZN, zn),
            6, "Zn");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_XN_DOT, xndot),
            7, "XnDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_YN_DOT, yndot),
            8, "YnDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_ZN_DOT, zndot),
            9, "ZnDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_XN_DOT_DOT, xndotdot),
            10, "XnDotDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_YN_DOT_DOT, yndotdot),
            11, "YnDotDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SIMULATED_ORBIT_GLONASS_ZN_DOT_DOT, zndotdot),
            12, "ZnDotDot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the cosine difference of latitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR cuc/Sets the cosine difference of latitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitCosineDifferenceOfLatitude(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  cuc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CUC %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, cuc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the sine difference of latitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR cus/Sets the sine difference of latitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitSineDifferenceOfLatitude(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  cus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CUS %.12lG", path, svID, p2standard, cus);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the cosine difference of orbital radius.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR crc/Sets the cosine difference of orbital radius.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitCosineDifferenceOfOrbitalRadius(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  crc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, crc, -1024, 1024),
        5, "crc");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CRC %.12lG", path, svID, p2standard, crc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the sine difference of orbital radius.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR crs/Sets the sine difference of orbital radius.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitSineDifferenceOfOrbitalRadius(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  crs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, crs, -1024, 1024),
        5, "crs");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CRS %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, crs);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the cosine difference of inclination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR cic/Sets the cosine difference of inclination.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitCosineDifferenceOfInclination(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  cic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CIC %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, cic);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the sine difference of inclination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR cis/Sets the sine difference of inclination.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedOrbitSineDifferenceOfInclination(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  cis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:ORB:CIS %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, cis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference week.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR referenceWeek/Sets the reference week.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedClockReferenceWeek(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   referenceWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceWeek, 0, 10000),
            5, "Reference Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:CLOC:WNOC %ld", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, referenceWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference time of week.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR toc/Sets the reference time of week.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedClockTimeOfClock(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  toc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, toc, 0, 982980),
            5, "toc");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:CLOC:TOC %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, toc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the group delay.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR tgd/Sets the group delay.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedClockGroupDelay(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  tgd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tgd, -0.0000001192, 0.0000001190),
        5, "tgd");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:CLOC:TGD %.12lG", path, svID, p2standard, tgd);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the values of af0, af1 or - af2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR af/Sets the AF.
/// HIPAR afValue/Sets the values of af0, af1 or - af2.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedClockAF(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   af,
                            ViReal64  afValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, af, 1, 2),
            5, "AF");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, afValue, -0.063, 0.062),
        6, "AF Value");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:SIM:CLOC:TOC:AF%ld %.12lG", path, svID, p2standard, af, afValue);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of valid satellites (SV IDs) of the selected almanac
/// HIFN  file for the navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR standard2/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR validSatellites/Queries the list of valid satellites (SV IDs) of the selected almanac
/// HIPAR validSatellites/file for the navigation standard.
ViStatus _VI_FUNC rsrfsiggen_SatNavGNSSValidSatellitesGPS(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   standard2,
                                                          ViInt32   svID,
                                                          ViChar    validSatellites[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard2, 0, 4),
            3, "Standard 2");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:%s:LIST?", satNavStandardArr[standard], svID, satNavStandardArr[standard2]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, validSatellites, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries if the selected SV ID is visible in the satellite
/// HIFN  constellation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR visible/Queries if the selected SV ID is visible in the satellite
/// HIPAR visible/constellation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSVisibleInTheConstellation(
                            ViSession  instrSession,
                            ViInt32    path,
                            ViInt32    standard,
                            ViInt32    svID,
                            ViBoolean* visible)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:VIS:STAT?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, visible));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates if the selected SV ID is healthy or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID.
/// HIPAR healthy/Indicates if the selected SV ID is healthy or not.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSSVIDHealthy(ViSession  instrSession,
                                                        ViInt32    path,
                                                        ViInt32    standard,
                                                        ViInt32    svID,
                                                        ViBoolean* healthy)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:HEAL?", path, svID, p2standard);
    checkErr(RsCore_QueryViBoolean(instrSession, cmd, healthy));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the list of all available SVIDs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Sets the Array Size.
/// HIPAR list/Queries the list of all available SVIDs.
/// HIPAR returnedValues/Displays the number of returned values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavGNSSSVIDListAll(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   standard,
                                                        ViInt32   arraySize,
                                                        ViInt32   list[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViPChar  p2standard;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2standard),
        3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID:%s:LIST:ALL?", path, p2standard);
    checkErr(RsCore_QueryIntegerArrayToUserBuffer(instrSession, cmd, arraySize, list, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the simulated clock date and time for PZ-90 coordinate reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects the path.
/// HIPAR svID/Selects the SV ID.
/// HIPAR year/Year. Range 1996 to 9999
/// HIPAR month/Month. Range 1 to 12
/// HIPAR day/Day. Range 1 to 31
/// HIPAR hour/Hour. Range 0 to 23
/// HIPAR minute/Minute. Range 0 to 59
/// HIPAR second/Second. Range 0 to 59.999, Increment 0.001
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSimulatedClockPZ90ReferenceDateTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1996, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "second");

    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", year, month, day);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_GLONASS_SIMULATED_CLOCK_DATE, param));
    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%.12lG", hour, minute, second);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_GLONASS_SIMULATED_CLOCK_TIME, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the reference date and time for EGNOS, WAAS, MSAS satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects the path.
/// HIPAR svID/Selects the SV ID.
/// HIPAR year/Year. Range 1996 to 9999
/// HIPAR month/Month. Range 1 to 12
/// HIPAR day/Day. Range 1 to 31
/// HIPAR hour/Hour. Range 0 to 23
/// HIPAR minute/Minute. Range 0 to 59
/// HIPAR second/Second. Range 0 to 59.999, Increment 0.001
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSBASSimulatedOrbitReferenceDateTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1996, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "second");

    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", year, month, day);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_REFERENCE_DATE, param));
    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%.12lG", hour, minute, second);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_REFERENCE_TIME, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the simulated clock date and time for EGNOS, WAAS, MSAS
/// HIFN  satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects the path.
/// HIPAR svID/Selects the SV ID.
/// HIPAR year/Year. Range 1996 to 9999
/// HIPAR month/Month. Range 1 to 12
/// HIPAR day/Day. Range 1 to 31
/// HIPAR hour/Hour. Range 0 to 23
/// HIPAR minute/Minute. Range 0 to 59
/// HIPAR second/Second. Range 0 to 59.999, Increment 0.001
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSBASSimulatedClockReferenceDateTime(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViInt32   year,
                            ViInt32   month,
                            ViInt32   day,
                            ViInt32   hour,
                            ViInt32   minute,
                            ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_MAX_MESSAGE_LEN] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1996, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            7, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            8, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.0, 59.999),
        9, "second");

    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%ld", year, month, day);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SBAS_SIMULATED_CLOCK_DATE, param));
    snprintf(param, RS_MAX_MESSAGE_LEN, "%ld,%ld,%.12lG", hour, minute, second);
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_SBAS_SIMULATED_CLOCK_TIME, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the simulated orbit parameters of the EGNOS, WAAS, MSAS
/// HIFN  satellites.
/// HIFN     The reference time can be set with
/// HIFN  rsrfsiggen_ConfigureSatNavGNSSSBASSimulatedClockReferenceDateTime.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects the path.
/// HIPAR svID/Selects the SV ID.
/// HIPAR xg/Sets the XG coordinate.
/// HIPAR yg/Sets the YG coordinate.
/// HIPAR zg/Sets the ZG coordinate.
/// HIPAR xgdot/Sets the velocity component X'G.
/// HIPAR ygdot/Sets the velocity component Z'G.
/// HIPAR zgdot/Sets the velocity component Z'G.
/// HIPAR xgdotdot/Sets the acceleration component X''G.
/// HIPAR ygdotdot/Sets the acceleration component Y''G.
/// HIPAR zgdotdot/Sets the acceleration component Z''G.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavGNSSSBASSimulatedOrbitParameters(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   svID,
                            ViReal64  xg,
                            ViReal64  yg,
                            ViReal64  zg,
                            ViReal64  xgdot,
                            ViReal64  ygdot,
                            ViReal64  zgdot,
                            ViReal64  xgdotdot,
                            ViReal64  ygdotdot,
                            ViReal64  zgdotdot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_satNavSVIDRepCap(instrSession, svID, repCap),
        3, "SV ID");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_XG, xg),
            4, "Xg");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_YG, yg),
            5, "Yg");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_ZG, zg),
            6, "Zg");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_XG_DOT, xgdot),
            7, "XgDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_YG_DOT, ygdot),
            8, "YgDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_ZG_DOT, zgdot),
            9, "ZgDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_XG_DOT_DOT, xgdotdot),
            10, "XgDotDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_YG_DOT_DOT, ygdotdot),
            11, "YgDotDot");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_SATNAV_GNSS_SBAS_SIMULATED_ORBIT_ZG_DOT_DOT, zgdotdot),
            12, "ZgDotDot");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the current status of signal generation for all trigger modes
/// HIFN  with Satellite Navigation modulation on.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR triggerStatus/Queries the current status of signal generation for all trigger modes
/// HIPAR triggerStatus/with Satellite Navigation modulation on.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerStatus(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViChar    triggerStatus[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:RMOD?", RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_QueryViString(instrSession, cmd, triggerStatus));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops signal generation for trigger modes Armed_Auto and
/// HIFN  Armed_Retrigger. A subsequent internal or external trigger event
/// HIFN  restarts signal generation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerArm(ViSession instrSession,
                                                       ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:ARM:EXEC", RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Executes a trigger. The internal trigger source must be selected using
/// HIFN  the command <subsystem>:TRIG:SOUR INT and a trigger mode other than
/// HIFN  AUTO must be selected using the command <subsystem>:TRIG:SEQ.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerExecute(ViSession instrSession,
                                                           ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:EXEC\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR triggerMode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerMode(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   triggerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerMode, 0, 4),
            3, "Trigger Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SEQ %s\n", satNavStandardArr[standard], triggerModeArr[triggerMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR triggerSource/Selects the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerSource(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   triggerSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, triggerSource, 0, 3),
            3, "Trigger Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:SOUR %s\n", satNavStandardArr[standard], satNavTriggerSourceArr[triggerSource]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the state of the signal synchronous to the external trigger
/// HIFN  event.
/// HIFN
/// HIFN     For or two or more R&S SMBVs configured to work in a master-slave mode
/// HIFN  for synchronous signal generation, configure this parameter depending
/// HIFN  on the provided system trigger event and the properties of the output
/// HIFN  signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR output/Sets the state of the signal synchronous to the external trigger
/// HIPAR output/event.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSynchronizationOutputtoExternalTrigger(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean output)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:EXT:SYNC:OUTP %s\n", satNavStandardArr[standard], (output == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the length of the signal sequence to be output in the Single
/// HIFN  trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR signalDuration/Defines the length of the signal sequence to be output in the Single
/// HIPAR signalDuration/trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerSignalDuration(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   signalDuration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:SLEN %ld\n", satNavStandardArr[standard], signalDuration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the
///HINF  RSRFSIGGEN_ATTR_SATNAV_GNSS_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTriggerDateTime(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   year,
                                                            ViInt32   month,
                                                            ViInt32   day,
                                                            ViInt32   hour,
                                                            ViInt32   minute,
                                                            ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_SATNAV_GNSS_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the trigger delay (expressed as a number of chips) for
/// HIFN  external triggering.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR externalTrigger/Sets the External Trigger.
/// HIPAR delay/Specifies the trigger delay (expressed as a number of chips) for
/// HIPAR delay/external triggering.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavExternalTriggerDelay(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   externalTrigger,
                            ViInt32   delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, externalTrigger, 1, 2),
            3, "External Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:EXT%ld:DEL %ld\n", satNavStandardArr[standard], externalTrigger, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the number of chips by which a restart is to be inhibited
/// HIFN  following a trigger event.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR externalTrigger/Sets the External Trigger.
/// HIPAR inhibit/Specifies the number of chips by which a restart is to be inhibited
/// HIPAR inhibit/following a trigger event.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavExternalTriggerInhibit(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   externalTrigger,
                            ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, externalTrigger, 1, 2),
            3, "External Trigger");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:EXT%ld:INH %ld\n", satNavStandardArr[standard], externalTrigger, inhibit);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the signal for the selected marker output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR mode/Defines the signal for the selected marker output.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerMode(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32   marker,
                                                       ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:MODE %s\n", satNavStandardArr[standard], marker, satNavMarkerModeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the divider for the pulsed marker signal in the setting
/// HIFN  SOURce:BB:GALileo:TRIGger:OUTPut:MODE PULSe. The pulse frequency is
/// HIFN  derived by dividing the symbol rate by the divider.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR pulseDivider/Sets the divider for the pulsed marker signal in the setting
/// HIPAR pulseDivider/SOURce:BB:GALileo:TRIGger:OUTPut:MODE PULSe. The pulse frequency is
/// HIPAR pulseDivider/derived by dividing the symbol rate by the divider.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerPulseDivider(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   marker,
                            ViInt32   pulseDivider)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:PULS:DIV %ld\n", satNavStandardArr[standard], marker, pulseDivider);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pulse frequency of the pulsed marker signal in the setting
/// HIFN  SOUR:BB:GAL:TRIG:OUTP:MODE PULS. The pulse frequency  is derived by
/// HIFN  dividing the symbol rate by the divider. The divider is defined with
/// HIFN  command :SOUR:BB:GALileo:TRIG:OUTP:PULS:DIV.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR pulseFrequency/Queries the pulse frequency of the pulsed marker signal in the setting
/// HIPAR pulseFrequency/SOUR:BB:GAL:TRIG:OUTP:MODE PULS. The pulse frequency  is derived by
/// HIPAR pulseFrequency/dividing the symbol rate by the divider. The divider is defined with
/// HIPAR pulseFrequency/command :SOUR:BB:GALileo:TRIG:OUTP:PULS:DIV.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMarkerPulseFrequency(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   marker,
                                                             ViReal64* pulseFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:PULS:FREQ?", satNavStandardArr[standard], marker);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, pulseFrequency));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of symbols in a period (ON time + OFF time) during
/// HIFN  which the marker signal in setting
/// HIFN  SOURce:BB:GALileo:TRIGger:OUTPut:MODE RATio on the marker outputs is
/// HIFN  OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR offTime/Sets the number of symbols in a period (ON time + OFF time) during
/// HIPAR offTime/which the marker signal in setting
/// HIPAR offTime/SOURce:BB:GALileo:TRIGger:OUTPut:MODE RATio on the marker outputs is
/// HIPAR offTime/OFF.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerOffTime(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   marker,
                                                          ViInt32   offTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:OFFT %ld\n", satNavStandardArr[standard], marker, offTime);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of symbols in a period (ON time + OFF time) during
/// HIFN  which the marker signal in setting
/// HIFN  SOURce:BB:GALileo:TRIGger:OUTPut:MODE RATio on the marker outputs is
/// HIFN  OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR onTime/Sets the number of symbols in a period (ON time + OFF time) during
/// HIPAR onTime/which the marker signal in setting
/// HIPAR onTime/SOURce:BB:GALileo:TRIGger:OUTPut:MODE RATio on the marker outputs is
/// HIPAR onTime/OFF.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerOnTime(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   marker,
                                                         ViInt32   onTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:ONT %ld\n", satNavStandardArr[standard], marker, onTime);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the number of symbols in a period (ON time + OFF time) during
/// HIFN  which the marker signal in setting
/// HIFN  SOURce:BB:GALileo:TRIGger:OUTPut:MODE RATio on the marker outputs is
/// HIFN  OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR pulseWidth/Sets the Pulse Width for 1PPS and 1PP2S marker mode
/// HIPAR pulseWidth/(SOUR:BB:GAL:TRIG:OUTP:MODE PPS | PP2S).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerPulseWidth(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   marker,
                                                             ViInt32   pulseWidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:PULS:WIDT %ld\n", satNavStandardArr[standard], marker, pulseWidth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the delay between the signal on the marker outputs and the
/// HIFN  start of the signal, expressed in terms of chips.
/// HIFN     Command :BB:GALileo:TRIGger:OUTPut:DELay:FIXed can be used to restrict
/// HIFN  the range of values to the dynamic range, i.e. the range within which a
/// HIFN  delay of the marker signals can be set without restarting the marker
/// HIFN  and signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR delay/Defines the delay between the signal on the marker outputs and the
/// HIPAR delay/start of the signal, expressed in terms of chips.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMarkerDelay(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   marker,
                                                        ViInt32   delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:DEL %ld\n", satNavStandardArr[standard], marker, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the delay between the signal on the marker outputs and the
/// HIFN  start of the signal, expressed in terms of chips.
/// HIFN     Command :BB:GALileo:TRIGger:OUTPut:DELay:FIXed can be used to restrict
/// HIFN  the range of values to the dynamic range, i.e. the range within which a
/// HIFN  delay of the marker signals can be set without restarting the marker
/// HIFN  and signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR fixMarkerDelay/Defines the delay between the signal on the marker outputs and the
/// HIPAR fixMarkerDelay/start of the signal, expressed in terms of chips.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavFixMarkerDelayToCurrentRange(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   marker,
                            ViBoolean fixMarkerDelay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:DEL:FIX %s\n", satNavStandardArr[standard], marker, (fixMarkerDelay == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the maximum marker delay for setting :BB:GAL:TRIG:OUTP:DEL:FIX
/// HIFN  ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR maximum/Queries the maximum marker delay for setting :BB:GAL:TRIG:OUTP:DEL:FIX
/// HIPAR maximum/ON.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavFixedMarkerDelayMaximum(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   marker,
                            ViInt32*  maximum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:DEL:MAX?", satNavStandardArr[standard], marker);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, maximum));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the minimum marker delay for setting :BB:GAL:TRIG:OUTP:DEL:FIX
/// HIFN  ON.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR marker/Sets the Marker.
/// HIPAR minimum/Queries the minimum marker delay for setting :BB:GAL:TRIG:OUTP:DEL:FIX
/// HIPAR minimum/ON.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavFixedMarkerDelayMinimum(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   marker,
                            ViInt32*  minimum)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marker, 1, 4),
            3, "Marker");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:TRIG:OUTP%ld:DEL:MIN?", satNavStandardArr[standard], marker);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, minimum));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the clock synchronization mode.
/// HIFN
/// HIFN     This parameter is used to enable generation of very precise
/// HIFN  synchronous signal of several connected R&S SMBVs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mode/Selects the clock synchronization mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavClockSynchronisationMode(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:CLOC:SYNC:MODE %s\n", satNavStandardArr[standard], clockSyncMode[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR source/Selects the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavClockSource(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, source, 0, 3),
            3, "Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:CLOC:SOUR %s\n", satNavStandardArr[standard], clockSourceLegacyArr[source]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the type of externally supplied clock (:BB:GAL:CLOCk:SOURce
/// HIFN  EXTernal).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mode/Enters the type of externally supplied clock (:BB:GAL:CLOCk:SOURce
/// HIPAR mode/EXTernal).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavClockMode(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:CLOC:MODE %s\n", satNavStandardArr[standard], satNavClockMode[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the type of externally supplied clock (:BB:GAL:CLOCk:SOURce
/// HIFN  EXTernal).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR multiplier/Enters the type of externally supplied clock (:BB:GAL:CLOCk:SOURce
/// HIPAR multiplier/EXTernal).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavClockMultiplier(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   multiplier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, multiplier, 1, 64),
            3, "Multiplier");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:CLOC:MULT %ld\n", satNavStandardArr[standard], multiplier);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the vehicle type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR vehicleType/Sets the vehicle type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavUserEnvironmentVehicleType(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   vehicleType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicleType, 0, 5),
            3, "Vehicle Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:VEH:TYPE %s\n", satNavStandardArr[standard], satNavVehicleTypeArr[vehicleType]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the predefined vehicle description files in the
/// HIFN  system directory. Only files with the file extension *.xvd are listed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the predefined vehicle description files in the
/// HIPAR catalog/system directory. Only files with the file extension *.xvd are listed.
ViStatus _VI_FUNC rsrfsiggen_SatNavUserEnvironmentVehiclePredefinedCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, 512, "SOUR:BB:%s:VEH:CAT:PRED?", satNavStandardArr[standard]);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the user-defined vehicle description files in the
/// HIFN  default directory. The default directory is set using command
/// HIFN  MMEM:CDIRectory. Only files with the file extension *.xvd are listed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the user-defined vehicle description files in the
/// HIPAR catalog/default directory. The default directory is set using command
/// HIPAR catalog/MMEM:CDIRectory. Only files with the file extension *.xvd are listed.
ViStatus _VI_FUNC rsrfsiggen_SatNavUserEnvironmentVehicleUserCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:VEH:CAT:USER?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads user environment vehicle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavUserEnvironmentVehicleLoad(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:VEH:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the predefined geographic locations.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the predefined geographic locations.
ViStatus _VI_FUNC rsrfsiggen_SatNavLocCatalog(ViSession instrSession,
                                              ViInt32   standard,
                                              ViInt32   arraySize,
                                              ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:CAT?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefind waypoint file. A waypoint file must have the
/// HIFN  extension .txt, .nmea or .xtd.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocWaypointFile(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:WAYP:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the geographic location of the GNSS
/// HIFN  receiver in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLocCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64* longitude,
                            ViReal64* latitude,
                            ViReal64* altitude)
{
    ViStatus  error = VI_SUCCESS;
    ViChar    cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViAReal64 results = NULL;
    ViInt32   returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:COOR:DEC?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, cmd, &results, &returnedValues));

    *longitude = results[0];
    *latitude = results[1];
    *altitude = results[2];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (results)
        free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the geographic location of the GNSS
/// HIFN  receiver in degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR DMSResults/Displays values of the DMS.
/// HIPAR returnedValues/Displays the number of returned values.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLocCoordinatesDMS(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   arraySize,
                                                          ViReal64  DMSResults[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:COOR:DMS?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, arraySize, DMSResults, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Yaw attitude parameter relative to the local horizon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR yaw/Sets the Yaw attitude parameter relative to the local horizon.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocYaw(ViSession instrSession,
                                                   ViInt32   standard,
                                                   ViReal64  yaw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, yaw, -180.0, 180.0), 3, "Yaw");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:YAW %.12lG\n", satNavStandardArr[standard], yaw);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Force the Yaw attitude parameter to motion direction. Usefull in
/// HIFN  automotive applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fromMotionYaw/Force the Yaw attitude parameter to motion direction. Usefull in
/// HIPAR fromMotionYaw/automotive applications.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocFromMotionYaw(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean fromMotionYaw)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:YAW:FMOT %s\n", satNavStandardArr[standard], (fromMotionYaw == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Pitch attitude parameter relative to the local horizon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR pitch/Sets the Pitch attitude parameter relative to the local horizon.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocPitch(ViSession instrSession,
                                                     ViInt32   standard,
                                                     ViReal64  pitch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, pitch, -180.0, 180.0), 3, "Pitch");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:PITC %.12lG\n", satNavStandardArr[standard], pitch);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Force the Pitch attitude parameter to motion direction. Usefull in
/// HIFN  automotive applications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fromMotionPitch/Force the Pitch attitude parameter to motion direction. Usefull in
/// HIPAR fromMotionPitch/automotive applications.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocFromMotionPitch(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean fromMotionPitch)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:PITC:FMOT %s\n", satNavStandardArr[standard], (fromMotionPitch == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Roll attitude parameter relative to the local horizon.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR roll/Sets the Roll attitude parameter relative to the local horizon.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocRoll(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViReal64  roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, roll, -180.0, 180.0), 3, "Roll");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:ROLL %.12lG\n", satNavStandardArr[standard], roll);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Simulates a constant rate of change of the roll.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR constantRate/Simulates a constant rate of change of the roll.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocConstantRate(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViReal64  constantRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, constantRate, -500.0, 500.0), 3, "Constant Rate");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:SPIN:RATE %.12lG\n", satNavStandardArr[standard], constantRate);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the start angles of rotation of the vehicle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR startRollAngle/Defines the start angles of rotation of the vehicle.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocStartRollAngle(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  startRollAngle)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, startRollAngle, -180.0, 180.0), 3, "Start Roll Angle");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:SPIN:SR %.12lG\n", satNavStandardArr[standard], startRollAngle);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables an internal algorithm to smooth the trajectory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR smoothMovement/Enables an internal algorithm to smooth the trajectory.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocSmoothMovement(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean smoothMovement)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:SMOV %s\n", satNavStandardArr[standard], (smoothMovement == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds an artificial delay (i.e. buffer time) in the response of the
/// HIFN  command <subsystem>:RT:HILPosition:LATency?. That is, the latency of
/// HIFN  the R&S SMBV response increases to the selected value.
/// HIFN
/// HIFN     This mechanism compensates the command jitter caused by the sending
/// HIFN  side (e.g. a Flight simulator). During the buffer time, the R&S SMBV
/// HIFN  aligns the commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR delay/Adds an artificial delay (i.e. buffer time) in the response of the
/// HIPAR delay/command <subsystem>:RT:HILPosition:LATency?. That is, the latency of
/// HIPAR delay/the R&S SMBV response increases to the selected value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocDelay(ViSession instrSession,
                                                     ViInt32   standard,
                                                     ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.02, 0.05), 3, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:DEL %.12lG\n", satNavStandardArr[standard], delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the way the waypoint file is to be read.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR roMode/Defines the way the waypoint file is to be read.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLocFileToBeRead(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   roMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, roMode, 0, 2),
            3, "RO Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOC:WAYP:ROM %s\n", satNavStandardArr[standard], satNavROModeArr[roMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefined obscuration model or enables the configuration of
/// HIFN  the near environment and physical model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR type/Selects a predefined obscuration model or enables the configuration of
/// HIPAR type/the near environment and physical model.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationType(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   type)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, 0, 12),
            3, "Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:TYPE %s\n", satNavStandardArr[standard], satNavObscurationTypeArr[type]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a near environment model for obscuration & auto multipath.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR nearEnvironment/Selects a near environment model for obscuration & auto multipath.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationNearEnvironment(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   nearEnvironment)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, nearEnvironment, 0, 4),
            3, "Near Environment");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:ENV %s\n", satNavStandardArr[standard], satNavObscurationEnvArr[nearEnvironment]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the physical effects to be simulated on the GNSS signal, i.e.
/// HIFN  obscuration only (OBSCuration) or obscuration and multipath propagation
/// HIFN  (OMPath).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR physicalModel/Selects the physical effects to be simulated on the GNSS signal, i.e.
/// HIPAR physicalModel/obscuration only (OBSCuration) or obscuration and multipath propagation
/// HIPAR physicalModel/(OMPath).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationPhysicalModel(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   physicalModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, physicalModel, 0, 1),
            3, "Physical Model");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:PMOD %s\n", satNavStandardArr[standard], satNavObscurationPhysModelArr[physicalModel]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the start position of a receiver in terms of height and X/Y
/// HIFN  offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR receiverOffsetX/Determines the start position of a receiver in terms of height and X/Y
/// HIPAR receiverOffsetX/offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationReceiverOffsetX(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  receiverOffsetX)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, receiverOffsetX, -1500.0, 1500.0), 3, "Receiver Offset X");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:ROFF:X %.12lG\n", satNavStandardArr[standard], receiverOffsetX);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the start position of a receiver in terms of height and X/Y
/// HIFN  offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR receiverOffsetY/Determines the start position of a receiver in terms of height and X/Y
/// HIPAR receiverOffsetY/offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationReceiverOffsetY(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  receiverOffsetY)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, receiverOffsetY, -1500.0, 1500.0), 3, "Receiver Offset X");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:ROFF:Y %.12lG\n", satNavStandardArr[standard], receiverOffsetY);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the start position of a receiver in terms of height and X/Y
/// HIFN  offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR heightOffset/Determines the start position of a receiver in terms of height and X/Y
/// HIPAR heightOffset/offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationReceiverHeightOffset(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  heightOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, heightOffset, 0.0, 500.0), 3, "Height Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:ROFF:HEIG %.12lG\n", satNavStandardArr[standard], heightOffset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Represents the map orientation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mapOrientation/Represents the map orientation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavObscurationMapOrientation(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  mapOrientation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, mapOrientation, 0.0, 359.9), 3, "Map Orientation");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:ROFF:MOR %.12lG\n", satNavStandardArr[standard], mapOrientation);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the predefined files in the system directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the predefined files in the system directory.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationObstaclesPredefinedCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:CAT:PRED?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the predefined files in the system directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the predefined files in the system directory.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationRoadsidePredefinedCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:RPL:CAT:PRED?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the user-defined files in the default directory.
/// HIFN  The default directory is set using command MMEM:CDIRectory.
/// HIFN
/// HIFN     Listed are only obstacles description files (*.rs_obst) respectively
/// HIFN  roadside buildings description files (*.rs_buil).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the user-defined files in the default directory.
/// HIPAR catalog/The default directory is set using command MMEM:CDIRectory.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationObstaclesUserCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:VOBS:CAT:USER?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the user-defined files in the default directory.
/// HIFN  The default directory is set using command MMEM:CDIRectory.
/// HIFN
/// HIFN     Listed are only obstacles description files (*.rs_obst) respectively
/// HIFN  roadside buildings description files (*.rs_buil).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the user-defined files in the default directory.
/// HIPAR catalog/The default directory is set using command MMEM:CDIRectory.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationRoadsideUserCatalog(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:RPL:CAT:USER?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Appends/inserts or deletes an obscured zone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR area/Sets the area parameter.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationAreaInsert(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   area)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, area, 0, 50),
            3, "Area");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:FULL:AREA%ld:INS\n", satNavStandardArr[standard], area);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Appends/inserts or deletes an obscured zone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR area/Sets the area parameter.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationAreaDelete(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   area)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, area, 0, 50),
            3, "Area");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:FULL:AREA%ld:DEL\n", satNavStandardArr[standard], area);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the reference starting position (in km) or time stamp (in s)
/// HIFN  of a specific obscured zone.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR area/Sets the area parameter.
/// HIPAR reference/Defines the reference starting position (in km) or time stamp (in s)
/// HIPAR reference/of a specific obscured zone.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationAreaReference(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   area,
                                                            ViReal64  reference)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, area, 0, 50),
            3, "Area");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, reference, 0.0, 1.0e+4), 4, "Reference");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:FULL:AREA%ld:REF %.12lG\n", satNavStandardArr[standard], area, reference);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Length of the obscuring zone, defined in km or sec.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR area/Sets the area parameter.
/// HIPAR length/Length of the obscuring zone, defined in km or sec.
ViStatus _VI_FUNC rsrfsiggen_SatNavObscurationAreaLength(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   area,
                                                         ViReal64  length)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, area, 0, 50),
            3, "Area");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, length, 1.0e-3, 50.0), 4, "Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:OBSC:FULL:AREA%ld:LENG %.12lG\n", satNavStandardArr[standard], area, length);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the generation of multipath signals for the satellite
/// HIFN  with the specified SV ID belonging to the specified GNSS standard.
/// HIFN  Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIFN  commands are considered only if the multipath configuration is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables the generation of multipath signals for the satellite
/// HIPAR enabled/with the specified SV ID belonging to the specified GNSS standard.
/// HIPAR enabled/Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIPAR enabled/commands are considered only if the multipath configuration is enabled.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathEnabledGPS(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:STAT %s\n", satNavStandardArr[standard], svID, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the generation of multipath signals for the satellite
/// HIFN  with the specified SV ID belonging to the specified GNSS standard.
/// HIFN  Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIFN  commands are considered only if the multipath configuration is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables the generation of multipath signals for the satellite
/// HIPAR enabled/with the specified SV ID belonging to the specified GNSS standard.
/// HIPAR enabled/Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIPAR enabled/commands are considered only if the multipath configuration is enabled.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathEnabledGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the generation of multipath signals for the satellite
/// HIFN  with the specified SV ID belonging to the specified GNSS standard.
/// HIFN  Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIFN  commands are considered only if the multipath configuration is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables the generation of multipath signals for the satellite
/// HIPAR enabled/with the specified SV ID belonging to the specified GNSS standard.
/// HIPAR enabled/Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIPAR enabled/commands are considered only if the multipath configuration is enabled.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathEnabledGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the generation of multipath signals for the satellite
/// HIFN  with the specified SV ID belonging to the specified GNSS standard.
/// HIFN  Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIFN  commands are considered only if the multipath configuration is enabled.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables the generation of multipath signals for the satellite
/// HIPAR enabled/with the specified SV ID belonging to the specified GNSS standard.
/// HIPAR enabled/Settings made with the subsequent <subsystem>:SVID<ch>:<GNSS>:MPATh
/// HIPAR enabled/commands are considered only if the multipath configuration is enabled.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathEnabledBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the paths number. The value determines the number of rows
/// HIFN  available for configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the paths number. The value determines the number of rows
/// HIPAR tap/available for configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathTapGPS(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViInt32   tap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:TAP:COUN %ld\n", satNavStandardArr[standard], svID, tap);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the paths number. The value determines the number of rows
/// HIFN  available for configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the paths number. The value determines the number of rows
/// HIPAR tap/available for configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathTapGALILEO(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViInt32   tap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:TAP:COUN %ld\n", satNavStandardArr[standard], svID, tap);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the paths number. The value determines the number of rows
/// HIFN  available for configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the paths number. The value determines the number of rows
/// HIPAR tap/available for configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathTapGLONASS(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViInt32   tap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:TAP:COUN %ld\n", satNavStandardArr[standard], svID, tap);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the paths number. The value determines the number of rows
/// HIFN  available for configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the paths number. The value determines the number of rows
/// HIPAR tap/available for configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathTapBeiDou(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   tap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:TAP:COUN %ld\n", satNavStandardArr[standard], svID, tap);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase, used in case of multipath simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR phase/Sets the carrier phase, used in case of multipath simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathCarrierPhaseGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.00, 6.28), 5, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:TAP%ld:CPH %.12lG\n", satNavStandardArr[standard], svID, tap, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase, used in case of multipath simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR phase/Sets the carrier phase, used in case of multipath simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathCarrierPhaseGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.00, 6.28), 5, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:TAP%ld:CPH %.12lG\n", satNavStandardArr[standard], svID, tap, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase, used in case of multipath simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR phase/Sets the carrier phase, used in case of multipath simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathCarrierPhaseGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.00, 6.28), 5, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:TAP%ld:CPH %.12lG\n", satNavStandardArr[standard], svID, tap, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase, used in case of multipath simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR phase/Sets the carrier phase, used in case of multipath simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathCarrierPhaseBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.00, 6.28), 5, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:TAP%ld:CPH %.12lG\n", satNavStandardArr[standard], svID, tap, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase, used in case of multipath simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR phase/Sets the carrier phase, used in case of multipath simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathCarrierPhaseQZSS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.00, 6.28), 5, "Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:MPAT:TAP%ld:CPH %.12lG\n", satNavStandardArr[standard], svID, tap, phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the additional Doppler shift of the simulated signal of the
/// HIFN  satellite, used to simulate multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR dopplerShift/Defines the additional Doppler shift of the simulated signal of the
/// HIPAR dopplerShift/satellite, used to simulate multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDopplerShiftGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10.0e-3, 10.0e+3), 5, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:TAP%ld:DSH %.12lG\n", satNavStandardArr[standard], svID, tap, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the additional Doppler shift of the simulated signal of the
/// HIFN  satellite, used to simulate multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR dopplerShift/Defines the additional Doppler shift of the simulated signal of the
/// HIPAR dopplerShift/satellite, used to simulate multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDopplerShiftGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10.0e-3, 10.0e+3), 5, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:TAP%ld:DSH %.12lG\n", satNavStandardArr[standard], svID, tap, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the additional Doppler shift of the simulated signal of the
/// HIFN  satellite, used to simulate multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR dopplerShift/Defines the additional Doppler shift of the simulated signal of the
/// HIPAR dopplerShift/satellite, used to simulate multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDopplerShiftGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10.0e-3, 10.0e+3), 5, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:TAP%ld:DSH %.12lG\n", satNavStandardArr[standard], svID, tap, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the additional Doppler shift of the simulated signal of the
/// HIFN  satellite, used to simulate multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR dopplerShift/Defines the additional Doppler shift of the simulated signal of the
/// HIPAR dopplerShift/satellite, used to simulate multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDopplerShiftBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10.0e-3, 10.0e+3), 5, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:TAP%ld:DSH %.12lG\n", satNavStandardArr[standard], svID, tap, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the additional Doppler shift of the simulated signal of the
/// HIFN  satellite, used to simulate multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR dopplerShift/Defines the additional Doppler shift of the simulated signal of the
/// HIPAR dopplerShift/satellite, used to simulate multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDopplerShiftQZSS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10.0e-3, 10.0e+3), 5, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:MPAT:TAP%ld:DSH %.12lG\n", satNavStandardArr[standard], svID, tap, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power of the satellite in dB, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR power/Sets the additional power of the satellite in dB, used to simulate
/// HIPAR power/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathPowerGPS(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViInt32   tap,
                                                          ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -10.0, 0.0), 5, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:TAP%ld:POW %.12lG\n", satNavStandardArr[standard], svID, tap, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power of the satellite in dB, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR power/Sets the additional power of the satellite in dB, used to simulate
/// HIPAR power/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathPowerGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -10.0, 0.0), 5, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:TAP%ld:POW %.12lG\n", satNavStandardArr[standard], svID, tap, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power of the satellite in dB, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR power/Sets the additional power of the satellite in dB, used to simulate
/// HIPAR power/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathPowerGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -10.0, 0.0), 5, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:TAP%ld:POW %.12lG\n", satNavStandardArr[standard], svID, tap, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power of the satellite in dB, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR power/Sets the additional power of the satellite in dB, used to simulate
/// HIPAR power/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathPowerBeiDou(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViInt32   tap,
                                                             ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -10.0, 0.0), 5, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:TAP%ld:POW %.12lG\n", satNavStandardArr[standard], svID, tap, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power of the satellite in dB, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR power/Sets the additional power of the satellite in dB, used to simulate
/// HIPAR power/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathPowerQZSS(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   tap,
                                                           ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -10.0, 0.0), 5, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:MPAT:TAP%ld:POW %.12lG\n", satNavStandardArr[standard], svID, tap, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional delay of the selected satellite, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR delay/Sets an additional delay of the selected satellite, used to simulate
/// HIPAR delay/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDelayGPS(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViInt32   tap,
                                                          ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.00000, 9.99999), 5, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:MPAT:TAP%ld:TSH %.12lG\n", satNavStandardArr[standard], svID, tap, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional delay of the selected satellite, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR delay/Sets an additional delay of the selected satellite, used to simulate
/// HIPAR delay/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDelayGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.00000, 9.99999), 5, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:MPAT:TAP%ld:TSH %.12lG\n", satNavStandardArr[standard], svID, tap, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional delay of the selected satellite, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR delay/Sets an additional delay of the selected satellite, used to simulate
/// HIPAR delay/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDelayGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   tap,
                            ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.00000, 9.99999), 5, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:MPAT:TAP%ld:TSH %.12lG\n", satNavStandardArr[standard], svID, tap, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional delay of the selected satellite, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR delay/Sets an additional delay of the selected satellite, used to simulate
/// HIPAR delay/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDelayBeiDou(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViInt32   tap,
                                                             ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.00000, 9.99999), 5, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:MPAT:TAP%ld:TSH %.12lG\n", satNavStandardArr[standard], svID, tap, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional delay of the selected satellite, used to simulate
/// HIFN  multipath conditions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tap/Sets the Tap parameter.
/// HIPAR delay/Sets an additional delay of the selected satellite, used to simulate
/// HIPAR delay/multipath conditions.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathDelayQZSS(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   tap,
                                                           ViReal64  delay)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            4, "Tap");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, delay, 0.00000, 9.99999), 5, "Delay");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:MPAT:TAP%ld:TSH %.12lG\n", satNavStandardArr[standard], svID, tap, delay);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the line-of-sight component.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR enabled/Activates the line-of-sight component.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSEnabled(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViInt32   vehicle,
                                                            ViInt32   antenna,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:ENAB %s", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the echoes number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echoesNumber/Sets the echoes number.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSEchoesNumber(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echoesNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echoesNumber, 0, 9),
            7, "Echoes Number");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECH:COUN %ld", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echoesNumber);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the angle of arrival azimuth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR angleOfArrivalAzimuth/Queries the angle of arrival azimuth.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSAngleOfArrivalAzimuth(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViReal64* angleOfArrivalAzimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:AAZ?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, angleOfArrivalAzimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the angle of arrival elevation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR angleOfArrivalElevation/Queries the angle of arrival elevation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSAngleOfArrivalElevation(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViReal64* angleOfArrivalElevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:AELEV?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, angleOfArrivalElevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries a code phase drift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR codePhaseDrift/Queries a code phase drift.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSCodePhaseDrift(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViReal64* codePhaseDrift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:CPDR?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, codePhaseDrift));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the carrier phase at the simulation start.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR carrierPhase/Queries the carrier phase at the simulation start.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSCarrierPhase(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViInt32   vehicle,
                                                             ViInt32   antenna,
                                                             ViReal64* carrierPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:CPH?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, carrierPhase));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries an additional Doppler shift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR dopplerShift/Queries an additional Doppler shift.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSDopplerShift(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViInt32   vehicle,
                                                             ViInt32   antenna,
                                                             ViReal64* dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:DSH?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, dopplerShift));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries an initial code phase for the selected echo.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR initialCodePhase/Queries an initial code phase for the selected echo.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSInitialCodePhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViReal64* initialCodePhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:ICPH?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, initialCodePhase));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the additional power offset for the selected echoes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR power/Queries the additional power offset for the selected echoes.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavMpathLOSPower(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   standard,
                                                      ViInt32   svID,
                                                      ViInt32   vehicle,
                                                      ViInt32   antenna,
                                                      ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:LOS:POW?", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, power));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the angle of arrival azimuth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR angleOfArrivalAzimuth/Sets the angle of arrival azimuth.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSAngleOfArrivalAzimuth(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  angleOfArrivalAzimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, angleOfArrivalAzimuth, 0, 6.28),
        8, "Angle of Arrival Azimuth");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:AAZ %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, angleOfArrivalAzimuth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the angle of arrival elevation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR angleOfArrivalElevation/Sets the angle of arrival elevation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSAngleOfArrivalElevation(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  angleOfArrivalElevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, angleOfArrivalElevation, 0, 6.28),
        8, "Angle of Arrival Elevation");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:AELEV %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, angleOfArrivalElevation);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets a code phase drift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR codePhaseDrift/Sets a code phase drift.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSCodePhaseDrift(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  codePhaseDrift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, codePhaseDrift, 0, 2000),
            8, "Code Phase Drift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:CPDR %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, codePhaseDrift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the carrier phase at the simulation start.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR carrierPhase/Sets the carrier phase at the simulation start.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSCarrierPhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  carrierPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, carrierPhase, 0, 6.28),
        8, "Carier Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:CPH %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, carrierPhase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an additional Doppler shift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR dopplerShift/Sets an additional Doppler shift.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSDopplerShift(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -10e3, 10e3),
        8, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:DSH %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets an initial code phase for the selected echo.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR initialCodePhase/Sets an initial code phase for the selected echo.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSInitialCodePhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   vehicle,
                            ViInt32   antenna,
                            ViInt32   echo,
                            ViReal64  initialCodePhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, initialCodePhase, -0, 3000),
        8, "Initial Code Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:ICPH %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, initialCodePhase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the additional power offset for the selected echoes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR vehicle/Sets the Vehicle parameter.
/// HIPAR antenna/Sets the antenna parameter.
/// HIPAR echo/Sets the echo parameter.
/// HIPAR power/Sets the additional power offset for the selected echoes.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavMpathLOSPower(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViInt32   vehicle,
                                                          ViInt32   antenna,
                                                          ViInt32   echo,
                                                          ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    if (standard == RSRFSIGGEN_VAL_SATNAV_STANDARD_QZSS)
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 193, 197),
                4, "SVID");
    }
    else
    {
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
                4, "SVID");
    }
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, vehicle, 1, 2),
            5, "Vehicle");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antenna, 1, 4),
            6, "Antenna");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, echo, 0, 9),
            7, "Echo");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, power, -15, 0),
        8, "Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SVID%ld:%s:MPAT:V%ld:A%ld:ECHO%ld:POW %.12lG", path, svID, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, vehicle, antenna, echo, power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the predefined antenna pattern files in the
/// HIFN  system directory. Only files with the file extension *.ant_pat are
/// HIFN  listed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the predefined antenna pattern files in the
/// HIPAR catalog/system directory. Only files with the file extension *.ant_pat are
/// HIPAR catalog/listed.
ViStatus _VI_FUNC rsrfsiggen_SatNavAntennaPredefinedCatalog(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   arraySize,
                                                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:APAT:CAT:PRED?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the names of the user-defined antenna pattern files in the
/// HIFN  default directory. The default directory is set using command
/// HIFN  MMEM:CDIRectory. Only files with the file extension *.ant_pat are
/// HIFN  listed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the names of the user-defined antenna pattern files in the
/// HIPAR catalog/default directory. The default directory is set using command
/// HIPAR catalog/MMEM:CDIRectory. Only files with the file extension *.ant_pat are
/// HIPAR catalog/listed.
ViStatus _VI_FUNC rsrfsiggen_SatNavAntennaUserCatalog(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViInt32   arraySize,
                                                      ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:APAT:CAT:USER?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected antenna pattern file. Only files with the file
/// HIFN  extension *.ant_pat are loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_SatNavAntennaLoad(ViSession instrSession,
                                               ViInt32   standard,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:APAT:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the antenna identifiers <antenna id> specified in the selected
/// HIFN  *.ant_pat file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the antenna identifiers <antenna id> specified in the selected
/// HIPAR catalog/*.ant_pat file.
ViStatus _VI_FUNC rsrfsiggen_SatNavAntennaList(ViSession instrSession,
                                               ViInt32   standard,
                                               ViInt32   arraySize,
                                               ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:APAT:ANT:LIST?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects one particular antenna by its antenna ID. To query the list of
/// HIFN  all antenna IDs, use the <subsystem>:APATtern:ANTenna:LIST?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR id/Selects one particular antenna by its antenna ID. To query the list of
/// HIPAR id/all antenna IDs, use the <subsystem>:APATtern:ANTenna:LIST?.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAntennaID(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViInt32   id)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:APAT:ANT:ID %ld\n", satNavStandardArr[standard], id);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the software to schedule a real time switch of the used
/// HIFN  antenna at a specific future time, as specified in the antenna pattern
/// HIFN  (*.ant_pat) file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR elapsedTime/Elapsed time from the simulation start time at which a switch to the
/// HIPAR elapsedTime/defined <AntennaId> is performed.
/// HIPAR antennaID/Use the command <subsystem>:APATtern:ANTenna:LIST? to query the
/// HIPAR antennaID/antenna IDs defined in the currently selected antenna pattern file.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAntennaUpdate(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViReal64  elapsedTime,
                                                          ViInt32   antennaID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elapsedTime, 0.0, 99999999.0), 3, "Elapsed Time");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, antennaID, 1, 100),
            4, "Antenna ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:UPD:ANT %.12lG,%ld\n", satNavStandardArr[standard], elapsedTime, antennaID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the data source for the navigation information.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR dataSource/Determines the data source for the navigation information.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationDataSource(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataSource, 0, 11),
            3, "Data Source");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:DATA %s\n", satNavStandardArr[standard], satNavDataSourceArr[dataSource]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the data list for <subsystem>:NAVigation:DATADLISt.
/// HIFN     The lists are stored as files with the fixed file extensions *.dm_iqd
/// HIFN  in a directory of the user's choice. The directory applicable to the
/// HIFN  following commands is defined with the command MMEMory:CDIR. To access
/// HIFN  the files in this directory, you only have to give the file name,
/// HIFN  without the path and the file extension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationDataList(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:DATA:DSEL '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the time basis used to enter the simulation start time. Per
/// HIFN  default, the UTC (Universal Time Coordinates) is used but the user may
/// HIFN  choose or switch to a different time basis at any time. The time is
/// HIFN  then automatically recalculated and displayed in the selected time
/// HIFN  format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR systemTime/Determines the time basis used to enter the simulation start time. Per
/// HIPAR systemTime/default, the UTC (Universal Time Coordinates) is used but the user may
/// HIPAR systemTime/choose or switch to a different time basis at any time. The time is
/// HIPAR systemTime/then automatically recalculated and displayed in the selected time
/// HIPAR systemTime/format.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationSystemTime(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   systemTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, systemTime, 0, 4),
            3, "System Time");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SIM:TBAS %s\n", satNavStandardArr[standard], ADGTimeBasisArr[systemTime]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the week number (WN) the navigation signal is generated for.
/// HIFN
/// HIFN     This setting is only available for
/// HIFN  <subsystem>:NAVigation:SIMulation:TBASis set to GPS or GST and
/// HIFN  <subsystem>:NAVigation:DATA set to RNData.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Enters the week number (WN) the navigation signal is generated for.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationWeekNumber(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SIM:WNUM %ld\n", satNavStandardArr[standard], weekNumber);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the simulation start time within the defined week (see
/// HIFN  <subsystem>:NAVigation:SIMulation:WNUMber).
/// HIFN
/// HIFN     This setting is only available for
/// HIFN  <subsystem>:NAVigation:SIMulation:TBASis set to GPS or GST.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeOfWeek/Defines the simulation start time within the defined week (see
/// HIPAR timeOfWeek/<subsystem>:NAVigation:SIMulation:WNUMber).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationTimeOfWeek(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  timeOfWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, standard, 0.0, 604799.999), 3, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SIM:TOW %.12lG\n", satNavStandardArr[standard], timeOfWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date for the simulation in DD.MM.YYYY format of the
/// HIFN  Gregorian calendar.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationDate(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   year,
                                                           ViInt32   month,
                                                           ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31), 5, "Day")

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SIM:DATE %ld,%ld,%ld\n", satNavStandardArr[standard], year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the exact simulation start time in UTC time format.
/// HIFN     This setting is only available for <subsystem>:NAVigation:DATA set to
/// HIFN  RNData and
/// HIFN     <subsystem>:NAVigation:SIMulation:TBASis set to UTC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavNavigationTime(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   hour,
                                                           ViInt32   minute,
                                                           ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Seconds");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SIM:TIME %ld,%ld,%lf\n", satNavStandardArr[standard], hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the leap second according to the simulation time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_SatNavTCONLeapSecondSynchronize(ViSession instrSession,
                                                             ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:LEAP:SYNC\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the currently used leap second.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR leapSecondCurrent/Defines the currently used leap second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONLeapSecondCurrent(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   leapSecondCurrent)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, leapSecondCurrent, 0, 50),
            3, "Leap Second Current");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:LEAP:SEC %ld\n", satNavStandardArr[standard], leapSecondCurrent);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the simulation of the leap second transition.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR leapsecondsimulatetransition/Enables/disables the simulation of the leap second transition.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONLeapSecondSimulateTransition(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean leapsecondsimulatetransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:LEAP:SLST %s\n", satNavStandardArr[standard], (leapsecondsimulatetransition == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the date of the next UTC time correction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceDate/Defines the date of the next UTC time correction.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavTCONUTCReferenceDate(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViChar    referenceDate[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:LEAP:DATE?", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 256, referenceDate, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant term of polynomial, A 0 .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR utcA0/Defines the constant term of polynomial, A 0 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONUTCA0(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViReal64  utcA0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, utcA0, -1024.0, 1023.0), 3, "UTC A0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:UTCS:AZER %.12lG\n", satNavStandardArr[standard], utcA0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant term of polynomial, A 1 .
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR utcA1/Defines the constant term of polynomial, A 1 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONUTCA1(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViReal64  utcA1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, utcA1, -512.0, 511.0), 3, "UTC A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:UTCS:AONE %.12lG\n", satNavStandardArr[standard], utcA1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant term of polynomial, A 0 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a0/Defines the constant term of polynomial, A 0 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA0GPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GPS:AZER %ld\n", satNavStandardArr[standard], a0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant term of polynomial, A 0 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a0/Defines the constant term of polynomial, A 0 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA0GALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GAL:AZER %ld\n", satNavStandardArr[standard], a0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the constant term of polynomial, A 0 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a0/Defines the constant term of polynomial, A 0 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA0GLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   a0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GLON:AZER %ld\n", satNavStandardArr[standard], a0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the 1 st  order term of polynomial, A 1 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a1/Defines the 1 st  order term of polynomial, A 1 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA1GPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a1, -8388608, 8388607), 3, "A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GPS:AONE %.12lG\n", satNavStandardArr[standard], a1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the 1 st  order term of polynomial, A 1 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a1/Defines the 1 st  order term of polynomial, A 1 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA1GALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a1, -8388608, 8388607), 3, "A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GAL:AONE %.12lG\n", satNavStandardArr[standard], a1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the 1 st  order term of polynomial, A 1 .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR a1/Defines the 1 st  order term of polynomial, A 1 .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersA1GLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  a1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, a1, -8388608, 8388607), 3, "A1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GLON:AONE %.12lG\n", satNavStandardArr[standard], a1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference time of week, t ot .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR tot/Defines the UTC data reference time of week, t ot .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersTotGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 255),
            3, "t ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GPS:TOT %ld\n", satNavStandardArr[standard], tot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference time of week, t ot .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR tot/Defines the UTC data reference time of week, t ot .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersTotGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 255),
            3, "t ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GAL:TOT %ld\n", satNavStandardArr[standard], tot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference time of week, t ot .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR tot/Defines the UTC data reference time of week, t ot .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersTotGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   tot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tot, 0, 255),
            3, "t ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GLON:TOT %ld\n", satNavStandardArr[standard], tot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference week number, WN t .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR wnot/Defines the UTC data reference week number, WN t .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersWNotGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   wnot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wnot, 0, 255),
            3, "WN ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GPS:WNOT %ld\n", satNavStandardArr[standard], wnot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference week number, WN t .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR wnot/Defines the UTC data reference week number, WN t .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersWNotGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   wnot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wnot, 0, 255),
            3, "WN ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GAL:WNOT %ld\n", satNavStandardArr[standard], wnot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the UTC data reference week number, WN t .
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR wnot/Defines the UTC data reference week number, WN t .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavTCONParametersWNotGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   wnot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, wnot, 0, 255),
            3, "WN ot");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:TCON:GLON:WNOT %ld\n", satNavStandardArr[standard], wnot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines whether the power is calculated automatically or is based
/// HIFN  on user defined settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR powerMode/Determines whether the power is calculated automatically or is based
/// HIPAR powerMode/on user defined settings.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatPowerMode(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   powerMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerMode, 0, 1),
            3, "Power Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:MODE %s\n", satNavStandardArr[standard], satNavPowerModeArr[powerMode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power level that is used as a reference for the calculation
/// HIFN  of the power level of the satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referencePower/Sets the power level that is used as a reference for the calculation
/// HIPAR referencePower/of the power level of the satellites.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatReferencePower(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  referencePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, referencePower, -145.00, 20.00), 3, "Reference Power");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:REF %.12lG\n", satNavStandardArr[standard], referencePower);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the satellite used as a reference for the calculation of the
/// HIFN  power levels of the satellites in auto power mode. Value -1 corresponds
/// HIFN  to N.A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceSatellite/Sets the satellite used as a reference for the calculation of the
/// HIPAR referenceSatellite/power levels of the satellites in auto power mode. Value -1 corresponds
/// HIPAR referenceSatellite/to N.A.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatReferenceSatellite(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   referenceSatellite)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceSatellite, 1, 24),
            3, "Reference Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:SREF %ld\n", satNavStandardArr[standard], referenceSatellite);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the total power of the GNSS signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR totalPower/Queries the total power of the GNSS signal.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatTotalPower(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViReal64* totalPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:TOT?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, totalPower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries or sets the reference standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceStandard/Queries or sets the reference standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatReferenceStandard(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   referenceStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, referenceStandard, 0, 2),
            3, "Reference Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:REF:STAN %s\n", satNavStandardArr[standard], satNavReferenceStandardArr[referenceStandard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the signal component of the current reference satellite used
/// HIFN  as reference for the power calculation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceSignal/Queries the signal component of the current reference satellite used
/// HIPAR referenceSignal/as reference for the power calculation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatReferenceSignal(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViReal64* referenceSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:REF:SIGN?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, referenceSignal));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the pseudorange of the reference satellite at the beginning of
/// HIFN  the simulation (t=t0).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR referenceDistance/Queries the pseudorange of the reference satellite at the beginning of
/// HIPAR referenceDistance/the simulation (t=t0).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatReferenceDistance(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViReal64* referenceDistance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:REF:DIST?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, referenceDistance));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the satellite's elevation mask, i.e. determines the elevation
/// HIFN  filter applied when <subsystem>:GOConstellation is performed, or, while
/// HIFN  using the Auto Localization mode, the filter used to filter all
/// HIFN  low-elevation satellites which are closer to the horizon and may
/// HIFN  witness in reality more unwanted multipath effects.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR elevationMask/Sets the satellite's elevation mask, i.e. determines the elevation
/// HIPAR elevationMask/filter applied when <subsystem>:GOConstellation is performed, or, while
/// HIPAR elevationMask/using the Auto Localization mode, the filter used to filter all
/// HIPAR elevationMask/low-elevation satellites which are closer to the horizon and may
/// HIPAR elevationMask/witness in reality more unwanted multipath effects.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatElevationMask(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViReal64  elevationMask)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, elevationMask, 2.5, 10.0), 3, "Elevation Mask");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SEM %.12lG\n", satNavStandardArr[standard], elevationMask);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the HDOP (Horizontal Dilution of Precision) of the selected
/// HIFN  satellite constellation at the beginning of the simulation.
/// HIFN     At least four different satellites have to be configured to get a
/// HIFN  reasonable value; otherwise -1 will be returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR hdop/Queries the HDOP (Horizontal Dilution of Precision) of the selected
/// HIPAR hdop/satellite constellation at the beginning of the simulation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatHDOP(ViSession instrSession,
                                                ViInt32   standard,
                                                ViReal64* hdop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:HDOP?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, hdop));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the PDOP (Position Dilution of Precision) of the selected
/// HIFN  satellite constellation at the beginning of the simulation.
/// HIFN     At least four different satellites have to be configured to get a
/// HIFN  reasonable value; otherwise -1 will be returned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR pdop/Displays the PDOP (Position Dilution of Precision) of the selected
/// HIPAR pdop/satellite constellation at the beginning of the simulation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatPDOP(ViSession instrSession,
                                                ViInt32   standard,
                                                ViReal64* pdop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:PDOP?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, pdop));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Retrieves an optimal satellite constellation for the selected
/// HIFN  Almanac/RINEX file, elevation mask and the enabled maximum number of
/// HIFN  satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatOptimalConstellation(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:GOC\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Retrieves an optimal satellite constellation for the selected
/// HIFN  Almanac/RINEX file, elevation mask and the enabled maximum number of
/// HIFN  satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR lastLatency/Time delay between the time specified with the parameter <ElapsedTime>
/// HIPAR lastLatency/in a HIL command (<subsystem>:RT:
/// HIPAR maxLatency/The largest latency value since the last time this query was sent.
/// HIPAR minLatency/
/// HIPAR noZeroValues/Number of non-zero latency values since the last time this query was
/// HIPAR noZeroValues/sent.
/// HIPAR totalValues/The number of received and processed HIL commands.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatLatencyStatistics(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64* lastLatency,
                            ViReal64* maxLatency,
                            ViReal64* minLatency,
                            ViInt32*  noZeroValues,
                            ViInt32*  totalValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:HILP:LAT:STAT?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    (void)sscanf(response, "%lf,%lf,%lf,%ld,%ld", lastLatency, maxLatency, minLatency, noZeroValues, totalValues);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relations between the signal components to 0 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningIntraZero(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INTR:GPS:PZER\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the same
/// HIFN  navigation standard according to the relation determined in the
/// HIFN  specifications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningIntraSetToStandard(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INTR:GPS:PST\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the same
/// HIFN  navigation standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR intraCAP/Sets the power relation between the signals belonging to the same
/// HIPAR intraCAP/navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningIntraCAP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  intraCAP)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, intraCAP, 0.00, 6.00), 3, "Intra CAP");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INTR:GPS:CAP %.12lG\n", satNavStandardArr[standard], intraCAP);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Inter-Standard Power relations to 0 dB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterZero(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:PZER\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the different
/// HIFN  navigation standard according to the relation determined in the
/// HIFN  specifications.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterSetToStandard(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:PST\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the different
/// HIFN  navigation standards. The power level of the GPS signal C/A is used as
/// HIFN  reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR caE1Def/Sets the power relation between the signals belonging to the different
/// HIPAR caE1Def/navigation standards. The power level of the GPS signal C/A is used as
/// HIPAR caE1Def/reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterCAE1Def(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  caE1Def)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, caE1Def, -6.00, 6.00), 3, "CA E1 Def");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:CAE1D %.12lG\n", satNavStandardArr[standard], caE1Def);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the different
/// HIFN  navigation standards. The power level of the GPS signal C/A is used as
/// HIFN  reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR caRCA/Sets the power relation between the signals belonging to the different
/// HIPAR caRCA/navigation standards. The power level of the GPS signal C/A is used as
/// HIPAR caRCA/reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterCARCA(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  caRCA)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, caRCA, -6.00, 6.00), 3, "CA RCA");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:CARCA %.12lG\n", satNavStandardArr[standard], caRCA);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the different
/// HIFN  navigation standards. The power level of the GPS signal C/A is used as
/// HIFN  reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR caTOGEO/Sets the power relation between the signals belonging to the different
/// HIPAR caTOGEO/navigation standards. The power level of the GPS signal C/A is used as
/// HIPAR caTOGEO/reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterCATOGEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  caTOGEO)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, caTOGEO, 0.00, 6.00), 3, "CA TO GEO");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:CABCAG %.12lG\n", satNavStandardArr[standard], caTOGEO);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power relation between the signals belonging to the different
/// HIFN  navigation standards. The power level of the GPS signal C/A is used as
/// HIFN  reference.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR caTOQZSSCA/Sets the power relation between the signals belonging to the different
/// HIPAR caTOQZSSCA/navigation standards. The power level of the GPS signal C/A is used as
/// HIPAR caTOQZSSCA/reference.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTuningInterCATOQZSSCA(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  caTOQZSSCA)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, caTOQZSSCA, 0.00, 6.00), 3, "CA TO QZSS CA");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:POW:INT:CAQCA %.12lG\n", satNavStandardArr[standard], caTOQZSSCA);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the tropospheric model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR troposhphericModel/Determines the tropospheric model.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericTroposphericModel(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   troposhphericModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, troposhphericModel, 0, 1),
            3, "Tropospheric Model");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:TROP:MOD %s\n", satNavStandardArr[standard], satNavTropoModelArr[troposhphericModel]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the applied ionospheric model.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR ionoshphericModel/Determines the applied ionospheric model.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericIonosphericModel(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   ionoshphericModel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ionoshphericModel, 0, 1),
            3, "Ionospheric Model");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:MOD %s\n", satNavStandardArr[standard], satNavIonModelArr[ionoshphericModel]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the solar flux level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR solarFlux/Sets the solar flux level.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericIonosphericSolarFlux(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  solarFlux)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, solarFlux, 0.0, 300.0), 3, "Solar Flux");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:NEQ:SFL %.12lG\n", satNavStandardArr[standard], solarFlux);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the sunspot number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR sunspotNumber/Sets the sunspot number.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericIonosphericSunspotNumber(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  sunspotNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sunspotNumber, -99.636, 248.870), 3, "Sunspot Number");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:NEQ:SUNS %.12lG\n", satNavStandardArr[standard], sunspotNumber);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the instrument to use the measured sunspot number value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR measuredSunspot/Enables the instrument to use the measured sunspot number value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericIonosphericMeasuredSunspot(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean measuredSunspot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:NEQ:UMSN %s\n", satNavStandardArr[standard], (measuredSunspot == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Select an ionospheric grid file with mixed ionospheric information on
/// HIFN  the enabled GNSS systems.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericIonosphericGridFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:MOPS:FILE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR alphaParameter/Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIPAR alphaParameter/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericGPSAlpha(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  alphaParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, alphaParameter, -128.0, 127.0), 4, "Alpha Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:GPS:ION:ALPH%ld %.12lG\n", satNavStandardArr[standard], alpha, alphaParameter);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR alphaParameter/Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIPAR alphaParameter/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericQZSSAlpha(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  alphaParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, alphaParameter, -128.0, 127.0), 4, "Alpha Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:QZSS:ION:ALPH%ld %.12lG\n", satNavStandardArr[standard], alpha, alphaParameter);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR klobucharAlpha/Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIPAR klobucharAlpha/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericKlobucharAlpha(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  klobucharAlpha)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, klobucharAlpha, -128.0, 127.0), 4, "Klobuchar Alpha");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:KLOB:ALPH%ld %.12lG\n", satNavStandardArr[standard], alpha, klobucharAlpha);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR beta/Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIPAR beta/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericGPSBeta(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  beta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, beta, -128.0, 127.0), 4, "Beta");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:GPS:ION:BETA%ld %.12lG\n", satNavStandardArr[standard], alpha, beta);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR beta/Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIPAR beta/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericBeiDouBeta(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  beta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, beta, -128.0, 127.0), 4, "Beta");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:BEID:ION:BETA%ld %.12lG\n", satNavStandardArr[standard], alpha, beta);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR beta/Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIPAR beta/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericQZSSBeta(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  beta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, beta, -128.0, 127.0), 4, "Beta");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:QZSS:ION:BETA%ld %.12lG\n", satNavStandardArr[standard], alpha, beta);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR klobucharBeta/Sets the parameter beta_0 .. beta_3 of the satellite's navigation
/// HIPAR klobucharBeta/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericKlobucharBeta(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  klobucharBeta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, klobucharBeta, -128.0, 127.0), 4, "Klobuchar Beta");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:ION:KLOB:BETA%ld %.12lG\n", satNavStandardArr[standard], alpha, klobucharBeta);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIFN  message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR alpha/Sets the Alpha parameter.
/// HIPAR alphaParameter/Sets the parameter alpha_0 .. alpha_3 of the satellite's navigation
/// HIPAR alphaParameter/message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericGALILEOAlpha(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   alpha,
                            ViReal64  alphaParameter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, alpha, 0, 3),
            3, "Alpha");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, alphaParameter, -8192.0, 8191.0), 4, "Alpha Parameter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:GAL:ION:AI%ld %.12lG\n", satNavStandardArr[standard], alpha, alphaParameter);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the parameters ionospheric disturbance flag for region 1 to 5 of
/// HIFN  the satellite's navigation message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the Region parameter.
/// HIPAR sf/Sets the parameters ionospheric disturbance flag for region 1 to 5 of
/// HIPAR sf/the satellite's navigation message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavAtmosphericGALILEOSF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   sf)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 1, 5),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sf, 0, 1),
            4, "SF");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ATM:GAL:ION:SF%ld %ld\n", satNavStandardArr[standard], region, sf);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Forces ephemeris projection for all satellites, i.e. performs the same
/// HIFN  action as the command
/// HIFN  <subsystem>:SVID<ch>:<GNSS>:NMESsage:RTPRojection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR ephemerisProjection/Forces ephemeris projection for all satellites, i.e. performs the same
/// HIPAR ephemerisProjection/action as the command
/// HIPAR ephemerisProjection/<subsystem>:SVID<ch>:<GNSS>:NMESsage:RTPRojection.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConfigurationEphemerisProjection(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean ephemerisProjection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT:GRTP %s\n", satNavStandardArr[standard], (ephemerisProjection == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the maximum number of satellites that can be simulated. The
/// HIFN  minimum allowed number depends on the selected simulation mode (see
/// HIFN  <subsystem>:SMODe on page 153) .
/// HIFN     Configurations with one satellite are allowed only for
/// HIFN  <subsystem>:SMODeSTATic. For <subsystem>:SMODeAUTO or USER, four
/// HIFN  satellites or more are required.
/// HIFN     Note: Generating the navigation signal with more than 6 satellites is
/// HIFN  enabled only for instruments equipped with option R&S SMBV-K91; more
/// HIFN  than 12 satellites require option R&S SMBV-K96.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR numberOfSatellites/Defines the maximum number of satellites that can be simulated. The
/// HIPAR numberOfSatellites/minimum allowed number depends on the selected simulation mode (see
/// HIPAR numberOfSatellites/<subsystem>:SMODe on page 153) .
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConfigurationNumberOfSatellites(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   numberOfSatellites)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfSatellites, 1, 24),
            3, "Number Of Satellites");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT:COUN %ld\n", satNavStandardArr[standard], numberOfSatellites);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates/deactivates the satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR enabled/Activates/deactivates the satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatEnabled(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32   satellite,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:STAT %s\n", satNavStandardArr[standard], satellite, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the navigation standard the corresponding satellite belongs
/// HIFN  to. The available GNSS standards depend on the selected Hybrid GNSS
/// HIFN  configuration and the installed options.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR satStandard/Selects the navigation standard the corresponding satellite belongs
/// HIPAR satStandard/to. The available GNSS standards depend on the selected Hybrid GNSS
/// HIPAR satStandard/configuration and the installed options.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatStandard(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   satellite,
                                                        ViInt32   satStandard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satStandard, 0, 4),
            4, "Sat Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:STAN %s\n", satNavStandardArr[standard], satellite, satNavStandardArr[satStandard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the type of signal the corresponding satellite is using.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR signals/Selects the type of signal the corresponding satellite is using.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatSignals(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32   satellite,
                                                       ViInt32   signals)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signals, 0, 5),
            4, "Signals");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SIGN %s\n", satNavStandardArr[standard], satellite, satNavSATSignalArr[signals]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Space Vehicle ID of the satellite to be simulated. This
/// HIFN  value is used to generate the corresponding spreading code.
/// HIFN     For <subsystem>:NAVigation:DATARNData, only the valid IDs which are
/// HIFN  listed in the almanac are selectable; for arbitrary data, any ID can be
/// HIFN  selected.
/// HIFN     SV ID set to -1 corresponds to the manual selection 'N.A. (not
/// HIFN  assigned)'. It is not allowed to assign the same SV ID to more than one
/// HIFN  satellite of one GNSS standard. Hence, the selection N.A. can be used
/// HIFN  to exchange the assigned SV ID to different satellites belonging to the
/// HIFN  same standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR spaceVehicleID/Defines the Space Vehicle ID of the satellite to be simulated. This
/// HIPAR spaceVehicleID/value is used to generate the corresponding spreading code.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatSpaceVehicleID(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   spaceVehicleID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, spaceVehicleID, 1, 40),
            4, "Space Vehicle ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SVID %ld\n", satNavStandardArr[standard], satellite, spaceVehicleID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the power offset of the satellite in dB. The offset determines
/// HIFN  the power ratio of the activated satellites.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR power/Queries the power offset of the satellite in dB. The offset determines
/// HIPAR power/the power ratio of the activated satellites.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatPower(ViSession instrSession,
                                                 ViInt32   standard,
                                                 ViInt32   satellite,
                                                 ViReal64* power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:POW?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, power));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the standard chip rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR standardChipRate/Queries the standard chip rate.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatStandardChipRate(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   satellite,
                                                            ViReal64* standardChipRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SCR?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, standardChipRate));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the modulation used for modulating the carrier signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR modulation/Defines the modulation used for modulating the carrier signal.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatModulation(ViSession instrSession,
                                                      ViInt32   standard,
                                                      ViInt32   satellite,
                                                      ViChar    modulation[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:MOD?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, modulation, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Displays the time the satellite's elevation is higher than 2.5, 5, 7.5
/// HIFN  or 10 starting at the first simulation moment of the satellite of
/// HIFN  interest, as selected with the command <subsystem>:SEMask.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR duration/Displays the time the satellite's elevation is higher than 2.5, 5, 7.5
/// HIPAR duration/or 10 starting at the first simulation moment of the satellite of
/// HIPAR duration/interest, as selected with the command <subsystem>:SEMask.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatDuration(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViInt32   satellite,
                                                    ViReal64* duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:DUR?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, duration));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the propagation delay from satellite to receiver in meters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR pseudorange/Defines the propagation delay from satellite to receiver in meters.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatPseudorange(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   satellite,
                                                           ViReal64  pseudorange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:PRAN %.12lG\n", satNavStandardArr[standard], satellite, pseudorange);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets a bias to the pseudorange of a satellite.
/// HIFN     This setting is only available for <subsystem>:SMODeAUTO or USER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR pseudorangeBIAS/Sets a bias to the pseudorange of a satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatPseudorangeBIAS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  pseudorangeBIAS)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, pseudorangeBIAS, -1000.0, 1000.0), 4, "Pseudorange BIAS");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:PRB %.12lG\n", satNavStandardArr[standard], satellite, pseudorangeBIAS);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the propagation delay from satellite to receiver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR timeShift/Defines the propagation delay from satellite to receiver.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatTimeShift(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   satellite,
                                                         ViReal64  timeShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeShift, 0.000, 2499999.99), 4, "Time Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:TSH %.12lG\n", satNavStandardArr[standard], satellite, timeShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the initial carrier phase.
/// HIFN     This setting is only available for <subsystem>:SMODeSTATic. The
/// HIFN  parameter is updated automatically for USER and AUTO.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR initialCarrierPhase/Sets the initial carrier phase.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatInitialCarrierPhase(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  initialCarrierPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, initialCarrierPhase, 0.0, 6.28), 4, "Initial Carrier Phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:ICPH %.12lG\n", satNavStandardArr[standard], satellite, initialCarrierPhase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the currently valid values for Doppler-shifted carrier
/// HIFN  frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR resultingStartFrequency/Queries the currently valid values for Doppler-shifted carrier
/// HIPAR resultingStartFrequency/frequency.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatResultingStartFrequency(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64* resultingStartFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:FREQ?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, resultingStartFrequency));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the currently valid values for the chip rate.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR resultingStartChipRate/Queries the currently valid values for the chip rate.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSatResultingStartChipRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64* resultingStartChipRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:CACR?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, resultingStartChipRate));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Number of navigation pages. Several navigation pages are available per
/// HIFN  satellite.
/// HIFN     If a RINEX file is used, the value of this parameter is retrieved from
/// HIFN  the file. A RINEX file may contain ephemeris information for up to 24
/// HIFN  hours. Since the ephemeris are updated every two hours, the RINEX file
/// HIFN  contains a maximum number of 12 navigation pages.
/// HIFN     This setting is only available for <subsystem>:SMODeUSER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR navigationPages/Number of navigation pages. Several navigation pages are available per
/// HIPAR navigationPages/satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageNavigationPages(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   navigationPages)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, navigationPages, 1, 12),
            4, "Navigation Pages");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:PAGE:COUN %ld\n", satNavStandardArr[standard], svID, navigationPages);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the instrument to internally perform a projection of the
/// HIFN  navigation message whenever this message is approaching its validity
/// HIFN  limit.
/// HIFN     This setting is only available for <subsystem>:SMODe USER and
/// HIFN  <subsystem>:SATellite:GRTProjection OFF.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR realTimeProjection/Enables the instrument to internally perform a projection of the
/// HIPAR realTimeProjection/navigation message whenever this message is approaching its validity
/// HIPAR realTimeProjection/limit.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageRealTimeProjection(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean realTimeProjection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:RTPR %s\n", satNavStandardArr[standard], svID, (realTimeProjection == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the L1-L2 correction term.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tgdCorrection/Defines the L1-L2 correction term.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageTGDCorrectionGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  tgdCorrection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tgdCorrection, -128.0, 127.0), 4, "TGD Correction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:TGD %.12lG\n", satNavStandardArr[standard], svID, tgdCorrection);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the L1-L2 correction term.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tgdCorrection/Defines the L1-L2 correction term.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageTGDCorrectionBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  tgdCorrection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tgdCorrection, -128.0, 127.0), 4, "TGD Correction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:NMES:CCOR:TGD %.12lG\n", satNavStandardArr[standard], svID, tgdCorrection);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the L1-L2 correction term.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tgdCorrection/Defines the L1-L2 correction term.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageTGDCorrectionQZSS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  tgdCorrection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tgdCorrection, -128.0, 127.0), 4, "TGD Correction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:NMES:CCOR:TGD %.12lG\n", satNavStandardArr[standard], svID, tgdCorrection);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIFN     This setting is only available for <subsystem>:SMODeUSER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSetToAlmanac(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the projection of the parameters of the navigation message.
/// HIFN  The values of the TOE and TOC are calculated according to the selected
/// HIFN  current simulation time (for 'System Time UTC').
/// HIFN     This setting is only available for <subsystem>:SMODeUSER.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageProjectNavigationMessage(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:PROJ\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Type of code for L2; This value does not have any impact on the actual
/// HIFN  used ranging code of the generated satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR codeOnL2/Type of code for L2; This value does not have any impact on the actual
/// HIPAR codeOnL2/used ranging code of the generated satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCodeOnL2(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   codeOnL2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, codeOnL2, 0, 2),
            4, "Code On L2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CLTM %s\n", satNavStandardArr[standard], svID, satNavCodeONL2Arr[codeOnL2]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Use of carrier L2 P data flag
/// HIFN     This value does not have an impact on whether data is really
/// HIFN  transmitted on the satellite's carrier L2 or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR l2PDataFlag/Use of carrier L2 P data flag
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageL2PDataFlag(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean l2PDataFlag)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:LTPD %s\n", satNavStandardArr[standard], svID, (l2PDataFlag == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates the curve-fit interval used by the CS (Control Segment) in
/// HIFN  determining the ephemeris parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR fitIntervalFlag/Indicates the curve-fit interval used by the CS (Control Segment) in
/// HIPAR fitIntervalFlag/determining the ephemeris parameters.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageFitIntervalFlag(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean fitIntervalFlag)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:FIFL %s\n", satNavStandardArr[standard], svID, (fitIntervalFlag == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  SV accuracy / URA Index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR uraIndex/SV accuracy / URA Index.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageURAIndex(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   uraIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, uraIndex, 0, 15),
            4, "URA Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:URA %ld\n", satNavStandardArr[standard], svID, uraIndex);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This value does not have an impact on the actual health status of the
/// HIFN  generated satellite.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR svHealth/This value does not have an impact on the actual health status of the
/// HIPAR svHealth/generated satellite.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSVHealth(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   svHealth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svHealth, 0, 31),
            4, "SV Health");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:HEAL %ld\n", satNavStandardArr[standard], svID, svHealth);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Issue of Data, Clock.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR iodc/Issue of Data, Clock.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageIODC(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   iodc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iodc, 0, 1023),
            4, "IODC");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:IODC %ld\n", satNavStandardArr[standard], svID, iodc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Issue of Data, Ephemeris
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR iode/Issue of Data, Ephemeris
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageIODE(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   iode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, iode, 0, 255),
            4, "IODE");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:IODE %ld\n", satNavStandardArr[standard], svID, iode);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Time Of Ephemeris.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR toe/Time Of Ephemeris.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageTOE(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViInt32   toe)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, toe, 0, 65535),
            4, "TOE");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:TOE %ld\n", satNavStandardArr[standard], svID, toe);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Mean Anomaly at Reference Time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR m0/Mean Anomaly at Reference Time.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageM0(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   svID,
                                                         ViInt32   m0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:MZER %ld\n", satNavStandardArr[standard], svID, m0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Mean Motion Difference From Computed Value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR nDelta/Mean Motion Difference From Computed Value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageNDelta(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViReal64  nDelta)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, nDelta, -32768.0, 32767.0), 4, "n Delta");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:NDEL %.12lG\n", satNavStandardArr[standard], svID, nDelta);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Eccentricity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR eccentricity/Eccentricity.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageEccentricity(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  eccentricity)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:ECC %lf\n", satNavStandardArr[standard], svID, eccentricity);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Square Root of the Semi-Major Axis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sqrt_a/Square Root of the Semi-Major Axis
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSQRTA(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViReal64  sqrt_a)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, sqrt_a, 100000, 4294967295),
            4, "SQRT(A)");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:SQRA %.12lG\n", satNavStandardArr[standard], svID, sqrt_a);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Longitude of Ascending Node of Orbit Plane at Weekly Epoch.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR omega0/Longitude of Ascending Node of Orbit Plane at Weekly Epoch.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageOmega0(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViInt32   omega0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:OZER %ld\n", satNavStandardArr[standard], svID, omega0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Inclination Angle at Reference Time
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR i0/Inclination Angle at Reference Time
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagei0(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   svID,
                                                         ViInt32   i0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:IZER %ld\n", satNavStandardArr[standard], svID, i0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Argument of Perigee.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR omega/Argument of Perigee.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageOmega(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   svID,
                                                            ViInt32   omega)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:OMEG %ld\n", satNavStandardArr[standard], svID, omega);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Rate of Right Ascension.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR omegaDOT/Rate of Right Ascension.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageOmegaDOT(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  omegaDOT)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, omegaDOT, -8388608.0, 8388607.0), 4, "Omega DOT");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:ODOT %.12lG\n", satNavStandardArr[standard], svID, omegaDOT);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Rate of Inclination Angle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR idot/Rate of Inclination Angle.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageIDOT(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViReal64  idot)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, idot, -8192.0, 8191.0), 4, "IDOT");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:IDOT %.12lG\n", satNavStandardArr[standard], svID, idot);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Cosine Harmonic Correction Term to the Argument of
/// HIFN  Latitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR cuc/Amplitude of the Cosine Harmonic Correction Term to the Argument of
/// HIPAR cuc/Latitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCuc(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  cuc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, cuc, -32768.0, 32767.0), 4, "Cuc");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CUC %.12lG\n", satNavStandardArr[standard], svID, cuc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Sine Harmonic Correction Term to the Argument of
/// HIFN  Latitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR cus/Amplitude of the Sine Harmonic Correction Term to the Argument of
/// HIPAR cus/Latitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCus(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  cus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, cus, -32768.0, 32767.0), 4, "Cus");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CUS %.12lG\n", satNavStandardArr[standard], svID, cus);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Cosine Harmonic Correction Term to the Orbit Radius.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR crc/Amplitude of the Cosine Harmonic Correction Term to the Orbit Radius.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCrc(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  crc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, crc, -32768.0, 32767.0), 4, "Crc");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CRC %.12lG\n", satNavStandardArr[standard], svID, crc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Sine Harmonic Correction Term to the Orbit Radius.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR crs/Amplitude of the Sine Harmonic Correction Term to the Orbit Radius.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCrs(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  crs)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, crs, -32768.0, 32767.0), 4, "Crs");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CRS %.12lG\n", satNavStandardArr[standard], svID, crs);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Cosine Harmonic Correction Term to the Angle of
/// HIFN  Inclination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR cic/Amplitude of the Cosine Harmonic Correction Term to the Angle of
/// HIPAR cic/Inclination.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCic(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  cic)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, cic, -32768.0, 32767.0), 4, "Cic");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CIC %.12lG\n", satNavStandardArr[standard], svID, cic);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Amplitude of the Sine Harmonic Correction Term to the Angle of
/// HIFN  Inclination.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR cis/Amplitude of the Sine Harmonic Correction Term to the Angle of
/// HIPAR cis/Inclination.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageCis(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViReal64  cis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, cis, -32768.0, 32767.0), 4, "Cis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:CIS %.12lG\n", satNavStandardArr[standard], svID, cis);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  SF1 Reserved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sf1Reserved1/SF1 Reserved.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSF1Reserved1(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   sf1Reserved1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sf1Reserved1, 0, 255),
            4, "SF1 Reserved 1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:SF1R1 %ld\n", satNavStandardArr[standard], svID, sf1Reserved1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  SF1 Reserved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sf1Reserved2/SF1 Reserved.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSF1Reserved2(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   sf1Reserved2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sf1Reserved2, 0, 255),
            4, "SF1 Reserved 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:SF1R2 %ld\n", satNavStandardArr[standard], svID, sf1Reserved2);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  SF1 Reserved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sf1Reserved3/SF1 Reserved.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSF1Reserved3(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   sf1Reserved3)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sf1Reserved3, 0, 255),
            4, "SF1 Reserved 3");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:SF1R3 %ld\n", satNavStandardArr[standard], svID, sf1Reserved3);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  SF1 Reserved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sf1Reserved4/SF1 Reserved.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSF1Reserved4(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   sf1Reserved4)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sf1Reserved4, 0, 255),
            4, "SF1 Reserved 4");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:SF1R4 %ld\n", satNavStandardArr[standard], svID, sf1Reserved4);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Age of Data Offset
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR aodo/Age of Data Offset
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageAODO(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViInt32   aodo)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, aodo, 0, 31),
            4, "AODO");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:EPH:AODO %ld\n", satNavStandardArr[standard], svID, aodo);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Issue Of Data (Almanacs)
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR ioda/Issue Of Data (Almanacs)
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageIODAGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   ioda)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, ioda, 0, 1023),
            4, "IODA");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:NMES:EPH:IODA %ld\n", satNavStandardArr[standard], svID, ioda);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagePresetGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagePresetGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagePresetGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagePresetBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  The navigation message's parameters are calculated according to the
/// HIFN  selected almanac.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessagePresetQZSS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:QZSS:NMES:PRES\n", satNavStandardArr[standard], svID);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Signal In Space Accuracy
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR sisa/Signal In Space Accuracy.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatMessageSignalInSpaceAccuracy(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   sisa)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sisa, 0, 15),
            4, "SISA");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:NMES:EPH:SISA %ld\n", satNavStandardArr[standard], svID, sisa);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the L1-L2 Correction Term. As this parameter affects the
/// HIFN  satellite's orbit, the value selected here also has an impact on the
/// HIFN  satellite's parameters 'Time Shift', 'Doppler Shift' and 'Duration
/// HIFN  (Elevation>2.5/5/7.5/10)'.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR tgd/Defines the L1-L2 Correction Term. As this parameter affects the
/// HIPAR tgd/satellite's orbit, the value selected here also has an impact on the
/// HIPAR tgd/satellite's parameters 'Time Shift', 'Doppler Shift' and 'Duration
/// HIPAR tgd/(Elevation>2.5/5/7.5/10)'.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatClockTGD(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViReal64  tgd)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, tgd, -128.0, 127.0), 4, "TGD");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:TGD %.12lG\n", satNavStandardArr[standard], svID, tgd);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the Clock Correction Parameter. As this parameter affects the
/// HIFN  satellite's orbit, the value selected here also has an impact on the
/// HIFN  satellite's parameters 'Time Shift', 'Doppler Shift' and 'Duration
/// HIFN  (Elevation>2.5/5/7.5/10)'
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR toc/Defines the Clock Correction Parameter. As this parameter affects the
/// HIPAR toc/satellite's orbit, the value selected here also has an impact on the
/// HIPAR toc/satellite's parameters 'Time Shift', 'Doppler Shift' and 'Duration
/// HIPAR toc/(Elevation>2.5/5/7.5/10)'
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatClocktOC(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViInt32   toc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, toc, 0, 65535),
            4, "tOC");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:TOC %ld\n", satNavStandardArr[standard], svID, toc);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clock Correction parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR af0/Clock Correction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatClockaf0(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViReal64  af0)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, af0, -128.0, 127.0), 4, "af0");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:AF0 %.12lG\n", satNavStandardArr[standard], svID, af0);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clock Correction parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR af1/Clock Correction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatClockaf1(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViReal64  af1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, af1, -128.0, 127.0), 4, "af1");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:AF1 %.12lG\n", satNavStandardArr[standard], svID, af1);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clock Correction parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR af2/Clock Correction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatClockaf2(ViSession instrSession,
                                                        ViInt32   standard,
                                                        ViInt32   svID,
                                                        ViReal64  af2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, af2, -128.0, 127.0), 4, "af2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:NMES:CCOR:AF2 %.12lG\n", satNavStandardArr[standard], svID, af2);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the receiver attitude parameters yaw, pitch, roll in degrees
/// HIFN  for the selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR yaw/Returns the Yaw parameter.
/// HIPAR pitch/Returns the Pitch parameter.
/// HIPAR roll/Returns the Roll parameter.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTAttitude(ViSession instrSession,
                                                   ViInt32   standard,
                                                   ViInt32   timeBasis,
                                                   ViInt32   year,
                                                   ViInt32   month,
                                                   ViInt32   day,
                                                   ViInt32   hour,
                                                   ViInt32   minutes,
                                                   ViReal64  seconds,
                                                   ViInt32   weekNumber,
                                                   ViReal64  timeOfWeek,
                                                   ViReal64* yaw,
                                                   ViReal64* pitch,
                                                   ViReal64* roll)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:RATT? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", satNavStandardArr[standard], timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", yaw, pitch, roll);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in decimal format for
/// HIFN  the selected moment of time. The required query parameters depend on
/// HIFN  the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitude/Queries the satellite longitude in the selected moment of time.
/// HIPAR latitude/Queries the satellite latitude in the selected moment of time.
/// HIPAR altitude/Queries the satellite altitude in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTDecimal(ViSession instrSession,
                                                  ViInt32   standard,
                                                  ViInt32   timeBasis,
                                                  ViInt32   year,
                                                  ViInt32   month,
                                                  ViInt32   day,
                                                  ViInt32   hour,
                                                  ViInt32   minutes,
                                                  ViReal64  seconds,
                                                  ViInt32   weekNumber,
                                                  ViReal64  timeOfWeek,
                                                  ViReal64* longitude,
                                                  ViReal64* latitude,
                                                  ViReal64* altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:RLOC:COOR:DEC? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", satNavStandardArr[standard], timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%lf,%lf,%lf", longitude, latitude, altitude);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the coordinates of the receiver location in DMS format for the
/// HIFN  selected moment of time. The required query parameters depend on the
/// HIFN  selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR longitudeDeg/Queries the satellite longitude degrees.
/// HIPAR longitudeMin/Queries the satellite longitude minutes in the selected moment of
/// HIPAR longitudeMin/time.
/// HIPAR longitudeSec/Queries the satellite longitude seconds in the selected moment of
/// HIPAR longitudeSec/time.
/// HIPAR longitudeDir/Queries the satellite longitude direction in the selected moment of
/// HIPAR longitudeDir/time.
/// HIPAR latitudeDeg/Queries the satellite latitude degrees in the selected moment of time.
/// HIPAR latitudeMin/Queries the satellite latitude minutes in the selected moment of time.
/// HIPAR latitudeSec/Queries the satellite latitude seconds in the selected moment of time.
/// HIPAR latitudeDir/Queries the satellite latitude direction in the selected moment of
/// HIPAR latitudeDir/time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTDMS(ViSession instrSession,
                                              ViInt32   standard,
                                              ViInt32   timeBasis,
                                              ViInt32   year,
                                              ViInt32   month,
                                              ViInt32   day,
                                              ViInt32   hour,
                                              ViInt32   minutes,
                                              ViReal64  seconds,
                                              ViInt32   weekNumber,
                                              ViReal64  timeOfWeek,
                                              ViInt32*  longitudeDeg,
                                              ViInt32*  longitudeMin,
                                              ViReal64* longitudeSec,
                                              ViChar    longitudeDir[],
                                              ViInt32*  latitudeDeg,
                                              ViInt32*  latitudeMin,
                                              ViReal64* latitudeSec,
                                              ViChar    latitudeDir[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:RLOC:COOR:DMS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf", satNavStandardArr[standard], timeBasisArr[timeBasis], year, month, day, hour, minutes, seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));
    (void)sscanf(response, "%ld,%ld,%lf,%s,%ld,%ld,%lf,%s", longitudeDeg, longitudeMin, longitudeSec, longitudeDir, latitudeDeg, latitudeMin, latitudeSec, latitudeDir);

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Switches between the available views.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR displayType/Switches between the available views.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavRTDisplayType(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   displayType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, displayType, 0, 4),
            3, "Display Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SPOT:DISP %s\n", satNavStandardArr[standard], satNavRTDisplayTypeArr[displayType]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the radius of the display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mapDeviation/Queries the radius of the display.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTMapDeviation(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32*  mapDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SPOT:MAP:DEV?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, mapDeviation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite azimuth in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR azimuth/Queries the satellite azimuth in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTAzimuth(ViSession instrSession,
                                                  ViInt32   standard,
                                                  ViInt32   svid,
                                                  ViInt32   system,
                                                  ViInt32   timeBasis,
                                                  ViInt32   year,
                                                  ViInt32   month,
                                                  ViInt32   day,
                                                  ViInt32   hour,
                                                  ViInt32   minutes,
                                                  ViReal64  seconds,
                                                  ViInt32   weekNumber,
                                                  ViReal64  timeOfWeek,
                                                  ViReal64* azimuth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:%s:SVID%ld:AZIM? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf\n", satNavStandardArr[standard], satNavStandardArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, azimuth));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the satellite's elevation in the selected moment of time. The
/// HIFN  required query parameters, depend on the selected time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR elevation/Queries the satellite's elevation in the selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTElevation(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViInt32   svid,
                                                    ViInt32   system,
                                                    ViInt32   timeBasis,
                                                    ViInt32   year,
                                                    ViInt32   month,
                                                    ViInt32   day,
                                                    ViInt32   hour,
                                                    ViInt32   minutes,
                                                    ViReal64  seconds,
                                                    ViInt32   weekNumber,
                                                    ViReal64  timeOfWeek,
                                                    ViReal64* elevation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            5, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            6, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            7, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            8, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            9, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            10, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 11, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            12, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 13, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:%s:SVID%ld:ELEV? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf\n", satNavStandardArr[standard], satNavStandardArr[system], svid, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, elevation));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the absolute power of a specified Tap of a specific satellite
/// HIFN  at a selected moment of time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svid/Sets the Satellite ID.
/// HIPAR system/Sets the SatNav System.
/// HIPAR tap/Sets the Tap value.
/// HIPAR timeBasis/Sets the selected Time Basis.
/// HIPAR year/Sets the Year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
/// HIPAR hour/Sets the Hour.
/// HIPAR minutes/Sets the minutes.
/// HIPAR seconds/Sets the Seconds.
/// HIPAR weekNumber/Sets the Week Number.
/// HIPAR timeOfWeek/Sets the Time Of Week.
/// HIPAR absolutePower/Queries the absolute power of a specified Tap of a specific satellite
/// HIPAR absolutePower/at a selected moment of time.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavRTPower(ViSession instrSession,
                                                ViInt32   standard,
                                                ViInt32   svid,
                                                ViInt32   system,
                                                ViInt32   tap,
                                                ViInt32   timeBasis,
                                                ViInt32   year,
                                                ViInt32   month,
                                                ViInt32   day,
                                                ViInt32   hour,
                                                ViInt32   minutes,
                                                ViReal64  seconds,
                                                ViInt32   weekNumber,
                                                ViReal64  timeOfWeek,
                                                ViReal64* absolutePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svid, 1, 50),
            3, "SVID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 3),
            4, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            5, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 3),
            6, "Time Basis");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            7, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            8, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            9, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            10, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minutes, 0, 59),
            11, "Minutes");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 12, "Seconds");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 529947),
            13, "Week Number");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 14, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:RT:%s:SVID%ld:TAP%ld:POW:ABS? %s,%ld,%ld,%ld,%ld,%ld,%lf,%ld,%lf\n", satNavStandardArr[standard], satNavStandardArr[system], svid, tap, timeBasisArr[timeBasis], year, month, day, hour, minutes,
        seconds, weekNumber, timeOfWeek);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, absolutePower));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the type of assistance data to be loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mode/Defines the type of assistance data to be loaded.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGMode(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 2),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:MODE %s\n", satNavStandardArr[standard], satNavStandardArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the affected parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSynchronizeAll(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GPS:SYNC\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the SVID parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR standard2/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSynchronizeSVIDGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   standard2)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard2, 0, 4),
            3, "Standard 2");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:SVID:SYNC\n", satNavStandardArr[standard], satNavStandardArr[standard2]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the Location parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGRefSynchronizeLocation(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   system)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:LOC:SYNC\n", satNavStandardArr[standard], satNavStandardArr[system]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGRefCoordinatesDecimal(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   system,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -10000.0, 1600000.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:LOC:COOR:DEC %lf,%lf,%lf", satNavStandardArr[standard], satNavStandardArr[system], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the geographic reference location in degrees, minutes and
/// HIFN  seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGRefCoordinatesDMS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   system,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", satNavStandardArr[standard], satNavStandardArr[system], longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Uncertainty Radius, i.e. sets the maximum radius of the area
/// HIFN  within which the two dimensional location of the UE is bounded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR uncertainlyRadius/Sets the Uncertainty Radius, i.e. sets the maximum radius of the area
/// HIPAR uncertainlyRadius/within which the two dimensional location of the UE is bounded.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGRefUncertainlyRadius(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   system,
                            ViReal64  uncertainlyRadius)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, uncertainlyRadius, 0.0, 1.0e+6), 4, "Uncertainly Radius");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:LOC:URAD %.12lG\n", satNavStandardArr[standard], satNavStandardArr[system], uncertainlyRadius);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Synchronizes the Assistance Data parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOASynchronize(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   system)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:SYNC\n", satNavStandardArr[standard], satNavStandardArr[system]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Determines the time basis used to enter the time of assistance data
/// HIFN  parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR timeBasis/Determines the time basis used to enter the time of assistance data
/// HIPAR timeBasis/parameters.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOATimeBasis(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   system,
                                                            ViInt32   timeBasis)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeBasis, 0, 4),
            4, "Time Basis");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:TBAS %s\n", satNavStandardArr[standard], satNavStandardArr[system], ADGTimeBasisArr[timeBasis]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enabled for GPS time basis
/// HIFN  (<subsystem>:ADGeneration:GPS:TOAData:TBASis).
/// HIFN     Sets the week number (WN) the assistance data is generated for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Enabled for GPS time basis
/// HIPAR weekNumber/(<subsystem>:ADGeneration:GPS:TOAData:TBASis).
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGTOAWeekNumber(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32   system,
                                                         ViInt32*  weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:WNUM?\n", satNavStandardArr[standard], satNavStandardArr[system]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, weekNumber));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enabled for GPS time basis
/// HIFN  (<subsystem>:ADGeneration:GPS:TOAData:TBASis).
/// HIFN     Determines the Time of Week (TOW) the assistance data is generated
/// HIFN  for.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR timeOfWeek/Enabled for GPS time basis
/// HIPAR timeOfWeek/(<subsystem>:ADGeneration:GPS:TOAData:TBASis).
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOATimeOfWeek(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   system,
                                                             ViReal64  timeOfWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, -604800.0, 604800.0), 4, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:TOW %.12lG\n", satNavStandardArr[standard], satNavStandardArr[system], timeOfWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the duration of the assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR duration/Sets the duration of the assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOADuration(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   system,
                                                           ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, duration, 1.0e-3, 5.0e+3), 4, "Duration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:DUR %.12lG\n", satNavStandardArr[standard], satNavStandardArr[system], duration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the resolution of the assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR resolution/Sets the resolution of the assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOAResolutionGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, resolution, 1.0e-3, 5.0), 3, "Resolution");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GPS:TOAD:RES %.12lG\n", satNavStandardArr[standard], resolution);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the resolution of the assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR resolution/Sets the resolution of the assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOAResolutionGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  resolution)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, resolution, 1.0e-3, 5.0), 3, "Resolution");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GAL:TOAD:RES %.12lG\n", satNavStandardArr[standard], resolution);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the date for the assistance data in DMS format of the Gregorian
/// HIFN  calendar.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR year/Sets the year parameter.
/// HIPAR month/Sets the month parameter.
/// HIPAR day/Sets the day parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOADate(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32   system,
                                                       ViInt32   year,
                                                       ViInt32   month,
                                                       ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            4, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            5, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            6, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:DATE %ld,%ld,%ld\n", satNavStandardArr[standard], satNavStandardArr[system], year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enters the exact start time for the assistance data in UTC time
/// HIFN  format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR hour/Sets the hour parameter.
/// HIPAR minute/Sets the minute parameter.
/// HIPAR seconds/Sets the seconds parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGTOATime(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViInt32   system,
                                                       ViInt32   hour,
                                                       ViInt32   minute,
                                                       ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            4, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            5, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 6, "Seconds");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:%s:TOAD:TIME %ld,%ld,%.12lG\n", satNavStandardArr[standard], satNavStandardArr[system], hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables a space vehicle with the selected SV ID for
/// HIFN  generation of assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables a space vehicle with the selected SV ID for
/// HIPAR enabled/generation of assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSVIDEnabledGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GPS:SVID%ld:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables a space vehicle with the selected SV ID for
/// HIFN  generation of assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables a space vehicle with the selected SV ID for
/// HIPAR enabled/generation of assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSVIDEnabledGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GAL:SVID%ld:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables a space vehicle with the selected SV ID for
/// HIFN  generation of assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables a space vehicle with the selected SV ID for
/// HIPAR enabled/generation of assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSVIDEnabledGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GLON:SVID%ld:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables a space vehicle with the selected SV ID for
/// HIFN  generation of assistance data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables/disables a space vehicle with the selected SV ID for
/// HIPAR enabled/generation of assistance data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGSVIDEnabledBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:BEID:SVID%ld:STAT %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries part of the current assistance data settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDAcquisitionBlockGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GPS:SVID%ld:ACQ:BLOC? %.12lG\n", satNavStandardArr[standard], svID, timeOffset);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries part of the current assistance data settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDAcquisitionBlockBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:BEID:SVID%ld:ACQ:BLOC? %.12lG\n", satNavStandardArr[standard], svID, timeOffset);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries selected number of symbols from the sensitivity assistance
/// HIFN  message for a specific satellite (SV ID) and at given specific time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR numberOfSymbols/Determines the length of the sensitivity assistance data to be
/// HIPAR numberOfSymbols/generated. The maximum length of the generated sensitivity assistance
/// HIPAR numberOfSymbols/data is then 125 subframes, i.e. 25 pages or one complete navigation
/// HIPAR numberOfSymbols/message.
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDSensitivityMessageGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViInt32   numberOfSymbols,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GPS:SVID%ld:SAD? %.12lG,%ld\n", satNavStandardArr[standard], svID, timeOffset, numberOfSymbols);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries selected number of symbols from the sensitivity assistance
/// HIFN  message for a specific satellite (SV ID) and at given specific time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR numberOfSymbols/Determines the length of the sensitivity assistance data to be
/// HIPAR numberOfSymbols/generated. The maximum length of the generated sensitivity assistance
/// HIPAR numberOfSymbols/data is then 125 subframes, i.e. 25 pages or one complete navigation
/// HIPAR numberOfSymbols/message.
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDSensitivityMessageGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViInt32   numberOfSymbols,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GAL:SVID%ld:SAD? %.12lG,%ld\n", satNavStandardArr[standard], svID, timeOffset, numberOfSymbols);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries selected number of symbols from the sensitivity assistance
/// HIFN  message for a specific satellite (SV ID) and at given specific time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR numberOfSymbols/Determines the length of the sensitivity assistance data to be
/// HIPAR numberOfSymbols/generated. The maximum length of the generated sensitivity assistance
/// HIPAR numberOfSymbols/data is then 125 subframes, i.e. 25 pages or one complete navigation
/// HIPAR numberOfSymbols/message.
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDSensitivityMessageGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViInt32   numberOfSymbols,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:GLON:SVID%ld:SAD? %.12lG,%ld\n", satNavStandardArr[standard], svID, timeOffset, numberOfSymbols);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries selected number of symbols from the sensitivity assistance
/// HIFN  message for a specific satellite (SV ID) and at given specific time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR timeOffset/
/// HIPAR numberOfSymbols/Determines the length of the sensitivity assistance data to be
/// HIPAR numberOfSymbols/generated. The maximum length of the generated sensitivity assistance
/// HIPAR numberOfSymbols/data is then 125 subframes, i.e. 25 pages or one complete navigation
/// HIPAR numberOfSymbols/message.
/// HIPAR block/Queries part of the current assistance data settings.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavADGSVIDSensitivityMessageBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViReal64  timeOffset,
                            ViInt32   numberOfSymbols,
                            ViChar    block[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:BEID:SVID%ld:SAD? %.12lG,%ld\n", satNavStandardArr[standard], svID, timeOffset, numberOfSymbols);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, block, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected almanac
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as almanac file with the specific file extensions
/// HIFN  *.rs_al or *.rs_yuma.
/// HIFN     The default extension is *.rs_al and can be omitted in the file name.
/// HIFN     To save an almanac file as file with extension *.rs_yuma, however,
/// HIFN  this extension has to be specified in the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGGenerateAlmanacFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:ALM:CRE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGGenerateNavigationFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:NAV:CRE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  navigation file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as navigation file with the specific file
/// HIFN  extensions *.rs_nav.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGGenerateAcquisitionFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:ACQ:CRE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected
/// HIFN  ionospheric file. The directory is set using command MMEM:CDIRectory. A
/// HIFN  path can also be specified, in which case the files in the specified
/// HIFN  directory are read. Only the file name has to be entered. Assistance
/// HIFN  data settings are stored as ionospheric file with the specific file
/// HIFN  extensions *.rs_ion.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGGenerateIonosphericFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:ION:CRE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current assistance data settings into the selected UTC
/// HIFN  file. The directory is set using command MMEM:CDIRectory. A path can
/// HIFN  also be specified, in which case the files in the specified directory
/// HIFN  are read. Only the file name has to be entered. Assistance data
/// HIFN  settings are stored as UTC file with the specific file extensions
/// HIFN  *.rs_utc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavADGGenerateUTCFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:ADG:UTC:CRE '%s'\n", satNavStandardArr[standard], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the configuration of all SV IDs simultaneously. Current
/// HIFN  settings are overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR coupling/Enables the configuration of all SV IDs simultaneously. Current
/// HIPAR coupling/settings are overwritten.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalSignalCoupling(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean coupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID:SIGN:COUP %s\n", satNavStandardArr[standard], (coupling == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of signal a SV ID will use in the moment the
/// HIFN  corresponding satellite becomes visible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR signal/Sets the type of signal a SV ID will use in the moment the
/// HIPAR signal/corresponding satellite becomes visible.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalSignalGPS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, 0, 3),
            4, "Signal");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GPS:SIGN %s\n", satNavStandardArr[standard], svID, satNavSATSignalArr[signal]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of signal a SV ID will use in the moment the
/// HIFN  corresponding satellite becomes visible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR signal/Sets the type of signal a SV ID will use in the moment the
/// HIPAR signal/corresponding satellite becomes visible.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalSignalGALILEO(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, 0, 3),
            4, "Signal");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GAL:SIGN %s\n", satNavStandardArr[standard], svID, satNavSATSignalArr[signal]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of signal a SV ID will use in the moment the
/// HIFN  corresponding satellite becomes visible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR signal/Sets the type of signal a SV ID will use in the moment the
/// HIPAR signal/corresponding satellite becomes visible.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalSignalGLONASS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, 0, 3),
            4, "Signal");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:GLON:SIGN %s\n", satNavStandardArr[standard], svID, satNavSATSignalArr[signal]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the type of signal a SV ID will use in the moment the
/// HIFN  corresponding satellite becomes visible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR signal/Sets the type of signal a SV ID will use in the moment the
/// HIPAR signal/corresponding satellite becomes visible.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalSignalBeiDou(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViInt32   signal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signal, 0, 3),
            4, "Signal");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:BEID:SIGN %s\n", satNavStandardArr[standard], svID, satNavSATSignalArr[signal]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the power a SV ID will use the moment the corresponding satellite
/// HIFN  becomes visible and is simulated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR system/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR power/Sets the power a SV ID will use the moment the corresponding satellite
/// HIPAR power/becomes visible and is simulated.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatGlobalPower(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   system,
                                                           ViInt32   svID,
                                                           ViReal64  power)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, system, 0, 4),
            3, "System");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            4, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SVID%ld:%s:POW %.12lG\n", satNavStandardArr[standard], svID, satNavStandardArr[system], power);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a Doppler profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR config/Selects a Doppler profile.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsConfig(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   config)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, config, 0, 2),
            4, "Config");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:CONF %s\n", satNavStandardArr[standard], satellite, satNavSDynConfigArr[config]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum acceleration of a predefined profile, or defines a
/// HIFN  user-specific value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR maxAcceleration/Sets the maximum acceleration of a predefined profile, or defines a
/// HIPAR maxAcceleration/user-specific value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsMaxAcceleration(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  maxAcceleration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, maxAcceleration, 0.01, 1000.0), 4, "Max Acceleration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:ACC:MAX %.12lG\n", satNavStandardArr[standard], satellite, maxAcceleration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant acceleration period of a predefined profile, or sets
/// HIFN  a user-specific value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR accelerationPeriod/Sets the constant acceleration period of a predefined profile, or sets
/// HIPAR accelerationPeriod/a user-specific value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsConstantAccelerationPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  accelerationPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, accelerationPeriod, 0.01, 10800.0), 4, "Acceleration Period");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:CAP %.12lG\n", satNavStandardArr[standard], satellite, accelerationPeriod);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the constant velocity period of a predefined profile, or sets a
/// HIFN  user-specific value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR velocityPeriod/Sets the constant velocity period of a predefined profile, or sets a
/// HIPAR velocityPeriod/user-specific value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsConstantVelocityPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  velocityPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, velocityPeriod, 0.01, 10800.0), 4, "Velocity Period");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:CVP %.12lG\n", satNavStandardArr[standard], satellite, velocityPeriod);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the Doppler shift for a constant signal profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR dopplerShift/Sets the Doppler shift for a constant signal profile.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsDopplerShift(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  dopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dopplerShift, -100.0e+3, 100.0e+3), 4, "Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:DSH %.12lG\n", satNavStandardArr[standard], satellite, dopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the unit for displaying the signal in the graph.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR dopplerShiftUnit/Selects the unit for displaying the signal in the graph.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsDopplerShiftUnit(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   dopplerShiftUnit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dopplerShiftUnit, 0, 1),
            4, "Doppler Shift Unit");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:DSH:UNIT %s\n", satNavStandardArr[standard], satellite, satNavDynDopplerShiftUnitArr[dopplerShiftUnit]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the doppler shift of a predefined high order profile. For
/// HIFN  user-defined dynamics profiles, you can determine the parameter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR initDopplerShift/Sets the doppler shift of a predefined high order profile. For
/// HIPAR initDopplerShift/user-defined dynamics profiles, you can determine the parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsInitDopplerShift(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  initDopplerShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, initDopplerShift, -19042.0, 19042.0), 4, "Init Doppler Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:IDSH %.12lG\n", satNavStandardArr[standard], satellite, initDopplerShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the maximum jerk of a predefined profile, or determines a
/// HIFN  user-specific value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR maximumJerk/Sets the maximum jerk of a predefined profile, or determines a
/// HIPAR maximumJerk/user-specific value.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsMaximumJerk(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  maximumJerk)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, maximumJerk, 0.1, 7.0e+4), 4, "Maximum Jerk");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:JERK:MAX %.12lG\n", satNavStandardArr[standard], satellite, maximumJerk);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a signal dynamics profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR profile/Selects a signal dynamics profile.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsProfile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   profile)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, profile, 0, 1),
            4, "Profile");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:PROF %s\n", satNavStandardArr[standard], satellite, satNavDynProfileArr[profile]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the time that elapses until the Doppler signal of a high order
/// HIFN  velocity profile repeats.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR dopplerPeriod/Queries the time that elapses until the Doppler signal of a high order
/// HIPAR dopplerPeriod/velocity profile repeats.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSignalDynamicsDopplerPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64* dopplerPeriod)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:RPER?\n", satNavStandardArr[standard], satellite);
    checkErr(RsCore_QueryViReal64(instrSession, cmd, dopplerPeriod));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets a time delay for the Doppler signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR startTimeOffset/Sets a time delay for the Doppler signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSignalDynamicsStartTimeOffset(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViReal64  startTimeOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, startTimeOffset, 0.0, 90000.0), 4, "Start Time Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:SDYN:TOFF %.12lG\n", satNavStandardArr[standard], satellite, startTimeOffset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the data rate of the satellite navigation signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR dataRate/Sets the data rate of the satellite navigation signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavModulationControlDataRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   dataRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, dataRate, 0, 3),
            4, "Data Rate");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:MCON:DRAT %s\n", satNavStandardArr[standard], satellite, satNavMConDataRateArr[dataRate]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables meandering, i.e. doubling the data rate of a
/// HIFN  GLONASS satellite navigation signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR meandering/Disables or enables meandering, i.e. doubling the data rate of a
/// HIPAR meandering/GLONASS satellite navigation signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavModulationControlMeanderingGLONASS(
                            ViSession instrSession,
                            ViInt32   satellite,
                            ViBoolean meandering)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            2, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:GLON:SAT%ld:MCON:MEAN %s\n", satellite, (meandering == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the time signal component of GLONASS signals.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR timeSequence/Disables or enables the time signal component of GLONASS signals.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavModulationControlTimeSequenceGLONASS(
                            ViSession instrSession,
                            ViInt32   satellite,
                            ViBoolean timeSequence)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            2, "Satellite");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:GLON:SAT%ld:MCON:TSEQ %s\n", satellite, (timeSequence == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables or enables the secondary code in the pilot channel of
/// HIFN  Galileo, or in the D1 Navigation Message for BeiDou.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR secondaryCode/Disables or enables the secondary code in the pilot channel of
/// HIPAR secondaryCode/Galileo, or in the D1 Navigation Message for BeiDou.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavModulationControlSecondaryCode(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   svID,
                            ViBoolean secondaryCode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            4, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:MCON:SEC%ld %s\n", satNavStandardArr[standard], satellite, svID, (secondaryCode == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables/enables the spreading signal component of a satellite
/// HIFN  navigation signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR satellite/Sets the Satellite parameter.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR spreading/Disables/enables the spreading signal component of a satellite
/// HIPAR spreading/navigation signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavModulationControlSpreading(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   satellite,
                            ViInt32   svID,
                            ViBoolean spreading)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, satellite, 1, 24),
            3, "Satellite");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            4, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:SAT%ld:MCON:SPR%ld %s\n", satNavStandardArr[standard], satellite, svID, (spreading == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables loading the dedicated files as source for the navigation data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables loading the dedicated files as source for the navigation data.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConstellationImportDedicatedFilesEnabled(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:%s:UDS %s", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, enabled ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the file from that the navigation data is extracted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/Sets the file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConstellationImportNavigationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:%s:FILE:NMES '%s'", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the file from that the satellites constellation and navigation
/// HIFN  data are extracted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileName/Sets the file name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConstellationImportConstellationFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:%s:FILE:CONS '%s'", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Triggers the import of constellation and navigation data from the
/// HIFN  selected files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConstellationImportExecute(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:%s:EXEC", path, RsRFSigGen_rngSatNavStandard.entries[standard].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Enables or disables the use constellation source file for all
///HINF  active GNSS systems.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR standard/Sets the Satellite Navigation standard.
///HIPAR enabled/Enables or disables the use constellation source file
///HIPAR enabled/for all active GNSS systems.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSatConstellationImportUseForAllSystems(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, RSRFSIGGEN_VAL_SATNAV_STANDARD_GPS, RSRFSIGGEN_VAL_SATNAV_STANDARD_NAVIC),
            3, "Standard");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSatNavStandard, standard, &p2string),
        3, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:%s:UALL %s", path, p2string, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mode/Sets the logging mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogMode(ViSession instrSession,
                                                    ViInt32   standard,
                                                    ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:MODE %s\n", satNavStandardArr[standard], satNavLogModeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries that the logged data is stored into files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR destination/Queries that the logged data is stored into files.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogDestination(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViChar    destination[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:DEST?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, destination, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the storage place.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR fileDirectory/Sets the storage place.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogDestinationFileDirectory(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  fileDirectory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:DEST:FILE:DIR '%s'\n", satNavStandardArr[standard], fileDirectory);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the file prefix.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR prefix/Sets the file prefix.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogDestinationFilePrefix(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViString  prefix)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:DEST:FILE:BNAM '%s'\n", satNavStandardArr[standard], prefix);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a time stamp to the file name.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeStamp/Adds a time stamp to the file name.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogDestinationFileTimeStamp(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean timeStamp)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:DEST:FILE:TAPP:STAT %s\n", satNavStandardArr[standard], (timeStamp == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the file format in that the logged data is stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR format/Sets the file format in that the logged data is stored.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogSatelliteFormat(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32*  format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:FORM?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, format));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the logging of the selected category.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the logging of the selected category.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogSatelliteEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the file name of the file with logged data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR file/Queries the file name of the file with logged data.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogSatelliteFile(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:FILE?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR step/Sets the logging step.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogSatelliteStep(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   step)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, step, 0, 6),
            3, "Step");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:STEP %s\n", satNavStandardArr[standard], satNavLogUMotionStepArr[step]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the file format in that the logged data is stored.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR format/Sets the file format in that the logged data is stored.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogUMotionFormat(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViInt32*  format)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:FORM?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, format));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the logging of the selected category.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the logging of the selected category.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogUMotionEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the file name of the file with logged data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR file/Sets the file name of the file with logged data.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogUMotionFile(ViSession instrSession,
                                                       ViInt32   standard,
                                                       ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:FILE?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR step/Sets the logging step.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogUMotionStep(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   step)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, step, 0, 6),
            3, "Step");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:STEP %s\n", satNavStandardArr[standard], satNavLogUMotionStepArr[step]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables all of the available SV IDs
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogUMotionEnableAllSVID(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:SEL:ALL\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables all of the available SV IDs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogUMotionDisableAllSVID(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:SEL:NONE\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables all of the available SV IDs
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogSatelliteEnableAllSVID(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:SEL:ALL\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Disables all of the available SV IDs.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogSatelliteDisableAllSVID(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:SEL:NONE\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogBeiDouEnabled(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViInt32   svID,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:BEID:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogGALILEOEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:GAL:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogGLONASSEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   svID,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:GLON:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogQZSSEnabled(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:QZSS:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogSBASEnabled(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   svID,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:SBAS:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a space vehicle with the selected SV ID for
/// HIFN  logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR svID/Sets the SV ID parameter.
/// HIPAR enabled/Enables or disables a space vehicle with the selected SV ID for
/// HIPAR enabled/logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogGPSEnabled(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   svID,
                                                          ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, svID, 1, 50),
            3, "SV ID");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:GPS:SVID%ld %s\n", satNavStandardArr[standard], svID, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Logging files are created and saved. Files with the same name are
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineAbort(ViSession instrSession,
                                                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:ABOR\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Logging files are created and saved. Files with the same name are
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineGenerate(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:GEN\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging time to the simulation start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineSynchronize(
                            ViSession instrSession,
                            ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:SYNC\n", satNavStandardArr[standard]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the time basis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR systemTime/Sets the time basis.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineTimeBasis(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   systemTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, systemTime, 0, 4),
            3, "System Time");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:TBAS %s\n", satNavStandardArr[standard], ADGTimeBasisArr[systemTime]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging start date.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR year/Sets the year.
/// HIPAR month/Sets the Month.
/// HIPAR day/Sets the Day.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineDate(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   year,
                                                           ViInt32   month,
                                                           ViInt32   day)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, year, 1980, 9999),
            3, "Year");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            4, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            5, "Day");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:DATE %ld,%ld,%ld\n", satNavStandardArr[standard], year, month, day);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging start time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR hour/Sets the Hour.
/// HIPAR minute/Sets the Minute.
/// HIPAR seconds/Sets the Seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineTime(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32   hour,
                                                           ViInt32   minute,
                                                           ViReal64  seconds)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            3, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            4, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, seconds, 0.0, 59.999), 5, "Seconds");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:TIME %ld,%ld,%.12lG\n", satNavStandardArr[standard], hour, minute, seconds);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging start week number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR weekNumber/Sets the logging start week number.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineWeekNumber(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   weekNumber)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weekNumber, 0, 9999),
            3, "Week Number");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:WNUM %ld\n", satNavStandardArr[standard], weekNumber);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the start time within the selected week number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR timeOfWeek/Sets the start time within the selected week number.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineTimeOfWeek(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  timeOfWeek)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, timeOfWeek, 0.0, 604799.999), 3, "Time Of Week");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:TOW %.12lG\n", satNavStandardArr[standard], timeOfWeek);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the logging duration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR duration/Sets the logging duration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogOfflineDuration(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViReal64  duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, duration, 0.0, 864000.0), 3, "Duration");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:DUR %.12lG\n", satNavStandardArr[standard], duration);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Querries the progress of the offline data logging generation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR progress/Querries the progress of the offline data logging generation.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavLogOfflineProgress(ViSession instrSession,
                                                           ViInt32   standard,
                                                           ViInt32*  progress)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:OFFL:PROG?\n", satNavStandardArr[standard]);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, progress));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or stops data logging. Files with the same name are
/// HIFN  overwritten.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Starts or stops data logging. Files with the same name are
/// HIPAR enabled/overwritten.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogRTEnabled(ViSession instrSession,
                                                         ViInt32   standard,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:RT:STAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatAccelerationECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:ACC:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatPositionECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:POS:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatVelocityECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:VEL:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatAzimuth(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:AZIM %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatCBLAS(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:CBI %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatCPhase(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:CPH %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatDShift(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:DSH %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatElevation(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:ELEV %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatIDelay(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:IDEL %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatPRange(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:PRAN %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatPRangeBIAS(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:PRAN:BIAS %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatPRangeBIASRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:PRAN:BIAS:RATE %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatPRangeRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:PRAN:RATE %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatRange(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:RANG %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatRangeRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:RANG:RATE %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatRRate(ViSession instrSession,
                                                             ViInt32   standard,
                                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:RRAT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatSLevel(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:SLEV %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamSatTDelay(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:SAT:TDEL %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionAccelerationECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:ACC:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionJerkECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:JERK:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionPositionECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:POS:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionVelocityECEF(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:VEL:ECEF %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionVelocityLNED(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:VEL:LNED %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionAttitude(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:ATT %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionAttitudeAcceleration(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:ATT:ACC %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionAttitudeJerk(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:ATT:JERK %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionAttitudeRate(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:ATT:RATE %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionGDOP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:GDOP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionGSpeed(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:GSP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionHDOP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:HDOP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionPDOP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:PDOP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionPositionENU(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:POS:ENU %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionPositionLLA(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:POS:LLA %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionTDOP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:TDOP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the parameter for logging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR enabled/Enables the parameter for logging.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavLogParamUMotionVDOP(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:LOGG:CAT:UMOT:VDOP %s\n", satNavStandardArr[standard], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the SBAS navigation data mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR mode/Sets the SBAS navigation data mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASNavigationDataMode(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mode, 0, 1),
            3, "Mode");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:NDM %s\n", satNavStandardArr[standard], SBASNavModeArr[mode]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the regional systems for that the SBAS messages will be
/// HIFN  generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Defines the regional systems for that the SBAS messages will be
/// HIPAR enabled/generated.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the regional systems for that the SBAS messages will be
/// HIFN  generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionAlmanacEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:ALM:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionAlmanacFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:ALM:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionAlmanacSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:ALM:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionAlmanacPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:ALM:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionRinexEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:RIN:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionRinexFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:RIN:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionRinexSpan(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   region,
                                                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:RIN:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionRinexPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:RIN:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR state/Enables generation of the particular SBAS correction data and regional
/// HIPAR state/system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionUTCOffset(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   region,
                                                            ViChar    state[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:UTC:STAT?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, state, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionUTCOffsetPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:UTC:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionIGridEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:IGR:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionIGridFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:IGR:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionIGridSpan(ViSession instrSession,
                                                            ViInt32   standard,
                                                            ViInt32   region,
                                                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:IGR:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionIGridPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:IGR:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionPRNMaskEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRNM:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionPRNMaskFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRNM:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionPRNMaskSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRNM:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionPRNMaskPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRNM:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCorrectionEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCOR:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCorrectionFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCOR:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionFCorrectionSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCOR:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCorrectionPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCOR:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionLTCorrectionEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:LTC:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionLTCorrectionFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:LTC:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionLTCorrectionSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:LTC:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionLTCorrectionPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:LTC:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionCECovarianceEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:CEC:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionCECovarianceFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:CEC:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionCECovarianceSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:CEC:SPAN?", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, 128, file));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionCECovariancePeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:CEC:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCDegradationEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCD:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCDegradationFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCD:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionFCDegradationSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCD:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionFCDegradationPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:FCD:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionDFactorEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:DFAC:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionDFactorFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:DFAC:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionDFactorSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:DFAC:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionDFactorPeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:DFAC:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionServiceEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:SERV:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS file for the corresponding SBAS regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionServiceFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:SERV:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR file/Queries the date and time span of the current SBAS file for the
/// HIPAR file/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionServiceSpan(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViChar    file[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:SERV:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region]);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, file, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the periodicity of the SBAS message.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR period/Sets the periodicity of the SBAS message.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionServicePeriod(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   period)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:SERV:PER %ld\n", satNavStandardArr[standard], SBASRegionArr[region], period);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables generation of the particular SBAS correction data and regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR enabled/Enables generation of the particular SBAS correction data and regional
/// HIPAR enabled/system.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionIODEEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:LTC:UIFG %s\n", satNavStandardArr[standard], SBASRegionArr[region], (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables a PRN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR prn/Sets the PRN.
/// HIPAR enabled/Enables or disables a PRN.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionPRNEnabled(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   prn,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, prn, 1, 1),
            4, "PRN");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRN%ld:STAT %s\n", satNavStandardArr[standard], SBASRegionArr[region], prn, (enabled == VI_TRUE) ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the duration of the current raw file per PRN and SBAS regional
/// HIFN  system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR prn/Sets the PRN.
/// HIPAR duration/Queries the duration of the current raw file per PRN and SBAS regional
/// HIPAR duration/system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionPRNDuration(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   prn,
                            ViInt32*  duration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, prn, 1, 1),
            4, "PRN");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRN%ld:DUR?\n", satNavStandardArr[standard], SBASRegionArr[region], prn);
    checkErr(RsCore_QueryViInt32(instrSession, cmd, duration));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the SBAS raw file for the corresponding SBAS regional system
/// HIFN  and PRN.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR prn/Sets the PRN.
/// HIPAR fileName/File Name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionPRNFile(
                            ViSession instrSession,
                            ViInt32   standard,
                            ViInt32   region,
                            ViInt32   prn,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, prn, 1, 1),
            4, "PRN");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRN%ld:FILE '%s'\n", satNavStandardArr[standard], SBASRegionArr[region], prn, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the date and time span of the current SBAS file for the
/// HIFN  selected regional system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR standard/Sets the Satellite Navigation standard.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR prn/Sets the PRN.
/// HIPAR span/Queries the date and time span of the current SBAS file for the
/// HIPAR span/selected regional system.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionPRNSpan(ViSession instrSession,
                                                          ViInt32   standard,
                                                          ViInt32   region,
                                                          ViInt32   prn,
                                                          ViChar    span[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, standard, 0, 4),
            2, "Standard");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, prn, 1, 1),
            4, "PRN");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:BB:%s:NAV:SBAS:%s:PRN%ld:SPAN?\n", satNavStandardArr[standard], SBASRegionArr[region], prn);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, 128, span, NULL));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds a set of
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     to an import file list in one step.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR directory/Sets the directory name.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionImportAddDirectory(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViString  directory)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:ADD:DIR '%s'", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, directory);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     to an import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR fileName/Sets the file name.
ViStatus _VI_FUNC rsrfsiggen_ConfigureSatNavSBASRegionImportAddFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:ADD:FILE '%s'", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, fileName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries all
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     of the import file list in a comma separated list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries all files of the import file list.
/// HIPAR returnedValues/Returns the number of values returned in the Response control.
ViStatus _VI_FUNC rsrfsiggen_QuerySatNavSBASRegionImportFileCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   arraySize,
                            ViChar    catalog[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:LIST?", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, returnedValues));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads all
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     from an import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
ViStatus _VI_FUNC rsrfsiggen_LoadSatNavSBASRegionImportFiles(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   region)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:EXEC", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes all
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     from the import files list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
ViStatus _VI_FUNC rsrfsiggen_RemoveSatNavSBASRegionImportAllFiles(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:REM:ALL", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Removes one particular
/// HIFN     *.ems files for EGNOS correction data
/// HIFN     *.nstb files for WAAS correction data
/// HIFN     at the n-th position from the import file list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR region/Sets the SBAS Region.
/// HIPAR position/Sets the n-th position from the import file list.
ViStatus _VI_FUNC rsrfsiggen_RemoveSatNavSBASRegionImportFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   region,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, region, 0, 2),
            3, "Region");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GNSS:SV:IMP:SBAS:%s:REM:FILE%ld", path, RsRFSigGen_rngSatNavSBASRegion.entries[region].cmdString, position);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables or disables the NFC standard.
/// HIFN
/// HIFN     AEnabling this standard disables all the other digital standards and
/// HIFN  digital modulation modes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR nfcEnabled/Activates or deactivates the NFC standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean nfcEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_ENABLED, nfcEnabled),
            3, "NFC Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the NFC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR technology/Selects the NFC or EMV technology.
/// HIPAR transmissionMode/Selects the transmission mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFC(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   technology,
                                          ViInt32   transmissionMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_TECHNOLOGY, technology),
            3, "Technology");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_TRANSMISSION_MODE, transmissionMode),
            4, "Transmission Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the NFC modulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR slope/Determines the transition between the modulated and unmodulated parts
/// HIPAR slope/(Edge or Slope).
/// HIPAR tlow/Defines the signals low time (below 5%) in s.
/// HIPAR modulationDepth/Sets the modulation depth in %.
/// HIPAR sampleRate/Enters the sample rate, i.e. the time resolution of the generated
/// HIPAR sampleRate/signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCModulation(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean slope,
                                                    ViReal64  tlow,
                                                    ViReal64  modulationDepth,
                                                    ViReal64  sampleRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_MODULATION_SLOPE, slope),
            3, "Slope");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_MODULATION_TLOW, tlow),
            4, "T Low");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_MODULATION_DEPTH, modulationDepth),
            5, "Modulation Depth");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_MODULATION_SAMPLE_RATE, sampleRate),
            6, "Sample Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the NFC predefined sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR predefinedSequence/Selects a predefined sequence.
/// HIPAR configureMarker1/Enables Marker 1 as shown in the picture of the dialog.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCPredefinedSequence(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   predefinedSequence,
                                                            ViBoolean configureMarker1)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_PREDEFINED_SEQUENCE, predefinedSequence),
            3, "Predefined Sequence");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_PREDEFINED_SEQUENCE_CONFIGURE_MARKER, configureMarker1),
            4, "Configure Marker 1");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the selected "Predefined Sequence" and marker.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCPredefinedSequenceApply(
                            ViSession instrSession,
                            ViInt32   path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_PREDEFINED_SEQUENCE_APPLY, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the NFC sequence.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sequence/Sets the sequence number.
/// HIPAR commandType/Selects the command type.
/// HIPAR repetition/Determines the number of times to repeat the generation of a frame.
/// HIPAR powerOffset/Determines the power offset value in dB.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCSequence(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   sequence,
                                                  ViInt32   commandType,
                                                  ViInt32   repetition,
                                                  ViReal64  powerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_output[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequence, 1, 100),
            3, "Sequence");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));
    snprintf(repCap_output, RS_REPCAP_BUF_SIZE, ",NFC%ld", sequence);
    strcat(repCap, repCap_output);

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_SEQUENCE_COMMAND_TYPE, commandType),
            4, "Command Type");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_SEQUENCE_REPETITION, repetition),
            5, "Repetition");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_SEQUENCE_POWER_OFFSET, powerOffset),
            6, "Power Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNFCClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_NFC_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables the ILS standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates or deactivates the ILS standard
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR component/Selects the ILS modulation type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILS(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   component)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_COMPONENT, component),
            3, "Component");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Glide Slope Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR DDMPolarity/Sets the polarity for DDM calculation.
/// HIPAR mode/Sets the operating mode for the ILS-GS modulation signal.
/// HIPAR upFrequency/Sets the modulation frequency of the antenna lobe arranged at
/// HIPAR upFrequency/the bottom viewed from the air plane for the ILS-GS modulation
/// HIPAR upFrequency/signal.
/// HIPAR downFrequency/Sets the modulation frequency of the antenna lobe arranged at
/// HIPAR downFrequency/the top viewed from the air plane (up frequency).
/// HIPAR upDownPhase/Sets the phase between the modulation signals of the upper and
/// HIPAR upDownPhase/lower antenna lobe of the ILS-GS signal. The zero crossing of
/// HIPAR upDownPhase/the lower lobe (150Hz) signal serves as a reference. The angle
/// HIPAR upDownPhase/refers to the period of the signal of the right antenna lobe.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSGlideSlopeSignal(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   DDMPolarity,
                                                          ViInt32   mode,
                                                          ViReal64  upDownPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_DDM_POLARITY, DDMPolarity),
            3, "DDM Polarity");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_SIGNAL_MODE, mode),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_PHASE, upDownPhase),
            5, "Up/Down Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Glide Slope ICAO Channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ICAOChannel/Selects the ICAO channel i.e., the ILS Glide Slope transmitting
/// HIPAR ICAOChannel/frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSGlideSlopeICAO(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   ICAOChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_ICAO));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_ICAO_CHANNEL, ICAOChannel),
            3, "ICAO Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Glide Slope user defined
/// HIFN carrier frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierFrequency/Sets the carrier frequency if Carrier Frequency Mode is set to
/// HIPAR carrierFrequency/User.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSGlideSlopeUserCarrierFrequency(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_USER));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Glide Slope Amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sumOfDepth/Sets the arithmetic sum of the modulation depths of the upper
/// HIPAR sumOfDepth/lobe (90Hz) and lower lobe (150Hz) for the ILS-GS signal
/// HIPAR sumOfDepth/contents.
/// HIPAR fly/Sets the simulation mode for the ILS-GS modulation signal.
/// HIPAR DDMStep/Sets the variation of the difference in depth of modulation via
/// HIPAR DDMStep/the rotary knob.
/// HIPAR DDMCurrent/Sets the DDM value alternatively as a current by means of the
/// HIPAR DDMCurrent/ILS indicating instrument.
/// HIPAR DDMDepth/Sets the difference in depth of modulation between the signal of
/// HIPAR DDMDepth/the upper/left lobe (90 Hz) and the lower/right lobe (150 Hz).
/// HIPAR DDMLogarithmic/Sets the depth of modulation value for the ILS-GS in dB.
/// HIPAR DDMPercent/Sets the difference in depth of modulation between the signal of
/// HIPAR DDMPercent/the upper lobe (90 Hz) and the lower lobe (150 Hz).
/// HIPAR DDMSDMCoupling/Selects if the DDM value is fixed or is changed with a change of
/// HIPAR DDMSDMCoupling/sum of modulation depths.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSGlideSlopeAmplitude(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  sumOfDepth,
                                                             ViInt32   fly,
                                                             ViInt32   DDMStep,
                                                             ViReal64  DDMDepth,
                                                             ViInt32   DDMSDMCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_SUM_OF_DEPTH, sumOfDepth),
            3, "Sum Of Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_FLY, fly),
            4, "Fly");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_DDM_STEP, DDMStep),
            5, "DDM Step");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_DDM_DEPTH, DDMDepth),
            6, "DDM Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_GLIDE_SLOPE_DDM_COUPLING, DDMSDMCoupling),
            7, "DDM-SDM Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Glide Slope Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR DDMPolarity/Sets the polarity for DDM calculation.
/// HIPAR mode/Sets the operating mode for the ILS-LOC modulation signal.
/// HIPAR leftFrequency/Sets the modulation frequency of the antenna lobe arranged at
/// HIPAR leftFrequency/the bottom viewed from the air plane for the ILS-LOC modulation
/// HIPAR leftFrequency/signal.
/// HIPAR rightFrequency/Sets the modulation frequency of the antenna lobe arranged at
/// HIPAR rightFrequency/the right viewed from the air plane.
/// HIPAR leftRightPhase/Sets the phase between the modulation signals of the upper and
/// HIPAR leftRightPhase/lower antenna lobe of the ILS-LOC signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerSignal(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   DDMPolarity,
                                                         ViInt32   mode,
                                                         ViReal64  leftRightPhase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_DDM_POLARITY, DDMPolarity),
            3, "DDM Polarity");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_SIGNAL_MODE, mode),
            4, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_PHASE, leftRightPhase),
            5, "Left/Right Phase");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the  ILS-LOC ICAO Channel..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ICAOChannel/Selects the ICAO channel i.e., the ILS-LOC transmitting
/// HIPAR ICAOChannel/frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerICAO(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   ICAOChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_ICAO));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_ICAO_CHANNEL, ICAOChannel),
            3, "ICAO Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS-LOC user defined
/// HIFN carrier frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierFrequency/Sets the carrier frequency if Carrier Frequency Mode is set to
/// HIPAR carrierFrequency/User.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerUserCarrierFrequency(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_USER));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS-LOC Amplitude.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sumOfDepth/Sets the arithmetic sum of the modulation depths of the upper
/// HIPAR sumOfDepth/lobe (90Hz) and lower lobe (150Hz) for the ILS-GS signal
/// HIPAR sumOfDepth/contents.
/// HIPAR fly/Sets the simulation mode for the ILS-GS modulation signal.
/// HIPAR DDMStep/Sets the variation of the difference in depth of modulation via
/// HIPAR DDMStep/the rotary knob.
/// HIPAR DDMCurrent/Sets the DDM value alternatively as a current by means of the
/// HIPAR DDMCurrent/ILS indicating instrument.
/// HIPAR DDMDepth/Sets the difference in depth of modulation between the signal of
/// HIPAR DDMDepth/the upper/left lobe (90 Hz) and the lower/right lobe (150 Hz).
/// HIPAR DDMLogarithmic/Sets the depth of modulation value for the ILS-GS in dB.
/// HIPAR DDMPercent/Sets the difference in depth of modulation between the signal of
/// HIPAR DDMPercent/the upper lobe (90 Hz) and the lower lobe (150 Hz).
/// HIPAR DDMSDMCoupling/Selects if the DDM value is fixed or is changed with a change of
/// HIPAR DDMSDMCoupling/sum of modulation depths.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerAmplitude(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViReal64  sumOfDepth,
                                                            ViInt32   fly,
                                                            ViInt32   DDMStep,
                                                            ViReal64  DDMDepth,
                                                            ViInt32   DDMSDMCoupling)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_SUM_OF_DEPTH, sumOfDepth),
            3, "Sum Of Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_FLY, fly),
            4, "Fly");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_DDM_STEP, DDMStep),
            5, "DDM Step");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_DDM_DEPTH, DDMDepth),
            6, "DDM Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_DDM_COUPLING, DDMSDMCoupling),
            7, "DDM-SDM Coupling");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the additional COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Switches on/off the addittional COM/ID signal.
/// HIPAR code/Sets the coding of the COM/ID signal by the international short
/// HIPAR code/name of the airport (e.g. MUC for the Munich airport).
/// HIPAR frequency/Sets the frequency of the COM/ID signal.
/// HIPAR period/Sets the period of the COM/ID signal in seconds.
/// HIPAR depth/Sets the AM modulation depth of the COM/ID signal.
/// HIPAR timeSchema/Sets the time schema for the COM/ID signal.
/// HIPAR dotLength/Sets the length of a morse dot in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerCOMID(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViBoolean state,
                                                        ViString  code,
                                                        ViReal64  frequency,
                                                        ViReal64  period,
                                                        ViReal64  depth,
                                                        ViInt32   timeSchema,
                                                        ViReal64  dotLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_ENABLED, state),
            3, "State");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_CODE, code),
            4, "Code");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_FREQUENCY, frequency),
            5, "Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_PERIOD, period),
            6, "Period");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_DEPTH, depth),
            7, "Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_TIME_SCHEMA, timeSchema),
            8, "Time Schema");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_DOT_LENGTH, dotLength),
            9, "Dot Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the parameters for User Time Schema of
/// HIFN COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dashLength/Sets the length of a morse dash in seconds.
/// HIPAR symbolSpace/Sets the length of the symbol space in seconds.
/// HIPAR letterSpace/Sets the length of a letter space in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSLocalizerCOMIDUserTimeSchema(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  dashLength,
                            ViReal64  symbolSpace,
                            ViReal64  letterSpace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_DASH_LENGTH, dashLength),
            3, "Dash Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_SYMBOL_SPACE, symbolSpace),
            4, "Symbol Space");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_LOCALIZER_COM_ID_LETTER_SPACE, letterSpace),
            5, "Letter Space");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the ILS Marker Beacon Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierFrequencyMode/Sets the carrier frequency mode of the ILS-MB modulation signal.
/// HIPAR carrierFrequency/Sets the carrier frequency for the ILS-MB modulation signal.
/// HIPAR markerFrequency/Sets the modulation frequency of the marker signal for the
/// HIPAR markerFrequency/ILS-MB modulation signal.
/// HIPAR markerDepth/Sets the modulation depth of the marker signal for the ILS-MB
/// HIPAR markerDepth/signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSMarkerBeaconsSignal(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViInt32   carrierFrequencyMode,
                                                             ViReal64  carrierFrequency,
                                                             ViReal64  markerFrequency,
                                                             ViReal64  markerDepth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_CARRIER_FREQUENCY_MODE, carrierFrequencyMode),
            3, "Carrier Frequency Mode");

    if (carrierFrequencyMode == RSRFSIGGEN_VAL_ILS_FREQUENCY_MODE_USER)
    {
        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_CARRIER_FREQUENCY, carrierFrequency),
                4, "Carrier Frequency");
    }
    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_MARKER_FREQUENCY, markerFrequency),
            5, "Marker Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_MARKER_DEPTH, markerDepth),
            6, "Marker Depth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the additional COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Switches on/off the addittional COM/ID signal.
/// HIPAR code/Sets the coding of the COM/ID signal by the international short
/// HIPAR code/name of the airport (e.g. MUC for the Munich airport).
/// HIPAR frequency/Sets the frequency of the COM/ID signal.
/// HIPAR period/Sets the period of the COM/ID signal in seconds.
/// HIPAR depth/Sets the AM modulation depth of the COM/ID signal.
/// HIPAR timeSchema/Sets the time schema for the COM/ID signal.
/// HIPAR dotLength/Sets the length of a morse dot in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSMarkerBeaconsCOMID(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViBoolean state,
                                                            ViString  code,
                                                            ViReal64  frequency,
                                                            ViReal64  period,
                                                            ViReal64  depth,
                                                            ViInt32   timeSchema,
                                                            ViReal64  dotLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_ENABLED, state),
            3, "State");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_CODE, code),
            4, "Code");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_FREQUENCY, frequency),
            5, "Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_PERIOD, period),
            6, "Period");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_DEPTH, depth),
            7, "Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_TIME_SCHEMA, timeSchema),
            8, "Time Schema");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_DOT_LENGTH, dotLength),
            9, "Dot Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the parameters for User Time Schema of
/// HIFN COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dashLength/Sets the length of a morse dash in seconds.
/// HIPAR symbolSpace/Sets the length of the symbol space in seconds.
/// HIPAR letterSpace/Sets the length of a letter space in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSMarkerBeaconsCOMIDUserTimeSchema(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  dashLength,
                            ViReal64  symbolSpace,
                            ViReal64  letterSpace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_DASH_LENGTH, dashLength),
            3, "Dash Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_SYMBOL_SPACE, symbolSpace),
            4, "Symbol Space");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_MARKER_BEACONS_COM_ID_LETTER_SPACE, letterSpace),
            5, "Letter Space");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureILSClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ILS_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables the VOR standard.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR path/Document parameter here.
/// HIPAR enabled/Document parameter here.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVOREnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the VOR Signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the operating mode for the VOR modulation signal.
/// HIPAR VARREFFrequency/Sets the frequency of the variable and the reference signal. As
/// HIPAR VARREFFrequency/the two signals must have the same frequency, the setting is
/// HIPAR VARREFFrequency/valid for both signals.
/// HIPAR VARDepth/Sets the AM modulation depth of the 30Hz variable signal.
/// HIPAR subcarrierFrequency/Document parameter here.
/// HIPAR subcarrierDepth/Document parameter here.
/// HIPAR REFDeviation/Document parameter here.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORSignal(ViSession instrSession,
                                                ViInt32   path,
                                                ViInt32   mode,
                                                ViReal64  VARREFFrequency,
                                                ViReal64  VARDepth,
                                                ViReal64  subcarrierFrequency,
                                                ViReal64  subcarrierDepth,
                                                ViReal64  REFDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_SIGNAL_MODE, mode),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_VARIABLE_REFERENCE_FREQUENCY, VARREFFrequency),
            4, "VAR/REF Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_VARIABLE_DEPTH, VARDepth),
            5, "VAR Depth");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_SUBCARRIER_FREQUENCY, subcarrierFrequency),
            6, "Subcarrier Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_SUBCARRIER_DEPTH, subcarrierDepth),
            7, "Subcarrier Depth");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_FREQUENCY_DEVIATION, REFDeviation),
            8, "REF Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the VOR ICAO Channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ICAOChannel/Selects the ICAO channel i.e., the VOR transmitting frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORICAO(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   ICAOChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_ICAO));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_ICAO_CHANNEL, ICAOChannel),
            3, "ICAO Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the VOR user defined
/// HIFN carrier frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR carrierFrequency/Sets the carrier frequency if Carrier Frequency Mode is set to
/// HIPAR carrierFrequency/User.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORUserCarrierFrequency(
                            ViSession instrSession,
                            ViInt32   path,
                            ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_CARRIER_FREQUENCY_MODE, RSRFSIGGEN_VAL_CARRIER_FREQUENCY_MODE_USER));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_CARRIER_FREQUENCY, carrierFrequency),
            3, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the different position settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR bearingAngle/Sets the phase angle between the 30Hz VAR signal and the 30Hz
/// HIPAR bearingAngle/reference signal.
/// HIPAR direction/Sets the reference position of the phase information.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORPosition(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViReal64  bearingAngle,
                                                  ViInt32   direction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_POSITION_BEARING_ANGLE, bearingAngle),
            3, "Bearing Angle");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_POSITION_BEARING_ANGLE_DIRECTION, direction),
            4, "Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the additional COM/ID signal.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Switches on/off the addittional COM/ID signal.
/// HIPAR code/Sets the coding of the COM/ID signal by the international short
/// HIPAR code/name of the airport (e.g. MUC for the Munich airport).
/// HIPAR frequency/Sets the frequency of the COM/ID signal.
/// HIPAR period/Sets the period of the COM/ID signal in seconds.
/// HIPAR depth/Sets the AM modulation depth of the COM/ID signal.
/// HIPAR timeSchema/Sets the time schema for the COM/ID signal.
/// HIPAR dotLength/Sets the length of a morse dot in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORCOMID(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean state,
                                               ViString  code,
                                               ViReal64  frequency,
                                               ViReal64  period,
                                               ViReal64  depth,
                                               ViInt32   timeSchema,
                                               ViReal64  dotLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_ENABLED, state),
            3, "State");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_CODE, code),
            4, "Code");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_FREQUENCY, frequency),
            5, "Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_PERIOD, period),
            6, "Period");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_DEPTH, depth),
            7, "Depth");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_TIME_SCHEMA, timeSchema),
            8, "Time Schema");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_DOT_LENGTH, dotLength),
            9, "Dot Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the parameters for User Time Schema of
/// HIFN COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dashLength/Sets the length of a morse dash in seconds.
/// HIPAR symbolSpace/Sets the length of the symbol space in seconds.
/// HIPAR letterSpace/Sets the length of a letter space in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORCOMIDUserTimeSchema(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  dashLength,
                                                             ViReal64  symbolSpace,
                                                             ViReal64  letterSpace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_DASH_LENGTH, dashLength),
            3, "Dash Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_SYMBOL_SPACE, symbolSpace),
            4, "Symbol Space");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_COM_ID_LETTER_SPACE, letterSpace),
            5, "Letter Space");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureVORClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_VOR_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function enables the DME standard.
/// HIRET Document return value here.
/// HIRET You may use multiple lines for documentation.
/// HIPAR instrSession/Document parameter here.
/// HIPAR path/Document parameter here.
/// HIPAR enabled/Document parameter here.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the mode of the DME modulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDME(ViSession instrSession,
                                          ViInt32   path,
                                          ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME ICAO Channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR ICAOChannel/Selects the ICAO channel i.e., the DME transmitting frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEICAO(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   ICAOChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_CHANNEL_MODE, RSRFSIGGEN_VAL_DME_CHANNEL_MODE_ICAO));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ICAO_CHANNEL, ICAOChannel),
            3, "ICAO Channel");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the carrier frequency of the signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the simulated channel.
/// HIPAR carrierFrequency/Sets the carrier frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEChannelFrequency(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   channel,
                                                          ViReal64  carrierFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    switch (channel)
    {
    case RSRFSIGGEN_VAL_DME_CHANNEL_MODE_X:
    case RSRFSIGGEN_VAL_DME_CHANNEL_MODE_Y:
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_CHANNEL_MODE, channel),
                3, "Channel");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channel), 3, "Channel");
        break;
    }
    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_CARRIER_FREQUENCY, carrierFrequency),
            4, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME Interrogation Pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pulseShape/Sets the pulse shape.
/// HIPAR pulseRise/Sets the rise time of the pulse (10% to 90% of peak voltage).
/// HIPAR pulseWidth/Sets the pulse width (50% to 50% of peak voltage).
/// HIPAR pulseFall/Sets the fall time of the pulse (90% to 10% of peak voltage).
/// HIPAR pulseSpacing/Sets the spacing between the first and second pulse of a pulse
/// HIPAR pulseSpacing/pair (time between halfvoltage points on the leading edge of
/// HIPAR pulseSpacing/each pulse).
/// HIPAR singlePulse/Activates/deactivates generation of a single test pulse.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEPulse(ViSession instrSession,
                                               ViInt32   path,
                                               ViInt32   pulseShape,
                                               ViReal64  pulseRise,
                                               ViReal64  pulseWidth,
                                               ViReal64  pulseFall,
                                               ViReal64  pulseSpacing,
                                               ViBoolean singlePulse)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_SHAPE, pulseShape),
            3, "Pulse Shape");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_RISE, pulseRise),
            4, "Pulse Rise");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_WIDTH, pulseWidth),
            5, "Pulse Width");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_FALL, pulseFall),
            6, "Pulse Fall");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_SPACING, pulseSpacing),
            7, "Pulse Spacing");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_SINGLE_PULSE, singlePulse),
            8, "Single Pulse");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function comprises the settings, necessary to configure the
/// HIFN receiving of the pulse.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pulsePairSpacingTolerance/Sets the pulse pair spacing tolerance.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEReceive(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViReal64  pulsePairSpacingTolerance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_PAIR_SPACING_TOLERANCE, pulsePairSpacingTolerance),
            3, "Pulse Pair Spacing Tolerance");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME Interrogation Signal
/// HIFN Parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pulseSquitter/Activates/deactivates the random pulse repetition rate.
/// HIPAR pulseRepetitionRate/Sets the number of DME pulse pairs per second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEInterrogationSignal(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViBoolean pulseSquitter,
                                                             ViInt32   pulseRepetitionRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_SQUITTER, pulseSquitter),
            3, "Pulse Squitter");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_REPETITION_RATE, pulseRepetitionRate),
            4, "Pulse Repetition Rate");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME Analysis Parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Activates/deactivates the DME analysis.
/// HIPAR expectedReplayDelay/Sets the expected reply delay.
/// HIPAR gateLength/Sets the gate length for the measurement window.
/// HIPAR measurementTime/Sets the DME analysis measurement time.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEAnalysis(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean state,
                                                  ViReal64  expectedReplayDelay,
                                                  ViReal64  gateLength,
                                                  ViReal64  measurementTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ANALYSIS_ENABLED, state),
            3, "State");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ANALYSIS_EXPECTED_REPLY_DELAY, expectedReplayDelay),
            4, "Expected Replay Delay");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ANALYSIS_GATE_LENGTH, gateLength),
            5, "Gate Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ANALYSIS_MEASUREMENT_TIME, measurementTime),
            6, "Measurement Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the DME Replay Signal Parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR pulseSquitter/Activates/deactivates the random pulse repetition rate.
/// HIPAR pulseRepetitionRate/Sets the number of DME pulse pairs per second.
/// HIPAR rangeDistance/Sets the simulated distance between interrogator and transponder
/// HIPAR rangeDistance/for reply mode.
/// HIPAR replyEfficiency/Sets the relation between reply pulse pairs and received trigger
/// HIPAR replyEfficiency/signals, e.g. with a set efficiency of 50% only every second
/// HIPAR replyEfficiency/trigger event leads to the generation of a reply pulse pair.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEReplaySignal(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViBoolean pulseSquitter,
                                                      ViInt32   pulseRepetitionRate,
                                                      ViReal64  rangeDistance,
                                                      ViReal64  replyEfficiency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_SQUITTER, pulseSquitter),
            3, "Pulse Squitter");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_PULSE_REPETITION_RATE, pulseRepetitionRate),
            4, "Pulse Repetition Rate");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_RANGE_DISTANCE, rangeDistance),
            5, "Range Distance");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_REPLY_EFFICIENCY, replyEfficiency),
            6, "Reply Efficiency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the additional COM/ID signal.
/// HIRET Document return value here.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR state/Switches on/off the addittional COM/ID signal.
/// HIPAR pairOfPulsePairs/Enables the transmission of a pair of pulse pairs during the set
/// HIPAR pairOfPulsePairs/"Rate".
/// HIPAR pulsePairSpacing/Sets the morse pulse pair spacing.
/// HIPAR code/Sets the coding of the COM/ID signal by the international short
/// HIPAR code/name of the airport (e.g. MUC for the Munich airport).
/// HIPAR rate/Sets the pulse repetition rate of the ID sequence.
/// HIPAR period/Sets the period of the COM/ID signal in seconds.
/// HIPAR depth/Sets the AM modulation depth of the COM/ID signal.
/// HIPAR timeSchema/Sets the time schema for the COM/ID signal.
/// HIPAR dotLength/Sets the length of a morse dot in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMECOMID(ViSession instrSession,
                                               ViInt32   path,
                                               ViBoolean state,
                                               ViBoolean pairOfPulsePairs,
                                               ViReal64  pulsePairSpacing,
                                               ViString  code,
                                               ViReal64  rate,
                                               ViReal64  period,
                                               ViInt32   timeSchema,
                                               ViReal64  dotLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_ENABLED, state),
            3, "State");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_PAIR_OF_PULSE_PAIRS, pairOfPulsePairs),
            4, "Pair Of Pulse Pairs");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_PULSE_PAIR_SPACING, pulsePairSpacing),
            5, "Pulse Pair Spacing");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_CODE, code),
            6, "Code");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_RATE, rate),
            7, "Rate");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_PERIOD, period),
            8, "Period");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_TIME_SCHEMA, timeSchema),
            9, "Time Schema");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_DOT_LENGTH, dotLength),
            10, "Dot Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the parameters for User Time Schema of
/// HIFN COM/ID signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dashLength/Sets the length of a morse dash in seconds.
/// HIPAR symbolSpace/Sets the length of the symbol space in seconds.
/// HIPAR letterSpace/Sets the length of a letter space in seconds.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMECOMIDUserTimeSchema(ViSession instrSession,
                                                             ViInt32   path,
                                                             ViReal64  dashLength,
                                                             ViReal64  symbolSpace,
                                                             ViReal64  letterSpace)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_ID_DASH_LENGTH, dashLength),
            3, "Dash Length");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_SYMBOL_SPACE, symbolSpace),
            4, "Symbol Space");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_DME_COM_ID_LETTER_SPACE, letterSpace),
            5, "Letter Space");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMETriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMETriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureDMEClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_DME_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the GBAS standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates or deactivates the GBAS standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function enables the configuration of multiple frequency
/// HIFN  channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates or deactivates the GBAS standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBAS(ViSession instrSession,
                                           ViInt32   path,
                                           ViBoolean multipleFrequencyChannels)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_MULTIPLE_FREQUENCY_CHANNELS, multipleFrequencyChannels),
            3, "Multiple Frequency Channels");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the GBAS VHF Data Broadcast (VDB)
/// HIFN  transmitter.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR enabled/Activates or deactivates the selected VHF Data Broadcast (VDB)
/// HIPAR enabled/transmitter.
/// HIPAR gbasID/Deletes the specified files with stored user channel codings.
/// HIPAR ssid/Sets the Station Slot Identifier SSID of the of the ground station.
/// HIPAR frequencyNumber/Sets the frequency number the corresponding VDB is using.
/// HIPAR data/Selects the data source, e.g. a sequence of 0 or 1, a pseudo-random
/// HIPAR data/sequence with different lenght, a pattern or a data list.
/// HIPAR numberOfFrames/Sets the number of frames of the selected VDB.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDB(ViSession instrSession,
                                              ViInt32   path,
                                              ViInt32   channel,
                                              ViBoolean enabled,
                                              ViString  gbasID,
                                              ViInt32   ssid,
                                              ViInt32   frequencyNumber,
                                              ViInt32   data,
                                              ViInt32   numberOfFrames)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_ID, gbasID),
            5, "GBAS ID");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_SSID, ssid),
            6, "SSID");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_FREQUENCY_NUMBER, frequencyNumber),
            7, "frequencyNumber");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_DATA, data),
            8, "Data");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_NUMBER_OF_FRAMES, numberOfFrames),
            9, "numberOfFrames");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function Appends a new VDB to the end of the VDB list.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBAppend(ViSession instrSession,
                                                    ViInt32   path)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_APPEND, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function inserts a new VDB before the selected one.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBInsert(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_INSERT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function deletes the selected VDB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBDelete(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_DELETE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the ground station reference location in
/// HIFN  decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType2ReferenceLocationConfigurationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 6, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LOC:COOR:DEC %lf,%lf,%lf", path, channel, longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the ground station reference location in
/// HIFN  degrees, minutes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
/// HIPAR altitude/Altitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType2ReferenceLocationConfigurationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, -10000.0, 1600000.0), 12, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s,%lf", path, channel, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir], altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the LTP/FTP in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FASLTPFTPLocationConfigurationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LFL:COOR:DEC %lf,%lf", path, channel, longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the LTP/FTP in degrees, minutes and
/// HIFN  seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FASLTPFTPLocationConfigurationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LFL:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s", path, channel, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the Delta FPAD location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FASDeltaFrapLocationConfigurationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:DFL:COOR:DEC %lf,%lf", path, channel, longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the Delta FPAD location in degrees, minutes
/// HIFN  and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FASDeltaFrapLocationConfigurationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:DFL:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s", path, channel, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the LTP/FTP in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4LocationConfigurationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LOC:COOR:DEC %lf,%lf", path, channel, longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the ground station reference location in
/// HIFN  degrees, minutes and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4LocationConfigurationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            4, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            5, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 6, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            7, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            8, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            9, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 10, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            11, "Latitude Dir");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:LOC:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s", path, channel, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the Delta DERP location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fdb/Sets the FDB parameter.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FDBDERPLocationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   fdb,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fdb, 1, 5),
            4, "FDB");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 5, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:FDB%ld:DDL:COOR:DEC %lf,%lf", path, channel, fdb, longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the Delta DERP location in degrees, minutes
/// HIFN  and seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fdb/Sets the FDB parameter.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FDBDERPLocationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   fdb,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fdb, 1, 5),
            4, "FDB");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            5, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            6, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 7, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            8, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            9, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            10, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 11, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            12, "Latitude Dir");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCON:FDB%ld:DDL:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s", path, channel, fdb, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the DP location in decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fdb/Sets the FDB parameter.
/// HIPAR longitude/Longitude parameter.
/// HIPAR latitude/Latitude parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FDBDPLocationDecimal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   fdb,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fdb, 1, 5),
            4, "FDB");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 5, "Longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 6, "Latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCONfig:FDB%ld:DPL:COOR:DEC %lf,%lf", path, channel, fdb, longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the coordinates of the DP location in degrees, minutes and
/// HIFN  seconds.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fdb/Sets the FDB parameter.
/// HIPAR longitudeDeg/Longitude Degrees parameter.
/// HIPAR longitudeMin/Longitude Minutes parameter.
/// HIPAR longitudeSec/Longitude Seconds parameter.
/// HIPAR longitudeDir/Longitude Direction parameter.
/// HIPAR latitudeDeg/Latitude Degrees parameter.
/// HIPAR latitudeMin/Latitude Minutes parameter.
/// HIPAR latitudeSec/Latitude Seconds parameter.
/// HIPAR latitudeDir/Latitude Direction parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4FDBDPLocationDMS(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViInt32   fdb,
                            ViInt32   longitudeDeg,
                            ViInt32   longitudeMin,
                            ViReal64  longitudeSec,
                            ViInt32   longitudeDir,
                            ViInt32   latitudeDeg,
                            ViInt32   latitudeMin,
                            ViReal64  latitudeSec,
                            ViInt32   latitudeDir)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, fdb, 1, 5),
            4, "FDB");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDeg, 0, 180),
            5, "Longitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeMin, 0, 59),
            6, "Longitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitudeSec, 0.0, 59.999), 7, "Longitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, longitudeDir, 0, 1),
            8, "Longitude Dir");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDeg, 0, 180),
            9, "Latitude Deg");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeMin, 0, 59),
            10, "Latitude Min");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitudeSec, 0.0, 59.999), 11, "Latitude Sec");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, latitudeDir, 0, 1),
            12, "Latitude Dir");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:BB:GBAS:VDB%ld:MCONfig:FDB%ld:DPL:COOR:DMS %ld,%ld,%lf,%s,%ld,%ld,%lf,%s", path, channel, fdb, longitudeDeg, longitudeMin, longitudeSec, longitudeDirArr[longitudeDir],
        latitudeDeg, latitudeMin, latitudeSec, latitudeDirArr[latitudeDir]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected predeifned file (extension *.txt).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4TAPWaypointPredefinedFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_TYPE_4_TAP_WAYPOINT_PREDEFINED_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected user-defined file (extension *.txt).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageType4TAPWaypointUserDefinedFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_TYPE_4_TAP_WAYPOINT_USER_DEFINED_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected predeifned file (extension *.rs_gbas).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageDifferentialGNSSPredefinedFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_DIFFERENTIAL_GNSS_PREDEFINED_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected user-defined file (extension *.rs_gbas).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageDifferentialGNSSUserDefinedFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_DIFFERENTIAL_GNSS_USER_DEFINED_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected predefined file (extension *.rs_scat).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageDifferentialGNSSSPRFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_DIFFERENTIAL_GNSS_SPR_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected user-defined file (extension *.rs_scat).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR channel/Selects the VDB channel.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASVDBMessageDifferentialGNSSSUSFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   channel,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channel, 1, 8),
            3, "Channel");
    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,VDBC%ld", pathArr[path], channel);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_VDB_MESSAGE_DIFFERENTIAL_GNSS_SUS_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function selects the trigger source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the trigger source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureGBASClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_GBAS_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the 5G standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the 5G standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the modulation type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR modulationType/Selects the modulation type.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMModulationType(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   modulationType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_MODULATION_TYPE, modulationType),
            3, "Modulation Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed. Provided are internal triggering by a command,
/// HIFN  external trigger signal via one of the provided local or global
/// HIFN  connectors and triggering by a signal from the other paths.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed. Provided are internal triggering by a command,
/// HIPAR source/external trigger signal via one of the provided local or global
/// HIPAR source/connectors and triggering by a signal from the other paths.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Enables/disables output of the signal synchronous to the external
/// HIPAR sync/trigger event.
/// HIPAR delay/Specifies the trigger delay for triggering by the signal from the
/// HIPAR delay/second path.
/// HIPAR inhibit/For triggering via the other path, specifies the number of samples by
/// HIPAR inhibit/which a restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViReal64  delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_OFDM_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMTriggerDateTime(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   year,
                                                          ViInt32   month,
                                                          ViInt32   day,
                                                          ViInt32   hour,
                                                          ViInt32   minute,
                                                          ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_OFDM_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_OFDM_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOFDMClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_OFDM_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the Radar Echo Generation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Enables/disables the Radar Echo Generation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureREGEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_REG_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the files with settings in the default directory. Listed are
/// HIFN  files with the file extension *.reg.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the files with settings in the default directory. Listed are
/// HIPAR catalog/files with the file extension *.reg.
/// HIPAR bytesRead/Returns number of bytes read from instrument. If the value returned is
/// HIPAR bytesRead/larger then 'Array Size', then the response stored in 'Catalog' is not
/// HIPAR bytesRead/complete. Resize 'Catalog' array to accomodate all values and call into
/// HIPAR bytesRead/the function again.
ViStatus _VI_FUNC rsrfsiggen_QueryREGFileCatalog(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   arraySize,
                                                 ViChar    catalog[],
                                                 ViUInt32* bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:REG:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    if (bytesRead)
        *bytesRead = (ViUInt32)strlen(catalog);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.reg.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Loads the selected file from the default or the specified directory.
/// HIPAR fileName/Loaded are files with extension *.reg.
ViStatus _VI_FUNC rsrfsiggen_ConfigureREGLoad(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_REG_RECALL_SETTINGS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current settings into the selected file; the file extension
/// HIFN  (*.reg) is assigned automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Stores the current settings into the selected file; the file extension
/// HIPAR fileName/(*.reg) is assigned automatically.
ViStatus _VI_FUNC rsrfsiggen_ConfigureREGStore(ViSession instrSession,
                                               ViInt32   path,
                                               ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_REG_SAVE_SETTINGS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries files with object setting in the default directory. Listed are files
/// HIFN  with the file extension *.reg_obj.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries files with object setting in the default directory. Listed are files
/// HIPAR catalog/with the file extension *.reg_obj.
/// HIPAR bytesRead/Returns number of bytes read from instrument. If the value returned is
/// HIPAR bytesRead/larger then 'Array Size', then the response stored in 'Catalog' is not
/// HIPAR bytesRead/complete. Resize 'Catalog' array to accomodate all values and call into
/// HIPAR bytesRead/the function again.
ViStatus _VI_FUNC rsrfsiggen_QueryREGObjectFileCatalog(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   arraySize,
                                                       ViChar    catalog[],
                                                       ViUInt32* bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:REG:OBJ:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    if (bytesRead)
        *bytesRead = (ViUInt32)strlen(catalog);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.reg_obj.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR object/This control selects the object.
/// HIPAR fileName/Loads the selected file from the default or the specified directory. Loaded are
/// HIPAR fileName/files with extension *.reg_obj.
ViStatus _VI_FUNC rsrfsiggen_ConfigureREGObjectLoad(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   object,
                                                    ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_object[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, object, 1, 12),
            3, "Object");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_REG_OBJECT_RECALL,
            "Object", object- 1, RS_REPCAP_BUF_SIZE, repCap_object));

    strcat(repCap, ",");
    strcat(repCap, repCap_object);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_REG_OBJECT_RECALL, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current settings into the selected file; the file extension
/// HIFN  (*.reg_obj) is assigned automatically.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR object/This control selects the object.
/// HIPAR fileName/Stores the current settings into the selected file; the file extension
/// HIPAR fileName/(*.reg_obj) is assigned automatically.
ViStatus _VI_FUNC rsrfsiggen_ConfigureREGObjectStore(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   object,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_object[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, object, 1, 12),
            3, "Object");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_REG_OBJECT_SAVE,
            "Object", object- 1, RS_REPCAP_BUF_SIZE, repCap_object));

    strcat(repCap, ",");
    strcat(repCap, repCap_object);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_REG_OBJECT_SAVE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates signal generation, and deactivates all digital standards,
/// HIFN  digital modulation modes and other sweeps in the corresponding path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates signal generation, and deactivates all digital standards,
/// HIPAR enabled/digital modulation modes and other sweeps in the corresponding path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the processing mode for the extended sequencer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Selects the processing mode for the extended sequencer.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqMode(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the available extended sequencer archive files in the default
/// HIFN  directory. Listed are files with the file extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the available extended sequencer archive files in the default
/// HIPAR catalog/directory. Listed are files with the file extension *.ps_arc.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqArchiveFileCatalog(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViInt32   arraySize,
                                                           ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:ARCH:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqArchiveLoad(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_ARCHIVE_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stores the current user mode configuration in the selected file,
/// HIFN  including all used extended sequencer files.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqArchiveStore(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_ARCHIVE_STORE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the available pulse sequencer files.
/// HIFN     As response, you get a string containing the existing files *.ps_def,
/// HIFN  separated by commas. To set the default directory, use command
/// HIFN  MMEM:CDIRectory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the available extended sequencer archive files in the default
/// HIPAR catalog/directory. Listed are files with the file extension *.ps_arc.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqPulseFileCatalog(ViSession instrSession,
                                                         ViInt32   path,
                                                         ViInt32   arraySize,
                                                         ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:PSEQ:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqPulseFileSelect(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   sequencer,
                                                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxPath = 0;
    ViChar   repCapSequencer[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSequencer = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &maxPath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, maxPath),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_PULSE_FILE_SELECT, "Path", path - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        2, "Path");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "ExtendedSequencer", &maxSequencer));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequencer, 1, maxSequencer),
            3, "Sequencer");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_PULSE_FILE_SELECT, "ExtendedSequencer", sequencer - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        3, "Sequencer");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapPath, repCapSequencer);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_PULSE_FILE_SELECT, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqDirectionFindingFileSelect(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   sequencer,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxPath = 0;
    ViChar   repCapSequencer[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSequencer = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &maxPath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, maxPath),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_DIRECTION_FINDING_FILE_SELECT, "Path", path - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        2, "Path");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "ExtendedSequencer", &maxSequencer));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequencer, 1, maxSequencer),
            3, "Sequencer");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_DIRECTION_FINDING_FILE_SELECT, "ExtendedSequencer", sequencer - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        3, "Sequencer");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapPath, repCapSequencer);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_DIRECTION_FINDING_FILE_SELECT, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads out the files extension *.inf_mswv from the default directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Reads out the files extension *.inf_mswv from the default directory.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqRTCIFileCatalog(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   arraySize,
                                                        ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:RTCI:WLIS:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Reads out the files extension *.wv in the default directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Reads out the files extension *.wv in the default directory.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqRTCIWaveformFileCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:RTCI:WAVE:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a waveform list from the default directory. If a waveform list
/// HIFN  with the specified name does not yet exist, it is created. The file
/// HIFN  extension *.inf_mswv may be omitted.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqRTCIFile(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   sequencer,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxPath = 0;
    ViChar   repCapSequencer[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSequencer = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &maxPath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, maxPath),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_FILE, "Path", path - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        2, "Path");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "ExtendedSequencer", &maxSequencer));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequencer, 1, maxSequencer),
            3, "Sequencer");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_FILE, "ExtendedSequencer", sequencer - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        3, "Sequencer");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapPath, repCapSequencer);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_FILE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Save and load the waveform list to the coder board, the file extension
/// HIFN  *.inf_mswv is automatically assigned.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqRTCISave(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   sequencer,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxPath = 0;
    ViChar   repCapSequencer[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  maxSequencer = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &maxPath));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, maxPath),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_SAVE, "Path", path - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        2, "Path");

    checkErr(RsCore_GetRepCapIdItemsCount(instrSession, "ExtendedSequencer", &maxSequencer));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, sequencer, 1, maxSequencer),
            3, "Sequencer");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_SAVE, "ExtendedSequencer", sequencer - 1, RS_REPCAP_BUF_SIZE, repCapPath),
        3, "Sequencer");

    (void)snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCapPath, repCapSequencer);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_RTCI_WAVEFORM_LIST_SAVE, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the available user defined waveform segment sequence files.
/// HIFN     As response, you get a string containing the existing files *.ps_seq,
/// HIFN  separated by commas. To set the default directory, use command
/// HIFN  MMEM:CDIRectory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the available user defined waveform segment sequence files.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqUserFileCatalog(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   arraySize,
                                                        ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:USER:SEQ:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqUserFileSelect(ViSession instrSession,
                                                           ViInt32   path,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_USER_FILE_SELECT, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the available hopping list files.
/// HIFN     As response, you get a string containing the existing files *.ps_seq,
/// HIFN  separated by commas. To set the default directory, use command
/// HIFN  MMEM:CDIRectory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the available hopping list files.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqUserHoppingFileCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:USER:HOT:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqUserHoppingFileSelect(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_USER_HOPPING_FILE_SELECT, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the available attenuation list files.
/// HIFN     As response, you get a string containing the existing files *.ps_seq,
/// HIFN  separated by commas. To set the default directory, use command
/// HIFN  MMEM:CDIRectory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Array size parameter.
/// HIPAR catalog/Queries the available attenuation list files.
ViStatus _VI_FUNC rsrfsiggen_QueryExtSeqUserAttenuationFileCatalog(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SOUR%ld:BB:ESEQ:USER:AOT:FILE:CAT?", path);
    checkErr(rsrfsiggen_QueryViString(instrSession, cmd, arraySize, catalog));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.ps_arc.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR attenuation/Selects attenuation.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqUserAttenuationFileSelect(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   attenuation,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, attenuation, 1, 2),
            3, "Attenuation");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,ESAtt%ld", pathArr[path], attenuation);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_EXTSEQ_USER_ATTENUATION_FILE_SELECT, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_EXTSEQ_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExtSeqTriggerDateTime(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   year,
                                                            ViInt32   month,
                                                            ViInt32   day,
                                                            ViInt32   hour,
                                                            ViInt32   minute,
                                                            ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_EXTSEQ_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_EXTSEQ_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the OneWeb standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the OneWeb standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the configuration mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the configuration mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebMode(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the transmission direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Sets the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebLinkDirection(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects and loads a predefined reference signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/Selects and loads a predefined reference signal.
ViStatus _VI_FUNC rsrfsiggen_OneWebLoadReferenceSignals(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_LOAD_REFERENCE_SIGNALS, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR user/Sets the user.
/// HIPAR fileName/Selects an existing data list file from the default directory or from
/// HIPAR fileName/the specific directory.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebDLFrameUserDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   user,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_path[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, user, 1, 4),
            3, "User");

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap_path));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,User%ld", repCap_path, user);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_DL_FRAME_USER_DATA_LIST, fileName),
            4, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR subframe/Sets the subframe.
/// HIPAR allocation/Sets the Allocation.
/// HIPAR user/Sets the user.
/// HIPAR fileName/Selects an existing data list file from the default directory or from
/// HIPAR fileName/the specific directory.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebDLSubframeAllocationUserDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   subframe,
                            ViInt32   allocation,
                            ViInt32   user,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_path[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap_path));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframe, 0, 39),
            3, "Subframe");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, allocation, 0, 111),
            4, "Allocation");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, user, 1, 4),
            5, "User");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,SF%ld,All%ld,User%ld", repCap_path, subframe, allocation, user);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_DL_SUBFRAME_ALLOCATION_USER_DATA_LIST, fileName),
            6, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR userEquipment/Sets the User Equipment.
/// HIPAR cell/Sets the cell.
/// HIPAR fileName/Selects an existing data list file from the default directory or from
/// HIPAR fileName/the specific directory.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebULUEPUACHDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   userEquipment,
                            ViInt32   cell,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_path[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap_path));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userEquipment, 1, 4),
            3, "User Equipment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cell, 1, 4),
            4, "Cell");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,UE%ld,Cell%ld", repCap_path, userEquipment, cell);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_UL_UE_PUACH_DATA_LIST, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR userEquipment/Sets the User Equipment.
/// HIPAR cell/Sets the cell.
/// HIPAR fileName/Selects an existing data list file from the default directory or from
/// HIPAR fileName/the specific directory.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebULUEPUSCHDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   userEquipment,
                            ViInt32   cell,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_path[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap_path));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, userEquipment, 1, 4),
            3, "User Equipment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, cell, 1, 4),
            4, "Cell");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,UE%ld,Cell%ld", repCap_path, userEquipment, cell);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_UL_UE_PUSCH_DATA_LIST, fileName),
            5, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebTriggerSource(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Enables signal output synchronous to the trigger event.
/// HIPAR delay/Sets the trigger delay for triggering by the trigger signal from the
/// HIPAR delay/other path.
/// HIPAR inhibit/For triggering via the other path, specifies the duration by which a
/// HIPAR inhibit/restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebTrigger(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean sync,
                                                    ViReal64  delay,
                                                    ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_TRIGGER_SYNCHRONIZE_OUTPUT, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureOneWebClockSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_ONEWEB_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the Verizon 5G standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the Verizon 5G standard.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GEnabled(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects a predefined configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR predefinedConfiguration/Selects a predefined configuration.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GPredefinedConfiguration(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  predefinedConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_PREDEFINED_CONFIGURATION, predefinedConfiguration),
            3, "Predefined COnfiguration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the transmission direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GLinkDirection(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GTriggerMode(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GTriggerSource(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Enables/disables output of the signal synchronous to the external
/// HIPAR sync/trigger event.
/// HIPAR delay/Specifies the trigger delay for triggering by the signal from the
/// HIPAR delay/second path.
/// HIPAR inhibit/For triggering via the other path, specifies the number of samples by
/// HIPAR inhibit/which a restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GTrigger(ViSession instrSession,
                                                 ViInt32   path,
                                                 ViBoolean sync,
                                                 ViReal64  delay,
                                                 ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_CLOCK_SYNCHRONIZATION_OUTPUT_TO_EXTERNAL_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureV5GClockSource(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_V5G_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the 5G New Radio standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the Verizon 5G standard.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNREnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Exports the signal configuration to an .allocation file that you can
/// HIFN  import into the NR 5G application on a signal and spectrum analyzer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/The filename for the exported the signal configuration.
ViStatus _VI_FUNC rsrfsiggen_Export5GNRSettingsToAnalyzerFile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_pathRepCap(instrSession, path, repCap),
        2, "Path");
    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_5G_NR_EXPORT_SETTINGS_TO_ANALYZER_FILE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines the transmission direction.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR linkDirection/Defines the transmission direction.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRLinkDirection(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   linkDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_LINK_DIRECTION, linkDirection),
            3, "Link Direction");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Loads the file from the default or the specified directory.
/// HIFN     Loaded are files with extension *.vaf or *.dat.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRFilterLoad(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_FILTER_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIFN     Refer to 'Accessing Files in the Default or Specified Directory' for
/// HIFN  general information on file handling in the default and in a specific
/// HIFN  directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR user/User parameter.
/// HIPAR cell/Cell parameter.
/// HIPAR bwp/Bandwidth part parameter.
/// HIPAR group/Group parameter.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRDLBWPRateMatchDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   user,
                            ViInt32   cell,
                            ViInt32   bwp,
                            ViInt32   group,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCapPath));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,NR5GUser%ld,NR5GCell%ld,NR5GBWP%ld,NR5GGroup%ld", repCapPath, user, cell, bwp, group);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_DL_BWP_RATE_MATCH_DATA_LIST, fileName),
            7, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR cell/Cell parameter.
/// HIPAR subframe/Subframe parameter.
/// HIPAR user/User parameter.
/// HIPAR bwp/Bandwidth part parameter.
/// HIPAR allocation/Allocation parameter.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRSchedulingCORESETDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   cell,
                            ViInt32   subframe,
                            ViInt32   user,
                            ViInt32   bwp,
                            ViInt32   allocation,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCapPath));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,NR5GCell%ld,NR5GSubframe%ld,NR5GUser%ld,NR5GBWP%ld,NR5GAL%ld", repCapPath, cell, subframe, user, bwp, allocation);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_SCHEDULING_ALLOCATION_CORESET_DATA_LIST, fileName),
            8, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an existing data list file from the default directory or from
/// HIFN  the specific directory.
/// HIFN     Refer to 'Accessing Files in the Default or Specified Directory' for
/// HIFN  general information on file handling in the default and in a specific
/// HIFN  directory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR cell/Cell parameter.
/// HIPAR subframe/Subframe parameter.
/// HIPAR user/User parameter.
/// HIPAR bwp/Bandwidth part parameter.
/// HIPAR allocation/Allocation parameter.
/// HIPAR dci/DCI parameter.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRSchedulingCORESETDCIDataList(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   cell,
                            ViInt32   subframe,
                            ViInt32   user,
                            ViInt32   bwp,
                            ViInt32   allocation,
                            ViInt32   dci,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCapPath[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCapPath));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,NR5GCell%ld,NR5GSubframe%ld,NR5GUser%ld,NR5GBWP%ld,NR5GAL%ld,NR5GDCI%ld", repCapPath, cell, subframe, user, bwp, allocation, dci);

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_SCHEDULING_ALLOCATION_CORESET_DCI_DATA_LIST, fileName),
            9, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Enables/disables output of the signal synchronous to the external
/// HIPAR sync/trigger event.
/// HIPAR delay/Specifies the trigger delay for triggering by the signal from the
/// HIPAR delay/second path.
/// HIPAR inhibit/For triggering via the other path, specifies the number of samples by
/// HIPAR inhibit/which a restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViReal64  delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_EXTERNAL_TRIGGER_SYNC_OUTPUT_TO_EXT_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the RSRFSIGGEN_ATTR_5G_NR_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRTriggerDateTime(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   year,
                                                          ViInt32   month,
                                                          ViInt32   day,
                                                          ViInt32   hour,
                                                          ViInt32   minute,
                                                          ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_5G_NR_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_5G_NR_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_Configure5GNRClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_5G_NR_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the standard and deactivates all the other digital standards
/// HIFN  and digital modulation modes in the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates the standard and deactivates all the other digital standards
/// HIPAR enabled/and digital modulation modes in the same path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the general settings of LoRa standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR bandwidth/Sets the channel bandwidth.
/// HIPAR idleInterval/Sets the time of the interval separating two frames.
/// HIPAR sequenceLength/Sets the sequence length of the signal in number of frames. The signal
/// HIPAR sequenceLength/is calculated in advance and output in the arbitrary waveform
/// HIPAR sequenceLength/generator.
/// HIPAR oversampling/Sets the oversampling factor of the generated waveform.
/// HIPAR sampleRateVariation/Sets the sample rate of the signal.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaGeneral(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViInt32   bandwidth,
                                                  ViReal64  idleInterval,
                                                  ViInt32   sequenceLength,
                                                  ViInt32   oversampling,
                                                  ViReal64  sampleRateVariation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_BANDWIDTH, bandwidth),
            3, "Bandwidth");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IDLE_INTERVAL, idleInterval),
            4, "Idle Interval");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_SEQUENCE_LENGTH, sequenceLength),
            5, "Sequence Length");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_OVERSAMPLING, oversampling),
            6, "Oversampling");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_SAMPLE_RATE_VARIATION, sampleRateVariation),
            7, "Sample Rate Variation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frame general settings of LoRa standard.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR payloadReducedCodingMode/Activates the payload reduced coding mode.
/// HIPAR syncMode/Sets the synchronization mode of the preamble.
/// HIPAR unmodulatedPreambleLength/Sets the unmodulated preamble length.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaFrameGeneral(ViSession instrSession,
                                                       ViInt32   path,
                                                       ViBoolean payloadReducedCodingMode,
                                                       ViInt32   syncMode,
                                                       ViInt32   unmodulatedPreambleLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_PAYLOAD_REDUCED_CODING_MODE, payloadReducedCodingMode),
            3, "Payload Reduced Coding Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_SYNC_MODE, syncMode),
            4, "Sync Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_UNMODULATED_PREAMBLE_LENGTH, unmodulatedPreambleLength),
            5, "Unmodulated Preamble Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the frame Modulation And Coding Configuration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR codingRate/Sets the coding rate.
/// HIPAR encoderActive/Activates encoding of the modulation symbols.
/// HIPAR spreadingFactor/Sets the spreading factor for the modulation.
/// HIPAR interleaverActive/Activates the interleaver in the frame.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaFrameModulationAndCodingConfiguration(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   codingRate,
                            ViBoolean encoderActive,
                            ViInt32   spreadingFactor,
                            ViBoolean interleaverActive)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_CODING_RATE, codingRate),
            3, "Coding Rate");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_ENCODER_ACTIVE, encoderActive),
            4, "Encoder Active");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_SPREADING_FACTOR, spreadingFactor),
            5, "Spreading Factor");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_INTERLEAVER_ACTIVE, interleaverActive),
            6, "Interleaver Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the LoRa frame data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR dataLength/Sets the data length of the payload in the frame.
/// HIPAR payloadCRC/Activates a cyclic redundancy check (CRC) of the payload.
/// HIPAR dataSource/Sets the data source for the payload data in a LoRa frame.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaFrameData(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   dataLength,
                                                    ViBoolean payloadCRC,
                                                    ViInt32   dataSource)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_LENGTH, dataLength),
            3, "Data Length");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_PAYLOAD_CRC, payloadCRC),
            4, "Payload CRC");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_SOURCE, dataSource),
            5, "Data Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the LoRa frame data header.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR headerActive/Activates the header data in the frame.
/// HIPAR burstMode/Activates the burst mode of header data in the frame.
/// HIPAR compressedMode/Activates the compressed mode of the header data in the frame.
/// HIPAR reversedBit/Enables the reserved bit in the frame header.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaFrameDataHeaderConfiguration(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean headerActive,
                            ViBoolean burstMode,
                            ViBoolean compressedMode,
                            ViBoolean reversedBit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_HEADER_ACTIVE, headerActive),
            3, "Header Active");

    if (headerActive == VI_TRUE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_BURST_MODE, burstMode),
                4, "Burst Mode");

        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_COMPRESSED_MODE, compressedMode),
                5, "Compressed Mode");

        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_FRAME_DATA_REVERSED_BIT, reversedBit),
                6, "Reversed Bit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the LoRa Impairments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR impairments/Activates impairments settings in the payload.
/// HIPAR symbolTimingError/Sets symbol timing error.
/// HIPAR frequencyError/Sets the frequency offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaImpairments(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViBoolean impairments,
                                                      ViInt32   symbolTimingError,
                                                      ViReal64  frequencyError)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_ENABLED, impairments),
            3, "Impairments");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_SYMBOL_TIMING_ERROR, symbolTimingError),
            4, "Symbol Timing Error");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_FREQUENCY_OFFSET, frequencyError),
            5, "Frequency Error");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the LoRa Impairments Frequency Drift.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR enabled/Activates frequency drift settings in the payload.
/// HIPAR type/Sets the frequency drift type.
/// HIPAR deviation/Sets the frequency deviation of the frequency drift.
/// HIPAR driftRate/Sets the rate of the carrier frequency drift.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaImpairmentsFrequencyDrift(
                            ViSession instrSession,
                            ViInt32   path,
                            ViBoolean enabled,
                            ViInt32   type,
                            ViReal64  deviation,
                            ViInt32   driftRate)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_FREQUENCY_DRIFT_ENABLED, enabled),
            3, "Enabled");

    if (enabled == VI_TRUE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_FREQUENCY_DRIFT_TYPE, type),
                4, "Type");

        viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_FREQUENCY_DRIFT_DEVIATION, deviation),
                5, "Deviation");

        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_IMPAIRMENTS_FREQUENCY_DRIFT_RATE, driftRate),
                6, "Drift Rate");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the trigger mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the trigger mode.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaTriggerMode(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaTriggerSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger delay and inhibit.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR sync/Enables/disables output of the signal synchronous to the external
/// HIPAR sync/trigger event.
/// HIPAR delay/Sets the trigger delay for triggering by the trigger signal from the
/// HIPAR delay/other path.
/// HIPAR inhibit/For triggering via the other path, specifies the number of samples by
/// HIPAR inhibit/which a restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaTrigger(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean sync,
                                                  ViReal64  delay,
                                                  ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_EXTERNAL_TRIGGER_SYNC_OUTPUT_TO_EXT_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the clock source.
/// HIFN
/// HIFN     Selecting EXTernal is only possible for path A, since the external
/// HIFN  clock source is permanently allocated to path A.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR source/Sets the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureLoRaClockSource(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_LORA_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the standard and deactivates all the other digital standards
/// HIFN  and digital modulation modes in the same path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR enabled/Activates the standard and deactivates all the other digital standards
/// HIPAR enabled/and digital modulation modes in the same path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBEnabled(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_ENABLED,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_ENABLED, enabled),
            3, "Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Confifures the HRP UWB general settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR mode/Sets the HRP UWB mode.
/// HIPAR channelNumber/Sets the channel number.
/// HIPAR idleInterval/Sets the time of the interval separating two frames.
/// HIPAR sequenceLength/Sets the sequence length of the signal in number of frames. The signal
/// HIPAR sequenceLength/is calculated in advance and output in the arbitrary waveform
/// HIPAR sequenceLength/generator. The maximum number of frames is calculated as follows:
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBGeneral(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViInt32   mode,
                                                    ViInt32   channelNumber,
                                                    ViReal64  idleInterval,
                                                    ViInt32   sequenceLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_MODE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_MODE, mode),
            3, "Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_CHANNEL_NUMBER, channelNumber),
            4, "Channel Number");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_IDLE_INTERVAL, idleInterval),
            5, "Idle Interval");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_SEQUENCE_LENGTH, sequenceLength),
            6, "Sequence Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the channel bandwidth.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR bandwidth/Queries the channel bandwidth.
ViStatus _VI_FUNC rsrfsiggen_QueryHRPUWBBandwidth(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViReal64* bandwidth)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_BANDWIDTH,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_GetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_BANDWIDTH, bandwidth),
            3, "Bandwidth");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures UWB HRP clipping settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR enabled/Activates level clipping (Clipping). The value is defined with the
/// HIPAR enabled/command RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_LEVEL, the mode of calculation
/// HIPAR enabled/with the command RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_MODE.
/// HIPAR level/Sets the limit for level clipping.
/// HIPAR mode/Sets the method for level clipping.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBClipping(ViSession instrSession,
                                                     ViInt32   path,
                                                     ViBoolean enabled,
                                                     ViInt32   level,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_ENABLED,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_LEVEL, level),
            4, "Level");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_CLIPPING_MODE, mode),
            5, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the impairments settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR enabled/Activates adding impairments to the signal. Impairments change the
/// HIPAR enabled/signal to simulate a non-ideal transmitter.
/// HIPAR chipClockError/Sets the chip clock error of the impairment symbols.
/// HIPAR frequencyOffset/Sets the frequency offset.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBImpairments(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViBoolean enabled,
                                                        ViInt32   chipClockError,
                                                        ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_IMPAIRMENTS_ENABLED,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_IMPAIRMENTS_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_IMPAIRMENTS_CHIP_CLOCK_ERROR, chipClockError),
            4, "Chip Clock Error");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_IMPAIRMENTS_FREQUENCY_OFFSET, frequencyOffset),
            5, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects trigger mode, i.e. determines the effect of a trigger event on
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR mode/Selects trigger mode, i.e. determines the effect of a trigger event on
/// HIPAR mode/the signal generation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBTriggerMode(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_TRIGGER_MODE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_TRIGGER_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the trigger signal source and determines the way the
/// HIFN  triggering is executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR source/Selects the trigger signal source and determines the way the
/// HIPAR source/triggering is executed.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBTriggerSource(ViSession instrSession,
                                                          ViInt32   path,
                                                          ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_TRIGGER_SOURCE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_TRIGGER_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the internal trigger settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR sync/Enables signal output synchronous to the trigger event.
/// HIPAR delay/Specifies the trigger delay (expressed as number of samples) for
/// HIPAR delay/triggering by the trigger signal from the other path (two-path
/// HIPAR delay/instruments only).
/// HIPAR inhibit/For triggering via the other path, specifies the number of samples by
/// HIPAR inhibit/which a restart is inhibited.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBTrigger(ViSession instrSession,
                                                    ViInt32   path,
                                                    ViBoolean sync,
                                                    ViReal64  delay,
                                                    ViInt32   inhibit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_EXTERNAL_TRIGGER_SYNC_OUTPUT_TO_EXT_TRIGGER,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_EXTERNAL_TRIGGER_SYNC_OUTPUT_TO_EXT_TRIGGER, sync),
            3, "Sync");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_INTERNAL_TRIGGER_DELAY, delay),
            4, "Delay");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_INTERNAL_TRIGGER_INHIBIT, inhibit),
            5, "Inhibit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the date and time for a time-based trigger signal. For
///HINF  trigger modes single or armed auto, you can activate triggering
///HINF  at this time via the
///HINF  RSRFSIGGEN_ATTR_HRP_UWB_TIME_BASED_TRIGGER.
///HIRET Returns the status code of this operation.
///HIPAR path/The path for which is this function performed.
///HIPAR year/Sets the year.
///HIPAR month/Sets the month.
///HIPAR day/Sets the day.
///HIPAR hour/Sets the hour.
///HIPAR minute/Sets the minute.
///HIPAR second/Sets the second.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBTriggerDateTime(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   year,
                                                            ViInt32   month,
                                                            ViInt32   day,
                                                            ViInt32   hour,
                                                            ViInt32   minute,
                                                            ViInt32   second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   dateParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";
    ViChar   timeParams[RS_MAX_SHORT_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(dateParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", year, month, day);
    snprintf(timeParams, RS_MAX_SHORT_MESSAGE_BUF_SIZE, "%d, %d, %d", hour, minute, second);

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_HRP_UWB_TRIGGER_DATE, dateParams));
    checkErr(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_HIDDEN_HRP_UWB_TRIGGER_TIME, timeParams));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the clock source.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR path/Selects repcap Path.
/// HIPAR source/Selects the clock source.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHRPUWBClockSource(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   source)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViInt32  pathMax = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetRepCapIdItemsCount(instrSession, "Path", &pathMax), 2, "Path");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, pathMax),
            2, "Path");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSRFSIGGEN_ATTR_HRP_UWB_CLOCK_SOURCE,
            "Path", path- 1, RS_REPCAP_BUF_SIZE, repCap),
            2, "Path");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_HRP_UWB_CLOCK_SOURCE, source),
            3, "Source");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates fading simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fadingEnabled/Activates fading simulation.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMEnabled(ViSession instrSession,
                                                  ViInt32   path,
                                                  ViBoolean fadingEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_ENABLED, fadingEnabled),
            3, "Fading Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Selects a predefined fading simulator setting which complies with the
/// HIFN test specifications found in the common mobile radio standards.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR standard/Selects a predefined fading simulator setting which complies with the
/// HIPAR standard/test specifications found in the common mobile radio standards.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMStandard(ViSession instrSession,
                                                   ViInt32   path,
                                                   ViInt32   standard)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_STANDARD, standard),
            3, "Standard");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures fading simulation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR configuration/Sets the fading configuration.
/// HIPAR signalDedicatedTo/Selects the RF frequency which is used as a basis for computing the
/// HIPAR signalDedicatedTo/Doppler shift.
/// HIPAR virtualRF/Determines the virtual RF frequency if the baseband signal is output
/// HIPAR virtualRF/(and not the RF signal) on the instrument. The entered value is used as
/// HIPAR virtualRF/the basis for computing the Doppler shift.
/// HIPAR ignoreRFChangesLt5PCT/Determines whether frequency changes < 5% are ignored. This enables
/// HIPAR ignoreRFChangesLt5PCT/faster frequency hopping.
/// HIPAR hoppingMode/Activates frequency hopping and determines how fading is resumed after
/// HIPAR hoppingMode/a frequency hop. This function is available only for the Standard Delay
/// HIPAR hoppingMode/and Fine Delay 30 MHz / 50 MHz fading configurations.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIM(ViSession instrSession,
                                           ViInt32   path,
                                           ViInt32   configuration,
                                           ViInt32   signalDedicatedTo,
                                           ViReal64  virtualRF,
                                           ViBoolean ignoreRFChangesLt5PCT,
                                           ViInt32   hoppingMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_FADING_CONFIGURATION, configuration),
            3, "Configuration");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_SIGNAL_DEDICATED_TO, signalDedicatedTo),
            4, "Ssignal Dedicated To");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_VIRTUAL_RF_FREQUENCY, virtualRF),
            5, "Virtual RF");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_IGNORE_RF_CHANGES_LT_5PCT, ignoreRFChangesLt5PCT),
            6, "Ignore RF Changes Lt 5 PCT");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_FREQUENCY_HOPPING_MODE, hoppingMode),
            7, "Hopping Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures insertion loss.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR mode/Sets the insertion loss of the fading simulator.
/// HIPAR insertionLoss/Sets the insertion loss of the fading simulator.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMInsertionLoss(ViSession instrSession,
                                                        ViInt32   path,
                                                        ViInt32   mode,
                                                        ViReal64  insertionLoss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_INSERTION_LOSS_MODE, mode),
            3, "mode");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_INSERTION_LOSS, insertionLoss),
            4, "Insertion Loss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the share of samples which were clipped due to
/// HIFN  the insertion loss setting.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR buffersize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR buffersize/Clipped Samples parameter.
/// HIPAR clippedSamples/Queries the share of samples which were clipped due to the insertion
/// HIPAR clippedSamples/loss setting.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMInsertionLossClippedSamples(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   buffersize,
                            ViChar    clippedSamples[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_CLIPPED_SAMPLES,
            buffersize, clippedSamples),
            4, "Clipped Samples");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures coupled parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR speedSetting/Couples the setting for the speed for the paths of both faders. A
/// HIPAR speedSetting/change in the speed in one fader leads to a corresponding change in the
/// HIPAR speedSetting/second fader. The Common Speed for all Paths setting is also coupled
/// HIPAR speedSetting/(SOURce:FSIM:CSPeed).
/// HIPAR localConstant/With lognormal fading couples the setting of the Local Constant for
/// HIPAR localConstant/the paths of both faders. A change in the local constant in one fader
/// HIPAR localConstant/leads to a corresponding change in the second fader.
/// HIPAR standardDeviation/Couples the setting for the lognormal standard deviation for the paths
/// HIPAR standardDeviation/of both faders. A change in the standard deviation in one fader leads
/// HIPAR standardDeviation/to a corresponding change in the other path.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMCoupledParameters(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViBoolean speedSetting,
                                                            ViBoolean localConstant,
                                                            ViBoolean standardDeviation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_SPEED_SETTING_COUPLED, speedSetting),
            3, "Speed Setting");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_LOCAL_CONSTANT_COUPLED, localConstant),
            4, "Local Constant");

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_STANDARD_DEVIATION_COUPLED, standardDeviation),
            5, "Standard Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures global fader coupling.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR startSeed/Sets the fading start seed. This value is global for the instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMGlobalFaderCoupling(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   startSeed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_GLOBAL_SEED, startSeed),
            3, "Start Seed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads the selected scenario file. The directory is set
/// HIFN  using command MMEM:CDIRectory. A path can also be specified, in which
/// HIFN  case the files in the specified directory are read. The file extension
/// HIFN  may be omitted. Only files with the file extension *.fad_udyn will be
/// HIFN  loaded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_FSIMScenarioLoad(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_SCENARIO_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function stores the current scenario settings into the selected
/// HIFN  file. The directory is set using command MMEM:CDIRectory. Only the file
/// HIFN  name has to be entered. Scenario files are stored as files with the
/// HIFN  specific file extension *.fad_udyn.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_FSIMScenarioSave(ViSession instrSession,
                                              ViInt32   path,
                                              ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_SCENARIO_SAVE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the Rx/Tx coordinates in ENU coordination system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR x/Sets the X coordinate.
/// HIPAR y/Sets the Y coordinate.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMUserTrajectoryENU(ViSession instrSession,
                                                            ViInt32   path,
                                                            ViInt32   rxTx,
                                                            ViReal64  x,
                                                            ViReal64  y,
                                                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, x, 0.0, 1000.0), 4, "X");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, y, 0.0, 1000.0), 5, "Y");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, 0.0, 1000.0), 6, "altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:USER:%s:TRAJ:FAP:COOR:XYZ %lf,%lf,%lf", path, rxTxArr[rxTx], x, y, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the coordinates of the Tx/Rx fixed vehicle in
/// HIFN  degrees, minutes and seconds format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR longitude/Sets coordinates of in degrees, minutes and seconds format.
/// HIPAR longitudeDirection/Selects a whether east or west longitude is used.
/// HIPAR latitude/Sets coordinates of in degrees, minutes and seconds format.
/// HIPAR latitudeDirection/Selects a whether east or west longitude is used.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMUserTrajectoryCoordinatesDegree(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   rxTx,
                            ViString  longitude,
                            ViString  longitudeDirection,
                            ViString  latitude,
                            ViString  latitudeDirection,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, 0.0, 1000.0), 8, "Altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:USER:%s:TRAJ:FAP:COOR:DMS %s,%s,%s,%s,%lf", path, rxTxArr[rxTx], longitude, longitudeDirection, latitude, latitudeDirection, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the coordinates of the Tx/Rx fixed vechicle in
/// HIFN  decimal format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMUserTrajectoryCoordinatesDecimalDegree(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   rxTx,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, 0.0, 1000.0), 6, "altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:USER:%s:TRAJ:FAP:COOR %lf,%lf,%lf", path, rxTxArr[rxTx], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the coordinates of the reference location in ENU
/// HIFN  coordination system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
/// HIPAR altitude/Sets the altitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMUserTrajectoryENUReferenceCoordinates(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   rxTx,
                            ViReal64  longitude,
                            ViReal64  latitude,
                            ViReal64  altitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "latitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, altitude, 0.0, 1000.0), 6, "altitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:USER:%s:TRAJ:TDF:ENU:COOR %lf,%lf,%lf", path, rxTxArr[rxTx], longitude, latitude, altitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the coordinates of the Tx/Rx ship in degrees,
/// HIFN  minutes and seconds format.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR longitude/Sets coordinates of in degrees, minutes and seconds format.
/// HIPAR longitudeDirection/Selects a whether east or west longitude is used.
/// HIPAR latitude/Sets coordinates of in degrees, minutes and seconds format.
/// HIPAR latitudeDirection/Selects a whether east or west longitude is used.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMShipToShipCoordinatesDegree(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   rxTx,
                            ViString  longitude,
                            ViString  longitudeDirection,
                            ViString  latitude,
                            ViString  latitudeDirection)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:SHIP:%s:COOR:DMS %s,%s,%s,%s",
            path, rxTxArr[rxTx], longitude, longitudeDirection, latitude, latitudeDirection);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the coordinates of the Tx/Rx ship in decimal
/// HIFN  format.
/// HIFN
/// HIFN     Note(s):
/// HIFN     (1) To set the altitude, use the parameter Height of Antenna.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR longitude/Sets the longitude.
/// HIPAR latitude/Sets the latitude.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMShipToShipCoordinatesDecimalDegree(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   rxTx,
                            ViReal64  longitude,
                            ViReal64  latitude)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            3, "Rx Tx");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, longitude, -180.0, 180.0), 4, "longitude");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, latitude, -90.0, 90.0), 5, "latitude");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DSS:SHIP:%s:COOR %lf,%lf", path, rxTxArr[rxTx], longitude, latitude);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the paramters of the custom fading profile.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR bandwidth/Sets the bandwidth of the original Doppler profile from which the
/// HIPAR bandwidth/resulting profile is created.
/// HIPAR offsetFrequency/Sets the frequency offset , i.e. the frequency offset used to shift
/// HIPAR offsetFrequency/the original profile.
/// HIPAR lowerCutoffFrequency/Sets the lower cut-off frequency.
/// HIPAR upperCutoffFrequency/Sets the upper cut-off frequency.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMCustomFadingProfile(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   group,
                            ViInt32   delayPath,
                            ViReal64  bandwidth,
                            ViReal64  offsetFrequency,
                            ViReal64  lowerCutoffFrequency,
                            ViReal64  upperCutoffFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, group, 1, 8),
            3, "group");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, delayPath, 1, 40),
            4, "Delay Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, bandwidth, 50.0, 40000.0), 5, "bandwidth");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offsetFrequency, -23950.0, 23950.0), 6, "Offset Frequency");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowerCutoffFrequency, -4000.0, 3950.0), 7, "Lower Cutoff Frequency");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, upperCutoffFrequency, -3950.0, 4000.0), 8, "Upper Cutoff Frequency");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:DEL:GRO%ld:PATH%ld:CUST:DATA %lf,%lf,%lf,%lf", path, group, delayPath, bandwidth, offsetFrequency,
        lowerCutoffFrequency, upperCutoffFrequency);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads a file with saved MIMO settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_FSIMMIMOLoad(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_MIMO_LOAD, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function loads a file with saved MIMO settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR fileName/File name parameter.
ViStatus _VI_FUNC rsrfsiggen_FSIMMIMOSave(ViSession instrSession,
                                          ViInt32   path,
                                          ViString  fileName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, repCap, RSRFSIGGEN_ATTR_FSIM_MIMO_SAVE, fileName),
            3, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value for the phase of the receiver correlation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR phase/Sets the phase.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMOKroneckerCorrelationPhase(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   rxTx,
                            ViInt32   matrixPath,
                            ViReal64  phase)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            4, "Rx Tx");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            5, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phase, 0.0, 360.0), 6, "phase");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:KRON:CORR:%s:%s:PHASe %lf", path, tap, rxTxArr[rxTx], pathMatrixArr[matrixPath], phase);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the ratio of the receiver correlation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR ratio/Sets the ratio.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMOKroneckerCorrelationRatio(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   rxTx,
                            ViInt32   matrixPath,
                            ViReal64  ratio)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            4, "Rx Tx");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            5, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, ratio, 0.0, 1.0), 6, "ratio");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:KRON:CORR:%s:%s:RAT %lf", path, tap, rxTxArr[rxTx], pathMatrixArr[matrixPath], ratio);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value for the real part of the receiver correlation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR real/Sets the real part.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMOKroneckerCorrelationReal(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   rxTx,
                            ViInt32   matrixPath,
                            ViReal64  real)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            4, "Rx Tx");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            5, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, real, -1.0, 1.0), 6, "real");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:KRON:CORR:%s:%s:REAL %lf", path, tap, rxTxArr[rxTx], pathMatrixArr[matrixPath], real);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the value for the imaginary part of the receiver correlation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR rxTx/Sets the Rx or Tx coordinates.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR imginary/Sets the imaginary part.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMOKroneckerCorrelationImaginary(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   rxTx,
                            ViInt32   matrixPath,
                            ViReal64  imginary)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxTx, RSRFSIGGEN_VAL_FSIM_COOR_RX, RSRFSIGGEN_VAL_FSIM_COOR_TX),
            4, "Rx Tx");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            5, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, imginary, -1.0, 1.0), 6, "imginary");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:KRON:CORR:%s:%s:IMAG %lf", path, tap, rxTxArr[rxTx], pathMatrixArr[matrixPath], imginary);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the relative gain in the selected path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR gain/Sets the gain.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMORelativeGainVectorMatrixGain(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   matrixPath,
                            ViReal64  gain)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            4, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, gain, -50.0, 0.0), 5, "gain");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:GVEC:%s:GAIN %lf", path, tap, pathMatrixArr[matrixPath], gain);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the phase shift of the selected path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR tap/Sets the current tap.
/// HIPAR matrixPath/This control selects the path.
/// HIPAR phaseShift/Sets the phase shift.
ViStatus _VI_FUNC rsrfsiggen_ConfigureFSIMMIMORelativeGainVectorMatrixPhaseShift(
                            ViSession instrSession,
                            ViInt32   path,
                            ViInt32   tap,
                            ViInt32   matrixPath,
                            ViReal64  phaseShift)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, tap, 1, 10),
            3, "Tap");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, matrixPath, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_AA, RSRFSIGGEN_VAL_FSIM_MIMO_PATH_HH),
            4, "Matrix Path");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, phaseShift, 0.0, 360.0), 5, "Phase Shift");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR%ld:FSIM:MIMO:TAP%ld:GVEC:%s:PHAS %lf", path, tap, pathMatrixArr[matrixPath], phaseShift);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the status of the required connections between the R&S SMW and
/// HIFN  the R&S SZU.
/// HIFN  R&S SZU is connected to the R&S SMW via USB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR arraySize/Sets the size of the Connections Status array.
/// HIPAR rfConnStatus/Returns the status of the required connections.
/// HIPAR iOUTConnStatus/Returns the status of the required connections.
/// HIPAR iBAROUTConnStatus/Returns the status of the required connections.
/// HIPAR qOUTConnStatus/Returns the status of the required connections.
/// HIPAR qBAROUTConnStatus/Returns the status of the required connections.
/// HIPAR triggerConnStatus/Returns the status of the required connections.
ViStatus _VI_FUNC rsrfsiggen_CheckConnectionsSettings(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32*  RFConnStatus,
                                                      ViInt32*  iOUTConnStatus,
                                                      ViInt32*  iBAROUTConnStatus,
                                                      ViInt32*  qOUTConnStatus,
                                                      ViInt32*  qBAROUTConnStatus,
                                                      ViInt32*  triggerConnStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  retCnt = 0;
    ViAInt32 values = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, path, 1, 4),
            2, "Path");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SCON:EXT:IQO%ld:CONN:CHEC?", path);
    checkErr(RsCore_QueryAsciiViInt32Array(instrSession, cmd, &values, &retCnt));

    *RFConnStatus = values[0];
    *iOUTConnStatus = values[1];
    *iBAROUTConnStatus = values[2];
    *qOUTConnStatus = values[3];
    *qBAROUTConnStatus = values[4];
    *triggerConnStatus = values[5];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (values) free(values);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds manually an external instrument to the list of available
/// HIFN  instruments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR instrumentName/Sets the instrument name.
/// HIPAR hardwareChannel/Hardware channel (USB or LAN) used by the remote channel to the
/// HIPAR hardwareChannel/external instrument
/// HIPAR address/IP address or hostname of the connected external instrument.
/// HIPAR rfPath/Determines the number of RF paths the externa instrument is equipped
/// HIPAR rfPath/with.
ViStatus _VI_FUNC rsrfsiggen_SystemConfigurationExternalAdd(ViSession instrSession,
                                                            ViString  instrumentName,
                                                            ViString  hardwareChannel,
                                                            ViString  address,
                                                            ViString  RFPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SCON:EXT:REM:ADD \"%s\",\"%s\",\"%s\",\"%s\"\n", instrumentName, hardwareChannel, address, RFPath);

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function changes the symbolic name of the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR currentName/Current name is a parameter indicating the actual name of the
/// HIPAR currentName/instrument.
/// HIPAR newName/New name is a parameter that indicats the new wanted name of the
/// HIPAR newName/instrument.
ViStatus _VI_FUNC rsrfsiggen_SystemConfigurationExternalRename(
                            ViSession instrSession,
                            ViString  currentName,
                            ViString  newName)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SCON:EXT:REM:REN \"%s\",\"%s\"", currentName, newName);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the test generator for the bit error rate measurement,
/// HIFN  or the block error rate measurement respectively.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR enabled/CActivates the test generator for the bit error rate measurement,
/// HIPAR enabled/or the block error rate measurement respectively.
ViStatus _VI_FUNC rsrfsiggen_BERTestGenState(ViSession instrSession,
                                             ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, "", RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_ENABLED, enabled),
            2, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the BER test generator settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measurementMode/Selects the measurement mode of the BER/BLER test generator.
/// HIPAR clockFrequency/Sets the clock frequency for the BER/BLER test generator.
/// HIPAR errorRate/Sets the error bit pattern of the BER/BLER test generator.
/// HIPAR trigger/Selects the trigger input mode for the BER/BLER test generator.
/// HIPAR dataEnableOnTime/Sets the data enable time interval of the BER/BLER test generator.
/// HIPAR dataEnableOffTime/Sets the data enable time interval of the BER/BLER test generator.
/// HIPAR restartDelay/Sets the delay time for the restart signal of the BER/BLER test generator.
ViStatus _VI_FUNC rsrfsiggen_BERTestGenConfig(ViSession instrSession,
                                              ViInt32   measurementMode,
                                              ViInt32   clockFrequency,
                                              ViReal64  errorRate,
                                              ViInt32   trigger,
                                              ViInt64   dataEnableOnTime,
                                              ViInt64   dataEnableOffTime,
                                              ViInt64   restartDelay)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_MEASUREMENT_MODE, measurementMode),
            2, "Measurement Mode");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_CLOCK_FREQUENCY, clockFrequency),
            3, "Clock Frequency");

    viCheckParm(rsrfsiggen_SetAttributeViReal64(instrSession, "", RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_ERROR_RATE, errorRate),
            4, "Error Rate");

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_TRIGGER, trigger),
            5, "Trigger");

    if (trigger == RSRFSIGGEN_VAL_TEST_GENERATOR_TRIGGER_DENEBLE)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt64(instrSession, "",
            RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_DATA_ENABLE_ON_TIME,
            dataEnableOnTime), 6, "Data Enable On Time");

        viCheckParm(rsrfsiggen_SetAttributeViInt64(instrSession, "",
            RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_DATA_ENABLE_OFF_TIME,
            dataEnableOffTime), 7, "Data Enable Off Time");
    }
    else if (trigger == RSRFSIGGEN_VAL_TEST_GENERATOR_TRIGGER_RESTART)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt64(instrSession, "",
            RSRFSIGGEN_ATTR_BERT_BLER_TEST_GENERATOR_RESTART_DELAY,
            restartDelay), 8, "Restart Delay");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the last BLER measurement. In the first measurement
/// HIFN  following the start, intermediate results for the number of data blocks,
/// HIFN  errors and error rate are also queried. In the following measurements
/// HIFN  (only for :BLER:SEQ AUTO) only the final results of each single measurement are queried.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataBits/Number of checked data bits.
/// HIPAR errors/Number of error bits.
/// HIPAR bitErrorRate/Error rate. If no termination criterion has been reached since
/// HIPAR bitErrorRate/the beginning of the measurement, the current quotient of "Number of error bits"
/// HIPAR bitErrorRate/vand "Number of data bits" is entered. As soon as at least one final result has been
/// HIPAR bitErrorRate/reached in continuous measurement, the most recent final result is entered.
/// HIPAR measurementStatus/Status of measurement.
/// HIPAR clockLineStatus/Status of clock line.
/// HIPAR dataLineStatus/Status of data line.
/// HIPAR synchronizationStatus/Synchronization status.
ViStatus _VI_FUNC rsrfsiggen_GetBLERResult(ViSession instrSession,
                                           ViInt32*  dataBits,
                                           ViInt32*  errors,
                                           ViReal64* bitErrorRate,
                                           ViInt32*  measurementStatus,
                                           ViInt32*  clockLineStatus,
                                           ViInt32*  dataLineStatus,
                                           ViInt32*  synchronizationStatus)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* results = NULL;
    ViInt32   retCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "BLER:RES?", &results, &retCnt));

    *dataBits = (ViInt32)results[0];
    *errors = (ViInt32)results[1];
    *bitErrorRate = results[2];
    *measurementStatus = (ViInt32)results[3];
    *clockLineStatus = (ViInt32)results[4];
    *dataLineStatus = (ViInt32)results[5];
    *synchronizationStatus = (ViInt32)results[6];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the result of the last BER measurement. In the first measurement
/// HIFN  following the start, intermediate results for the number of data blocks,
/// HIFN  errors and error rate are also queried. In the following measurements
/// HIFN  (only for :BLER:SEQ AUTO) only the final results of each single measurement are queried.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR dataBits/Number of checked data bits.
/// HIPAR errors/Number of error bits.
/// HIPAR bitErrorRate/Error rate. If no termination criterion has been reached since
/// HIPAR bitErrorRate/the beginning of the measurement, the current quotient of "Number of error bits"
/// HIPAR bitErrorRate/vand "Number of data bits" is entered. As soon as at least one final result has been
/// HIPAR bitErrorRate/reached in continuous measurement, the most recent final result is entered.
/// HIPAR measurementStatus/Status of measurement.
/// HIPAR clockLineStatus/Status of clock line.
/// HIPAR dataLineStatus/Status of data line.
/// HIPAR synchronizationStatus/Synchronization status.
ViStatus _VI_FUNC rsrfsiggen_GetBERResult(ViSession instrSession,
                                          ViInt32*  dataBits,
                                          ViInt32*  errors,
                                          ViReal64* bitErrorRate,
                                          ViInt32*  measurementStatus,
                                          ViInt32*  clockLineStatus,
                                          ViInt32*  dataLineStatus,
                                          ViInt32*  synchronizationStatus)
{
    ViStatus  error = VI_SUCCESS;
    ViReal64* results = NULL;
    ViInt32   retCnt = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryAsciiViReal64Array(instrSession, "BERT:RES?", &results, &retCnt));

    *dataBits = (ViInt32)results[0];
    *errors = (ViInt32)results[1];
    *bitErrorRate = results[2];
    *measurementStatus = (ViInt32)results[3];
    *clockLineStatus = (ViInt32)results[4];
    *dataLineStatus = (ViInt32)results[5];
    *synchronizationStatus = (ViInt32)results[6];

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    if (results) free(results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status of the remote connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR remoteConnectionState/Returns the status of the remote connection.
ViStatus _VI_FUNC rsrfsiggen_QueryCoderRemoteConnection(ViSession  instrSession,
                                                        ViInt32    path,
                                                        ViBoolean* remoteConnectionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_GetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EXTERNAL_CODER_REMOTE_CONNECTION_ENABLED, remoteConnectionState),
            3, "Remote Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the status of the I/Q connection of the digital
/// HIFN  interfaces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR iqConnectionState/Returns the status of the I/Q connection of the digital interfaces.
ViStatus _VI_FUNC rsrfsiggen_QueryBBMMRemoteConnection(ViSession  instrSession,
                                                       ViInt32    path,
                                                       ViBoolean* iqConnectionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    viCheckParm(rsrfsiggen_GetAttributeViBoolean(instrSession, repCap, RSRFSIGGEN_ATTR_EXTERNAL_BBMM_CONNECTION_ENABLED, iqConnectionState),
            3, "IQ Connection State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects an external instrument for the selected connector and triggers
/// HIFN  connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR instrumentName/Instrument alias name.
/// HIPAR rfPath/Used RF output of the external instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureBBMMSelectedConnectorTriggerConnection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  instrumentName,
                            ViString  rfPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", instrumentName, rfPath);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_EXTERNAL_BBMM_SELECTED_CONNECTOR_TRIGGER_CONNECTION, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an external instrument for the selected connector and triggers
/// HIFN  connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR instrumentName/Instrument alias name.
/// HIPAR rfPath/Used RF output of the external instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExternalCoderSelectedConnectorTriggerConnection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  instrumentName,
                            ViString  rfPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", instrumentName, rfPath);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_EXTERNAL_CODER_SELECTED_CONNECTOR_TRIGGER_CONNECTION, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an external instrument for the selected connector and triggers
/// HIFN  connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR instrumentName/Instrument alias name.
/// HIPAR rfPath/Used RF output of the external instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExternalFaderSelectedConnectorTriggerConnection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  instrumentName,
                            ViString  rfPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", instrumentName, rfPath);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_EXTERNAL_FADER_SELECTED_CONNECTOR_TRIGGER_CONNECTION, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an external instrument for the selected connector and triggers
/// HIFN  connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR instrumentName/Instrument alias name.
/// HIPAR rfPath/Used RF output of the external instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExternalIQSelectedConnectorTriggerConnection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  instrumentName,
                            ViString  rfPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", instrumentName, rfPath);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_EXTERNAL_IQ_SELECTED_CONNECTOR_TRIGGER_CONNECTION, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects an external instrument for the selected connector and triggers
/// HIFN  connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR instrumentName/Instrument alias name.
/// HIPAR rfPath/Used RF output of the external instrument.
ViStatus _VI_FUNC rsrfsiggen_ConfigureExternalRFSelectedConnectorTriggerConnection(
                            ViSession instrSession,
                            ViInt32   path,
                            ViString  instrumentName,
                            ViString  rfPath)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    snprintf(param, RS_PARAM_BUF_SIZE, "'%s','%s'", instrumentName, rfPath);

    viCheckParm(RsCore_SetAttributeViRawString(instrSession, repCap,
                                                 RSRFSIGGEN_ATTR_HIDDEN_EXTERNAL_RF_SELECTED_CONNECTOR_TRIGGER_CONNECTION, RS_VAL_DIRECT_USER_CALL,
                                                 param), 3, "Wrong parameters");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the selected remote connection.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR remoteConnection/Selected remote connection.
ViStatus _VI_FUNC rsrfsiggen_ConfigureCoderConnection(ViSession instrSession,
                                                      ViInt32   path,
                                                      ViInt32   remoteConnection)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_pathRepCap(instrSession, path, repCap));

    switch (remoteConnection)
    {
    case RSRFSIGGEN_VAL_EXTERNAL_CODER_DISCONNECT:
        attribute = RSRFSIGGEN_ATTR_EXTERNAL_CODER_DISCONNECT_CONNECTION;
        break;

    case RSRFSIGGEN_VAL_EXTERNAL_CODER_CONNECT:
        attribute = RSRFSIGGEN_ATTR_EXTERNAL_CODER_TRIGGER_CONNECTION;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, remoteConnection), 3, "Remote Connection");
        break;
    }
    if (attribute != 0)
    {
        viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, repCap, attribute, remoteConnection),
                3, "Remote Connection");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
/// HIFN  Loads the selected predefined file.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR fileName/Loads the selected predefined file.
ViStatus _VI_FUNC rsrfsiggen_LoadRFPortsAlignmentPredefinedFile(
                            ViSession instrSession,
                            ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_RF_PORTS_ALIGNMENT_LOAD_PREDEFINED_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
/// HIFN  Loads the selected file from the default or the specified directory.
/// HIFN  Loaded are files with extension *.rfsa.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR fileName/Loads the selected file from the default or the specified directory.
/// HIPAR fileName/Loaded are files with extension *.rfsa.
ViStatus _VI_FUNC rsrfsiggen_LoadRFPortsAlignmentSetupFile(ViSession instrSession,
                                                           ViString  fileName)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_RF_PORTS_ALIGNMENT_LOAD_SETUP_FILE, fileName),
            2, "File Name");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
/// HIFN  Queries the frequency list for calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR arraySize/Array size parameter.
/// HIPAR frequencyList/Queries the frequency list for calibration.
/// HIPAR returnedValues/Returns the number of values returned in the Frequency List.
ViStatus _VI_FUNC rsrfsiggen_QueryRFPortsAlignmentCalibrationFrequencyList(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  frequencyList[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, ":SCON:RFAL:SET:INFO:CAL:FREQ:LIST:VAL?",
        arraySize, frequencyList, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
/// HIFN  Queries the PEP list for calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR arraySize/Array size parameter.
/// HIPAR PEPList/Queries the PEP list for calibration.
/// HIPAR returnedValues/Returns the number of values returned in the PEP List.
ViStatus _VI_FUNC rsrfsiggen_QueryRFPortsAlignmentCalibrationPEPList(
                            ViSession instrSession,
                            ViInt32   arraySize,
                            ViReal64  PEPList[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, ":SCON:RFAL:SET:INFO:CAL:POW:LIST:VAL?",
        arraySize, PEPList, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Adds or modifies a key-value pair (device tag).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR id/ID number of the tag you want to modify or query.
/// HIPAR key/String containing key name of the queried tag.
/// HIPAR value/String containing value of the queried tag.
ViStatus _VI_FUNC rsrfsiggen_ConfigureHUMSTagValue(ViSession instrSession,
                                                   ViInt32   id,
                                                   ViString  key,
                                                   ViString  tagValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, id, 0, 31),
            2, "ID");

    snprintf (param, RS_PARAM_BUF_SIZE, "%ld,'%s','%s'", id, key, tagValue);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_DIAGNOSTIC_HUMS_TAGS_VALUE, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Adds or modifies a key-value pair (device tag).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR id/ID number of the tag you want to modify or query.
/// HIPAR arraySize/Array size parameter.
/// HIPAR tagsValues/Queries the tags value.
ViStatus _VI_FUNC rsrfsiggen_QueryHUMSTagValue(ViSession instrSession,
                                               ViInt32   id,
                                               ViInt32   arraySize,
                                               ViChar    tagsValues[])
{
    ViStatus error = VI_SUCCESS;
    ViInt32  respLen = 0;
    ViChar   param[RS_PARAM_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, id, 0, 31),
            2, "ID");

    snprintf (param, RS_PARAM_BUF_SIZE, "%ld", id);

    checkErr(RsCore_GetAttributeViRawStringWithQparam(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_QUERY_DIAGNOSTIC_HUMS_TAGS_VALUE, param, arraySize, tagsValues, &respLen));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Defines an SNMP user profile.
///HIRET Returns the status code of this operation.
///HIPAR name/String containing name of the user.
///HIPAR access/Defines the access right a user can have.
///HIPAR level/Defines the security level.
///HIPAR authenticationPassword/String containing the authentication
///HIPAR authenticationPassword/password.
///HIPAR privacyPassword/String containing the privacy password.
ViStatus _VI_FUNC rsrfsiggen_AddHUMSSNMPUser(ViSession instrSession,
                                             ViString  name,
                                             ViInt32   access,
                                             ViInt32   level,
                                             ViString  authenticationPassword,
                                             ViString  privacyPassword)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  accessRngVal;
    ViChar*  levelRngVal;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, access, RSRFSIGGEN_VAL_HUMS_SNMP_ACCESS_RIGHTS_RO, RSRFSIGGEN_VAL_HUMS_SNMP_ACCESS_RIGHTS_RW),
            3, "Access");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngHUMSSNMPAccessRights, access, &accessRngVal),
        3, "Access");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, level, RSRFSIGGEN_VAL_HUMS_SNMP_SECURITY_LEVEL_NOA, RSRFSIGGEN_VAL_HUMS_SNMP_SECURITY_LEVEL_PRIV),
            4, "Level");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngHUMSSNMPSecurityLevel, level, &levelRngVal),
        4, "Level");


    checkErr(RsCore_LockSession(instrSession));

   switch (level)
   {
   case RSRFSIGGEN_VAL_HUMS_SNMP_SECURITY_LEVEL_NOA:
      snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "'%s', %s, %s", name, accessRngVal, levelRngVal);
      break;
   case RSRFSIGGEN_VAL_HUMS_SNMP_SECURITY_LEVEL_AUTH:
      snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "'%s', %s, %s, '%s'", name, accessRngVal, levelRngVal, authenticationPassword);
      break;
   case RSRFSIGGEN_VAL_HUMS_SNMP_SECURITY_LEVEL_PRIV:
      snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "'%s', %s, %s, '%s', '%s'", name, accessRngVal, levelRngVal, authenticationPassword, privacyPassword);
      break;
   default:
      break;
   }

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_DIAGNOSTIC_HUMS_ADD_SNMP_USER, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Queries the number of users and a list of all SNMP users for
///HINF  SNMPv3.
///HINF
///HINF  Prerequisites:
///HINF  RSRFSIGGEN_ATTR_DIAGNOSTIC_HUMS_ENHANCED_PROTOCOL_SNMP must be set to SNMPv3.
///HIRET Returns the status code of this operation.
///HIPAR bufferSize/Array size parameter.
///HIPAR count/Total number of registered SNMP users.
///HIPAR users/Array of all user names
///HIPAR returnedValues/Returns the number of values returned in the
///HIPAR returnedValues/Users.
ViStatus _VI_FUNC rsrfsiggen_QueryHUMSSNMPUsers(ViSession instrSession,
                                                ViInt32   bufferSize,
                                                ViInt32*  count,
                                                ViChar    users[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_GetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_DIAGNOSTIC_HUMS_USER_TABLE, RS_MAX_MESSAGE_BUF_SIZE, response));

    if(strchr(response, ',') != NULL)
    {
        p2buf = strtok(response, ",");

        *count = RsCore_Convert_String_To_ViInt32(p2buf);

        p2buf = strtok(NULL, "\n");

        if(returnedValues != NULL)
            *returnedValues = (ViInt32) strlen(p2buf);

        strncpy (users, p2buf, (size_t)bufferSize);
    }
    else
    {
        *count = 0;
        *returnedValues = 0;
        memset(users, 0, bufferSize);
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets a specific RGBA color for the screen.
///HIRET Returns the status code of this operation.
///HIPAR redChannel/Sets the light intesity of the red channel.
///HIPAR greenChannel/Sets the light intesity of the green channel.
///HIPAR blueChannel/Sets the light intesity of the black channel.
///HIPAR alphaChannel/Sets the alpha channel, which represents the
///HIPAR alphaChannel/transparency information.
ViStatus _VI_FUNC rsrfsiggen_ConfigureScreenColor(ViSession instrSession,
                                                  ViInt32   redChannel,
                                                  ViInt32   greenChannel,
                                                  ViInt32   blueChannel,
                                                  ViInt32   alphaChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "%d, %d, %d, %d", redChannel, greenChannel, blueChannel, alphaChannel);

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_TEST_PIXEL_RGBA_COLOR, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sends a command to trigger the RF signal generator.
/// HIFN  Call this function if you set a trigger source to a software trigger
/// HIFN  value.  Below is a list of attributes and functions which can set the
/// HIFN  trigger source to a software trigger.
/// HIFN
/// HIFN     Functions:
/// HIFN     rsrfsiggen_ConfigureArbTriggerSource
/// HIFN     rsrfsiggen_ConfigureSweep
/// HIFN     rsrfsiggen_ConfigureCDMATriggerSource
/// HIFN     rsrfsiggen_ConfigureTDMATriggerSource
/// HIFN
/// HIFN     Attributes:
/// HIFN     RSRFSIGGEN_ATTR_ARB_TRIGGER_SOURCE
/// HIFN     RSRFSIGGEN_ATTR_SWEEP_TRIGGER_SOURCE
/// HIFN     RSRFSIGGEN_ATTR_CDMA_TRIGGER_SOURCE
/// HIFN     RSRFSIGGEN_ATTR_TDMA_TRIGGER_SOURCE
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     This function does not check the instrument status.   Typically, you
/// HIFN  call this function only in a sequence of calls to other low-level
/// HIFN  driver functions.  The sequence performs one operation.  You use the
/// HIFN  low-level functions to optimize one or more aspects of interaction with
/// HIFN  the instrument.  If you want to check the instrument status, call the
/// HIFN  rsrfsiggen_error_query function at the conclusion of the sequence.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rsrfsiggen_SendSoftwareTrigger(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_INIT_SW_TRIGGER, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rsrfsiggen_readToFile(ViSession instrSession,
                                        ViString  source,
                                        ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEMory:DATA? '%s'\n", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function is used to read data from the host computer and write it
/// HIFN  to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/control computer to instrument applies.
ViStatus _VI_FUNC rsrfsiggen_writeFromFile(ViSession instrSession,
                                           ViString  source,
                                           ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument.  The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rsrfsiggen_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rsrfsiggen_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rsrfsiggen_DefaultInstrSetup(instrSession));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  Triggers an instrument reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rsrfsiggen_Preset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function runs the instrument's self test routine and returns the test result(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR testResult/Contains the value returned from the instrument self test.
/// HIPAR testMessage/Returns the self-test response string from the instrument.
ViStatus _VI_FUNC rsrfsiggen_self_test(ViSession instrSession,
                                       ViInt16*  testResult,
                                       ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, testResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, testResult, 48, testMessage));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the driver and instrument revisions.
/// HIRET /// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR driverRev/Returns the instrument driver software revision numbers in the form of a string.
/// HIPAR instrRev/Returns the instrument firmware revision numbers in the form of a string.
ViStatus _VI_FUNC rsrfsiggen_revision_query(ViSession instrSession,
                                            ViChar    driverRev[],
                                            ViChar    instrRev[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads an error code and a message from the
/// HIFN instrument's error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR errCode/Returns the error code read from the instrument's error queue.
/// HIPAR errMessage/Returns the error message string read from the instrument's error message queue.
ViStatus _VI_FUNC rsrfsiggen_error_query(ViSession instrSession,
                                         ViInt32*  errCode,
                                         ViChar    errMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errCode, 1024, errMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Returns the list of static error (if any). Static errors are generally more severe than normal error conditions, which can be queried with SYSTem:ERRor[:NEXT]?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR numberOfBytesToRead/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument and stored in the Read Buffer.
ViStatus _VI_FUNC rsrfsiggen_SystemStaticError(ViSession instrSession,
                                               ViInt32   bufferSize,
                                               ViChar    buffer[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, 1024),
            2, "Buffer Size");
    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_SYSTEM_STATIC_ERRORS,
            bufferSize, buffer),
            3, "Buffer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rsrfsiggen_error_message(ViSession instrSession,
                                           ViStatus  errorCode,
                                           ViChar    errorMessage[])
{
    return rsrfsiggen_utility_error_message(instrSession, errorCode, errorMessage);
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rsrfsiggen_ConfigureErrorChecking(ViSession instrSession,
                                                    ViBoolean optionChecking,
                                                    ViBoolean rangeChecking,
                                                    ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrfsiggen_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rsrfsiggen_IDQueryResponse(ViSession instrSession,
                                             ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_QueryViString(instrSession, "*IDN?", 256, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rsrfsiggen_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rsrfsiggen_QueryOPC(ViSession instrSession,
                                      ViInt32*  OPC)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsrfsiggen_QueryViInt32(instrSession, "*OPC?", OPC));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rspwrmeter_init or
/// HIPAR instrSession/rspwrmeter_InitWithOptions function.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rsrfsiggen_SetTimeout(ViSession instrSession,
                                        ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32)timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rsrfsiggen_GetTimeout(ViSession instrSession,
                                        ViUInt32* timeout)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  timeoutInt;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, &timeoutInt));
    *timeout = (ViUInt32)timeoutInt;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rsrfsiggen_SetVISATimeout(ViSession instrSession,
                                            ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rsrfsiggen_GetVISATimeout(ViSession instrSession,
                                            ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function calibrates the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR path/The path for which is this function performed.
/// HIPAR timeout/Sets the timeout.
/// HIPAR result/Returns the calibration result.
ViStatus _VI_FUNC rsrfsiggen_Calibrate(ViSession instrSession,
                                       ViInt32   path,
                                       ViInt32   timeout,
                                       ViInt32*  result)
{
    ViStatus error = VI_SUCCESS;
    ViInt32  old_timeout = -1;
             path;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, timeout, 0, 600000),
            2, "Timeout");

    checkErr(rsrfsiggen_GetOPCTimeout(instrSession, &old_timeout));
    checkErr(rsrfsiggen_SetOPCTimeout(instrSession, timeout));
    viCheckParm(rsrfsiggen_GetAttributeViInt32(instrSession, NULL, RSRFSIGGEN_ATTR_CALIBRATION_ALL, result),
            3, "Result");

Error:
    if (old_timeout >= 0)
        rsrfsiggen_SetOPCTimeout(instrSession, old_timeout);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the date of the last factory calibration.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR calibrationDate/Returns the date of the last factory calibration.
ViStatus _VI_FUNC rsrfsiggen_LastFactoryCalibration(ViSession instrSession,
                                                    ViInt32   bufferSize,
                                                    ViChar    calibrationDate[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, 1024),
            2, "Buffer Size");
    viCheckParm(rsrfsiggen_GetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_LAST_FACTORY_CALIBRATION,
            bufferSize, calibrationDate),
            3, "Calibration Date");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function locks or unlocks the front panel keyboard or manual
/// HIFN  operation via the display the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR operation/Locks or unlocks the front panel keyboard or manual operation via the
/// HIPAR operation/display the instrument.
/// HIPAR lockState/Memory protection state.
ViStatus _VI_FUNC rsrfsiggen_DisplayLock(ViSession instrSession,
                                         ViInt32   operation,
                                         ViBoolean lockState)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attribute = 0;

    checkErr(RsCore_LockSession(instrSession));

    switch (operation)
    {
    case RSRFSIGGEN_VAL_SYSTEM_LOCK_DISPLAY:
        attribute = RSRFSIGGEN_ATTR_SYSTEM_DISPLAY_LOCK;
        break;

    case RSRFSIGGEN_VAL_SYSTEM_LOCK_KEYBOARD:
        attribute = RSRFSIGGEN_ATTR_SYSTEM_KEYBOARD_LOCK;
        break;

    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, operation), 2, "Operation");
        break;
    }
    if (attribute != 0)
    {
        viCheckParm(rsrfsiggen_SetAttributeViBoolean(instrSession, "", attribute, lockState),
                3, "Lock State");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  To unlock the user interface for manual operation, use one of the
/// HIFN  following:
/// HIFN
/// HIFN     - On the instrument's keypad or external keyboard, press any key.
/// HIFN     The instrument prompts you to enter the security password for
/// HIFN  unlocking.
/// HIFN
/// HIFN     Note: The character of the first key you pressed is immediately added
/// HIFN  in the input field. Delete the entry before inserting the password.
/// HIFN     Enter the security password 123456.
/// HIFN
/// HIFN     - In remote control mode, send the command SYST:ULOC ENABled to
/// HIFN  release all locks at once.
/// HIFN     Alternatively:
/// HIFN     - Send the command SYST:KLOC OFF to unlock the keyboard and
/// HIFN  touchscreen
/// HIFN     - Send the command SYST:DLOC OFF to release all locks.
/// HIFN
/// HIFN     Via remote control, there is no password required.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR userInterface/Locks or unlocks the user interface of the instrument.
ViStatus _VI_FUNC rsrfsiggen_LockUserInterface(ViSession instrSession,
                                               ViInt32   userInterface)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsrfsiggen_SetAttributeViInt32(instrSession, "", RSRFSIGGEN_ATTR_SYSTEM_USER_INTERFACE_LOCK, userInterface),
            2, "User Interface");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rsrfsiggen_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rsrfsiggen_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                         ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rsrfsiggen_ConfigureMultiThreadLocking(ViSession instrSession,
                                                         ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN This function configures attribute data type mismatch reporting
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR state/Enables/disables data type mismatch reporting.
ViStatus _VI_FUNC rsrfsiggen_ConfigureAttributeDataTypeMismatchReporting(
                            ViSession instrSession,
                            ViBoolean state)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->reportAttrDataTypeMismatch = state;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches OPC-synchronization ON/OFF for writes.
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for writes.
ViStatus _VI_FUNC rsrfsiggen_OPCSyncWriteEnable(ViSession instrSession,
                                                ViBoolean enable)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrWrites = enable;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Switches OPC-synchronization ON/OFF for queries.
///HIRET Returns the status code of this operation.
///HIPAR enable/Switches OPC-synchronization ON/OFF for queries.
ViStatus _VI_FUNC rsrfsiggen_OPCSyncQueryEnable(ViSession instrSession,
                                                ViBoolean enable)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->opcSyncAttrQueries = enable;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Sets the mechanism which is used for OPC-synchronised queries
///HINF  (valid for read attributes and all OPC-synchronised queries):
///HINF  Default value: 'useEsb' = true, 'clsBefore' = true, 'useMav' =
///HINF  false
///HINF
///HINF  Scenario A: useEsb = true:
///HINF  The query command is appended by the ';*OPC', and the wait
///HINF  loop ends if the ESB bit signals that the command was completed,
///HINF  or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore has no effect and is always set to true.
///HINF      - set the 'useMav' to true, to enable the opc-wait loop
///HINF  sucessfully finish if the MAV bit signals, that the response is
///HINF  available, even if the ESB bit is still 0.
///HINF
///HINF  Scenario B: useEsb = false:
///HINF  The query command is not appended by the ';*OPC', and the wait
///HINF  loop ends if the MAV bit signals that the message is
///HINF  available, or the ERRQ (bit 2) signals an error.
///HINF      - clsBefore causes sending query *STB? before sending the
///HINF  command to clear the error queue and output buffers.
///HINF      - useMav has no effect, and is always set to true.
///HINF
///HIRET Returns the status code of this operation.
///HIPAR useEsb/Switches queries OPC-synchronization to use the Event
///HIPAR useEsb/Status Bit.
///HIPAR clsBefore/Switches queries OPC-synchronization to use call
///HIPAR clsBefore/the Clear Before Read procedure before sending the
///HIPAR clsBefore/query.
///HIPAR useMav/Switches queries OPC-synchronization to react on
///HIPAR useMav/Message Available bit (MAV)
ViStatus _VI_FUNC rsrfsiggen_SetOpcSyncQueryMechanism(ViSession instrSession,
                                                      ViBoolean useEsb,
                                                      ViBoolean clsBefore,
                                                      ViBoolean useMav)
{
    return RsCore_SetOpcSyncQueryMechanism(instrSession, useEsb, clsBefore, useMav);
}

/// HIFN  Activates volatile mode, so that no user data can be written to the
/// HIFN  internal memory permanently.
/// HIFN
/// HIFN     To enable volatile mode, reboot the instrument. Otherwise the change
/// HIFN  has no effect.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR password/Current security password.
/// HIPAR memoryProtectionState/Memory protection state.
ViStatus _VI_FUNC rsrfsiggen_SecurityVolatileMode(ViSession instrSession,
                                                  ViString  password,
                                                  ViBoolean memoryProtectionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":SYST:SEC:VOLM '%s',%s", password, memoryProtectionState ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the automatic signature verification for firmware installation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR password/Current security password.
/// HIPAR updatePolicy/Type of the update policy.
ViStatus _VI_FUNC rsrfsiggen_SecurityUpdatePolicy(ViSession instrSession,
                                                  ViString  password,
                                                  ViInt32   updatePolicy)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];
    ViChar*  p2updatePolicyString;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &RsRFSigGen_rngSecurityUpdatePolicy, updatePolicy, &p2updatePolicyString),
        2, "Update Policy");

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\'%s\',%s", password, p2updatePolicyString);

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_SECURITY_UPDATE_POLICY, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sanitizes the internal memory.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RsRfSigGen_init or
/// HIPAR instrSession/RsRfSigGen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR password/Current security password.
/// HIPAR memoryProtectionState/Protection state
ViStatus _VI_FUNC rsrfsiggen_SecuritySanitize(ViSession instrSession,
                                              ViString  password,
                                              ViBoolean memoryProtectionState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   param[RS_PARAM_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    (void)snprintf(param, RS_PARAM_BUF_SIZE, "\'%s\',%s", password, memoryProtectionState ? "ON":"OFF");

    checkErr(RsCore_SetAttributeViRawString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_SECURITY_SANITIZE, RS_VAL_DIRECT_USER_CALL, param));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Disables the LAN interface in general, including all
///HINF  services.
///HIRET Returns the status code of this operation.
///HIPAR password/Sets current security password.
///HIPAR enabled/Enables or disables the LAN interface in general.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNetworkInterface(ViSession instrSession,
                                                       ViString  password,
                                                       ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "'%s', %s", password, (enabled == VI_TRUE) ? "ON" : "OFF");

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, NULL, RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_LAN_ENABLED, params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

///HINF  Enables the supported LAN interface services individually.
///HIRET Returns the status code of this operation.
///HIPAR service/Selects the LAN interface service.
///HIPAR password/Sets current security password.
///HIPAR enabled/Enables or disables remote support for the concrete
///HIPAR enabled/LAN interface.
ViStatus _VI_FUNC rsrfsiggen_ConfigureNetworkServices(ViSession instrSession,
                                                      ViInt32   service,
                                                      ViString  password,
                                                      ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViAttr   attributes[] =
             {
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_AVAHI,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_FTP,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_HTTP,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_RAW_SOCKET_PORT,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_REMOTE_SUPPORT,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_REMOTE_PROCEDURE_CALLS,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_SMB,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_SCPI_OVER_LAN,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_SSH,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_SOFTWARE_UPDATE,
             RSRFSIGGEN_ATTR_HIDDEN_SECURITY_NETWORK_VNC
             };
    ViChar   params[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    snprintf(params, RS_MAX_MESSAGE_BUF_SIZE, "'%s', %s", password, (enabled == VI_TRUE) ? "ON" : "OFF");

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, service, RSRFSIGGEN_VAL_LAN_SERVICES_AVAHI, RSRFSIGGEN_VAL_LAN_SERVICES_VNC),
            3, "Access");

    checkErr(rsrfsiggen_SetAttributeViString(instrSession, NULL, attributes[service], params));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function sets the Enable, NTransition, and PTransition bits
/// HIFN of the Main status questionable and operating registers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR enable/This control sets the enable bits of the status register.
/// HIPAR PTransition/This control sets the PTransition bits (edge detectors) of the
/// HIPAR PTransition/status register.
/// HIPAR NTransition/This control sets the NTransition bits (edge detectors) of the
/// HIPAR NTransition/status register.
ViStatus _VI_FUNC rsrfsiggen_SetStatusRegister(ViSession instrSession,
                                               ViInt32   registerOperation,
                                               ViInt32   questionableRegister,
                                               ViInt32   enable,
                                               ViInt32   PTransition,
                                               ViInt32   NTransition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    switch (registerOperation)
    {
    case RSRFSIGGEN_VAL_STAT_REG_PRESET:
        checkErr(rsrfsiggen_SetAttributeViString(instrSession, "", RSRFSIGGEN_ATTR_STATUS_PRESET, NULL));
        break;
    case RSRFSIGGEN_VAL_STAT_REG_QUESTIONABLE:
        switch (questionableRegister)  // NOLINT(hicpp-multiway-paths-covered)
        {
        case RSRFSIGGEN_VAL_STAT_REG_MAIN:
            viCheckParm(RsCore_InvalidViInt32Range(instrSession, enable, 0, 65535),
                    4, "Enable");
            viCheckParm(RsCore_InvalidViInt32Range(instrSession, PTransition, 0, 65535),
                    5, "PTransition");
            viCheckParm(RsCore_InvalidViInt32Range(instrSession, NTransition, 0, 65535),
                    6, "NTransition");

            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:QUES:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition, NTransition);
            checkErr(RsCore_Write(instrSession, cmd));
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 3, "Questionable Operation");
        }
        break;
    case RSRFSIGGEN_VAL_STAT_REG_OPERATION:
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, enable, 0, 65535),
                4, "Enable");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, PTransition, 0, 65535),
                5, "PTransition");
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, NTransition, 0, 65535),
                6, "NTransition");

        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "STAT:OPER:ENAB %ld;PTR %ld;NTR %ld", enable, PTransition, NTransition);
        checkErr(RsCore_Write(instrSession, cmd));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function contains the commands for the querying status
/// HIFN reporting system.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR registerOperation/Specifies the operation to perform.
/// HIPAR questionableRegister/This control selects the registers that will be configured.
/// HIPAR registerPart/Sets the register part.
/// HIPAR registerValue/Returns content of selected register.
ViStatus _VI_FUNC rsrfsiggen_GetStatusRegister(ViSession instrSession,
                                               ViInt32   registerOperation,
                                               ViInt32   questionableRegister,
                                               ViInt32   registerPart,
                                               ViInt32*  registerValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString statusRegister[] = { "COND","EVEN" };

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, registerPart, 0, 1),
            4, "Register Part");

    switch (registerOperation)
    {
    case RSRFSIGGEN_VAL_STAT_REG_QUESTIONABLE:
        switch (questionableRegister)  // NOLINT(hicpp-multiway-paths-covered)
        {
        case RSRFSIGGEN_VAL_STAT_REG_MAIN:
            snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:QUES:%s?", statusRegister[registerPart]);
            checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 3, "Questionable Operation");
        }

        break;
    case RSRFSIGGEN_VAL_STAT_REG_OPERATION:
        snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":STAT:OPER:%s?", statusRegister[registerPart]);
        checkErr(RsCore_QueryViInt32(instrSession, cmd, registerValue));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, registerOperation), 2, "Register Operation");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function retrieves and then clears the IVI error information for
/// HIFN  the session or the current execution thread. One exception exists: If
/// HIFN  the BufferSize parameter is 0, the function does not clear the error
/// HIFN  information. By passing 0 for the buffer size, the caller can ascertain
/// HIFN  the buffer size required to get the entire error description string and
/// HIFN  then call the function again with a sufficiently large buffer.
/// HIFN
/// HIFN     If the user specifies a valid IVI session for the instrSession
/// HIFN  parameter, Get Error retrieves and then clears the error information
/// HIFN  for the session.  If the user passes NULL for the instrSession
/// HIFN  parameter, this function retrieves and then clears the error
/// HIFN  information for the current execution thread.  If the instrSession
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error. Normally, the error information describes the first error
/// HIFN  that occurred since the user last called rsrfsiggen_GetError or
/// HIFN  rsrfsiggen_ClearError.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Description parameter.
/// HIPAR description/Returns the error description for the IVI session or execution thread.
/// HIPAR description/ If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rsrfsiggen_GetError(ViSession instrSession,
                                      ViStatus* errorCode,
                                      ViInt32   bufferSize,
                                      ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rsrfsiggen_utility_error_message, errorCode, bufferSize, description));

Error:
    // Unlock Session
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rsrfsiggen_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}

/// HIFN This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rsrfsiggen_WriteInstrData(ViSession instrSession,
                                            ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
ViStatus _VI_FUNC rsrfsiggen_QueryViBoolean(ViSession  instrSession,
                                            ViString   command,
                                            ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, command, value));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViInt32 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value.
ViStatus _VI_FUNC rsrfsiggen_QueryViInt32(ViSession instrSession,
                                          ViString  command,
                                          ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, command, value));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViReal64 value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value.
ViStatus _VI_FUNC rsrfsiggen_QueryViReal64(ViSession instrSession,
                                           ViString  command,
                                           ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, command, value));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViString value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Value parameter.
/// HIPAR value/Returns the string value.
ViStatus _VI_FUNC rsrfsiggen_QueryViString(ViSession instrSession,
                                           ViString  command,
                                           ViInt32   bufferSize,
                                           ViChar    value[])
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, command, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLength(instrSession, command, &pbuffer));

    checkErr(rsrfsiggen_CheckStatus(instrSession));

    checkErr(RsCore_CopyToUserBufferAsciiData(value, bufferSize, pbuffer));

Error:
    if (pbuffer) free(pbuffer);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to set the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrfsiggen_SetAttributeViInt32(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to set the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrfsiggen_SetAttributeViInt64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViInt64   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to set the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrfsiggen_SetAttributeViReal64(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to set the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrfsiggen_SetAttributeViString(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to set the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rsrfsiggen_SetAttributeViBoolean(ViSession instrSession,
                                                   ViString  repeatedCapabilityName,
                                                   ViAttr    attributeId,
                                                   ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViInt32 variable.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViInt32(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViInt64 variable.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViInt64(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViAttr    attributeId,
                                                 ViInt64*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViInt64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViReal64 variable.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViReal64(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR bufSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute.  The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViString(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViAttr    attributeId,
                                                  ViInt32   bufSize,
                                                  ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringOrRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

// This function queries the value of a ViRawString attribute.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViRawString(ViSession instrSession,
                                                     ViString  repeatedCapabilityName,
                                                     ViAttr    attributeId,
                                                     ViInt32   bufSize,
                                                     ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
        attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

// Read string attribute with dynamic buffer length
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViStringUnknownLen(ViSession instrSession,
                                                            ViString  repeatedCapabilityName,
                                                            ViAttr    attributeId,
                                                            ViString* outValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViStringUnknownLen(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, outValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

// Read raw string attribute with dynamic buffer length
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViRawStringUnknownLen(
                            ViSession instrSession,
                            ViString  repeatedCapabilityName,
                            ViAttr    attributeId,
                            ViString* outValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawStringUnknownLen(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, outValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR repeatedCapabilityName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR repeatedCapabilityName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR repeatedCapabilityName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViBoolean variable.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeViBoolean(ViSession  instrSession,
                                                   ViString   repeatedCapabilityName,
                                                   ViAttr     attributeId,
                                                   ViBoolean* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rsrfsiggen_GetAttributeRepCapName(ViSession instrSession,
                                                    ViAttr    attributeId,
                                                    ViString  repCapNameId,
                                                    ViInt32   index,
                                                    ViInt32   bufferSize,
                                                    ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rsrfsiggen_GetAttributeRepeatedCapabilityIds(
                            ViSession instrSession,
                            ViAttr    attributeID,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}

/// HIFN This function returns the repeated capability name(s) that
/// HIFN belongs to the attribute and single repeated capability id you
/// HIFN may specify. More then one name is listed in comma separated
/// HIFN string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace, this parameter defines the Id of the repeated capability on which to query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdNames/The buffer in which the function returns the repeated capability id name(s) belonging to the attribute.
ViStatus _VI_FUNC rsrfsiggen_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViAttr    attributeId,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeId,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}

/// HIFN  This function performs the following operations:
/// HIFN
/// HIFN     - Closes the instrument I/O session.
/// HIFN
/// HIFN     - Destroys the instrument driver session and all of its attributes.
/// HIFN
/// HIFN     - Deallocates any memory resources the driver uses.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) You must unlock the session before calling rsrfsiggen_close.
/// HIFN
/// HIFN     (2) After calling rsrfsiggen_close, you cannot use the instrument
/// HIFN  driver again until you call rsrfsiggen_init or
/// HIFN  rsrfsiggen_InitWithOptions.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsrfsiggen_init or
/// HIPAR instrSession/rsrfsiggen_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rsrfsiggen_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
